/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** 
    This file defines pragmas for BIAS WIN32 compilation.

    It is used as a central place to disable some warnings for pedantic W4 compilation 
    instead of spreading pragmas over all source files directly
    (e.g. minpack and other dirty code)
    Please include it *only* in CPP files directy to avoid side effects as
    #include <Base/Common/BIASpragma.hh>

    If you need it in *header* files, use: 
    \verbatim
      #include <Base/Common/BIASpragmaStart.hh>
      //... the bad code, bad foreign headers etc...
      #include <Base/Common/BIASpragmaEnd.hh>
    \endverbatim
    to avoid sie effects with client code.

    You can switch on/off each individual warning messages globally, here
    while "dirty code" exists in BIAS/MIP.
    This way you can compile your own project with W4
    without taking care of possible internal problems of BIAS or third party libraries including warning noise from STL.

    And if you want to improve BIAS, just enable warning one by one and fix them in the code.

    The idea is to enable warnings one by one until all BIAS/MIP are finally fixed.

    @author Jan Woetzel 08/2005
*/

#if defined(WIN32) && defined(_MSC_VER)

/* suppress STL warning noise (Jan Woetzel 2005)
 see:  http://www.codeproject.com/vcpp/stl/stl_without_warnings.asp?df=100&forumid=2917&exp=0&select=114698&tid=80903
 warning numbers get enabled in yvals.h: */
/* #include <yvals.h>  */

#ifndef _CRT_SECURE_NO_DEPRECATE
#  define _CRT_SECURE_NO_DEPRECATE
#endif

#pragma warning (disable: 4005)
/* #pragma warning (disable: 4018) */ /* signed/unsigned mismatch */
#pragma warning (disable: 4056) /* overflow in floating-point constant arithmetic*/
#pragma warning (disable: 4099) /* .pdb not found, no debugging information (not catchable in source) */
#pragma warning (disable: 4100) /* unreferenced formal parameter */
#pragma warning (disable: 4127) /* conditional expression is constant */
#pragma warning (disable: 4131)
#pragma warning (disable: 4204)
#pragma warning (disable: 4210)
#pragma warning (disable: 4221) /* Linker warning: no public symbols found, members will be inaccessible */
#pragma warning (disable: 4231)
#pragma warning (disable: 4238)
#pragma warning (disable: 4244)
#pragma warning (disable: 4245) /* conversion from 'type1' to 'type2', signed/unsigned mismatch */
#pragma warning (disable: 4251) /* missing dll interface */
#pragma warning (disable: 4275)
#pragma warning (disable: 4290)
#pragma warning (disable: 4305)
#pragma warning (disable: 4310)
#pragma warning (disable: 4355) /*'this' : used in base member initializer list */
#pragma warning (disable: 4389)
#pragma warning (disable: 4309) /*truncation of constant value */
#pragma warning (disable: 4505)
#pragma warning (disable: 4511)
#pragma warning (disable: 4512) /* 'class' : assignment operator could not be generated */
#pragma warning (disable: 4530)
/* #pragma warning (disable: 4661) */  /* no suitable def. for explicit instantiation */
/* #pragma warning (disable: 4663) */  /* C++ language change: to explicitly specialize class template 'vector' */
#pragma warning (disable: 4701)
#pragma warning (disable: 4702) /* unreachable code */
#pragma warning (disable: 4706)
/* #pragma warning (disable: 4710) */ /* function not inlined */
/* #pragma warning (disable: 4786) */ /* identifier was truncated to 'number' characters in the debug information */
/* #pragma warning (disable: 4798) */ /* linker tool confict (debug/release, threadde/single etc) (not catchable in source) */
/* #pragma warning (disable: 4799) */ /* .pdb not found, no debugging information (not catchable in source) */
#pragma warning (disable: 4996) /* deprecated warning (for VS2005) */

#endif /* WIN32 & MSC */
