#include "ProgressSemaphore.hh"

#include <Base/Debug/Error.hh>

using namespace BIAS;

ProgressSemaphore::ProgressSemaphore()
{
#ifdef BIAS_HAVE_PTHREADS
	pthread_mutex_init(&Lock_, NULL);
#endif
	Reset();
}


ProgressSemaphore::~ProgressSemaphore()
{}
 

float ProgressSemaphore::GetProgress()
{
  float ratio;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  if (NumSteps_>0) 
    ratio = float(CurrentStep_)/float(NumSteps_);
  else
    ratio=0.0f;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
  BIASASSERT(ratio>=0. && ratio<=1.);
  return ratio;
}
 

void ProgressSemaphore::SetNumSteps(unsigned int steps)
{
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  NumSteps_ = steps;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
}
 

void ProgressSemaphore::SetCurrentStep(unsigned int currentstep)
{
  BIASASSERT(currentstep<=NumSteps_);
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  CurrentStep_ = currentstep;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
}

void ProgressSemaphore::IncrementStep()
{
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  CurrentStep_++;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
  BIASASSERT(CurrentStep_<=NumSteps_);
}


void ProgressSemaphore::SetTerminateFlag()
{
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  Terminate_ = true;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
}

void ProgressSemaphore::ResetTerminateFlag()
{
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  Terminate_ =false;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
}

bool ProgressSemaphore::IsTerminationRequested()
{
  bool flag;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  flag = Terminate_;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
  return flag;
}

void ProgressSemaphore::Reset()
{
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_lock(&Lock_);
#endif
  NumSteps_ = 0;
  CurrentStep_ = 0;
  Terminate_ = false;
#ifdef BIAS_HAVE_PTHREADS
  pthread_mutex_unlock(&Lock_);
#endif
}
