#ifndef _CycleCounter_hh_64825420763_
#define _CycleCounter_hh_64825420763_



namespace BIAS {



  /** @brief processor cycles Time Stamp Counter  on >= Pentium like CPU's. 

  High resolution CPU timing information based on x86 assembly rdtsc command.
  Works for single core machines. 
  Please note that the cycle counter may be unreliable 
  -with speed step etc. on notebooks 
  -with multicore/multiprocessor systems as the cyclec ounters may not be in sync
   and the schneduler may move thread execution between processing kernels.


  Windows:
  Assembler function without prolog and epilog. 
  Should work on Intel Pentium and newer AMD CPU's, see rdtsc assembler instruction. 
  Return the number of clock cycles since the CPU was powered up or reset. 
  This functions can't be class member inlined because it collides with naked fastcall modifier. 
  int64 has no ANSI equivalent.

  Linux: 
  Same approach avoiding int64 usage.

  See http://en.wikipedia.org/wiki/RDTSC

  \todo: For x64 see http://www3.intel.com/cd/ids/developer/asmo-na/eng/257129.htm

  @author Felix Woelk(Linux), Jan Woetzel(Windows) */

#ifdef WIN32
#ifdef _WIN64
	__forceinline __int64 __fastcall GetTSC(){ return __rdtsc(); };
#else
    __forceinline __int64 __fastcall GetTSC(){ __asm rdtsc };
#endif

  /** @brief compute the difference between two int64 cycle counts using 12 bit overflow mask .
  @author Jan Woetzel 12/2004 */
  __forceinline __int64 
    GetTSCTicks(__int64 start, __int64 end) 
  {
    return ((end - start) & 0xFFFFFFFFFFFF);
  };

#else// WIN32


  inline 
    void GetTSC_hilo(unsigned int *hi, unsigned int *lo)
  {
#if !defined(WIN32) && defined(__i386__)

    asm("rdtsc; movl %%edx,%0; movl %%eax,%1"  /* Read cycle counter */
      : "=r" (*hi), "=r" (*lo)                 /* and move results to */
      : /* No input */                         /* the two outputs */
    : "%edx", "%eax");
#else
    *hi = *lo = 0; // dummy implementation
#endif
  }

#endif // WIN32

} // namespace


#endif
