/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @example ExampleDebug.cpp
    @class ExampleDebug
    @relates Debug
    @brief demonstrates the new usage of debuglevels   
    the new concept avoids multiple definitions of debuglevels.
    Example:

    // file father.hh
    #define DL_FATHER 0x1
    class father{
      foo(){ BIASCDOUT(DL_FATHER, "bla");}
    };
    // file child.hh
    #include "father.hh"
    #define DL_CHILD 0x1
    class child : public father
    {
      foo2(){ foo(); BIASCDOUT(DL_CHILD, "fasel"); }
    };
      
    calling 
      child.SetDebugLevel(DL_CHILD);
      child.foo2();
    
    results in the unexpected debug output "bla".
    To avoid this use new debugconcept:
    @ingroup g_examples
    @author woelk 09/2006 */

#include <Base/Debug/Debug.hh>

using namespace BIAS;
using namespace std;


class myclass : public Debug
{
public:
  myclass() : myint(0) 
  {
    // tell the class abouts its debuglevels
    NewDebugLevel("DL1");
    NewDebugLevel("DL2");
  }

  inline ~myclass() {}

  void foo(int arg){
    // use new debug output macro 
    BCDOUT(DL1, "foo: arg = "<<std::dec<<arg<<endl);
    myint += arg;
    BCDOUT(DL2, "foo: myint = "<<myint<<endl);
  }
protected:
  int myint; ///< some internal variable
};



int main(int argc, char *argv[])
{
  myclass mc;
  
  cout << "show me the available debug levels: "<<endl;
  mc.ShowDebugLevel();


  cout << "without debug output: "<<endl;
  mc.foo(5);
  cout << "with debuglevel \"DL1\": "<<endl;
  mc.AddDebugLevel("DL1");
  mc.foo(7);
  cout << "with debuglevel \"DL1\" and \"DL2\": "<<endl;
  mc.AddDebugLevel("DL2");
  mc.foo(-42);
  cout << "again without debug output: "<<endl;
  mc.SetDebugLevel(0);
  mc.foo(666);
  cout << "with debuglevel \"DL2\": "<<endl;
  mc.AddDebugLevel("DL2");
  mc.foo(-42);
  cout << "print current debug level: "<<endl;
  mc.PrintDebugLevel();
 

  return 0;
}
