#ifndef __EuclideanTransf3D_hh__
#define __EuclideanTransf3D_hh__

#include "Base/Math/Matrix4x4.hh"
#include "Base/Math/Vector3.hh"
#include "Base/Geometry/RMatrixBase.hh"

namespace BIAS {

  /** @class EuclideanTransf3D
      @ingroup g_geometry
      @brief Euclidean transformation for 3D points
      @author woelk 08/2004 */
  class BIASGeometryBase_EXPORT EuclideanTransf3D : public Matrix4x4<double>
  {
  public:
    EuclideanTransf3D();

    inline EuclideanTransf3D(const Matrix4x4<double>& m)
    { Set(m); }
    
    ~EuclideanTransf3D();

    void Set(const Matrix4x4<double>& m);

    void Set(const RMatrixBase& R, const Vector3<double>& t);
    
    void SetXYZ(const Vector3<double>& EulerAng, const Vector3<double>& t);

    void GetR(RMatrixBase& R) const;

    void GetT(Vector3<double>& T) const;

    void GetAnglesXYZ(Vector3<double>& angles) const;

    void Dump(std::ostream& os = std::cout) const;

  protected:
    

  }; // class

} // namespace



#endif // __EuclideanTransf3D_hh__
