/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __TrifocalTensorBase_hh__
#define __TrifocalTensorBase_hh__

#include <Base/Math/Tensor3D3x3x3.hh>
#include <Base/Math/Matrix3x3.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Geometry/RMatrixBase.hh>
#include <Base/Geometry/PMatrixBase.hh>
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Geometry/HomgPoint3D.hh>

#define TRIFOCALTENSOR_TYPE double

namespace BIAS {

  class BIASGeometryBase_EXPORT TrifocalTensorBase
    : public Tensor3D3x3x3<TRIFOCALTENSOR_TYPE>
  {
  public:
    TrifocalTensorBase();
    TrifocalTensorBase(const TrifocalTensorBase& t);

    ~TrifocalTensorBase();

    void Dump();

    /** composition for "normalized" (with K^-1) image points */
    void Compose(RMatrixBase& R1, RMatrixBase& R2, RMatrixBase& R3,
                 Vector3<double>& C1, Vector3<double>& C2, 
                 Vector3<double>& C3);
    void Compose(RMatrixBase& R1, RMatrixBase& R2, RMatrixBase& R3,
                 HomgPoint3D& C1, HomgPoint3D& C2, HomgPoint3D& C3);

    /** composition for R1=I and C1=0 */
    void Compose(RMatrixBase& R2, RMatrixBase& R3, Vector3<double>& C2, 
                 Vector3<double>& C3);
    void Compose(RMatrixBase& R2, RMatrixBase& R3,
                 HomgPoint3D& C2, HomgPoint3D& C3);
   
    void CheckPointCorr(HomgPoint2D& p1, HomgPoint2D& p2, HomgPoint2D& p3,
                        Matrix3x3<double>& res);
    


    TrifocalTensorBase& operator=(const TrifocalTensorBase& t);

  protected:
    
  }; // class


} // namespace

#endif // __TrifocalTensorBase_hh__
