/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @example ExampleIpl.cpp
   @class ExampleIpl ImageConvert
   @brief OpenCV/IPL image format acces.
   @ingroup g_examples
   @author Thomsen / Woetzel
*/

#include <iostream>

#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#ifdef BIAS_HAVE_OPENCV
#  include <highgui.h>
#endif

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
#ifdef BIAS_HAVE_OPENCV
  Image<unsigned char> im, im2;
  IplImage *dst=NULL;

  if (argc!=2){
    BIASERR(argv[0]<<" <image-file>");
    return -1;
  }
  
  if (ImageIO::Load(argv[1], im)!=0){
    BIASERR("error loading "<<argv[1]<<" (Load)");
    return -2;
  }

  if (ImageConvert::BIAS2ipl(im, dst)!=0){
    BIASERR("error converting bias -> ipl");
    return -2;
  }

  cvNamedWindow( "window", 1 );
  cvShowImage( "window", dst );
  const int delay=3000; 
  //cvWaitKey(0);
  std::cout<<"will close gui window after "<<delay<<" msecs."<<endl;
  cvWaitKey(4000); // msec
  std::cout<<"closed window."<<endl;

  if (ImageConvert::Ipl2BIAS(dst, im2)!=0){
    BIASERR("error converting ipl -> bias");
    return -2;
  }
  
  //if (ImageIO::Save("out_bias-ipl-bias.mip", im2)!=0){
  if (ImageIO::Save("out_bias-ipl-bias.mip", im2)!=0){
    BIASERR("error writing bias-ipl-bias.mip");
    return -1;
  }
#else
    std::cout<<"you compiled without OpenCV."<<endl;
#endif
  return 0;
}
