/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example ExampleJPEGDecompress.cpp
   @class ExampleJPEGDecompress
   @relates CompressJpeg
   @brief Reads a JPEG image from disk and prints image info.
   @ingroup g_examples
   @author Arne Petersen
*/

//#include <Base/Common/LeakChecking.h>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/CompressJpeg.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  if (argc != 2) {
    cout << "Usage: " << argv[0] << " <image file>" << endl;
    return -1;
  }

  int res = 0;
  BIAS::Image<unsigned char> image;
  JpegHandler jHandler;
  jHandler.Init();

  res = jHandler.ReadJPEG(argv[1]);
  if (res < 0) {
    BIASERR("Reading JPEG failed (returned "<< res <<")!");
    return -1;
  }
  res = jHandler.Decompress(image, true);
  if (res < 0) {
    BIASERR("Decompression failed (returned "<< res <<")!");
    return -1;
  }

  cout << "Image size: " << image.GetWidth() << "x"
       << image.GetHeight() << endl;
  cout << "Channel count: " << image.GetChannelCount() << endl;

  if (image.GetMetaData()) {
    cout << "Image metadata:" << endl;
    image.GetMetaData()->Dump();
  } else {
    cout << "No image metadata found." << endl;
  }

  return 0;
}
