#include "bias_config.h"

///
/// solve explicit instantiation
/// this file is included by ImageConvert.cpp
/// @author Jan Woetzel
///

#define INSTANCE_ImageConvert(type)\
template BIASImageBase_EXPORT int ImageConvert::IP_ToGrey(Image<type> &img);\
template BIASImageBase_EXPORT int ImageConvert::ToGrey(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::GetChannel(Image<type>& image, int channel, bool strip);\
template BIASImageBase_EXPORT int ImageConvert::GetChannel(const Image<type>& source, Image<type>& dest, unsigned int channel);\
template BIASImageBase_EXPORT int ImageConvert::ToRGB(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToRGBA(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToHSV(const Image<type>& source,Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToHSL(const Image<type>& source,Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::TohsL(const Image<type>& source,Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleaved(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleavedRGB(const Image<type>& red, const Image<type>& green, const Image<type>& blue, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToPlanar(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::FromInterleaved(const Image<type>& source, Image<type>& Image1, Image<type>& Image2, Image<type>& Image3);\
template BIASImageBase_EXPORT int ImageConvert::IP_RGBToGrey_(Image<type> &img);\
template BIASImageBase_EXPORT int ImageConvert::IP_YUV422ToGrey_(Image<type> &img);\
template BIASImageBase_EXPORT int ImageConvert::BGRToRGB_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::YUV422ToRGB_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::GreyToRGB_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::RGBToGrey_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::HSLToGrey_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::YUV422ToGrey_(const Image<type> &source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToPlanarYUYV422_(const Image<type>& source, Image<type>& dest);    \
template BIASImageBase_EXPORT int ImageConvert::ToPlanarRGB_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleavedYUYV422_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::ToInterleavedRGB_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::FromInterleavedYUYV422_(const Image<type>& source, Image<type>& Y, Image<type>& U, Image<type>& V);\
template BIASImageBase_EXPORT int ImageConvert::FromInterleavedRGB_(const Image<type>& source, Image<type>& R,  Image<type>& G, Image<type>& B);\
template BIASImageBase_EXPORT int ImageConvert::BayerToGrey_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::BayerToRGB_ (const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBSlow_(const Image<type>& source, Image<type>& dest);\
template BIASImageBase_EXPORT int ImageConvert::XYZToRGB(const Image<float>& source, Image<type>& dest, float scale);\
template BIASImageBase_EXPORT int ImageConvert::LUVToRGB(const Image<float>& source, Image<type>& dest, float scale);

#ifdef BIAS_HAVE_OPENCV
#define INSTANCE_ImageConvert_OPENCV(cvt)\
template BIASImageBase_EXPORT int ImageConvert::BIAS2ipl(const Image<cvt>& source, IplImage* &dest);\
template BIASImageBase_EXPORT int ImageConvert::Ipl2BIAS(IplImage* source, Image<cvt>& dest);
#else
#define INSTANCE_ImageConvert_OPENCV(cvt)
#endif

#ifdef BIAS_HAVE_DV
#define INSTANCE_ImageConvert_DV(dvtype)\
template BIASImageBase_EXPORT int ImageConvert::DVToRGB_(const Image<dvtype>& source, Image<dvtype>& dest,unsigned char quality);
#else
#define INSTANCE_ImageConvert_DV(dvtype)
#endif

// all instances for one type:
#define INSTI(atype)   INSTANCE_ImageConvert(atype) INSTANCE_ImageConvert_OPENCV(atype) INSTANCE_ImageConvert_DV(atype)

INSTI(unsigned char)
template BIASImageBase_EXPORT int ImageConvert::BayerToRGB (const Image<unsigned char>& source, Image<unsigned char>& dest, BayerDemosaicMethod method, bool flip);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBSimple_(const unsigned char* source, unsigned char* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBSimple(const unsigned char* source, unsigned char* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBBilinear_(const unsigned char* source, unsigned char* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBNearestNeighbour_(const unsigned char* source, unsigned char* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBAHD_(const unsigned char* source, unsigned char* dest, int sx, int sy, ImageBase::EColorModel tile, int bits);
template BIASImageBase_EXPORT void ImageConvert::ClearBorders_<unsigned char>(unsigned char *rgb, int sx, int sy, int w);
template BIASImageBase_EXPORT void ImageConvert::CLIP(unsigned char& in, unsigned char& out);
template BIASImageBase_EXPORT void ImageConvert::CLIP16(unsigned char& in, unsigned char& out, unsigned char bits);

INSTI(float)

#ifdef BUILD_IMAGE_INT
INSTI(int)
#endif
#ifdef BUILD_IMAGE_CHAR
INSTI(char)
#endif
#ifdef BUILD_IMAGE_SHORT
INSTI(short)
#endif
#if defined(BUILD_IMAGE_USHORT)
INSTI(unsigned short)
template BIASImageBase_EXPORT int ImageConvert::BayerToRGB (const Image<unsigned short>& source, Image<unsigned short>& dest, BayerDemosaicMethod method, bool flip);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBSimple_(const unsigned short* source, unsigned short* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBBilinear_(const unsigned short* source, unsigned short* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBNearestNeighbour_(const unsigned short* source, unsigned short* dest, int sx, int sy, ImageBase::EColorModel tile);
template BIASImageBase_EXPORT int ImageConvert::BayerToRGBAHD_(const unsigned short* source, unsigned short* dest, int sx, int sy, ImageBase::EColorModel tile, int bits);
template BIASImageBase_EXPORT void ImageConvert::ClearBorders_(unsigned short *rgb, int sx, int sy, int w);
template BIASImageBase_EXPORT void ImageConvert::CLIP(unsigned short& in, unsigned short& out);
template BIASImageBase_EXPORT void ImageConvert::CLIP16(unsigned short& in, unsigned short& out, unsigned short bits);
#endif
#ifdef BUILD_IMAGE_DOUBLE
INSTI(double)
#endif
#ifdef BUILD_IMAGE_UINT
INSTI(unsigned int)
#endif

