/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __Distribution_hh__
#define __Distribution_hh__
#include <Base/Common/BIASpragmaStart.hh>

#include <Base/Debug/Error.hh>
#include <vector>
#include <cmath>


namespace BIAS {
  
  /** @brief class for calculating mean, variance and possibly other parameters 
      @author woelk 08/2005 */
  class BIASMathBase_EXPORT Distribution {
  public:
    Distribution();

    ~Distribution();
    
    void Reset();

    void AddData(const std::vector<double>& data);

    inline unsigned GetCount()
    { return Count_; }

    inline double GetMean()
    { BIASASSERT(Count_!=0); return Sum_/(double)Count_; }

    inline double GetVariance()
    { BIASASSERT(Count_>0); 
      //double mean = GetMean(); // unused
      return (SquaredSum_ - Sum_*Sum_/(double)(Count_))/(double)(Count_-1); }

    inline double GetSigma()
    { return sqrt(GetVariance()); }

  protected:
    double Sum_;
    double SquaredSum_;
    unsigned Count_;

  }; // class


} // namespace

#include <Base/Common/BIASpragmaEnd.hh>

#endif
