#include<iostream>
#include<Base/Math/SparseUmfMatrix.hh>

using namespace std;
using namespace BIAS;

SparseUmfMatrix CreateMatrix();
SparseUmfMatrix CreateIdentityMatrix();


/**
  @example ExampleSparseUmfMatrix.cpp
  @class ExampleSparseUmfMatrix
  @relates SparseUmfMatrix
  @brief   Example for sparse matrix with umfpack
  
  @see SparseUmfMatrix.hh and umfpack userguide.pdf for more details on
       the sparse matrix methods

  @ingroup g_examples
  @author sedlazeck
  @date 06/2009
 */
int main(int argc, char* argv[])
{
	cout << "####################################################" << endl;
	cout << " SparseUmfMatrix example." << endl;
	cout << " Shows the usage of SparseUmfMatrix class" << endl;
	cout << endl;
	cout << "-------------------------- Create Example Matrix 'A'" << endl;
	// creates a test matrix
	SparseUmfMatrix A = CreateMatrix();
	// Print takes a boolean flag. If true, only first 20 entries are printed
	A.Print(true);

	cout << "--------------------------------------- Compress 'A'" << endl;
	// the created matrix is still in triplet form. Most algorithms prefer
	// the compressed column form. Column compression goes here:
	A.Compress();
	A.Print(true);

	cout << "------------------------------- Transpose 'A' to At'" << endl;
	// Create a new matrix where we want to store the transposed of our
	// test matrix. The default constructor won't allocate any data.
	SparseUmfMatrix At = SparseUmfMatrix();

	// Transpose the test matrix. Result will be stored in At. The algorithm
	// will also take care of data allocation.
	A.Transpose(At);
	At.Print(true);

	cout << "---------------------------------- Create Identity I" << endl;
	SparseUmfMatrix I = CreateIdentityMatrix();
	I.Print(true);

	cout << "--------------------------------------- Compress 'I'" << endl;
	// Again, compress the columns.
	I.Compress();
	I.Print(true);

	cout << "----------------------------------------- B = A * I" << endl;
	// Create result matrix.
	SparseUmfMatrix B = SparseUmfMatrix();

	// Calculate B = A * I
	A.Multiply(I, B);
	B.Print(true);

	return 0;
}

SparseUmfMatrix CreateMatrix()
{
	// define the dimensions of a matrix. Here a 4 by 4 matrix
	// is created that can store up to 16 elements. In this
	// example the matrix does not have any nonzero entry, so
	// numNonZeros is set to 16.
	int numRows = 4;
	int numColumns = 4;
	int numNonZeros = 16;
	// Create the matrix structure in triplet form.
	bool inTripletForm = true;

	// Called constructor will allocate space for 16 entries.
	SparseUmfMatrix A = SparseUmfMatrix(
			numRows, numColumns, numNonZeros, inTripletForm);

	// Add 16 entries to the created matrix. 'Entriy' can add
	// a value to a matrix in triplet form only. Also, it
	// does not check if a value for a given address does already
	// exist.
	A.Entry(0, 0, 1.0);
	A.Entry(0, 1, 2.0);
	A.Entry(0, 2, 3.0);
	A.Entry(0, 3, 4.0);
	A.Entry(1, 0, 5.0);
	A.Entry(1, 1, 6.0);
	A.Entry(1, 2, 7.0);
	A.Entry(1, 3, 8.0);
	A.Entry(2, 0, 9.0);
	A.Entry(2, 1, 10.0);
	A.Entry(2, 2, 11.0);
	A.Entry(2, 3, 12.0);
	A.Entry(3, 0, 13.0);
	A.Entry(3, 1, 14.0);
	A.Entry(3, 2, 15.0);
	A.Entry(3, 3, 16.0);

	// putting more than 16 elements into the matrix
	// will force the matrix to reallocate itself and
	// will double the allocated space. This is useful
	// when the number of entries is not known.

	return A;
}

SparseUmfMatrix CreateIdentityMatrix()
{
	int numRows = 4;
	int numColumns = 4;
	int numNonZeros = 4;
	bool inTripletForm = true;

	SparseUmfMatrix A = SparseUmfMatrix(
			numRows, numColumns, numNonZeros, inTripletForm);

	A.Entry(0, 0, 1.0);
	A.Entry(1, 1, 1.0);
	A.Entry(2, 2, 1.0);
	A.Entry(3, 3, 1.0);

	return A;
}
