#include<iostream>

#include <Base/Math/SparseUmfMatrix.hh>
#include <iostream>
#include <Base/Math/Random.hh>
#include <Base/Math/SparseMatrix.hh>
#include <MathAlgo/SVD.hh>
#include <Base/Debug/TimeMeasure.hh>

using namespace std;
using namespace BIAS;

void RunTranspose(int m, int n, double treshold);
void RunMultiply(int size, double treshold);

/**
  @example ExampleSparseUmfMatrix3.cpp
  @class ExampleSparseUmfMatrix3
  @brief   Example for sparse matrix with umfpack
  
  @see SparseMatrix.hh and umfpack userguide.pdf for more details on
       the sparse matrix methods

  @ingroup g_examples
  @author sedlazeck
  @date 06/2009
 */
int main(int argc, char* argv[])
{
	cout << "####################################################" << endl;
	cout << " SparseUmfMatrix example." << endl;
	cout << " Measures the execution time of common algorithms " << endl;
	cout << endl;
	RunTranspose(1000, 10000, 0.9);
	RunMultiply(1000, 0.9);
	return 0;
}

void RunTranspose(int m, int n, double treshold)
{
	cout << "CREATE DENSE MATRIX M=" << m << " N=" << n << endl;
	// create dense
	Matrix<double> A_dense(m, n);

	Random rand;
	double randElem;
	for(int i=0; i < m; i++){
	  for(int j=0; j < n; j++){
	    randElem = rand.GetNormalDistributed(0.0, 1.0);
	    if(randElem > treshold){
	    	A_dense[i][j] = randElem * 100.0;
	    } else {
	    	A_dense[i][j] = 0.0;
	    }
	  }
	}

	TimeMeasure tc;
	cout << "CREATE SPARSE MATRIX" << endl;
	SparseMatrix A_sparse(A_dense);
	SparseMatrix B_sparse(A_dense);


	/********************** TEST TRANSPOSE ***********************/
	cout << "START SLOW TRANSPOSE" << endl;
	tc.Start();
	A_sparse.Transpose(B_sparse, false);
	tc.Stop();
	tc.Print();

	tc.Reset();

	cout << "START FAST TRANPOSE" << endl;
	tc.Start();
	A_sparse.Transpose(B_sparse, true);
	tc.Stop();
	tc.Print();

}

void RunMultiply(int size, double treshold)
{
	int m = size;
	int n = size;
	cout << "CREATE DENSE MATRIX M=" << m << " N=" << n << endl;
	// create dense
	Matrix<double> A_dense(m, n);

	Random rand;
	double randElem;
	for(int i=0; i < m; i++){
	  for(int j=0; j < n; j++){
	    randElem = rand.GetNormalDistributed(0.0, 1.0);
	    if(randElem > treshold){
	    	A_dense[i][j] = randElem * 100.0;
	    } else {
	    	A_dense[i][j] = 0.0;
	    }
	  }
	}

	TimeMeasure tc;
	cout << "CONVERT TO SPARSE MATRIX" << endl;
	SparseMatrix A_sparse(A_dense);
	SparseMatrix B_sparse(A_dense);


	/********************** TEST MULTIPLY ***********************/
//	A_sparse.DisableSparseUmfMatrix();
//	cout << endl << "START SLOW MULTIPLICATION" << endl;
//	tc.Start();
//	A_sparse.Multiply(A_sparse, B_sparse);
//	tc.Stop();
//	tc.Print();
//
//	tc.Reset();

//	A_sparse.EnableSparseUmfMatrix(m, n);
	cout << endl << "START FAST MULTIPLICATION" << endl;
	tc.Start();
	A_sparse.Multiply(A_sparse, B_sparse);
	tc.Stop();
	tc.Print();
}
