#
# Try to find libDC1394 for IEEE1394 camera 
# Once run this will define: 
#
# DC1394_FOUND
# DC1394_INCLUDE_DIR
# DC1394_LIBRARIES
#
# Jan Woetzel
# www.mip.informatik.uni-kiel.de/~jw

IF(NOT UNIX)
  # MESSAGE("FindDC1394.cmake: libdc1394 only available for Unix.")
  SET(DC1394_FOUND FALSE)
ELSE(NOT UNIX)
  FIND_PATH(DC1394_INCLUDE_DIR libdc1394/dc1394_control.h
    $ENV{DC1394_HOME}/include
    /usr/include )
  
  FIND_LIBRARY(DC1394_LIBRARY
    NAMES dc1394_control
    PATHS 
    $ENV{DC1394_HOME}/lib
    /usr/lib
    )
  
  #MESSAGE("DC1394_V2=${DC1394_V2}")
  IF (NOT DC1394_LIBRARY OR DC1394_V2)
    #MESSAGE("Did not find libdc1394 version 1.x, searching for version 2")
    FIND_PATH(DC1394_INCLUDE_DIR dc1394/dc1394.h
      $ENV{DC1394_HOME}/include
      /usr/include )
    
    FIND_LIBRARY(DC1394_LIBRARY
      NAMES dc1394
      PATHS 
      $ENV{DC1394_HOME}/lib
      /usr/lib
      )
    IF (DC1394_LIBRARY)
      SET(DC1394_V2 TRUE CACHE BOOL "foo")
      #MESSAGE("setting DC1394_V2=true")
    ENDIF (DC1394_LIBRARY)
  ELSE (NOT DC1394_LIBRARY OR DC1394_V2)
    SET(DC1394_V2 FALSE CACHE BOOL "foo")
  ENDIF (NOT DC1394_LIBRARY OR DC1394_V2)
   
  #MESSAGE("DBG DC1394_LIBRARY=${DC1394_LIBRARY}")
  #MESSAGE("DBG DC1394_INCLUDE_DIR=${DC1394_INCLUDE_DIR}")  
 
  # --------------------------------
  
  IF(DC1394_LIBRARY)
    SET(DC1394_LIBRARIES ${DC1394_LIBRARY})
  ELSE(DC1394_LIBRARY)
    MESSAGE(STATUS "libdc1394 library not found.")
  ENDIF(DC1394_LIBRARY)
  
  IF(NOT DC1394_INCLUDE_DIR)
    MESSAGE(STATUS "libdc1394 include dir not found.")
  ENDIF(NOT DC1394_INCLUDE_DIR)
  
  IF(DC1394_LIBRARIES AND DC1394_INCLUDE_DIR)
    SET(DC1394_FOUND TRUE)
  ELSE(DC1394_LIBRARIES AND DC1394_INCLUDE_DIR)
    SET(DC1394_FOUND FALSE)
  ENDIF(DC1394_LIBRARIES AND DC1394_INCLUDE_DIR)
  
  MARK_AS_ADVANCED(
    DC1394_INCLUDE_DIR
    DC1394_LIBRARIES
    DC1394_LIBRARY
    DC1394_V2
    )
ENDIF(NOT UNIX)
