#
# try to find glut library and include files
#
# GLUT_INCLUDE_DIR, where to find GL/glut.h, etc.
# GLUT_LIBRARIES, the libraries to link against to use GLUT.
# GLUT_FOUND, If false, do not try to use GLUT.

# also defined, but not for general use are
# GLUT_glut_LIBRARY = the full path to the glut library.
# GLUT_Xmu_LIBRARY  = the full path to the Xmu library if available.
# GLUT_Xi_LIBRARY   = the full path to the Xi Library if available.
# GLUT_EXTRA_BIN_FILES = dlls to be included into installer

IF (WIN32)

  IF(CYGWIN)

    FIND_PATH( GLUT_INCLUDE_DIR GL/glut.h
      /usr/include
    )

    FIND_LIBRARY( GLUT_glut_LIBRARY glut32
      ${OPENGL_LIBRARY_DIR}
      /usr/lib
      /usr/lib/w32api
      /usr/local/lib
      /usr/X11R6/lib
    )
    
  ELSE(CYGWIN)
	
	# set from environment variable
	# if not set (Jan Woetzel Feb/2004)
	IF(NOT GLUT_ROOT_PATH)
	  SET(GLUT_ROOT_PATH $ENV{GLUT_ROOT_PATH})
	ENDIF(NOT GLUT_ROOT_PATH)
	
	
    FIND_PATH( GLUT_INCLUDE_DIR GL/glut.h
      "${GLUT_ROOT_PATH}/include"
      ${GLUT_ROOT_PATH}
      ${GLUT_DIR}/include
      "$ENV{EXTERN_LIBS_DIR}/glut/include"
      $ENV{EXTRA}/include
      $ENV{EXTRA}
      $ENV{ProgramFiles}\\Microsoft\ Visual\ Studio\ .NET\ 2003\\Vc7\\PlatformSDK\\Include\\gl
      DOC "directory where glut header GL/glut.h can be found."
    )
    
	# added glut32 in addition to glut
	# added NAMES PATHS syntax (Jan Woetzel Feb/2004)
    FIND_LIBRARY( GLUT_glut_LIBRARY 
      NAMES glut glut32
      PATHS
      ${GLUT_ROOT_PATH}/Release
      ${GLUT_ROOT_PATH}/Debug
      ${GLUT_ROOT_PATH}/lib
      ${GLUT_ROOT_PATH}
      ${GLUT_DIR}/lib
      ${OPENGL_LIBRARY_DIR}
      "$ENV{EXTERN_LIBS_DIR}/glut/lib"      
      $ENV{EXTRA}/lib
      $ENV{EXTRA}
      C:\\Program\ Files\\Microsoft\ Visual\ Studio\ .NET\ 2003\\Vc7\\PlatformSDK\\Lib
      DOC "directory where glut library can be found."
    )

  FIND_FILE( GLUT_DLL glut32.dll
    "${GLUT_ROOT_PATH}/bin"
    ${GLUT_ROOT_PATH}
    #  ${GLUT_DIR}/bin
    "$ENV{EXTERN_LIBS_DIR}/glut/bin"
    )
  SET(GLUT_EXTRA_BIN_FILES ${GLUT_DLL})
  FIND_FILE( GLU_DLL glu32.dll
    "${GLUT_ROOT_PATH}/bin"
    ${GLUT_ROOT_PATH}
    #  ${GLUT_DIR}/bin
    "$ENV{EXTERN_LIBS_DIR}/glut/bin"
    )
  LIST(APPEND GLUT_EXTRA_BIN_FILES ${GLU_DLL})
  #  MESSAGE("GLUT_ROOT_PATH=${GLUT_ROOT_PATH}")
  #  MESSAGE("GLUT_INCLUDE_DIR=${GLUT_INCLUDE_DIR}")
  #  MESSAGE("GLUT_glut_LIBRARY=${GLUT_glut_LIBRARY}")
  #MESSAGE("GLUT_EXTRA_BIN_FILES=${GLUT_EXTRA_BIN_FILES}")
  ENDIF(CYGWIN)
  
ELSE (WIN32)

  IF (APPLE)
# These values for Apple could probably do with improvement.
    FIND_PATH( GLUT_INCLUDE_DIR GL/glut.h
      ${OPENGL_LIBRARY_DIR}
    )
    SET(GLUT_glut_LIBRARY "-framework Glut" CACHE STRING "GLUT library for OSX")
  ELSE (APPLE)

    FIND_PATH( GLUT_INCLUDE_DIR GL/glut.h
      /usr/include
      /usr/include/GL
      /usr/local/include
      /usr/openwin/share/include
      /usr/openwin/include
      /usr/X11R6/include
      /usr/include/X11
      /opt/graphics/OpenGL/include
      /opt/graphics/OpenGL/contrib/libglut
    )

    FIND_LIBRARY( GLUT_glut_LIBRARY glut
      /usr/lib
      /usr/local/lib
      /usr/openwin/lib
      /usr/X11R6/lib
    )

    FIND_LIBRARY( GLUT_Xi_LIBRARY Xi
      /usr/lib
      /usr/local/lib
      /usr/openwin/lib
      /usr/X11R6/lib
    )

    FIND_LIBRARY( GLUT_Xmu_LIBRARY Xmu
      /usr/lib
      /usr/local/lib
      /usr/openwin/lib
      /usr/X11R6/lib
    )

  ENDIF (APPLE)

ENDIF (WIN32)

SET( GLUT_FOUND "NO" )
IF(GLUT_INCLUDE_DIR)
  IF(GLUT_glut_LIBRARY)
    # Is -lXi and -lXmu required on all platforms that have it?
    # If not, we need some way to figure out what platform we are on.
    SET( GLUT_LIBRARIES
      ${GLUT_glut_LIBRARY}
      ${GLUT_Xmu_LIBRARY}
      ${GLUT_Xi_LIBRARY} 
    )
    SET( GLUT_FOUND "YES" )

    # store linkdir for external use
    # TODO: Xmu, Xi linkdirs?
    IF    (GLUT_glut_LIBRARY)
      GET_FILENAME_COMPONENT(GLUT_LINK_DIRECTORIES ${GLUT_glut_LIBRARY} PATH)
    ENDIF (GLUT_glut_LIBRARY)
    #MESSAGE("DBG GLUT_LINK_DIRECTORIES=${GLUT_LINK_DIRECTORIES}")
    
    
#The following deprecated settings are for backwards compatibility with CMake1.4
    SET (GLUT_LIBRARY ${GLUT_LIBRARIES})
    SET (GLUT_INCLUDE_PATH ${GLUT_INCLUDE_DIR})
    
  ENDIF(GLUT_glut_LIBRARY)
ENDIF(GLUT_INCLUDE_DIR)

MARK_AS_ADVANCED(
  GLUT_INCLUDE_DIR
  GLUT_glut_LIBRARY
  GLUT_Xmu_LIBRARY
  GLUT_Xi_LIBRARY
)

# make FIND_PACKAGE case sensitive compatible
SET(GLUTjw_FOUND ${GLUT_FOUND})
