# -Try to find Lapack library  
# Once run this will define: 
# 
# LAPACK_FOUND
# LAPACK_LIBRARIES
# LAPACK_LINK_DIRECTORIES
# LAPCK_INCLUDE_DIRECTORIES
# 
# Friso, Jan Woetzel 05/2004
# www.mip.informatik.uni-kiel.de/~jw
# 
# --------------------------------

SET(LAPACK_POSSIBLE_LIBRARY_PATHS
  ${LAPACK_HOME}
  ${LAPACK_HOME}/lib
  $ENV{LAPACK_HOME}
  $ENV{LAPACK_HOME}/lib
  ${LAPACK_DIR}
  ${LAPACK_DIR}/lib
  $ENV{LAPACK_DIR}
  $ENV{LAPACK_DIR}/lib
    
  "$ENV{EXTERN_LIBS_DIR}/lapack/lib"  
  $ENV{EXTRA}/lib
  /usr/lib
  /usr/local/lib
  /usr/lib/gcc/i586-suse-linux/4.1.2 # libgfortran is here on Suse 10.2
  /usr/lib/gcc/i586-suse-linux/4.2.1 # libgfortran is here on Suse 10.3
  /usr/lib/gcc/i586-suse-linux/4.3 # libgfortran is here on Suse 11.1
  /usr/lib/gcc/i486-linux-gnu/4.2 # libgfortran is here on ubuntu hardy
  /usr/lib/gcc/i486-linux-gnu/4.3 # libgfortran is here on ubuntu intrepid
  /usr/lib/gcc/i486-linux-gnu/4.4 # libgfortran is here on ubuntu karmic
  /usr/lib/gcc/x86_64-linux-gnu/4.2/ # libgfortran is here on ubuntu hardy 64
  /usr/lib/gcc/x86_64-linux-gnu/4.3/ #libgfortran is here on ubuntu intrepid 64
  /usr/lib/gcc/x86_64-linux-gnu/4.4/ #libgfortran is here on ubuntu karmic 64
  /usr/lib64/gcc/x86_64-suse-linux/4.3/ #opensuse 11.1-64
  )

SET(LAPACK_POSSIBLE_INCLUDE_PATHS
  ${LAPACK_DIR}
  ${LAPACK_DIR}/include
  $ENV{LAPACK_DIR}
  $ENV{LAPACK_DIR}/include
  ${LAPACK_HOME}
  ${LAPACK_HOME}/include
  $ENV{LAPACK_HOME}
  $ENV{LAPACK_HOME}/include  
  "$ENV{EXTERN_LIBS_DIR}/lapack/include"
  $ENV{EXTRA}/include
  )



IF (WIN32)
  FIND_PATH(LAPACK_INCLUDE_DIRECTORIES Lapack.h
    PATHS ${LAPACK_POSSIBLE_INCLUDE_PATHS}
    )

FIND_PATH(LAPACK_BIN_DIR NAMES "LAPACK.dll" PATHS 
  ${LAPACK_DIR}
  ${LAPACK_DIR}/bin
  $ENV{LAPACK_DIR}
    $ENV{LAPACK_DIR}/bin
    ${LAPACK_HOME}
    ${LAPACK_HOME}/bin
    $ENV{LAPACK_HOME}
    $ENV{LAPACK_HOME}/bin
    "$ENV{EXTERN_LIBS_DIR}/blas/bin"    
    $ENV{EXTRA}/bin
    $ENV{EXTRA}
    )

ENDIF (WIN32)
#MESSAGE("DBG LAPACK_INCLUDE_DIRECTORIES=${LAPACK_INCLUDE_DIRECTORIES}")


FIND_LIBRARY(LAPACK_LIBRARY
  NAMES lapack LAPACK
  # mkl_lapack
  lapack3 liblapack.so.3 liblapack.so
  PATHS ${LAPACK_POSSIBLE_LIBRARY_PATHS}
  )
#MESSAGE("DBG LAPACK_LIBRARY=${LAPACK_LIBRARY}")


IF (UNIX)
  # f2c or g2c are needed on Linux because lapack was is f2c compiled
  # and we link "EXTERN C" without a valid c++ header (JW)
  #MESSAGE("looking for libgfortran in ${LAPACK_POSSIBLE_LIBRARY_PATHS}")
  FIND_LIBRARY(LAPACK_FORTRAN_TO_C_LIBRARY
    NAMES gfortran # gfortran for gcc4
    PATHS ${LAPACK_POSSIBLE_LIBRARY_PATHS}
    )  
  #MESSAGE("resulted in \"${LAPACK_FORTRAN_TO_C_LIBRARY}\"")

  MARK_AS_ADVANCED(LAPACK_FORTRAN_TO_C_LIBRARY)
  #MESSAGE("DBG LAPACK_FORTRAN_TO_C_LIBRARY=${LAPACK_FORTRAN_TO_C_LIBRARY}")
  IF(NOT LAPACK_FORTRAN_TO_C_LIBRARY)
    MESSAGE(FATAL_ERROR "gfortran required for LAPACK")
  ENDIF(NOT LAPACK_FORTRAN_TO_C_LIBRARY)
ENDIF(UNIX)
IF(WIN32)
  # lapack for windows needs f2c header for various declarations
  FIND_FILE(LAPACK_FORTRAN_TO_C_HEADER f2c.h
    NAMES f2c.h
    PATHS ${LAPACK_POSSIBLE_INCLUDE_PATHS}
    )
  MARK_AS_ADVANCED(LAPACK_FORTRAN_TO_C_HEADER)
  #MESSAGE("DBG LAPACK_FORTRAN_TO_C_HEADER=${LAPACK_FORTRAN_TO_C_HEADER}")
ENDIF(WIN32)


# --------------------------------

IF(LAPACK_LIBRARY)

  # OK, we've got the main" library"  
  GET_FILENAME_COMPONENT(LAPACK_LINK_DIRECTORIES ${LAPACK_LIBRARY} PATH)
  # TODO: add link directory of f2/g2 for Linux here, too? (JW)
  
  IF (UNIX AND LAPACK_FORTRAN_TO_C_LIBRARY)
    # additional g2c/f2c/... is required  on Unix (see above)
    SET(LAPACK_FOUND TRUE)
    SET(LAPACK_LIBRARIES ${LAPACK_LIBRARY} ${LAPACK_FORTRAN_TO_C_LIBRARY})
  ENDIF (UNIX AND LAPACK_FORTRAN_TO_C_LIBRARY)

  IF (WIN32 AND LAPACK_INCLUDE_DIRECTORIES AND LAPACK_FORTRAN_TO_C_HEADER)
    # On Windows mkl or VCLapack is sufficient
    SET(LAPACK_FOUND TRUE)
    SET(LAPACK_LIBRARIES ${LAPACK_LIBRARY})
  ENDIF (WIN32 AND LAPACK_INCLUDE_DIRECTORIES AND LAPACK_FORTRAN_TO_C_HEADER)

ELSE(LAPACK_LIBRARY)
  MESSAGE(
    # SEND_ERROR 
    STATUS 
    "FindLapack.cmake: Could not find LAPACK_LIBRARY")
ENDIF(LAPACK_LIBRARY)


MARK_AS_ADVANCED(
  #  LAPACK_INCLUDE_DIR
  LAPACK_LIBRARY
  LAPACK_LIBRARIES
  LAPACK_INCLUDE_DIRECTORIES
  )


# ==========================================
IF(NOT LAPACK_FOUND)
  # make FIND_PACKAGE friendly
  IF(NOT LAPACK_FIND_QUIETLY)
    IF(LAPACK_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "LAPACK required, please specify it's location.")
    ELSE(LAPACK_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: LAPACK was not found.")
    ENDIF(LAPACK_FIND_REQUIRED)
  ENDIF(NOT LAPACK_FIND_QUIETLY)
ENDIF(NOT LAPACK_FOUND)
