# 
# This module finds OpenSceneGraph (see www.openscenegraph.org/) 
# and determines where the include files and libraries are. 
##
# This code sets the following variables:
##
# OPENSCENEGRAPH_FOUND = system has OPENSCENEGRAPH lib
##
# OPENSCENEGRAPH_LIBRARIES   = full path to the libraries
##
# OPENSCENEGRAPH_INCLUDEDIR      = where to find headers 
# OPENSCENEGRAPH_BINDIR = where to find binaries 
##
# OPENSCENEGRAPH_LIBDIR = link directories, useful for rpath on Unix
# OPENSCENEGRAPH_LDFLAGS = linker flags on Unix
#
# www.mip.informatik.uni-kiel.de
##
IF(WIN32)
  ###########################  Windows ###############################  
  SET(OPENSCENEGRAPH_POSSIBLE_ROOT_PATHS 
    "$ENV{EXTERN_LIBS_DIR}/OpenSceneGraph"
    "$ENV{EXTERN_LIBS_DIR}/OpenSceneGraph-2.8.0"
    "$ENV{EXTERN_LIBS_DIR}/OpenSceneGraph-2.8.1"
    "$ENV{EXTERN_LIBS_DIR}/OpenSceneGraph-2.8.2"	
    )
  
  FIND_PATH(OPENSCENEGRAPH_ROOT_DIR include/osg/Node
    ${OPENSCENEGRAPH_POSSIBLE_ROOT_PATHS})
    
  IF(OPENSCENEGRAPH_ROOT_DIR)    
    SET(OPENSCENEGRAPH_INCLUDEDIR "${OPENSCENEGRAPH_ROOT_DIR}/include" CACHE INTERNAL "openscenegraph include dir")
    SET(OPENSCENEGRAPH_LIBDIR "${OPENSCENEGRAPH_ROOT_DIR}/lib" CACHE INTERNAL "openscenegraph lib dir")
    SET(OPENSCENEGRAPH_BINDIR "${OPENSCENEGRAPH_ROOT_DIR}/bin" CACHE INTERNAL "openscenegraph bin dir")
    SET(OPENSCENEGRAPH_LDFLAGS "" CACHE INTERNAL  "openscenegraph ldflags")
    SET(OPENSCENEGRAPH_LIBRARIES 
     debug osgd         optimized osg
     debug osgAnimationd       optimized osgAnimation
     debug osgDBd       optimized osgDB
     debug osgFXd       optimized osgFX
     debug osgGAd       optimized osgGA
     debug osgManipulatord       optimized osgManipulator
     debug osgParticled optimized osgParticle
     
     debug osgSimd      optimized osgSim
     debug osgShadowd   optimized osgShadow
     debug osgTerraind     optimized osgTerrain
     debug osgTextd     optimized osgText
     debug osgUtild     optimized osgUtil
     debug osgViewerd   optimized osgViewer
     debug osgVolumed   optimized osgVolume
     debug osgWidgetd   optimized osgWidget
     debug OpenThreadsd optimized OpenThreads
      CACHE INTERNAL "openscenegraph libraries")
	  # kicked a.. : debug osgPresentationd optimized osgPresentation
    SET(OPENSCENEGRAPH_FOUND TRUE)
  ELSE(OPENSCENEGRAPH_ROOT_DIR)
    SET(OPENSCENEGRAPH_FOUND FALSE)    
  ENDIF(OPENSCENEGRAPH_ROOT_DIR)    

ELSE(WIN32)

  IF(APPLE)
    MACRO(FIND_OSG NAME)
      FIND_LIBRARY(OSG_${NAME} ${NAME})
      IF (OSG_${NAME})
	SET(OPENSCENEGRAPH_LIBRARIES ${OPENSCENEGRAPH_LIBRARIES} ${OSG_${NAME}})
      ENDIF(OSG_${NAME})
    ENDMACRO(FIND_OSG NAME)
    
    FIND_OSG( osg)
    FIND_OSG( osgAnimation)
    FIND_OSG( osgDB)
    FIND_OSG( osgFX)
    FIND_OSG( osgGA)
    FIND_OSG( osgIntrospection)
    FIND_OSG( osgManipulator)
    FIND_OSG( osgParticle)
    FIND_OSG( osgShadow)
    FIND_OSG( osgSim)
    FIND_OSG( osgTerrain)
    FIND_OSG( osgText)
    FIND_OSG( osgUtil)
    FIND_OSG( osgViewer)
    FIND_OSG( osgVolume)
    FIND_OSG( osgWidget)
    FIND_OSG( OpenThreads)
    
    #MESSAGE("OPENSCENEGRAPH_LIBRARIES = ${OPENSCENEGRAPH_LIBRARIES}")
    IF(OPENSCENEGRAPH_LIBRARIES)
      SET(OPENSCENEGRAPH_FOUND TRUE)
    ELSE(OPENSCENEGRAPH_LIBRARIES)
      SET(OPENSCENEGRAPH_FOUND FALSE)
      MESSAGE("did not find OSG: OPENSCENEGRAPH_LIBRARIES = ${OPENSCENEGRAPH_LIBRARIES}")
    ENDIF(OPENSCENEGRAPH_LIBRARIES)

  ELSE(APPLE)
     ###########################  Linux ###############################
     INCLUDE( ${CMAKE_ROOT}/Modules/FindPkgConfig.cmake)
    
    IF (!PKG_CONFIG_FOUND)
      MESSAGE("PKG CONFIG NOT FOUND, required for finding openscenegraph")
    ENDIF(!PKG_CONFIG_FOUND)
    #MESSAGE("${CMAKE_BUILD_TYPE}")
    STRING(COMPARE EQUAL ${CMAKE_BUILD_TYPE} "Debug" IS_DEBUG)
    IF (IS_DEBUG)
      pkg_search_module (OPENSCENEGRAPH openscenegraphdbg)
      IF (NOT OPENSCENEGRAPH_FOUND)
        MESSAGE("no debug package of OpenSceneGraph found - using Release version OpenSceneGraph")
        pkg_search_module (OPENSCENEGRAPH openscenegraph)
      ENDIF (NOT OPENSCENEGRAPH_FOUND)
    ELSE (IS_DEBUG)
      pkg_search_module (OPENSCENEGRAPH openscenegraph)
    ENDIF (IS_DEBUG)
    IF (OPENSCENEGRAPH_FOUND)
      #MESSAGE("OPENSCENEGRAPH_LIBRARIES = ${OPENSCENEGRAPH_LIBRARIES}")
      SET(OPENSCENEGRAPH_FOUND TRUE)
    ELSE(OPENSCENEGRAPH_FOUND)
      MESSAGE("OpenSceneGraph not found in FindOpenSceneGraph.cmake, Line 45, check whether you find it manually:     pkg-config --debug --exists openscenegraph")
      SET(OPENSCENEGRAPH_FOUND FALSE)    
    ENDIF (OPENSCENEGRAPH_FOUND)

  ENDIF(APPLE)

ENDIF(WIN32)

#MESSAGE("OPENSCENEGRAPH_LIBDIR: ${OPENSCENEGRAPH_LIBDIR}")
#MESSAGE("OPENSCENEGRAPH_INCLUDEDIR: ${OPENSCENEGRAPH_INCLUDEDIR}")
#MESSAGE("OPENSCENEGRAPH_LDFLAGS: ${OPENSCENEGRAPH_LDFLAGS}")
#MESSAGE("OPENSCENEGRAPH_CFLAGS: ${OPENSCENEGRAPH_CFLAGS}")
#MESSAGE("OPENSCENEGRAPH_LIBRARIES: ${OPENSCENEGRAPH_LIBRARIES}")
