# Try to find SwissRanger camera driver library  
#
# Available from:
# www.mesa-imaging.ch
#
# Once run this will define: 
#
# SWISSRANGER_FOUND
# SWISSRANGER_INCLUDE_DIR
# SWISSRANGER_LIBRARIES
# SWISSRANGER_HEADER       = This is a hack to allow command completion in MSVS
#
#  (SWISSRANGER_LINK_DIRECTORIES: not yet...)
# 
# NOTES
# tested with version 1.012.583 (on Linux and WindowsXP)
#
# AUTHOR
# ischiller <ischiller@mip.informatik.uni-kiel.de> (2007)

FIND_PATH(SWISSRANGER_INCLUDE_DIR 
  NAMES libusbSR.h libMesaSR.h
  PATHS
  $ENV{SWISSRANGER_DIR}/include
  $ENV{SWISSRANGER_DIR}
  $ENV{SWISSRANGER_HOME}/include
  $ENV{SWISSRANGER_HOME}
  "$ENV{EXTERN_LIBS_DIR}/libusbSR"  
  $ENV{EXTRA}/include
  /usr/local/include
  /usr/include
  $ENV{EXTRA}
)
#MESSAGE("DBG SWISSRANGER_INCLUDE_DIR=${SWISSRANGER_INCLUDE_DIR}")  
  
FIND_LIBRARY(SWISSRANGER_LIBRARY
  #NAMES libusbSR libusbSR.so usbSR libmesasr.so libmesasr
  NAMES libmesasr.so libmesasr  
  PATHS 
  $ENV{SWISSRANGER_DIR}/lib
  $ENV{SWISSRANGER_DIR}
  $ENV{SWISSRANGER_HOME}/lib
  $ENV{SWISSRANGER_HOME}
  "$ENV{EXTERN_LIBS_DIR}/libusbSR"
  /usr/local/lib
  /usr/lib
  $ENV{EXTRA}/lib
  $ENV{EXTRA}
)
#MESSAGE("DBG SWISSRANGER_LIBRARY=${SWISSRANGER_LIBRARY}")
  
# --------------------------------

IF(SWISSRANGER_LIBRARY)
  SET(SWISSRANGER_LIBRARIES ${SWISSRANGER_LIBRARY})
ELSE(SWISSRANGER_LIBRARY)
  MESSAGE(STATUS "SWISSRANGER library not found.")
ENDIF(SWISSRANGER_LIBRARY)

IF(NOT SWISSRANGER_INCLUDE_DIR)
  MESSAGE(STATUS "SWISSRANGER include dir not found.")
ENDIF(NOT SWISSRANGER_INCLUDE_DIR)

IF(SWISSRANGER_LIBRARIES AND SWISSRANGER_INCLUDE_DIR)
  SET(SWISSRANGER_FOUND TRUE)
  
  # useful to see the headers files in IDE projects
  SET( SWISSRANGER_HEADER 
	  "${SWISSRANGER_INCLUDE_DIR}/definesSR.h"
	  "${SWISSRANGER_INCLUDE_DIR}/libMesaSR.h"
  )
  #MESSAGE("DBG SWISSRANGER_HEADER=${SWISSRANGER_HEADER}")
  SOURCE_GROUP(SWISSRANGER FILES ${SWISSRANGER_HEADER})
  
ELSE(SWISSRANGER_LIBRARIES AND SWISSRANGER_INCLUDE_DIR)
  SET(SWISSRANGER_FOUND FALSE)
ENDIF(SWISSRANGER_LIBRARIES AND SWISSRANGER_INCLUDE_DIR)

MARK_AS_ADVANCED(
  SWISSRANGER_INCLUDE_DIR
  SWISSRANGER_LIBRARY
  SWISSRANGER_LIBRARIES
)

# make FIND_PACKAGE case sensitive compatible
SET(SWISSRANGER_FOUND ${SWISSRANGER_FOUND})
