/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __ContourDetectorBase_hh__
#define __ContourDetectorBase_hh__

#include <bias_config.h>
#include <string>
#include <Base/Image/Image.hh>
#include <Base/Math/Vector2.hh>


namespace BIAS {

  /** @class BIASContour    
		  @author ischiller
      @brief
      contourPixel: all pixels contained in the contour 
      freemanCode: the direction from the first to the second pixel
        is the first value in freemanCode
        list of difference vectors coded in FreeManCode,
        that means: <br>
        0 = step right --<br>
        1 = step right & step down --<br>
        2 = step down --<br>
        3 = step left & step down --<br>
        ... --<br>
        6 = step up --<br>
        7 = step right & step up<br>
   */
  class BIASFeatureDetector_EXPORT BIASContour{
  
  public:
  	BIASContour() {length =0;};
  	~BIASContour(){};
  	  	
    std::vector<BIAS::Vector2<double> > contourPixel;
    std::vector<int>                    freemanCode;
    unsigned length;
  } ;


  /**
      @class ContourDetectorBase
      @ingroup g_feature
      @brief purely virtual interface defining class for contour detectors
      @author ischiller 08/2008
  */
  template <class StorageType>
  class BIASFeatureDetector_EXPORT ContourDetectorBase : public Debug
  {
  public:
    ContourDetectorBase();
    virtual ~ContourDetectorBase();

    /** purely virtual interface definition */
    virtual int Detect(Image<StorageType>& image,
                       std::vector<BIAS::BIASContour>& contours)=0;

    ///////////////////////////////////////////////////////////////////////

    inline void SetMaxNumContours(const int maxnum){
      dMaxNumContours_ = maxnum;
    }

    /**
        @brief return maximum number of contours
        @author ischiller
    */
    inline int GetMaxNumContours(){
      return dMaxNumContours_;
    }

  protected:

    /// maximum number of corners to return
    int dMaxNumContours_;

    std::vector<BIAS::BIASContour> detectedContours_;
  }; // class


} // namespace

#endif // __ContourDetectorBase_hh__
