/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "CornerDetectorFoerstner.hh"

using namespace std;
using namespace BIAS;

//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////

template <class StorageType, class CalculationType>
CornerDetectorFoerstner<StorageType, CalculationType>::
CornerDetectorFoerstner(): 
CornerDetectorGradient<StorageType, 
                         CalculationType>()
{
}

template <class StorageType, class CalculationType>
CornerDetectorFoerstner<StorageType,CalculationType>::
~CornerDetectorFoerstner()
{ 
}

template <class StorageType, class CalculationType> int
CornerDetectorFoerstner<StorageType, CalculationType>::
_ComputeCornerness(Image<CalculationType>& Cornerness)
{
  // preserver ROI
  int tlx, tly, brx, bry;
  this->_sgxx.GetROI()->GetCorners(tlx, tly, brx, bry);
  Cornerness.GetROI()->SetCorners(tlx, tly, brx, bry);

  const unsigned int width=Cornerness.GetWidth();
  const unsigned int height=Cornerness.GetHeight();

  // work pointers to summed multiplied derivatives
  CalculationType *psgxx=this->_sgxx.GetImageData(), 
    *psgyy=this->_sgyy.GetImageData(), 
    *psgxy=this->_sgxy.GetImageData();
  CalculationType *ptr, *end, tmp;

  // init min und max with impossible values
  //_max = FLT_MIN;
  //_min = FLT_MAX;
  
  // init all pointers to first usable position (skip upper two lines)
  /// \todo use ROI instead of border=2
  unsigned offs = 2*width+2;
  unsigned int x=2,y=2;
  ptr = Cornerness.GetImageData()+offs;
  psgxx += offs;
  psgxy += offs;
  psgyy += offs;
  end   = Cornerness.GetImageData()+width*(height-2)-2;
  const double me=(double)this->_MinCornerness;
  Feat fp; 
  this->_FeatList.clear(); 
  this->_FeatList.reserve(end-ptr);
  // run over all pixels, dont care about borders
  // run over all pixels, dont care about borders
  while (ptr<end) {
    const CalculationType xx = (CalculationType) *psgxx++;
    const CalculationType xy = (CalculationType) *psgxy++;
    const CalculationType yy = (CalculationType) *psgyy++;

    // cornerness = det / trace
    tmp = xx*yy-xy*xy / (xx+yy);

    // save highest/lowest values for quality analysis later
    //if (tmp>_max) _max = tmp;
    //else if (tmp<_min) _min=tmp;
    if (tmp*(CalculationType)4.0/(xx+yy)>me){
        fp.x=x;
        fp.y=y;
        fp.val=(tmp>(double)INT_MAX)?(INT_MAX):((int)tmp);
        this->_FeatList.push_back(fp);
        //cout <<"Found a point above min cornerness: "<<endl;
      }
    *ptr++ = tmp;
    if (++x>=width){
      y++;
      x=0;
    }
  }
  return 0;
}

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////

template class CornerDetectorFoerstner<unsigned char, float>; 
template class CornerDetectorFoerstner<float, float>; 

// fill in instances as required
#ifdef BUILD_IMAGE_INT
template class CornerDetectorFoerstner<unsigned char, int>; 
#endif
#ifdef BUILD_IMAGE_CHAR
template class CornerDetectorFoerstner<unsigned char, char>; 
#endif
#ifdef BUILD_IMAGE_SHORT
#endif
#ifdef BUILD_IMAGE_USHORT
template class CornerDetectorFoerstner<unsigned short, float>; 
#ifdef BUILD_IMAGE_INT
template class CornerDetectorFoerstner<unsigned short, int>; 
#endif
#endif
#ifdef BUILD_IMAGE_UINT
#endif
#ifdef BUILD_IMAGE_DOUBLE
#endif
