/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example ExampleBlobDetectorCCA.cpp
   @class ExampleBlobDetectorCCA
   @relates BlobDetectorCCA
   @brief Example for usage of class BlobDetectorCCA
   @ingroup g_examples
   @author ischiller
*/
#include "../BlobDetectorCCA.hh"
#include "Base/Image/ImageIO.hh"

using namespace std;


int main(int argc, char** argv)
{
  if (argc<=2) {
    std::cout<<"usage: "<<argv[0]<<" segmentedImg outputFile"<<std::endl;
    exit(1);
  }

  BIAS::BlobDetectorCCA<unsigned char> cca;
  
  BIAS::Image<unsigned char> img;
  BIAS::ImageIO::Load(argv[1],img);
  float percent = 0.1f;
  cca.SetPercent(percent);
  std::vector<BIAS::BIASBlob> blobs;
  cca.Detect(img,blobs);
  BIAS::Image<int> labelImg;
  cca.GetLabelImage(labelImg);
  //BIAS::ImageIO::Save(argv[2],labelImg);
  BIAS::ImageIO::Save(argv[2],labelImg);
  std::vector<BIAS::Vector2<unsigned int> > medians;
  cca.GetMedians(medians);

  for (unsigned int i=0;i<blobs.size();i++) {
    std::cout<<"blob "<<i<<": weight "<<blobs[i].weight<<
      " UL:"<<blobs[i].UL<<" br:"<<blobs[i].LR<<"   centerofmass: "
             <<blobs[i].centerofmass<<" median: "<<medians[i]<< std::endl;
  }
  
}
