/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __LinearRegionDetector_hh__
#define __LinearRegionDetector_hh__

#include "CornerDetectorGradient.hh"

namespace BIAS {

  /** @class LinearRegionDetector
      @ingroup g_feature
      @brief try to detect linear regions (const gradient) in images, 
             experimental code for testing
      @author koeser 09/2004 */
  template <class StorageType, class CalculationType>
  class BIASFeatureDetector_EXPORT LinearRegionDetector 
    : public CornerDetectorGradient<StorageType, CalculationType>
  {
  public:
    LinearRegionDetector();
    ~LinearRegionDetector();

    protected:
    /** @brief Computes the larger eigenvalue of the structure tensor
        points are accepted only if smaller eigenvalue is significantly smaller
        than the larger one

        c = max(e1,e2)
        Also fills up _FeatList.
        @author koeser 09/2004 
        @todo optimize for ROI usage */
      virtual int _ComputeCornerness(Image<CalculationType>& im);

 
  }; // class

} // namespace


#endif // __LinearRegionDetector_hh__
