/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __CENSUSTRANSFORM_HH_
#define __CENSUSTRANSFORM_HH_

#include "FilterBase.hh"
#include "FilterNToN.hh"

namespace BIAS {
/** @class CensusTransform
 @ingroup g_filter
 @brief 3x3 Census Transform of greyscale image

 @author fkellner 08/2008 */

template <class InputStorageType, class OutputStorageType>
class BIASFilter_EXPORT CensusTransform
  : public FilterNToN<InputStorageType, OutputStorageType>
{
public:
  CensusTransform();
  CensusTransform(const CensusTransform<InputStorageType, OutputStorageType>& other);
  virtual ~CensusTransform();

  virtual int Filter(const Image<InputStorageType>& src,
                     Image<OutputStorageType>& dst);
  virtual int FilterInt(const Image<InputStorageType>& src,
                        Image<OutputStorageType>& dst);
  virtual int FilterFloat(const Image<InputStorageType>& src,
                          Image<OutputStorageType>& dst);

  virtual FilterNToN<InputStorageType, OutputStorageType>* Clone() const
  { return new CensusTransform<InputStorageType, OutputStorageType>(*this); }

  virtual void GetBordersValid_(int &border_x, int &border_y) const;

protected:
private:

}; // class

}
// namespace

#endif /*__CENSUSTRANSFORM_HH_*/

