#include "DistTransform.hh"

//BIAS header
#include "Base/Image/ImageConvert.hh"
  
// std includes
#include <limits>

using namespace std;
using namespace BIAS;


template <class InputStorageType, class OutputStorageType>
DistTransform<InputStorageType, OutputStorageType>::
DistTransform()
  : FilterNToN<InputStorageType, OutputStorageType>()
{
  zeroLevel_ = 0;
}

template <class InputStorageType, class OutputStorageType>
DistTransform<InputStorageType, OutputStorageType>::
DistTransform(const DistTransform<InputStorageType, OutputStorageType>& other)
  : FilterNToN<InputStorageType, OutputStorageType>(other),
    zeroLevel_(other.zeroLevel_)
{
}

template <class InputStorageType, class OutputStorageType> 
void 
DistTransform<InputStorageType, OutputStorageType>::
InitDistanceImage_(BIAS::Image<OutputStorageType>& img)
{
  OutputStorageType *data = img.GetImageData();
  int pixels = img.GetPixelCount();
  OutputStorageType maxVal=
    OutputStorageType(OutputStorageType(img.GetWidth()+img.GetHeight())/
                      2.0*9.0); 

  // initiate with max possible distance infinity
  for (int i = 0; i < pixels; i++)
    if (data[i]==zeroLevel_) data[i] = 0;
    else data[i] = maxVal;
}
         

template <class InputStorageType, class OutputStorageType> 
void DistTransform<InputStorageType, OutputStorageType>::
MakeDistanceImage_(BIAS::Image<OutputStorageType>& img)
{
#define mymin(a,b) ((a<=b)?a:b)
#define minDEF(s0,s1,s2,s3,s4) mymin(s0, mymin(s1, mymin(s2, mymin(s3, s4))))

  OutputStorageType **data=img.GetImageDataArray();

  int rows = img.GetHeight();
  int cols = img.GetWidth();
  
  // forward pass
  for (int k = 1; k <= rows-1; k++)
    for (int j = 1; j <= cols-1-1; j++)
      data[k][j]= minDEF(data[k][j],                         // origin
                      OutputStorageType(data[k][j-1]+3), 
                      OutputStorageType(data[k-1][j]+3),     // left, top
                      OutputStorageType(data[k-1][j-1]+4), 
                      OutputStorageType(data[k-1][j+1]+4) ); // top diags  
  
  // backward pass
  for (int k = rows-1-1; k >= 0; k--)
    for (int j = cols-1-1; j >= 1; j--)
      data[k][j]= 
            minDEF(data[k][j],                         // origin
                OutputStorageType(data[k][j+1]+3), 
                OutputStorageType(data[k+1][j]+3),     // right, bottom
                OutputStorageType(data[k+1][j+1]+4), 
                OutputStorageType(data[k+1][j-1]+4) ); // bottom diags
}


template <class InputStorageType, class OutputStorageType> 
int DistTransform<InputStorageType, OutputStorageType>::
Filter(const BIAS::Image<InputStorageType>& src_img,
       BIAS::Image<OutputStorageType>& dist_img)
{
#ifdef BIAS_DEBUG
  if ( (dist_img.GetStorageType()==ImageBase::ST_unsignedchar) ) {
    BIASERR("OutputType unsigned char is invalid for distance images, "
            "use unsigned short or float!");
    return -1;
  }
#endif
  ImageConvert::ConvertST(src_img,dist_img,dist_img.GetStorageType());
  InitDistanceImage_(dist_img);
  MakeDistanceImage_(dist_img);
  return 0;
}

template <class InputStorageType, class OutputStorageType> 
int DistTransform<InputStorageType, OutputStorageType>::
FilterInt(const Image<InputStorageType>& src, Image<OutputStorageType>& dst) 
{
  BIASERR("FilterInt makes no sense here, using general Filter interface.");
  return Filter(src, dst);
}

template <class InputStorageType, class OutputStorageType> 
int DistTransform<InputStorageType, OutputStorageType>::
FilterFloat(const Image<InputStorageType>& src, Image<OutputStorageType>& dst) 
{
  BIASERR("FilterFloat makes no sense here, using general Filter interface.");
  return Filter(src, dst);
}

template <class InputStorageType, class OutputStorageType> 
void DistTransform<InputStorageType, OutputStorageType>::
GetBordersValid_(int & /*border_x*/, int & /*border_y*/) const
{
  BIASERR("Unfinished code");
  BIASABORT;
}

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////
#define FILTER_INSTANTIATION_CLASS DistTransform
#include "Filterinst.hh"
