/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2004
 Johan Skoglund
 skoglund@isy.liu.se
 CVL/isy
 university of Link"oping
 sweden

 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** 
	@example ExampleMedian.cpp
	@class ExampleMedian
	@brief Example for median filtering images
  @relates Median
	@ingroup g_examples 
	@author MIP
*/
#include "Filter/Median.hh"
#include "Base/Image/ImageIO.hh"
#include <Base/Image/ImageConvert.hh>
#include <iostream>

#include <Base/Debug/TimeMeasure.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[]) {

  Median<float, float> median;

  ImageBase ucim;
  Image<float> im;
  Image<float> med;

  if (argc<4) {
    cerr << "num args " << argc << " usage: filtersizeX filtersizeY sourceimage  destimage \n";
    return -2;
  }

  if (ImageIO::Load(argv[3], ucim)!=0) {
    BIASERR("error loading image "<<argv[3]);
    return -1;
  } else {
    cerr << "read "<<argv[3]<<endl;
  }

  if (ImageConvert::ConvertST(ucim, im, ImageBase::ST_float)!=0) {
    BIASERR("error converting image "<<argv[3]);
  }

  median.SetSize(atoi(argv[1]),atoi(argv[2]));

  if (im.GetChannelCount() == 1) {
    median.Filter(im, med);
  } else {
    median.FilterColorImg(im, med);
  }

  cout << "saving to " << string(argv[4]) << endl;
  ImageIO::Save(string(argv[4]), med);
  return 0;
}
