/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2004 (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel

BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Filter/FilterNToNFactory.hh>
#include <Filter/Bilateral.hh>
#include <Filter/Binomial.hh>
#include <Filter/CensusTransform.hh>
#include <Filter/Convolution.hh>
#include <Filter/DeInterlace.hh>
#include <Filter/DistTransform.hh>
#include <Filter/Gauss.hh>
#include <Filter/GaussThreshold.hh>
#include <Filter/Mean.hh>
#include <Filter/Median.hh>
#include <Filter/Thinning.hh>
#include <Filter/TukeyWindow.hh>

using namespace BIAS;
using namespace std;

template <class InputStorageType, class OutputStorageType>
FilterNToN<InputStorageType, OutputStorageType>*
FilterNToNFactory<InputStorageType, OutputStorageType>::
Generate(const std::string& className)
{
  if (className == "Bilateral")
    return new Bilateral<InputStorageType,OutputStorageType>;
  if (className == "Binomial")
    return new Binomial<InputStorageType,OutputStorageType>;
  if (className == "CensusTransform")
    return new CensusTransform<InputStorageType,OutputStorageType>;
  if (className == "Convolution")
    return new Convolution<InputStorageType,OutputStorageType>;
  if (className == "DeInterlace")
    return new DeInterlace<InputStorageType,OutputStorageType>;
  if (className == "DistTransform")
    return new DistTransform<InputStorageType,OutputStorageType>;
  if (className == "Gauss")
    return new Gauss<InputStorageType,OutputStorageType>;
  if (className == "GaussThreshold")
    return new GaussThreshold<InputStorageType,OutputStorageType>;
  if (className == "Mean")
    return new Mean<InputStorageType,OutputStorageType>;
  if (className == "Median")
    return new Median<InputStorageType,OutputStorageType>;
  if (className == "Thinning")
    return new Thinning<InputStorageType,OutputStorageType>;
  if (className == "TukeyWindows")
    return new TukeyWindow<InputStorageType,OutputStorageType>;

  BIASERR("Invalid FilterNToN class: " << className);
  return NULL;
}

#define FILTER_INSTANTIATION_CLASS FilterNToNFactory
#include "Filterinst.hh"
