/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __Thinning_hh__
#define __Thinning_hh__

#include "FilterNToN.hh"

// data types, in which intermediate values are computed
#define THIN_FLOAT float
#define THIN_INT   int

namespace BIAS {

  /** @class Thinning
      @brief generic Thinning class, uses ethe Rosenfeld algorithm to 
      thin structures in images without erasing them, unlike the erosion filter.
      @ingroup g_filter
      @author Aleksandr Mattal 07/2006 */
  template <class InputStorageType, class OutputStorageType>
  class BIASFilter_EXPORT Thinning : public FilterNToN<InputStorageType, OutputStorageType>
  {
  public:
    Thinning();
    Thinning(const Thinning<InputStorageType, OutputStorageType>& other);
    virtual ~Thinning();

    /** @brief makes Rosenfeld-Thinning on src-Image and puts it in Destionation-Image */  
      virtual int Filter(const Image<InputStorageType> &src,
                 Image<OutputStorageType> &dst);
      
    /** @brief only calls filter, for consistency of params */
    virtual int FilterInt(const Image<InputStorageType>& src, 
                          Image<OutputStorageType>& dst); 
    
    /** @brief only calls filter, for consistency of params */
    virtual int FilterFloat(const Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst);

    virtual FilterNToN<InputStorageType, OutputStorageType>* Clone() const
    { return new Thinning<InputStorageType, OutputStorageType>(); }
  
  protected:  
     virtual void GetBordersValid_(int& border_x, int& border_y) const {};
  }; // class 


} // namespace

#endif // __Thinning_hh__
