#ifndef __CONTEXTWX_HH__
#define __CONTEXTWX_HH__

#include <bias_config.h>
#include <Gui/biasgl.h>

//stupid header messes up glxext
//#include <Gui/bias_wx_glcanvas.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error Your BIAS needs WX. Please recompile BIAS with USE_WXWIDGETS.
#endif

#include "RenderContextBase.hh"

#include <Base/Common/BIASpragmaStart.hh>
#include <wx/wx.h>
#include <wx/glcanvas.h> 
#include <Base/Common/BIASpragmaEnd.hh>

// check defines from wx/setup.h implicitly
#if defined(wxUSE_GLCANVAS) && wxUSE_GLCANVAS
// OK, wx compiled in configuration with wxgl lib
#else
#  error wx has not been compiled with wxUSE_GLCANVAS=1. Please set wx/setup.h wxUSE_GLCANVAS=1 and recompile WX.
#endif

namespace BIAS {

  /** \class ContextWX 
  \brief Context implementation for wx.
  \attention call InitializationComplete() in order to enable rendering.
  \ingroup g_glviewer   
  \author Jan Kollmann
  */
  class BIASGLviewer_EXPORT ContextWX : public wxGLCanvas, 
    public RenderContextBase
  {
  public:
    static int defaultAttribs[];

    ContextWX(wxWindow* parent, wxWindowID id = -1,
      const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize,
      long style = 0, const wxString& name = wxT("GLCanvas"),
      int* attribList = defaultAttribs,
      const wxPalette& palette = wxNullPalette);

    ContextWX(wxWindow* parent, wxGLContext* shared, wxWindowID id = -1,
      const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize,
      long style = 0, const wxString& name = wxT("GLCanvas"),
      int* attribList = defaultAttribs,
      const wxPalette& palette = wxNullPalette);

    virtual void Redisplay() { 
      Refresh(); 
    }

    virtual void MakeGLContextCurrent();

    /** delay rendering until this method has been called. */
    virtual void InitializationComplete() { initialized_ = true; }
    virtual void SetTimer(unsigned int interval);
    virtual void DeactivateTimer();

  protected:
    bool initialized_;

  private:
    void OnPaint(wxPaintEvent& event);
    void OnLeftDown(wxMouseEvent& event);
    void OnLeftUp(wxMouseEvent& event);
    void OnRightDown(wxMouseEvent& event);
    void OnLeftDClick(wxMouseEvent& event);
    void OnRightDClick(wxMouseEvent& event);
    void OnMotion(wxMouseEvent& event);
    void OnEnterWindow(wxMouseEvent& event);
    void OnLeaveWindow(wxMouseEvent& event);    
    void OnSize(wxSizeEvent& event);
    void OnKey(wxKeyEvent& event);
    void OnKeyDown(wxKeyEvent& event);
    void OnTimer(wxTimerEvent& event);
    void OnEraseBG(wxEraseEvent& event);
    int _OldMouseX, _OldMouseY;
    bool _FirstPaint;
    wxTimer Timer_;

    DECLARE_EVENT_TABLE()
  };

} // namespace BIAS

#endif // __CONTEXTWX_HH__
