#ifndef __CONTROL_INTERFACE_HH__
#define __CONTROL_INTERFACE_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include <Base/Common/W32Compat.hh>
#include <Base/Debug/Debug.hh>
#include <Base/Debug/Error.hh>

typedef enum {
    KEY_F1  = 0,
    KEY_F2,
    KEY_F3,
    KEY_F4,
    KEY_F5,
    KEY_F6,
    KEY_F7,
    KEY_F8,
    KEY_F9,
    KEY_F10,
    KEY_F11,
    KEY_F12,
    KEY_HOME,
    KEY_PGUP,
    KEY_PGDOWN
} SpecialKey;

typedef enum {
  NONE = 0,
  ALT,
  CTRL,
  SHIFT
} modifiers;

namespace BIAS {

  class BIASGLviewer_EXPORT GLProjectionParametersInterface;

  /** \class ControlInterface
      \brief Defines the common interface used for communication between
      rendering components.
      All presented method return a boolean.
      Returning false means, that the called method has not
      reacted to the call.
      \author bartczak
      \ingroup g_glviewer
  */
  class BIASGLviewer_EXPORT ControlInterface : public BIAS::Debug
  {
  public:
    ControlInterface() : listenerName("unnamed control listener") {
      controlledObject_=NULL;
    }

    std::string listenerName;
    virtual std::string Name() { return listenerName; }
    virtual ~ControlInterface(){}


    virtual bool MouseWheelUsed(double stepsize){return false;}

    /** .
     * Does not handle modifiers.
     */
    virtual bool LeftMouseDoubleClicked(int /*x*/, int /*y*/, int m) { return false; }

    /**.
     * Does not handle modifiers.
     */
    virtual bool LeftMouseSingleClicked(int /*x*/, int /*y*/) { return false; }

    /** .
     * Does not handle modifiers.
     */
    virtual bool LeftMouseMoved(int /*x1*/, int /*x2*/,int /*y1*/, int /*y2*/)
    { return false; }

    virtual bool LeftMouseDown(int , int){ return false; };

    virtual bool LeftMouseUp(int , int){ return false; };

    /**.
     * Does not handle modifiers.
     */
    virtual bool RightMouseSingleClicked(int /*x*/, int /*y*/) { return false; }

    /** .
     * Does not handle modifiers.
     */
    virtual bool RightMouseDoubleClicked(int /*x*/, int /*y*/) { return false; }


    /** .
     * Does not handle modifiers.
     */
    virtual bool RightMouseMoved(int /*x1*/, int /*x2*/,
                                 int /*y1*/, int /*y2*/)
    { return false; }

    /** .
     * Does not handle modifiers.
     */
    virtual bool MiddleMouseMoved(int /*x1*/, int /*x2*/,
                                  int /*y1*/, int /*y2*/)
    { return false; }

    virtual bool LeftAndRightMouseMoved(int /*x1*/, int /*x2*/,
                                        int /*y1*/, int /*y2*/)
    { return false; }

    virtual bool MouseMoved(int /*x*/, int /*y*/)
    {return false; }

    virtual bool SpecialKeyPressed(int /*key*/)
    { return false; }

    virtual bool MouseLeftWindow()
    { return false; }

    /// ascii-coded charakter plus pointer position
    virtual bool StandardKeyPressed(unsigned char /*key*/, int, int)
    { return false; }

    virtual bool TimerExpired()
    { return false; };

    virtual bool WindowReshape(int /*width*/, int /*height*/)
    { return false; };

    virtual bool FinishedDraw(){return false;};

    virtual void SetGLProjectionParametersInterface( BIAS::GLProjectionParametersInterface* interface_ ){
      if(interface_ != NULL)
        controlledObject_ =  interface_;
      else
        BIASERR("Given interface is NULL");
    };

    virtual BIAS::GLProjectionParametersInterface* GetGLProjectionParametersInterface(){
    	return controlledObject_;
    }

  protected:
    BIAS::GLProjectionParametersInterface* controlledObject_;
  };

}//end of namespace

#endif //__CONTROL_INTERFACE_HH__
