#ifndef __DistanceMeasureControl_HH__
#define __DistanceMeasureControl_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer


#ifdef BIAS_HAVE_FONTGL
#include <Base/Common/BIASpragmaStart.hh>
#include <Utils/DrawTextGL.hh>
#include <Base/Common/BIASpragmaEnd.hh>
#endif

#include <Base/Math/Vector3.hh>
#include <GLviewer/Controller/TrackballControl.hh>
#include <GLviewer/Scenes/SceneBase.hh>

namespace BIAS {
  /** 
      \class DistanceMeasureControl
      \author Ingo Schiller 
      \date 02/2010
      \ingroup g_glviewer
      \brief class for distance measurements in 3d model, derives from TrackballControl and SceneBase
  */
  class BIASGLviewer_EXPORT DistanceMeasureControl : public TrackballControl, public SceneBase
  {   
  public:
  	DistanceMeasureControl();
    ~DistanceMeasureControl() {};
       

    /**
    * @brief Sets first or second click measurement positions 
    */
    bool LeftMouseDoubleClicked(int x, int y, int m);
    
    /**
      * @brief Sets intermediate coords 
    */
    bool MouseMoved(int x, int y);
    
    /**
    * @brief Activate or deactivate distance measurement
    * @param [in] bool on, activates or deactivates distance measurement
    */
    void SetDistanceMeasure(bool on);
    
    /**
    * @brief Return if the measurement mode is activated
    */
    bool GetDistanceMeasure();
    
    /**
     * @brief overloaded method from SceneBase. Here the line in 3D is drawn.
     */
    virtual void Draw();
    
  protected:
  	bool bDistMeasureActivated_;
  	unsigned dNumClick_;
    BIAS::HomgPoint3D firstHomgPoint_,secondHomgPoint_;
    int x_,y_;
    bool bIsInitialized_;
   
#ifdef BIAS_HAVE_FONTGL
    BIAS::DrawTextGL drawer_;
#endif

  };
}//end of namespace
#endif //__TrackballControl_HH__
