#ifndef __OnlyRotationControl_HH__
#define __OnlyRotationControl_HH__

#include <bias_config.h>

#include <GLviewer/Controller/ControlInterface.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS {
  /** 
      \class OnlyRotationControl
      \author haase
      \ingroup g_glviewer
      \brief class for camera movement in fixed position, only rotation mode.
  */
  class BIASGLviewer_EXPORT OnlyRotationControl : public ControlInterface
  {   
  public:
    OnlyRotationControl(); 
    /**
       performs a up/down (pitch) rotation (mouse moving up-down) for animations without 
       changing the viewport  position,steps of 0.1 degree by now 
    **/
    bool RightMouseMoved(int x1, int x2, int y1, int y2);


    /**
       performs a left/right (yaw) rotation (mouse moving left-right) for animations without
       changing the viewport 
       position,steps of 1 degree by now  
    **/
    bool LeftMouseMoved(int x1, int x2, int y1, int y2);

    /**
       performs a left/right (roll) rotation (mouse moving left-right) for animations without
       changing the viewport 
       position,steps of 1 degree by now  
    **/
    bool MiddleMouseMoved(int x1, int x2, int y1, int y2);

    /**
       performs a reset for (roll) rotation  without
       changing the viewport position  
    **/
    bool LeftMouseDoubleClicked(int x, int y);

    /**
       performs a minimal changing of the viewport position  
    **/
    bool LeftAndRightMouseMoved(int x1, int x2, int y1, int y2);
 
    /**
       performs an invert of up vector  -to correct vrml scenes  
    **/
    bool RightMouseDoubleClicked(int /*x*/, int /*y*/);
           
  protected: 
  };
}//end of namespace
#endif
