#ifndef __TrackballControl_HH__
#define __TrackballControl_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include <Base/Math/Vector3.hh>
#include <GLviewer/Controller/TrackballBase.hh>

namespace BIAS {
  /**
      \class TrackballControl
      \author cmenk
      \ingroup g_glviewer
      \brief class for camera movement in trackball mode.
  */
  class BIASGLviewer_EXPORT TrackballControl : public TrackballBase
  {
  public:
    TrackballControl();
    ~TrackballControl() {};

    /**
     * Moves the camera forward to point of interest if y2-y1 is positive
     * else the camera is moved away from point of interest.
     */
    bool RightMouseMoved(int x1, int x2, int y1, int y2);
    void MoveInViewingDirection(double stepSize);
    /**
     * Rotates the camera around the point of interest. x1, x2, y1, y2 are the
     * mouse coordinates and width, height must be the size of the window. The
     * trackball contains the whole window. The radius of the trackball is
     * sqrt from width*width/4 + height*height/4.
     */
    bool LeftMouseMoved(int x1, int x2, int y1, int y2);

    void RotateAroundPOI(BIAS::Vector3<double> axis, double angleRAD);

    /**
     * Computes from the coordinates x,y a 3D point in the scene and tries to
     * set it to the point of interest.
     */
    virtual bool LeftMouseDoubleClicked(int x, int y, int m);

    bool LeftAndRightMouseMoved(int x1, int x2, int y1, int y2);

    virtual bool MouseMoved(int x, int y){return false;}

    /**
       Handles Mouse Wheel events and zooms in or out
    */
    bool MouseWheelUsed(double stepsize);

    /**
     * Moves the camera left/right/up/down and sets new point of interest.
     */
    bool MiddleMouseMoved(int x1, int x2, int y1, int y2);

    void RotateAroundViewingDirection(double rotationAngleRAD);

    BIAS::Vector3<double> GetPoI() { return PointOfInterest_; }
    void SetPointOfInterest(const BIAS::Vector3<double>& poi);

    void SetGLProjectionParametersInterface(BIAS::GLProjectionParametersInterface* projInterface);

  protected:

    BIAS::Vector3<double> PointOfInterest_;
  };
}//end of namespace
#endif //__TrackballControl_HH__
