#ifndef __OPNESCENEGRAPHHELPER_HH__
#define __OPNESCENEGRAPHHELPER_HH__
#include <bias_config.h>

#include <Base/Image/Image.hh>
#include <Utils/TriangleMesh.hh>
#include <string>

namespace osg {
  class Node;
  class Group;
  class Geode;
  class Geometry;
  class Image;
  class Matrixd;
  class Vec3f;
  template <class T> class ref_ptr;
  typedef class Vec3f Vec3;
  typedef class Matrixd Matrix;
} 

// why the forward declarations? breaks on win32. fkellner
/*
namespace BIAS {
  class TriangleMesh;
  template <class T> class Image;
  template <class T> class Matrix4x4;
}
*/


namespace BIAS {

 
/** 
 * @class OpenSceneGraphHelper
 * @brief Utility class for using OpenSceneGraph 
 * @ingroup g_glviewer
 * @author MIP 
*/
  class BIASGLviewer_EXPORT OpenSceneGraphHelper {
  public:
    OpenSceneGraphHelper(){};
    ~OpenSceneGraphHelper(){};
    
   
    
    /** @brief conversion function */
    static bool BIASimageToOSGimage(const BIAS::Image<unsigned char>& in,
                                    osg::Image* out, bool copydata);
    
    static bool OSGimageToBIASimage(const osg::Image* in,
                                    BIAS::Image<unsigned char>& out);

    /** this is the new method which takes a triangle mesh and creates
    an osg branch avoiding the copying into ThreeDOut.
    */
    static osg::Node* CreateBranchIndexedFaceSets(const BIAS::TriangleMesh& mesh
						  ,const std::string& name);
   

    static bool OSG2BIASMatrix( const osg::Matrixd& OsgMat,
                                BIAS::Matrix4x4<double>& BiasMat);
    
    static bool BIAS2OSGMatrix(const BIAS::Matrix4x4<double>& BiasMat,
                               osg::Matrixd& OsgMat);
    
    /** checks max texture size and adjusts  */
    static bool AdjustImageSizeToGPULimit(BIAS::Image<unsigned char>& tex);


    /** Checks if the node contains additional information in form of a 
        key value pair. Throws an exception when node is NULL. 
        @author woelk 11/2009 */
    static bool HasInfo(const osg::ref_ptr<osg::Node> node, 
                        const std::string &key, std::string &value);

    /** Removes key value pair from node. Returns false if info was not 
        contained in the node. Throws an exception when node is NULL. 
        @author woelk 11/2009 */
    static bool RemoveInfo(osg::ref_ptr<osg::Node> node, 
                           const std::string &key);

    /** Adds or sets information to the node. Returns false if info is 
        overwritten. Throws an exception when node is NULL. 
        @author woelk 11/2009 */
    static bool AddOrSetInfo(osg::ref_ptr<osg::Node> node, 
                             const std::string &key, 
                             const std::string &value);

    /** returns the associated info as key_value pairs */
    static void GetInfo(const osg::ref_ptr<osg::Node> node, 
                        std::map<std::string, std::string> &key_val);

    /** Finds a node downwards of node who has a matching key value 
        pair. Returns NULL if no node with a matching key value pair is found. 
        Throws an exception when node is NULL. 
        @author woelk 11/2009 */
    static osg::ref_ptr<osg::Node> 
      FindNodeWithInfo(const osg::ref_ptr<osg::Node> node, 
                       const std::string &key, 
                       const std::string &value);
    
    /** Prints the content of the info associated with node to cout */
    static void DumpInfo(const osg::ref_ptr<osg::Node> node);
    
    /** Returns a vector of all nodes containing an info characterised by key.
        The search is conducted starting by node and proceeding downwards of 
        node.
        @author woelk 11/2009 */
    static std::vector<osg::ref_ptr<osg::Node> > 
      FindNodesWithInfo(const osg::ref_ptr<osg::Node> node, 
                        const std::string &key);

  protected:
    /// helper function for FindNodesWithInfo
    static void
      FindNodesWithInfoHelper_(const osg::ref_ptr<osg::Node> CurrentNode, 
                               const std::string &key,
                               std::vector<osg::ref_ptr<osg::Node> > &result);
    
  }; //class OpenSceneGraphHelper

  /// stream operator for osg matrix (woelk 10/2009)
  std::ostream &operator<<(std::ostream& os, const osg::Matrix& m);
  /// stream operator for osg vector (woelk 10/2009)
  std::ostream &operator<<(std::ostream& os, const osg::Vec3& v);



}//namespace
#endif
