#include <GLviewer/OutputAttachments/GLProjectionOutputAttachment.hh>

using namespace std;
using namespace BIAS;

GLProjectionOutputAttachment::GLProjectionOutputAttachment(BIAS::glfRenderTarget *out) {
  target_ = out;
  ShaderInited_ = false;
  ClearColor_ = Vector4<float>(0.0f,0.0f,0.0f,0.0f);
  InitShader_();
}

GLProjectionOutputAttachment::~GLProjectionOutputAttachment() {

}

int GLProjectionOutputAttachment::Draw() {
  if (!ShaderInited_) {
    InitShader_();
  }

  return Draw_();
}

int GLProjectionOutputAttachment::InitShader(std::string vertexCode, std::string fragmentCode) {
  Vector4<int> vp;
  glGetIntegerv(GL_VIEWPORT, vp.GetData());
  
  int stencilbits;
  glGetIntegerv(GL_STENCIL_BITS, &stencilbits);
  int alphaBits;
  glGetIntegerv(GL_ALPHA_BITS, &alphaBits);

  unsigned int imgWidth = vp[2] - vp[0];
  unsigned int imgHeight = vp[3] - vp[1];

  specialDisplayTex_.Create();
  specialDisplayTex_.Set(GL_LINEAR, GL_LINEAR, GL_CLAMP, GL_CLAMP, GL_TEXTURE0);
  specialDisplayTex_.SetTextureNr(GL_TEXTURE0);
  if (alphaBits > 0)
    specialDisplayTex_.Allocate(imgWidth, imgHeight, GL_RGBA8);
  else 
    specialDisplayTex_.Allocate(imgWidth, imgHeight, GL_RGB8);
  specialDisplayDepth_.Create();
  specialDisplayDepth_.Set(GL_NEAREST, GL_NEAREST, GL_CLAMP, GL_CLAMP, GL_TEXTURE1);
  specialDisplayDepth_.SetTextureNr(GL_TEXTURE1);
  if (stencilbits == 0)
    specialDisplayDepth_.Allocate(imgWidth, imgHeight, GL_DEPTH_COMPONENT);
  else 
    specialDisplayDepth_.Allocate(imgWidth, imgHeight, GL_DEPTH24_STENCIL8_EXT);

  specialDisplayRenderingFBO_.Create();
  specialDisplayRenderingFBO_.AttachTexture(specialDisplayTex_, GL_COLOR_ATTACHMENT0_EXT);
  specialDisplayRenderingFBO_.AttachTexture(specialDisplayDepth_, GL_DEPTH_ATTACHMENT_EXT);
  if (stencilbits != 0)
    specialDisplayRenderingFBO_.AttachTexture(specialDisplayDepth_, GL_STENCIL_ATTACHMENT_EXT);
  specialDisplayRenderingFBO_.CheckComplete();

  shaderF_.Create(GL_FRAGMENT_SHADER, fragmentCode.c_str());
  shaderV_.Create(GL_VERTEX_SHADER, vertexCode.c_str());

  shaderProg_.Create();
  shaderProg_.AttachShader(shaderF_);
  shaderProg_.AttachShader(shaderV_);
  shaderProg_.Link();
  ShaderInited_ = true;
  return 0;
}

void GLProjectionOutputAttachment::SetClearColor(const BIAS::Vector4<float>& cc)
{
  ClearColor_ = cc;
}

int GLProjectionOutputAttachment::StandardDraw_() {
  Vector4<int> vp;
  glGetIntegerv(GL_VIEWPORT, vp.GetData());

  unsigned int imgWidth = vp[2] - vp[0];
  unsigned int imgHeight = vp[3] - vp[1];
  //  cout << imgWidth << " " << imgHeight << endl;
  //  bind render target
  target_->ClearColorBuffer(ClearColor_[0],ClearColor_[1],ClearColor_[2],ClearColor_[3]);
  target_->ClearDepthBuffer();
  target_->Bind();
  shaderProg_.Bind();
  shaderProg_.SetUniform("tex", specialDisplayTex_.GetTextureNrInUniformFormat());
  shaderProg_.SetUniform("dep", specialDisplayDepth_.GetTextureNrInUniformFormat());
  shaderProg_.SetUniform("w", (float)imgWidth);
  shaderProg_.SetUniform("h", (float)imgHeight);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(0, imgWidth, 0, imgHeight);
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();

  // quad to render texture on
  glPushAttrib(GL_ENABLE_BIT);
  glEnable(GL_TEXTURE_2D);
  specialDisplayTex_.BindTU();
  specialDisplayDepth_.BindTU();
  glBegin(GL_QUADS);
    glColor3f(1.0, 0.0, 0.0);
    glTexCoord2f(0.0, 0.0);
    glVertex2f(0.0, 0.0);

    glColor3f(0.0, 1.0, 0.0);
    glTexCoord2f(0.0, 1.0);
    glVertex2f(0.0, imgHeight);

    glColor3f(0.0, 0.0, 1.0);
    glTexCoord2f(1.0, 1.0);
    glVertex2f(imgWidth, imgHeight);

    glColor3f(1.0, 1.0, 1.0);
    glTexCoord2f(1.0, 0.0);
    glVertex2f(imgWidth, 0.0);
  glEnd();
  glFlush();
  glPopAttrib();
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glUseProgram(0);

  GLF_THROW_ON_OPENGL_ERROR

  return 0;
}

