#ifndef GLPROJECTION_OUTPUT_ATTACHMENT_HH_
#define GLPROJECTION_OUTPUT_ATTACHMENT_HH_
#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include <Gui/biasgl.h>
#include <Base/Debug/Debug.hh>
#include <GLviewer/Scenes/SceneBase.hh>
//#include <GLviewer/SceneBGImage.hh>
#include <OpenGLFramework/Base/glfCommon.hh>
#include <OpenGLFramework/Base/glfShader.hh>
#include <OpenGLFramework/Base/glfShaderProgram.hh>
#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/Base/glfRenderTarget.hh>
#include <OpenGLFramework/Base/glfTexture2D.hh>

namespace BIAS {

  /**
   * @brief Class for extra rendering pass in GLProjection (for now only used in glutGLviewer)
   *
   * GLProjection will first render into FBO of this attachment and afterwards
   * the attached shader program is executed on the result.
   * See OutputHaze, OutputWowDisplay for examples on how to derive specialized versions
   * @ingroup g_glviewer
   * @author fkellner
   */
  class BIASGLviewer_EXPORT GLProjectionOutputAttachment : public BIAS::Debug {
    public:

      // identificators for glviewer
      enum GLProjectionOutputAttachmentType {
        WoWVX = 0,
        Haze,
        WoWVX2,
        Underwater,
        LensDistortion
      };

     /// constructor is given final render target (GLProjection defaults to "screen")
     GLProjectionOutputAttachment(BIAS::glfRenderTarget *out);
      virtual ~GLProjectionOutputAttachment();

      /// used by GLProjection to find rendering target of first pass
      BIAS::glfFramebufferObject* GetOutputFBO() {

        return &(specialDisplayRenderingFBO_);
      }

      /// rendering second pass (for example to "screen"
      int Draw();

      /// can be called to create output attachment without having to derive new subclass
      /// will also create FBO and attached textures. gl viewport information has to be already set up here.
      int InitShader(std::string vertexCode, std::string fragmentCode);
      
      void SetClearColor(const BIAS::Vector4<float>& cc);

    protected:
      virtual int InitShader_() { return -1;};
      virtual int Draw_() { return StandardDraw_();};
      /// this function suffices for simple shaders that only need the standard uniforms to be set.
      /// these are one texture attachment, one depth attachment (the first pass render result), and their width and height
      int StandardDraw_();

      BIAS::glfRenderTarget *target_;
      BIAS::glfFramebufferObject specialDisplayRenderingFBO_;
      BIAS::glfTexture2D specialDisplayTex_, specialDisplayDepth_;
      BIAS::glfShader shaderF_;
      BIAS::glfShader shaderV_;
      BIAS::glfShaderProgram shaderProg_;
      BIAS::Vector4<float> ClearColor_;

      bool ShaderInited_;
  };

}


#endif

