#include <GLviewer/OutputAttachments/OutputUnderwater.hh>

using namespace std;
using namespace BIAS;

OutputUnderwater::OutputUnderwater(BIAS::glfRenderTarget *out) 
: GLProjectionOutputAttachment(out)
{
  InitShader_();
}

OutputUnderwater::~OutputUnderwater() {
}

int OutputUnderwater::
InitShader_() {
  string vertexCode =
      "void main()\r\n"
      "{\r\n"
      "  gl_TexCoord[0] = gl_MultiTexCoord0;\r\n"
      "  gl_Position = ftransform();\r\n"
      "}\r\n";
  string fragmentCode =
      "uniform sampler2D tex;\r\n"
      "uniform sampler2D dep;\r\n"
      "uniform float w;\r\n"
      "uniform float h;\r\n"
      "uniform float qualityR;\r\n"
      "uniform float qualityG;\r\n"
      "uniform float qualityB;\r\n"
      "uniform float zNear;\r\n"
      "uniform float zFar;\r\n"
      "uniform float colorR;\r\n"
      "uniform float colorG;\r\n"
      "uniform float colorB;\r\n"
      "uniform float alphaR;\r\n"
      "uniform float alphaG;\r\n"
      "uniform float alphaB;\r\n"
      "void main()\r\n"
      "{\r\n"
      "  vec4 color;\r\n"
      "  vec2 pos = gl_TexCoord[0].st;\r\n"
      "  color = texture2D(tex,pos);\r\n"
      "  float depthZ = texture2D(dep,pos).z;\r\n"
      "  float depth = zNear * zFar / (zFar - depthZ * (zFar - zNear));\r\n"
      "  vec4 B = vec4(colorR,colorG,colorB,1.0);\r\n"
      "  vec4 A = vec4(alphaR,alphaG,alphaB,1.0);\r\n"
//      "  vec4 B = vec4(0.2,0.25,0.35,1.0);\r\n"
      "  vec4 mu = vec4(qualityR, qualityR, qualityB, 1.0);\r\n"
      "  vec4 t = vec4(exp(-mu[0]*depth),exp(-mu[1]*depth),exp(-mu[2]*depth),1.0);\r\n"
      "  vec4 t2 = vec4(exp(-2.0*mu[0]*depth),exp(-2.0*mu[1]*depth),exp(-2.0*mu[2]*depth),1.0);\r\n"
      "  color = A*(color*t2 + B*(vec4(1.0) - t));\r\n"
      "  color[3] = 1.0;\r\n"
      "  if(depthZ >= 1.0) gl_FragColor = B;\r\n"
      "  else gl_FragColor = color;\r\n"
      "}\r\n";

  return GLProjectionOutputAttachment::InitShader(vertexCode, fragmentCode);
}

void OutputUnderwater::
UpdateParams(double zNear,double zFar,std::vector<int> &waterParams) {

  waterQualityR_ = (float)waterParams[0]/100.0f;
  waterQualityG_ = (float)waterParams[1]/100.0f;
  waterQualityB_ = (float)waterParams[2]/100.0f;
  zNear_ = (float)zNear;
  zFar_ = (float)zFar;
  waterColorR_ = (float)waterParams[3]/100.0f;
  waterColorG_ = (float)waterParams[4]/100.0f;
  waterColorB_ = (float)waterParams[5]/100.0f;

  alphaR_ = (float)waterParams[6]/100.0f;
  alphaG_ = (float)waterParams[7]/100.0f;
  alphaB_ = (float)waterParams[8]/100.0f;

  //cout << waterQuality_ << endl;
}

int OutputUnderwater::
Draw_() {
  cout<<"OutputUnderwater!"<<endl;
  shaderProg_.SetUniform("qualityR", waterQualityR_);
  shaderProg_.SetUniform("qualityG", waterQualityG_);
  shaderProg_.SetUniform("qualityB", waterQualityB_);
  shaderProg_.SetUniform("zNear", zNear_);
  shaderProg_.SetUniform("zFar", zFar_);
  shaderProg_.SetUniform("colorR", waterColorR_);
  shaderProg_.SetUniform("colorG", waterColorG_);
  shaderProg_.SetUniform("colorB", waterColorB_);
  shaderProg_.SetUniform("alphaR", alphaR_);
  shaderProg_.SetUniform("alphaG", alphaG_);
  shaderProg_.SetUniform("alphaB", alphaB_);

  return GLProjectionOutputAttachment::StandardDraw_();
}
