#ifndef __PointLightScene_HH__
#define __PointLightScene_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer


#include <GLviewer/Scenes/SceneBase.hh>
#include <GLviewer/Controller/ControlInterface.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS {



  /** \class SceneBase 
      \brief Base class for all scenes.
      \ingroup g_glviewer
      \author bartczak
   */
  class BIASGLviewer_EXPORT PointLightScene : public SceneBase
  {
  public:
    PointLightScene();
    
    void SetLightPosition(BIAS::Vector3<double> pos){
      lightPos_ = pos;
    };

    void SetAmbient(BIAS::Vector4<double> am){
      ambient_[0] = am[0];     ambient_[1] = am[1];
      ambient_[2] = am[2];     ambient_[3] = am[3];
    }
    void SetDiffuse(BIAS::Vector4<double> di){
      diffuse_[0]=di[0];       diffuse_[1]=di[1];
      diffuse_[2]=di[2];       diffuse_[3]=di[3];
    }
    void SetSpecular(BIAS::Vector4<double> sp){
      specular_[0]=sp[0];       specular_[1]=sp[1];
      specular_[2]=sp[2];       specular_[3]=sp[3];
    }

    void SetDrawLightPosition(bool drawLight){
      drawLight_ = drawLight;
    };
    void SetLightAsHeadLight(bool asHeadLight){
      lightAsHeadALight_ = asHeadLight;
    };

  protected:
    /**
       overloaded function from SceneBase
     */
    virtual void Draw();

    void DrawLightPosition_();
  
  protected:
    BIAS::Vector3<double> lightPos_;
    bool drawLight_;
    bool lightAsHeadALight_;   
    float diffuse_[4];
    float ambient_[4];
    float specular_[4];    
  };
}//end namespace

#endif
