#ifndef __SCENEBGIMAGE_HH__
#define __SCENEBGIMAGE_HH__

#include <GLviewer/Scenes/SceneBase.hh>
#include <bias_config.h>
#include <Gui/biasgl.h>
#include <Base/Image/Image.hh>
#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS
{
/** \class SceneBGImage
    \brief Scene that renders a background image behind all other scenes
    \ingroup g_glviewer
    \author MIP
 */
  class BIASGLviewer_EXPORT SceneBGImage : public SceneBase
  {
  public:
    virtual ~SceneBGImage(){}
    SceneBGImage();
    int SetImage(const BIAS::Image<unsigned char> &img);

    /// Query the maximum texture size supported by OpenGL. Only valid after calling SetImage() at least once.
    int GetMaxTextureSize(){return MaxTextureSize_;};
    /// set filter for minification and magnification , use GL_NEAREST, GL_LINEAR and co.
    void SetMinMagFilter(int MinFilter,int MagFilter);
  protected:
    virtual void Draw();

    // take a _copy_ of the image to allow resizing
    int InitializeTexture_(BIAS::Image<unsigned char> img);
    int MaxTextureSize_;
    bool IsInitialized_;
    GLuint BackgroundImageTex_;
    GLint MinFilter_,MagFilter_;
    unsigned int ImgWidth_,ImgHeight_;
    BIAS::Vector2<float> TexCoord_[4];
  };
}//end of namespace

#endif
