#include "SceneGlutPrimitives.hh"

#include <Gui/biasgl.h> 
#include <Base/Math/Vector3.hh>
#include <Utils/GlutInitWrapper.hh>

using namespace BIAS;
using namespace std;

SceneGlutPrimitives::
SceneGlutPrimitives(){
	 bTeapot_ = true;		
	 bCube_ = false;
	 bIcosahedron_ = false;
	 bTetrahedron_ = false;
	 fCubeSize_ =0.3;
	 fTeapotSize_=0.3;
}

void SceneGlutPrimitives::
GetBoundingBox(BIAS::Vector3<double> & themin,
               BIAS::Vector3<double> & themax)
{
  themin = BIAS::Vector3<double>(-fTeapotSize_, -fTeapotSize_, -fTeapotSize_);
  themax = BIAS::Vector3<double>(fTeapotSize_, fTeapotSize_, fTeapotSize_);
}


void SceneGlutPrimitives::Draw() 
{
	BIAS::GlutInitWrapper::GetInstance()->Init();
	
	float colorBronzeDiff[4] = { 0.8, 0.6, 0.0, 1.0 };
	float colorBronzeSpec[4] = { 1.0, 1.0, 0.4, 1.0 };
	float colorBlue[4]       = { 0.0, 0.2, 1.0, 1.0 };
	float colorBlueSpec[4]   = { 1.0, 1.0, 0.2, 1.0 };
	float colorNone[4]       = { 0.0, 0.0, 0.0, 0.0 };
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  
  if(bCube_){
  	glMaterialfv(GL_FRONT, GL_DIFFUSE, colorBlue);
  	glMaterialfv(GL_FRONT, GL_SPECULAR, colorNone);
  	glMaterialf(GL_FRONT, GL_SHININESS, 50.0);
  	glColor4fv(colorBlue);
  	
  	glFrontFace(GL_CW);
  	glutSolidCube(fCubeSize_);
  	glFrontFace(GL_CCW);
  }
  if(bTeapot_){  
  	glMaterialfv(GL_FRONT, GL_DIFFUSE, colorBronzeDiff);
  	glMaterialfv(GL_FRONT, GL_SPECULAR, colorBronzeSpec);
  	glMaterialf(GL_FRONT, GL_SHININESS, 50.0);
  	glColor4fv(colorBronzeDiff);	

  	glFrontFace(GL_CW);
  	glutSolidTeapot(fTeapotSize_);
  	glFrontFace(GL_CCW);
  }
 
  if(bIcosahedron_){
  	glMaterialfv(GL_FRONT, GL_DIFFUSE, colorBronzeDiff);
  	glMaterialfv(GL_FRONT, GL_SPECULAR, colorBronzeSpec);
  	glMaterialf(GL_FRONT, GL_SHININESS, 50.0);
  	glColor4fv(colorBronzeSpec);	
  	glFrontFace(GL_CW);
  	glutSolidIcosahedron();
  	glFrontFace(GL_CCW);
  }
  if(bTetrahedron_){
  	glMaterialfv(GL_FRONT, GL_DIFFUSE, colorBronzeDiff);
  	glMaterialfv(GL_FRONT, GL_SPECULAR, colorBronzeSpec);
  	glMaterialf(GL_FRONT, GL_SHININESS, 50.0);
  	glColor4fv(colorBlueSpec);	
  	glFrontFace(GL_CW);
  	glutSolidTetrahedron();
  	glFrontFace(GL_CCW);
  }
  
  glPopMatrix();
}
