/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @class  SceneGlutPrimitives   
   @ingroup g_glviewer
   @relates SceneBase
   @brief  Class for rendering the Glut primitives like teapot, cube, Tetrahedron and  Icosahedron
   mainly used to test rendering without Scenegraph library.
  
   @author Ingo Schiller
   @date 02/2010
*/

#ifndef __SCENEGLUTPRMITIVES_H__
#define __SCENEGLUTPRMITIVES_H__

#include <bias_config.h>

#include <GLviewer/Scenes/SceneBase.hh>

namespace BIAS{
	class BIASGLviewer_EXPORT SceneGlutPrimitives : public BIAS::SceneBase
	{
	public:
		virtual ~SceneGlutPrimitives() {};
		SceneGlutPrimitives();
	
		virtual void Draw();
		
    virtual void GetBoundingBox(BIAS::Vector3<double> & themin,
																	  BIAS::Vector3<double> & themax);

    void SetDrawTeapot(bool draw, double size){bTeapot_ = draw; fTeapotSize_ = size;}
    void SetDrawCube(bool draw, double size){bCube_ = draw; fCubeSize_ = size;}
    void SetDrawTetrahedron(bool draw){bTetrahedron_=draw;}
    void SetDrawIcosahedron(bool draw){bIcosahedron_=draw;}
  
	private:	
		
		bool bTeapot_;		
		double fTeapotSize_;
		
		bool bCube_;		
		double fCubeSize_;
		
		bool bTetrahedron_;
		bool bIcosahedron_;
	};
}
#endif
