/*
 * SceneNurbsSurface.cpp
 *
 *  Created on: Nov 18, 2009
 *      Author: bruenger
 */
#include <iostream>
#include <fstream>
#include <Base/Common/XMLIO.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/Image.hh>
#include "SceneNurbsSurface.hh"

using namespace std;
using namespace BIAS;

SceneNurbsSurface::SceneNurbsSurface() {
	theNurb_ = gluNewNurbsRenderer();
    gluNurbsProperty(theNurb_, GLU_SAMPLING_TOLERANCE, 25.0);
    gluNurbsProperty(theNurb_, GLU_DISPLAY_MODE, GLU_FILL);
    animated_ = false;
    initialized_ = false;
    textured_ = false;
    showpoints_ = false;
}

SceneNurbsSurface::~SceneNurbsSurface() {
}

void SceneNurbsSurface::Init()
{
	if(!surface_.GetInitialized()){
		cout << "SceneNurbsSurface::Init - Please initialize the Surface first" << endl;
		BIASABORT;
	}

	dimu_ = surface_.GetDimu();
	dimv_ = surface_.GetDimv();
	order_ = surface_.GetOrder();


	int i, j, k;

	knotsu_ = new GLfloat[dimu_ + order_ + 1];

	for(i = 0; i < (dimu_ + order_ + 1); i++){
		knotsu_[i] = (GLfloat)surface_.GetKnotu(i);
	}

	knotsv_ = new GLfloat[dimv_ + order_ + 1];

	for(i = 0; i < (dimv_ + order_ + 1); i++){
		knotsv_[i] = (GLfloat)surface_.GetKnotv(i);
	}


	Vector3<float> vec;

	ctrlpoints_ = new GLfloat[dimu_*dimv_*3];

	for(k = 0; k < dimu_; k++){
		for(j = 0; j < dimv_; j++){
			vec = surface_.GetCtrlPoint(k,j);
			for(i = 0; i < 3; i++){
				ctrlpoints_[(k*dimv_+j)*3 + i] = (GLfloat)vec[i];
			}
		}
	}


	int t = 1;

	for(k = 0; k < dimu_; k++){
		for(j = 0; j < dimv_; j++){
			cout << "CTRL id: " << t << " - x:" << ctrlpoints_[(k*dimv_+j)*3] << ", y:" << ctrlpoints_[(k*dimv_+j)*3 + 1] << ", z:" << ctrlpoints_[(k*dimv_+j)*3 + 2] << endl;
			t++;
		}
	}

	initialized_ = true;
}

void SceneNurbsSurface::LoadSurfaceFromXML(std::string xmlfile)
{
	surface_.LoadFromXML(xmlfile);
	Init();
}

GLuint SceneNurbsSurface::LoadTextureRAW(std::string texturefile)
{

	Image<unsigned char> texture;
  ImageIO::Load(texturefile, texture);

	int width = texture.GetWidth();
	int height = texture.GetHeight();
	unsigned char **data = texture.GetImageDataArray();

	// allocate a texture name
	glGenTextures( 1, &tex_ );

	// select our current texture
	glBindTexture( GL_TEXTURE_2D, tex_ );

	// select modulate to mix texture with color for shading
	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	// when texture area is small, bilinear filter the closest mipmap
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
			GL_LINEAR_MIPMAP_NEAREST );
	// when texture area is large, bilinear filter the first mipmap
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );


	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );

	// build our texture mipmaps
	gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width, height,
			GL_RGB, GL_UNSIGNED_BYTE, data[0] );

	textured_ = true;
	return 0;//return something, just to please win32-compiler
}

void SceneNurbsSurface::LoadAnimationFromXML(std::string xmlfile)
{

	if(!initialized_){
		cout << "SceneNurbsSurface::LoadAnimationFromXML - Please initialize the Surface first" << endl;
		BIASABORT;
	}

	if (xmlfile == "") {
		cout << "SceneNurbsSurface::LoadAnimationFromXML - No XML-File given" << endl;
		BIASABORT;
	}



	XMLIO myXML;
	xmlNodePtr rootNode;

	rootNode = myXML.read(xmlfile);

	if (rootNode == NULL) {
		cout << "SceneNurbsSurface::LoadAnimationFromXML - Error while parsing XML-File" << endl;
		BIASABORT;
	}


	//Get count of given Frames
	std::vector<xmlNodePtr> count;
	myXML.GetChildren(rootNode,"frame", count);
	frames_ = myXML.getAttributeValueInt(count.back(),"nr");
	framescount_ = 0;

	cout << "Lade " << frames_ << " Frames. Davon " << count.size() << " Frames definiert" << endl;


	//Fill Array for the initial Frame
	int i, j, k, f;

	Vector3<float> vec;

	ctrlpoints_ = new GLfloat[(frames_+1)*dimu_*dimv_*3];

	//Set Frame 0 from Surface-Data
	for(k = 0; k < dimu_; k++){
		for(j = 0; j < dimv_; j++){
			vec = surface_.GetCtrlPoint(k,j);
			for(i = 0; i < 3; i++){
				ctrlpoints_[(k*dimv_+j)*3 + i] = (GLfloat)vec[i];
			}
		}
	}


	//Now step from given frame to given frame and interpolate
	unsigned int c, ctrlcount = 1;
	int lastframe = 0, currframe;
	xmlNodePtr currentNode;
	xmlNodePtr ctrlpointsNode;
	xmlAttrPtr ctrlpointsAttr;

	for(c = 0; c < count.size(); c++){
		currframe = myXML.getAttributeValueInt(count[c],"nr");
		cout << "Interpoliere " << abs(currframe - lastframe) << " Frames zwischen Frame " << lastframe << " und " << currframe << endl;
		currentNode = myXML.getChild(count[c],"ctrlpoints");
		for(f = (lastframe + 1); f < (currframe + 1); f++){
			//cout << "Berechne Werte fuer Frame " << f << endl;
			for(k = 0; k < dimu_; k++){
				for(j = 0; j < dimv_; j++){

					//Check first if current ctrlpoint is given in animation
          std::stringstream ctrlcountstring;
					ctrlcountstring << ctrlcount ;
					ctrlpointsNode = myXML.GetChildWithAttributeValueString(currentNode,"ctrlpoint","id",ctrlcountstring.str());
					if(ctrlpointsNode != NULL){

						//Check which attributes are given for the defined CtrlPoint

						//If Attr "x" is given, interpolate, else use position from last frame
						ctrlpointsAttr = myXML.getAttributeByName(ctrlpointsNode, "x");
						if(ctrlpointsAttr != NULL){
							ctrlpoints_[((f*dimu_+k)*dimv_+j)*3] = ((GLfloat)ctrlpoints_[((lastframe*dimu_+k)*dimv_+j)*3] * (GLfloat)(((abs(currframe - lastframe)) - (f - lastframe))/(GLfloat)(abs(currframe - lastframe)))) + ((GLfloat)myXML.getAttributeValueDouble(ctrlpointsNode,"x") * (GLfloat)((f - lastframe)/(GLfloat)(abs(currframe - lastframe))));
						}else{
							ctrlpoints_[((f*dimu_+k)*dimv_+j)*3] = ctrlpoints_[(((f-1)*dimu_+k)*dimv_+j)*3];
						}

						//If Attr "y" is given, interpolate, else use position from last frame
						ctrlpointsAttr = myXML.getAttributeByName(ctrlpointsNode, "y");
						if(ctrlpointsAttr != NULL){
							ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 1] = ((GLfloat)ctrlpoints_[((lastframe*dimu_+k)*dimv_+j)*3 + 1] * (GLfloat)(((abs(currframe - lastframe)) - (f - lastframe))/(GLfloat)(abs(currframe - lastframe)))) + ((GLfloat)myXML.getAttributeValueDouble(ctrlpointsNode,"y") * (GLfloat)((f - lastframe)/(GLfloat)(abs(currframe - lastframe))));
						}else{
							ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 1] = ctrlpoints_[(((f-1)*dimu_+k)*dimv_+j)*3 + 1];
						}

						//If Attr "z" is given, interpolate, else use position from last frame
						ctrlpointsAttr = myXML.getAttributeByName(ctrlpointsNode, "z");
						if(ctrlpointsAttr != NULL){
							ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 2] = ((GLfloat)ctrlpoints_[((lastframe*dimu_+k)*dimv_+j)*3 + 2] * (GLfloat)(((abs(currframe - lastframe)) - (f - lastframe))/(GLfloat)(abs(currframe - lastframe)))) + ((GLfloat)myXML.getAttributeValueDouble(ctrlpointsNode,"z") * (GLfloat)((f - lastframe)/(GLfloat)(abs(currframe - lastframe))));
						}else{
							ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 2] = ctrlpoints_[(((f-1)*dimu_+k)*dimv_+j)*3 + 2];
						}

					}else{
						//If current ctrlpoint is not given in this frame use position from last frame
						ctrlpoints_[((f*dimu_+k)*dimv_+j)*3] = ctrlpoints_[(((f-1)*dimu_+k)*dimv_+j)*3];
						ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 1] = ctrlpoints_[(((f-1)*dimu_+k)*dimv_+j)*3 + 1];
						ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 2] = ctrlpoints_[(((f-1)*dimu_+k)*dimv_+j)*3 + 2];
					}
					ctrlcount++;
				}
			}
			ctrlcount = 1;
		}
		lastframe = currframe;
	}

	/*//-----------------------------
	int t = 1;

	for(f = 0; f <= frames_; f++){
		cout << "Frame: " << f << endl;
		for(k = 0; k < dimu_; k++){
			for(j = 0; j < dimv_; j++){
			cout << "CTRL id: " << t << " - x:" << ctrlpoints_[((f*dimu_+k)*dimv_+j)*3] << ", y:" << ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 1] << ", z:" << ctrlpoints_[((f*dimu_+k)*dimv_+j)*3 + 2] << endl;
			t++;
			}
		}
	}
	//---------------------------*/
	animated_ = true;

}

int SceneNurbsSurface::NextFrame()
{
	if (!animated_) {
		cout << "SceneNurbsSurface::NextFrame - No Frames available because the scene is not animated" << endl;

	}else{
		framescount_++;
		if(framescount_ > frames_){
			framescount_ = 0;
		}
	}

	return framescount_;
}

void SceneNurbsSurface::Draw() {

	if(initialized_){

		float tknots[4] = { 0.0, 0.0, 1.0, 1.0 };
		float tcoords[8] = {0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0, 1.0};

		if(textured_){
			glEnable(GL_TEXTURE_2D);
		}
		glEnable(GL_COLOR_MATERIAL);


		glPushMatrix();

		gluBeginSurface(theNurb_);

		if(!textured_){
			glColor3f(1.0, 0.0, 0.0);
		}else{
			glColor3f(1.0, 1.0, 1.0);
			gluNurbsSurface(theNurb_, 4, tknots, 4, tknots, 2*2, 2, &tcoords[0], 2, 2, GL_MAP2_TEXTURE_COORD_2);
		}

		gluNurbsSurface(theNurb_, (dimu_ + order_ + 1), knotsu_, (dimv_ + order_ + 1), knotsv_, dimv_ * 3, 3, &ctrlpoints_[framescount_*(dimu_*dimv_*3)], (order_ + 1), (order_ + 1), GL_MAP2_VERTEX_3);

		gluEndSurface(theNurb_);

		if(showpoints_){
			int k, j;

			glPointSize(5.0);
			glDisable(GL_LIGHTING);
			if(glIsEnabled(GL_TEXTURE_2D)){
				glDisable(GL_TEXTURE_2D);
			}
			glEnable(GL_COLOR_MATERIAL);
			if(!textured_){
				glColor3f(0.0, 1.0, 0.0);
			}else{
				glColor3f(0.0, 0.0, 1.0);
			}
			glBegin(GL_POINTS);
			for(k = 0; k < dimu_; k++){
				for(j = 0; j < dimv_; j++){
					glVertex3f(ctrlpoints_[((framescount_*dimu_+k)*dimv_+j)*3], ctrlpoints_[((framescount_*dimu_+k)*dimv_+j)*3 + 1], ctrlpoints_[((framescount_*dimu_+k)*dimv_+j)*3 + 2]);
				}
			}
			glEnd();
			glDisable(GL_COLOR_MATERIAL);
			glEnable(GL_LIGHTING);
		}

		glPopMatrix();

		if(textured_){
			glDisable(GL_TEXTURE_2D);
		}
		glDisable(GL_COLOR_MATERIAL);


	}
}
