/*
 * SceneNurbsSurface.hh
 *
 *  Created on: Nov 18, 2009
 *      Author: bruenger
 */

#ifndef SCENENURBSSURFACE_HH_
#define SCENENURBSSURFACE_HH_


#include <MathAlgo/NurbsSurface.hh>
#include "SceneBase.hh"

namespace BIAS {
/** \class SceneNurbsSurface
    \brief Scene that renders a NURBS surface 
    \ingroup g_glviewer
    \author Johannes Bruenger
    \date 02/2010
 */
class BIASGLviewer_EXPORT SceneNurbsSurface : public SceneBase {
public:
	SceneNurbsSurface();
	virtual ~SceneNurbsSurface();
	void Init();
	void LoadSurfaceFromXML(std::string xmlfile);
	void LoadAnimationFromXML(std::string xmlfile);
	GLuint LoadTextureRAW(std::string texturefile);

	int NextFrame();
	inline bool GetShowpoints() const {	return showpoints_;	}
	inline void SetShowpoints(bool showpoints_) 	{	this->showpoints_ = showpoints_;	}
    inline bool getInitialized() const
    {
        return initialized_;
    }

    inline void setInitialized(bool initialized_)
    {
        this->initialized_ = initialized_;
    }

	NurbsSurface surface_;
protected:
	virtual void Draw();
	GLUnurbsObj *theNurb_;
	GLfloat *ctrlpoints_;
	GLfloat *knotsu_;
	GLfloat *knotsv_;
	int dimu_;
	int dimv_;
	int order_;
	bool animated_;
	bool initialized_;
	bool textured_;
	bool showpoints_;
	int frames_;
	int framescount_;
	GLuint tex_;
};

}

#endif /* SCENENURBSSURFACE_HH_ */
