#include "SceneText.hh"
#include <GLviewer/ContextWX.hh>

#ifdef BIAS_HAVE_OPENGL
#  include <Gui/biasgl.h>
#ifdef __APPLE__
# include <OpenGL/glu.h>
#else // __APPLE_
# include <GL/glu.h>
#endif // __APPLE__
#  include <Utils/StatekeeperGL.hh>
#endif


using namespace std;
using namespace BIAS;


SceneText::SceneText(bool MakeDemoText): isInitialized_(false) {
	NextIndex_ =0;
	if (MakeDemoText) {
    textPositions_[NextIndex_]=HomgPoint3D(0.0,0.0,1.0);
    textContents_[NextIndex_]="Northern Star";
    textColors_[NextIndex_]=RGBf(1.0f, 0.0f, 0.0f);
	NextIndex_++ ;
  }

  fontName_ = "-*-helvetica-bold-r-normal--12-*-*-*-p-*-iso8859-1";//DrawTextGL_DEFAULT_FONTNAME;
  fontHeight_  = 12;
  Context_ = NULL;
  
}

unsigned int SceneText::AddText(const HomgPoint3D &textPositions,const string& textContents,
				const RGBf& colors)
{
  textPositions_[NextIndex_]=textPositions;
  textContents_[NextIndex_]=textContents;
  textColors_[NextIndex_]=colors;
  return NextIndex_++;
}


void SceneText::SetText(const std::vector<BIAS::HomgPoint3D>& textPositions,
			      const std::vector<std::string>& textContents,
				     const std::vector<BIAS::RGBf>& colors)
{
	for (unsigned int i=0;i<textPositions.size();i++)
		AddText(textPositions[i],textContents[i],colors[i]);
}


void SceneText::RemoveText(unsigned int i)
{
textPositions_.erase(i);
textContents_.erase(i);
textColors_.erase(i);
}


void SceneText::UpdateText(unsigned int index,const BIAS::HomgPoint3D& textPositions,
                 const std::string& textContents,
                 const BIAS::RGBf& colors )
{
  BIASASSERT(textPositions_.find(index) != textPositions_.end());
  textPositions_[index]=textPositions;
  textContents_[index]=textContents;
  textColors_[index]=colors;
  
}


void SceneText::Draw()
{
// #ifdef WIN32
//   BIASWARNONCE("windows font selection mechanism not yet implmented here.");
//   return;
// #endif
#ifdef BIAS_HAVE_OPENGL

  if (!isInitialized_) {

    void *handleDC = NULL;
#ifdef WIN32
    BIASASSERT(Context_ != NULL);
    ContextWX *CWX = dynamic_cast<ContextWX *>(Context_);
    BIASASSERT(CWX != NULL);
    handleDC = GetDC(HWND(CWX->GetHandle())); //Use CWX->GetHWDN()?
#endif

#ifdef BIAS_HAVE_FONTGL
    if (drawer_.InitFont(handleDC, fontName_, fontHeight_)!=0) {
      cerr<<"init of font failed ..."<<endl;
      return;
    }
#else
    BIASWARNONCE("FontGL is disabled in BIAS.");
    return;
#endif
    isInitialized_ = true;
  }

  map<unsigned int,BIAS::HomgPoint3D>::iterator p3dit=textPositions_.begin();
  map<unsigned int,std::string>::iterator textit=textContents_.begin();
  map<unsigned int,BIAS::RGBf>::iterator colorit=textColors_.begin();


  while (p3dit != textPositions_.end()) {
    glColor3f(colorit->second[0], colorit->second[1], colorit->second[2]);
#ifdef BIAS_HAVE_FONTGL
    drawer_.Print3D(textit->second, p3dit->second[0], 
                     p3dit->second[1],p3dit->second[2]);

    

    //drawer_.Print3D(".", textPositions_[i][0], 
    //                textPositions_[i][1], textPositions_[i][2]);
    glBegin(GL_POINTS);
    glVertex3f(p3dit->second[0], p3dit->second[1],p3dit->second[2]);
    glEnd();
#endif
    p3dit++;
    textit++;
    colorit++;
  }
#endif
 
}
 





#ifdef BIAS_HAVE_XML2

int SceneText::XMLGetClassName(std::string& TopLevelTag,
                               double& Version) const {
  TopLevelTag = "SceneText";
  Version = 0.1;
  return 0;
}

int SceneText::XMLOut(const xmlNodePtr mainNode, XMLIO& XMLObject) const {
	std::map<unsigned int,BIAS::HomgPoint3D>::const_iterator itP= textPositions_.begin();
	std::map<unsigned int,std::string>::const_iterator itC= textContents_.begin();
    std::map<unsigned int,BIAS::RGBf>::const_iterator itCC=  textColors_.begin();
  for (; itP!=textPositions_.end(); ) {
    xmlNodePtr childNode = XMLObject.addChildNode(mainNode, "Text");
     
	XMLObject.addAttribute(childNode, "r", itCC->second[0]);
    XMLObject.addAttribute(childNode, "g", itCC->second[1]);
    XMLObject.addAttribute(childNode, "b", itCC->second[2]);

    XMLObject.addAttribute(childNode, "x", itP->second[0]);
    XMLObject.addAttribute(childNode, "y", itP->second[1]);
    XMLObject.addAttribute(childNode, "z", itP->second[2]);
    XMLObject.addAttribute(childNode, "w", itP->second[3]);
    
	XMLObject.addContent(childNode, itC->second);
	itP++;
	itC++;
	itCC++;
  }
  return 0;
}

int SceneText::XMLIn(const xmlNodePtr Node, XMLIO& XMLObject) {
  textColors_.clear();
  textPositions_.clear();
  textContents_.clear();
  xmlNodePtr childNode = XMLObject.getFirstChild(Node);
  while (!(childNode==NULL)) {
    if (XMLObject.getNodeName(childNode)=="Text") {
      RGBf thecolor;
      HomgPoint3D theposition;
     
      thecolor[0] = XMLObject.getAttributeValueDouble(childNode, "r");
      thecolor[1] = XMLObject.getAttributeValueDouble(childNode, "g");
      thecolor[2] = XMLObject.getAttributeValueDouble(childNode, "b");

      theposition[0] = XMLObject.getAttributeValueDouble(childNode, "x");
      theposition[1] = XMLObject.getAttributeValueDouble(childNode, "y");
      theposition[2] = XMLObject.getAttributeValueDouble(childNode, "z");
      theposition[3] = XMLObject.getAttributeValueDouble(childNode, "w");
      string theContent = "___"+XMLObject.getNodeContentString(childNode);
	  AddText(theposition,theContent,thecolor);
      cout<<"Found "<<theContent<<" at "<<theposition<<" with color "
          <<thecolor<<endl;      
    }
    childNode = XMLObject.getNextChild();
  }
  cout<<"Read "<<textColors_.size()<<" texts"<<endl;  
 
  return 0;
}

#endif // BIAS_HAVE_XML2




