#ifndef __SCENE_TEXT_HH__
#define __SCENE_TEXT_HH__

#include <bias_config.h>

#ifdef BIAS_HAVE_FONTGL
#ifdef BIAS_HAVE_OPENGL
#include <Base/Common/BIASpragmaStart.hh>
#include <Utils/DrawTextGL.hh>
#include <Base/Common/BIASpragmaEnd.hh>
#endif
#endif

#ifdef BIAS_HAVE_XML2
#include <Base/Common/XMLBase.hh>
#endif

#include <GLviewer/Scenes/SceneBase.hh>
#include <Base/Geometry/HomgPoint3D.hh>
#include <Base/Math/RGB.hh>
#include <string>
#include <vector>

namespace BIAS {


	class BIASGLviewer_EXPORT RenderContextBase;
  /** \class SceneText 
      \brief wrapping bias::drawtextgl to BIAS::Scene
      \author koeser
      \ingroup g_glviewer
   */
  class BIASGLviewer_EXPORT SceneText : public SceneBase
#ifdef BIAS_HAVE_XML2
      , public BIAS::XMLBase
#endif
  {
  public:
    virtual ~SceneText() {}
    SceneText(bool MakeDemoText = true);
	
	// a context is required to get a DC handle for init
	void SetContext(RenderContextBase *c){Context_ = c;};
    
    void SetText(const std::vector<BIAS::HomgPoint3D>& textPositions,
                 const std::vector<std::string>& textContents,
                 const std::vector<BIAS::RGBf>& colors);
  
    unsigned int AddText(const BIAS::HomgPoint3D& textPositions,
                 const std::string& textContents,
                 const BIAS::RGBf& colors) ;

    void RemoveText(unsigned int);
    void UpdateText(unsigned int,const BIAS::HomgPoint3D& textPositions,
                 const std::string& textContents,
                 const BIAS::RGBf& colors );

    void Clear() {
      textPositions_.clear();
      textContents_.clear();
      textColors_.clear();
    }
  protected:
    virtual void Draw(); 
    

#ifdef BIAS_HAVE_XML2
    /** @brief specialization of XML block name function */
    virtual int XMLGetClassName(std::string& TopLevelTag,
                                double& Version) const;
    
    /** @brief specialization of XML write function */
    virtual int XMLOut(const xmlNodePtr Node, BIAS::XMLIO& XMLObject) const;
    
    /** @brief specialization of XML read function */
    virtual int XMLIn(const xmlNodePtr Node, BIAS::XMLIO& XMLObject); 
#endif
    
#ifdef BIAS_HAVE_OPENGL
#ifdef BIAS_HAVE_FONTGL
    BIAS::DrawTextGL drawer_;
#endif
#endif
    std::map<unsigned int,BIAS::HomgPoint3D> textPositions_;
    std::map<unsigned int,std::string> textContents_;
    std::map<unsigned int,BIAS::RGBf> textColors_;
    unsigned int NextIndex_;
    std::string fontName_;
    int  fontHeight_;
    bool isInitialized_;
    RenderContextBase *Context_;
  };  
}//end of namespace

#endif
