#ifndef __SCENE_TEXT_SCREEN_HH__
#define __SCENE_TEXT_SCREEN_HH__

#include <bias_config.h>

#ifdef BIAS_HAVE_OPENGL
#ifdef BIAS_HAVE_FONTGL
#include <Utils/DrawTextGL.hh>
#endif
#endif

#ifdef BIAS_HAVE_XML2
#include <Base/Common/XMLBase.hh>
#endif

#include <GLviewer/Scenes/SceneBase.hh>
#include <Base/Geometry/HomgPoint3D.hh>
#include <Base/Math/RGB.hh>
#include <string>
#include <vector>

namespace BIAS {
  /** \class SceneTextScreen 
      \brief wrapping bias::drawtextgl to BIAS::Scene
      \author haase
      \ingroup g_glviewer
   */
  class BIASGLviewer_EXPORT SceneTextScreen : public SceneBase
#ifdef BIAS_HAVE_XML2
      , public BIAS::XMLBase
#endif
  {
  public:
    virtual ~SceneTextScreen() {}
    SceneTextScreen(bool MakeDemoText = true);
      
    
    void SetText(const std::vector<BIAS::HomgPoint3D>& textPositions,
                 const std::vector<std::string>& textContents,
                 const std::vector<BIAS::RGBf>& colors) {
      textPositions_ = textPositions;
      textContents_  = textContents;
      textColors_    = colors;
    }



    void Free(); 
  protected:
    virtual void Draw(); 
      

#ifdef BIAS_HAVE_XML2
    /** @brief specialization of XML block name function */
    virtual int XMLGetClassName(std::string& TopLevelTag,
                                double& Version) const;
    
    /** @brief specialization of XML write function */
    virtual int XMLOut(const xmlNodePtr Node, BIAS::XMLIO& XMLObject) const;
    
    /** @brief specialization of XML read function */
    virtual int XMLIn(const xmlNodePtr Node, BIAS::XMLIO& XMLObject); 
#endif
    
#ifdef BIAS_HAVE_OPENGL
#ifdef BIAS_HAVE_FONTGL
    BIAS::DrawTextGL drawer_;
#endif
#endif
    std::vector<BIAS::HomgPoint3D> textPositions_;
    std::vector<std::string> textContents_;
    std::vector<BIAS::RGBf> textColors_;

    std::string fontName_;
    int  fontHeight_;
    bool isInitialized_;
  };  
}//end of namespace

#endif
