#ifndef __SCENE_THREEDOUT_WRAPPER_HH__
#define __SCENE_THREEDOUT_WRAPPER_HH__

#include <bias_config.h>

#include <Utils/ThreeDOut.hh>
#include <GLviewer/Scenes/SceneBase.hh>
#include <Base/Math/Vector3.hh>
#include <GLviewer/TriangleMeshSplatRendering.hh>

// These are the flags for the protected variable RenderFlags_
enum {
  RENDER_CAMERA               = 1<<0,
  DRAW_ELLIPSOIDS             = 1<<1,
  DRAW_CAM_ELLIPSOIDS         = 1<<2
};

namespace BIAS {
  /** \class SceneThreeDOutWrapper 
      \brief Wrapping ThreeDOut OpenGL-functionality.
      \author bartczak
      \ingroup g_glviewer
   */
  class BIASGLviewer_EXPORT SceneThreeDOutWrapper : public SceneBase
  {
  public:
    virtual ~SceneThreeDOutWrapper() { 
      if (ownerOfThreeDOut_) 
        if (threeDOutObject_!=NULL) delete threeDOutObject_;
      threeDOutObject_ = NULL;
    }
    
    SceneThreeDOutWrapper() { threeDOutObject_ = NULL;
                              RenderFlags_ = RENDER_CAMERA; 
                              ownerOfThreeDOut_ = false; }
    

    void LinkWithThreeDOut(BIAS::ThreeDOut* threeDOutObject)
    {
      threeDOutObject_ = threeDOutObject;
      triangleSplatting_.AddThreeDOut(threeDOutObject_);
    }
    /**
    * Set the specified flag to true or false.
    */
    void                          SetFlag(unsigned long int Flag);
    void                          UnsetFlag(unsigned long int Flag);  
    bool                          GetFlag(unsigned long int Flag);
    virtual void                  GetBoundingBox(BIAS::Vector3<double> & min,
                                                 BIAS::Vector3<double> & max);
    virtual BIAS::Vector3<double> GetBoundingBoxCenter();
    inline  bool                  HasObject() const {return(threeDOutObject_!=NULL);};    
    virtual void                  MoveScene(const double& delta = 1.0);
    void                          SetThreeDOutObjectOwnership(bool o){ownerOfThreeDOut_=o;};
      
  protected:
    virtual void                  Draw();

  protected:
    BIAS::TriangleMeshSplatRendering triangleSplatting_;
    BIAS::ThreeDOut*                threeDOutObject_;
    unsigned long int               RenderFlags_;
    bool                            ownerOfThreeDOut_;  
  };  
}//end of namespace

#endif
