#ifndef __ScreenShotListener_hh__
#define __ScreenShotListener_hh__

#include <GLviewer/Controller/ControlInterface.hh>
#include <GLviewer/RenderContextBase.hh>

namespace BIAS {
  /** \class ScreenShotListener
      \brief If KEY_F5 is activated screenshot is stored to given basename +
      screenshot count.
      \author bartczak
      \ingroup g_glviewer
  */
  class BIASGLviewer_EXPORT ScreenShotListener : public ControlInterface {
  public :
    ScreenShotListener();
    void Init(RenderContextBase* context,
              const std::string& baseName,
              const std::string& extension,
              const std::string& descriptor = "",
              bool storeDepthMap = false,
              bool storeZBuffer = false,              
              bool storeMetricZMap = false);
    ScreenShotListener(RenderContextBase* context,
                       const std::string& baseName,
                       const std::string& extension,
                       const std::string& descriptor = "",
                       bool storeDepthMap = false,
                       bool storeZBuffer = false,
                       bool storeMetricZMap = false);
    virtual bool SpecialKeyPressed(int key);
    void StoreDepthMap(bool on = true) { storeDepthMap_ = on; }
    void StoreCameraParams(bool on = false){storeParams_ = on;}
    void StoreProjectionParams(bool on = false){storeProjectionParams_ = on;}
    void ScreenShotOnTimer(bool active = true) { screenShotOnTimer_ = active; }
    bool TimerExpired();
    
    virtual std::string Name() { return "ScreenShotListener"; }  

    void AllowGreyImages(bool storeGrey = true) {storeGrey_ = storeGrey;}

    bool StoreScreenShot() {return StoreNextScreenShot_();}

    void SetDescriptor(const std::string& descriptor) {
      descriptor_ = descriptor;
    }
    
    /** the parameters written into the metadata of screenshot are either
     *  taken from the slot specified. Which will then be the parameters
     *  with an absolute pose (relative to rig on the contrary).
     *  If slot is invalid (e.g negative) the whole projection is written into 
     *  image metadata.
     */
    void UseSlotParameters(int slot) {activeSlotParameter_ = slot; }

    void ResetCounter() {
      screenshotCounter_=0;
    }

    void SetCounter(int counter){
      screenshotCounter_=counter;
    }

  protected :
    int activeSlotParameter_;
    bool StoreNextScreenShotCameraParams_();
    bool StoreNextScreenShot_();
    RenderContextBase* context_;
    int screenshotCounter_;
    std::string screenShotBaseName_;
    std::string descriptor_;
    std::string screenShotExtension_;
    bool storeDepthMap_;
    bool storeMetricZMap_;
    bool storeZBuffer_;
    bool screenShotOnTimer_;
    bool storeGrey_;
    bool storeParams_;
    bool storeProjectionParams_;
  };

}
#endif

