#ifndef __ThreeDOutOpenSceneGraph_hh__
#define __ThreeDOutOpenSceneGraph_hh__

#include <bias_config.h>
#include <Utils/ThreeDOut.hh>

namespace osg {
  class Node;
  class Group;
  class Geode;
  class Geometry;

  template <class T> class ref_ptr;
}


namespace BIAS {
   /** @class ThreeDOutOpenSceneGraph
       @brief a class for exporting ThreeDOut objects to OSG scene graphs
       @ingroup g_glviewer
       @author Grauel 10/2007 
   */

  class BIASGLviewer_EXPORT ThreeDOutOpenSceneGraph
    : public BIAS::ThreeDOut
  {
  public:
    ThreeDOutOpenSceneGraph();
    ~ThreeDOutOpenSceneGraph();
    /** @brief save ThreeDOut object as OpenSceneGraph
        saves an Open Scene Graph and the needed textures
        @param filename filename of scene graph (XXXX.osg or XXXXXXX.ive)
        OpenScenegraphOut() saves the textures contained in ThreeDOut
        to disk, since they can't be embedded in .osg-files.
        the filename will be used as prefix for the texture file names.
        In case of saving an osg binary format file (.ive)
        the textures will be included in this file.
    */
    int OpenSceneGraphOut(const std::string &filename) const;

    /** @brief Get ThreeDOut object as OpenSceneGraph structure
        @param sg an empty ref_ptr 
        after calling GetOpenSceneGraph sg points to the scene graph structure 
    */
    int GetOpenSceneGraph(osg::ref_ptr<osg::Group>& sg, bool exporttextures=false) const;

    /** @brief Set the file extension and type of the saved textures
        @param ext a string with no leading point. e.g. "jpeg"
        if OpenScenegraphOut() is called the textures contained in ThreeDOut
        are saved to disk. the default type of this textures is "tif".
        By calling SetImageFileExtension() the file type/extension can be
        changed. */
    void SetImageFileExtension(std::string ext)
    { TexImgExtension_ = ext; }


  protected:    
    int GetSceneGraph_(osg::ref_ptr<osg::Group>& sg,
                       bool exporttextures,
                       const std::string &filename ) const;

    void AddOSGLines_(osg::ref_ptr<osg::Group>& Rootnode) const;

    //    copy internal points (from ThreeDOut) into a new subtree of
    //    @param Rootnode
    void AddOSGPoints_(osg::ref_ptr<osg::Group>& Rootnode) const;

    /** create one transform (IndexedFaceSets), add another transform
     for each indexed face set, then convert all indexed face-set data
     from ThreeDOut  */
    osg::Node * AddOSGIndexedFaceSets_(bool exporttextures,
                                std::string graphfilename) const;

   
 
    void AddOSGImage_(osg::ref_ptr<osg::Group>& Rootnode,
                      bool exportTextures,
                      std::string graphfilename) const;
    
    //can be used for standard and camera elliposoids
    void AddOSGEllipsoids_(osg::ref_ptr<osg::Group>& rootNode, 
                      bool camEllipsoids=false) const;
    void AddOSGPMatrix_(osg::ref_ptr<osg::Group>& rootNode) const;

    std::string 
       GenerateFilename_(std::string graphfilename,
                         std::string subname,
                         unsigned int texnumber) const;

    std::string TexImgExtension_;
    
  };


} // namespace


#endif // __ThreeDOutOpenSceneGraph_hh__
