#include "View.hh"
#include <algorithm>

using namespace std;
using namespace BIAS;
using namespace BIAS;

View::
View()
{
  viewport_  = Vector4<int>(0, 0, 640, 480);
  clearColor_ = Vector4<float>(0.0f, 0.0f, 0.0f, 0.0f);
  camera_ = NULL;
  backgroundImageScene_ = NULL;
}

void View::
SetViewport(int x, int y, int width, int height)
{
  viewport_ = Vector4<int>(x, y, width, height);
}

void View::
SetClearColor(float red, float green, float blue, float alpha)
{
  clearColor_ = Vector4<float>(red, green, blue, alpha);
}

void View::
SetBackgroundImageScene(SceneBGImage* bg)
{
  backgroundImageScene_ = bg;
}

void View::
SetGLProjectionParametersInterface(GLProjectionParametersInterface* camera)
{
  camera_ = camera;

  for (unsigned int i = 0; i < scenes_.size(); i++) {
    scenes_[i]->SetGLProjectionParametersInterface(camera);
  }
}

void View::
AppendScene(SceneBase* scene)
{
  BIASASSERT(scene != NULL);
  scene->SetGLProjectionParametersInterface(camera_);
  scenes_.push_back(scene);
}

void View::
RemoveScene(SceneBase* scene)
{
  vector<SceneBase*>::iterator itor = find(scenes_.begin(), scenes_.end(), scene);
  if (itor == scenes_.end()) {
    BIASERR("Scene to be removed is not contained by the view");
    return;
  }
  scenes_.erase(itor);
}

void View::
InsertSceneAt(SceneBase* scene, int index)
{
  if (index <= (int)scenes_.size()) {
    vector<SceneBase*>::iterator itor = scenes_.begin();
    for (int i = 0; i < index; i++) {
      itor++;
    }
    scenes_.insert(itor, scene);
  } else {
    scenes_.push_back(scene);
  }
  scene->SetGLProjectionParametersInterface(camera_);
}

void View::
RemoveSceneAt(SceneBase* scene, int index)
{
  if (index < 0 || index >= (int)scenes_.size()) {
    BIASERR("Scene index is out of range");
    return;
  }
  
  vector<SceneBase*>::iterator itor = scenes_.begin();
  for (int i = 0; i < index; i++) {
    itor++;
  }
  scenes_.erase(itor);
}

int View::
Render()
{
  // get viewport to be restored after the view was rendered
  GLint oldViewport[4];
  glGetIntegerv(GL_VIEWPORT, oldViewport);
  
  // set viewport
  glViewport(viewport_[0], viewport_[1], viewport_[2], viewport_[3]);

  // draw background quad to clear the depth buffer and color buffer in the viewport
  glPushAttrib(GL_DEPTH_BUFFER_BIT);
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_ALWAYS);
  
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  
  glBegin(GL_QUADS);
  glColor4f(clearColor_[0], clearColor_[1], clearColor_[2], clearColor_[3]);
  glVertex3f(-1, -1, 1);
  glVertex3f( 1, -1, 1);
  glVertex3f( 1,  1, 1);
  glVertex3f(-1,  1, 1);
  glEnd();

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  
  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();

  glPopAttrib();

  // draw scenes through the GLProjection
  int result = 0;
  if (camera_ != NULL) {
    result = camera_->Draw(scenes_, backgroundImageScene_);
    /*
    for (int i = 0; i < (int)scenes_.size(); i++) {
      scenes_[i]->FinishedDraw();
    }
    */
  }

  // restore old viewport
  glViewport(oldViewport[0],
             oldViewport[1],
             oldViewport[2],
             oldViewport[3]);

  return result;
}
