#ifndef __View_hh__
#define __View_hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include "Scenes/SceneBase.hh"
#include "GLProjectionParametersInterface.hh"
#include <Base/Common/W32Compat.hh>

#include <vector>


namespace BIAS {

  /**
   * @brief A view is a rectangular part of a render context in which a set of scenes
   * is rendered using a GLProjection.
   * @author jkollmann
   * @ingroup  g_glviewer
   */
  class BIASGLviewer_EXPORT View {
  public:
    View();

    void SetViewport(int x, int y, int width, int height);
    void SetClearColor(float red, float green, float blue, float alpha = 1.0f);

    /** Sets the GLProjection to be used for rendering this view.
        Every GLProjection can only be added to one view! */
    void SetGLProjectionParametersInterface(GLProjectionParametersInterface* camera);

    void SetBackgroundImageScene(SceneBGImage* bg);

    /** Appends a scene to the list of scenes to be rendered in this view.
        Every scene can only be added to one view! */
    void AppendScene(SceneBase* scene);

    /** Removes a scene from the list of scenes. */
    void RemoveScene(SceneBase* scene);

    /** Inserts a scene to the given position in the list of scenes.
        Every scene can only be added to one view!*/
    void InsertSceneAt(SceneBase* scene, int index);

    /** Removes the scene with the given index. */
    void RemoveSceneAt(SceneBase* scene, int index);

    int Render();
    
  private:
    BIAS::Vector4<int>          viewport_;
    BIAS::Vector4<float>        clearColor_;
    GLProjectionParametersInterface*      camera_;
    std::vector<SceneBase*>     scenes_;
    SceneBGImage*               backgroundImageScene_;
  };
  
  
} // namespace BIAS

#endif // __View_hh__
