/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example ExampleExtractKFromMipImage.cpp
   @class ExampleExtractKFromMipImage
   @relates KMatrix,Image,
   @brief Example for extracting the K-Matrix from a mip image
   @ingroup g_examples
   @author MIP
*/

#include <iostream>
#include <sstream>

#include <Image/Camera.hh>
#include <Base/Image/ImageIO.hh>
#include <Geometry/Projection.hh>
#include <Geometry/ProjectionParametersPerspective.hh>
#include <Base/Geometry/KMatrix.hh>

using namespace BIAS;
using namespace std;


int main(int argc, char** argv) {
  
  // argument specified?
  if (argc != 2) {
    BIASERR("don't know which image to use!/n/n  usage: ExampleExtractKFromMipImage <path_to_image>");
    BIASABORT;
  }
  
  Camera<unsigned char> image;
  
  // load image
  if (ImageIO::Load(argv[1], image) != 0) {
    BIASERR("specified image could not be loaded!");
    BIASABORT;
  }
  cout << "image loaded successfully!" << endl;
  
  // check for valid projection
  image.ParseMetaData();
  if (!image.IsProjValid()) {
    BIASERR("image contains no valid projection!");
    BIASABORT;
  }
  
  // extract projection and K matrix
  cout << "getting K... ";
  Projection proj = image.GetProj();
  ProjectionParametersPerspective* ppp;
  ppp = (ProjectionParametersPerspective*)proj.GetParameters();
  KMatrix K = ppp->GetK();
  
  cout << "done!" << endl;
  
  // save K matrix
  ostringstream filenameK;
  filenameK << argv[1] << ".mat";
  cout << "saving K as " << filenameK.str() << endl;
  K.Save(filenameK.str());
  
  return 0;
}
