/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @example ExampleProjectionParametersIO.cpp
   @class ExampleProjectionParametersIO
   @relates ProjectionParametersIO
   @brief small example demonstrating the ProjectionParametersIO
   @ingroup g_examples
   @author woelk 12/2006
*/

#include <Geometry/ProjectionParametersIO.hh>
#include <iostream>

using namespace BIAS;
using namespace std;

int main (int argc, char* argv[])
{
  CameraData cd, cd2;
  vector<CameraData> vcd, vcd2;
#ifdef BIAS_HAVE_XML2
  if (argc<2) {
    cout <<"no file given, using some build-in data"<<endl;
    ProjectionParametersIO::SetData_OMNITECH_ORIFL_190_3(cd);
  }
  else {
    if (ProjectionParametersIO::ReadCameraData(argv[1],cd)<0) {
      cout << "Error reading file:" << argv[1] << endl;
    } else {
      cout << "Read file:" << argv[1] << endl;
    }
  }
  
  if (argc<3) {
    cout <<"no rig file given, using some build-in data"<<endl;
    ProjectionParametersIO::SetData_DummyRig(vcd);
  } else {
    BIASASSERT(argc>1);
    if (ProjectionParametersIO::ReadRigData(argv[2],vcd)<0) {
      cout << "Error reading file:" << argv[1] << endl;
    } else {
      cout << "Read file:" << argv[1] << endl;
    }
  }
  cout << "File contents: ";
  if (argc>1) 
    cout<<argv[1]  ;
  cout << "-------------------" << endl;
  cout << cd;
  ProjectionParametersIO::WriteCameraData("CamDataTest.xml",cd);
  cout <<"CamDataTest.xml created" << endl;

  ProjectionParametersIO::ReadCameraData("CamDataTest.xml",cd2);
  cout << "File contents: " << "CamDataTest.xml" 
       << "-------------------" << endl;
  cout << cd2;
  ProjectionParametersIO::WriteCameraData("CamDataTestRewrite.xml", cd2);
  cout <<"CamDataTestRewrite.xml created" << endl;

  cout << "File contents: ";
  if (argc>2) 
    cout<<argv[2];  
  cout<< "-------------------" << endl;
  cout << vcd;
  ProjectionParametersIO::WriteRigData("RigDataTest.xml",vcd);
  cout <<"RigDataTest.xml created" << endl;

  ProjectionParametersIO::ReadRigData("RigDataTest.xml",vcd2);
  cout << "File contents: " << "RigDataTest.xml"  
       << "-------------------" << endl;
  cout << vcd2;
  ProjectionParametersIO::WriteRigData("RigDataTestRewrite.xml", vcd2);
  cout <<"RigDataTestRewrite.xml created" << endl;
#endif
  return 0;  
}
