/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Geometry/ProjectionParametersPerspective.hh>
#include <Utils/ThreeDOut.hh>
#include <MathAlgo/SVD.hh>

namespace BIAS {


class BIASGeometry_EXPORT ProjectionError
{
  public:
    enum ELeftRight
    {
      Left,
      Right
    };
    enum EInnerOuter
    {
      Inner,
      Outer
    };
    
    ProjectionError();
    /** @brief Initiates cameras and calculates the length of the baseline based
               on minDepth and angle.
        @param depth Depth for which the baseline shall be calculated.
        @param angle camera1 - object - camera2 (degree) */
    void Init(int res, double depth, double angle);
    void Init(int res, double baseline);
    double ComputeDiff(double z, double offset);
    const ProjectionParametersPerspective* GetCam(ELeftRight which);
    double GetBaseline() const;
    
  private:
	  std::vector<ProjectionParametersPerspective> cam;
    double baseline;
};

} // namespace
