/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include <Geometry/SphericalCoordinates.hh>
  
using namespace BIAS;
using namespace std;


void SphericalCoordinates:: 
SetAffineBase(const CoordinateTransform3D& newAffineBase)
{
  affineReferenceFrame_ = newAffineBase;
}

const CoordinateTransform3D& SphericalCoordinates::
GetAffineBase() const
{
  return affineReferenceFrame_;
}


void SphericalCoordinates::
GetSphericalCoordinatesFullPhi(const HomgPoint3D& point, 
                               double& rho, 
                               double& phi, double& theta) const
{
  HomgPoint3D localPoint = affineReferenceFrame_.GlobalToLocal(point);
  localPoint[3] = 0;
  rho = localPoint.NormL2();

  const double x = localPoint[0];
  const double y = localPoint[1];
  const double z = localPoint[2];

  if((rho<1e-12) || (fabs(x)<1e-12 && fabs(y)<1e-12)) {
    rho = 0;
    phi = 0;
    theta = 0;

  } else {
    phi = atan2(y,x);  
    theta = acos(z/rho);  
  }

  BIASASSERT(phi>-M_PI);
  BIASASSERT(phi<=M_PI);
  BIASASSERT(theta>=0);
  BIASASSERT(theta<=M_PI);

}


void SphericalCoordinates::
GetCartesianRayFromFullPhi(const double& phi, const double& theta,
        HomgPoint3D& ray) const
{
  ray[0] = sin(theta)*cos(phi);
  ray[1] = sin(theta)*sin(phi);
  ray[2] = cos(theta);
  ray[3] = 0;

  ray = affineReferenceFrame_.LocalToGlobal(ray);
}



