/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __TFTensorEstimation_hh__
#define __TFTensorEstimation_hh__

#include <Base/Common/BIASpragmaStart.hh>

#include "TrifocalTensor.hh"
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Geometry/Normalization.hh>

#include <vector>

namespace BIAS
{
  /** class TFTensorEstimation
      @brief compute a trifocal tensor from image correspondences, not robust
      against outliers
      @author Matthias Dunda
      @ingroup g_estimation */
  class BIASGeometry_EXPORT TFTensorEstimation
  {
  public:
    TFTensorEstimation();
    ~TFTensorEstimation();

    /** @brief Estimate a trifocal tensor given correnspondences
        over three images.
        @param HartleyNormalization normalizes correspondences for
        each view according to Hartley
        @return !=0 on error
        @author Matthias Dunda 01/2004 */
    int Compute( BIAS::TrifocalTensor &T,
                 const std::vector<BIAS::HomgPoint2D> &x1,
                 const std::vector<BIAS::HomgPoint2D> &x2,
                 const std::vector<BIAS::HomgPoint2D> &x3,
                 bool HartleyNormalization = true );
  protected:
    BIAS::Normalization Norm_;
    int AlgebraicMinimization(BIAS::TrifocalTensor &Initial,
                              BIAS::TrifocalTensor &Result);
                              
  }; // class

} // namespace

#include <Base/Common/BIASpragmaEnd.hh>

#endif /* __TFTensorEstimation_hh__ */

