/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Vision N GmbH
   Schauenburgerstr. 116
   24118 Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/
#ifndef __DrawTextWx_hh__
#define __DrawTextWx_hh__

#include "bias_config.h"
#include <Base/Image/ColourRGB.hh>

#include <string>

#define DTWX_OK 0
#define DTWX_OUT_OF_IMAGE 1
#define DTWX_INVALID_ARGUMENT -2
#define DTWX_WX_ERROR -3

namespace BIAS {
  
  template <class T> class Image;
  template <class T> class ColourRGB;

  /** @class DrawTextWx
      @brief Writing a text in an bias image usig a wx device context 
      This class uses wxDC and thus it can only be used in wx GUI applications.
      !! It does not work for console applications !!!
      @author woelk 09/2009 (c) www.vision-n.de */
  template <class StorageType>
  class BIASGui_EXPORT DrawTextWx
  {
  public:
    enum EVerticalAlign { V_ALIGN_TOP, V_ALIGN_CENTER, V_ALIGN_BOTTOM };

    enum EHorizontalAlign { H_ALIGN_LEFT, H_ALIGN_CENTER, H_ALIGN_RIGHT };

    DrawTextWx() {};
    
    ~DrawTextWx() {};
    
    /** @brief returns size of the bounding box of the text */
    static int GetTextExtend(const std::string &text,
                             unsigned &width, unsigned &height,
                             const unsigned point_size = 8);

    /** @brief write text in an image
        !!! Can only be used in wx-GUI applications which implement wxApp !!! */
    static int Text(Image<StorageType> &im, const std::string &text,
                     const unsigned &pos_x, const unsigned &pos_y, 
                     const ColourRGB<StorageType> &foreground_color = COLOR_WHITE,
                     const ColourRGB<StorageType> &background_color = COLOR_BLACK,
                     const unsigned point_size = 8,
                     const enum EVerticalAlign vertical_align = V_ALIGN_TOP,
                     const enum EHorizontalAlign horizontal_align = H_ALIGN_LEFT);

  }; // class


} // namespace

#endif // __DrawTextWx_hh__
