/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Vision N GmbH
   Schauenburgerstr. 116
   24118 Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/** 
 * @example ExampleGLShared.cpp
 * @ingroup g_examples
 * @brief OpenGL example for 
 * - using multiple canvases sharing the same canvas inlcuding didplay lists etc.
 * - drawing text in OpenGL Using fonts.
 * Demonstrates
 * - usage of wx OpenGL canvas,
 * - setting up a shared context,
 * - using fonts with OpenGL in absolute and relative mode. Resize the window to see the effect.
 * @author Jan Woetzel 
 */

//BIAS
#include <Gui/OpenGLCanvas.hh>
#ifdef BIAS_HAVE_FONTGL
#  include <Utils/DrawTextGL.hh>
#endif
#include <wx/wx.h>


using namespace BIAS;
using namespace std;


class ExampleGLSharedApp 
  : public /*wxApp*/ wxGLApp
{
  virtual bool OnInit(); 
}; // end of ExampleGLSharedApp -----------------------


enum _ExampleGLSharedMyFrameIDs_ {
  ID_ABOUT=wxID_ABOUT,
  ID_QUIT
};


// --------------------------------------------------------


/** Implement your own OopenGLCanvas derived class
and override one of the Display functions
to plug in your own GL calls / GL scene
@author Jan Woetzl */
class MyGLCanvas : public OpenGLCanvas {
public:
  MyGLCanvas(wxFrame* parent);

  /// constructor using a shared context 
  MyGLCanvas(
    wxFrame* parent
    ,wxGLContext* sharedContext
    ,wxWindowID id = -1
    ,const wxPoint& pos = wxDefaultPosition
    ,const wxSize& size = wxDefaultSize
    ,long style=0
    ,const wxString& name=wxT("MyGLCanvas sharedContext")
    ,int* attribList = 0
    ,const wxPalette& palette = wxNullPalette
    );

  // override to fire our own Display calls: 
  void DisplayCalls();

  //
  // additional members:
  //

  // additional function to load and init geometry, dipsla ylistst etc. JW
  void InitGeometry();

  // data mebers:
  int dplist;
#ifdef BIAS_HAVE_FONTGL
  DrawTextGL font;
#endif
};


class MyGLCanvas2 : public OpenGLCanvas {
public:
  MyGLCanvas2(wxFrame* parent);

  /// constructor using a shared context 
  MyGLCanvas2(
    wxFrame* parent
    ,wxGLContext* sharedContext
    ,wxWindowID id = -1
    ,const wxPoint& pos = wxDefaultPosition
    ,const wxSize& size = wxDefaultSize
    ,long style=0
    ,const wxString& name=wxT("MyGLCanvas sharedContext")
    ,int* attribList = 0
    ,const wxPalette& palette = wxNullPalette
    );

  // override to fire our own Display calls: 
  void DisplayCalls();

  int dplist;
};


// --------------------------------------------------------

class MyFrame: public wxFrame {
public:
  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);

  // data member ---
  //BIAS::OpenGLCanvas * p_canvas;
  MyGLCanvas * p_canvas;
  MyGLCanvas2 * p_2nd_canvas;

  DECLARE_EVENT_TABLE()
}; // end of MyFrame --------------------------------------

