/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Vision N GmbH
   Schauenburgerstr. 116
   24118 Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/**
   @example ExampleImageCanvas.cpp
   @class ExampleImageCanvas
   @relates ImageCanvas
   @brief Simple example for ImageCanvas Usage
   @ingroup g_examples
   @author MIP
*/


#include <bias_config.h>

#include <wx/wx.h>
#include <Gui/ImageCanvas.hh>
#include <Gui/StringConv.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

enum { ID_Quit = 1, ID_About, ID_Load};

class MyApp: public wxApp
{
  virtual bool OnInit();
};

IMPLEMENT_APP(MyApp)

class MyFrame: public wxFrame
{
public:

  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnLoad(wxCommandEvent& event);

  ImageCanvas *_ic;

  DECLARE_EVENT_TABLE()
};

bool MyApp::OnInit()
{
  MyFrame *frame = new MyFrame( wxT("ImageCanvas"), wxPoint(50,50), 
                                wxSize(450,340) );
  frame->Show(TRUE);
  SetTopWindow(frame);
  return TRUE;
} 

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(ID_Quit, MyFrame::OnQuit)
  EVT_MENU(ID_About, MyFrame::OnAbout)
  EVT_MENU(ID_Load, MyFrame::OnLoad)
END_EVENT_TABLE()


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
  wxMenu *menuFile = new wxMenu;
  
  menuFile->Append( ID_About, wxT("&About...") );
  menuFile->Append( ID_Load, wxT("&Load...") );
  menuFile->AppendSeparator();
  menuFile->Append( ID_Quit, wxT("E&xit") );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  SetMenuBar(menuBar);

  CreateStatusBar(2);
  SetStatusText(wxT("ImageName"), 0);

  wxBoxSizer *box=new wxBoxSizer(wxVERTICAL);
  _ic=new ImageCanvas(this, GetStatusBar(), 1);
  box->Add(_ic, 1, wxALL| wxGROW |wxALIGN_CENTER, 5);
  box->SetSizeHints(this);
  SetSizer(box);

  //const string fn("ucim.pgm");
  const string fn(BIAS_TESTS_DATA "r4.jpg" );
  Image<unsigned char> im;
  int res=ImageIO::Load(fn, im);
  if (res!=0){
    cout<<"could not load "<<fn<<endl;
    wxMessageBox(_T( "could not load image. aborting. ") );
    BIASABORT;
  }
  SetStatusText(_T("image"), 0);
  _ic->Show(im, "image" );
  FitInside();
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{ Close(TRUE); }

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  wxMessageBox(wxT("This is a wxWidgets Hello world sample"),
             wxT("About Hello World"), wxOK | wxICON_INFORMATION, this);
}

void MyFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
{
  wxFileDialog fileDialog(this, wxT("Choose a file"), wxT("."), wxT("ucim.pgm"));
  if (fileDialog.ShowModal() == wxID_OK){
    string fname=WxToAscii(fileDialog.GetPath());
    Image<unsigned char> im;
    if (ImageIO::Load(fname, im)!=0){
      BIASERR("errro loading "<<fname);
    } else {
      SetStatusText(fileDialog.GetFilename(), 0);
      _ic->Show(im, WxToAscii(fileDialog.GetFilename()));
    }
  } else {
    cerr << "canceled loadin image" << endl;
  }
}

