/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FILTER_DIALOG_MEAN_HH_
#define __FILTER_DIALOG_MEAN_HH_


#include <Gui/FilterDialogBase.hh>
#include <Filter/Mean.hh>
#include <wx/spinctrl.h>
#include <wx/slider.h>
#include <wx/stattext.h>


namespace BIAS {

/**
 * @class FilterDialogMean
 * @ingroup g_gui
 * @brief Implementation of a Mean filter dialog
 * @author jfernand 08/09
 */
template <class InputST, class OutputST>
class BIASGui_EXPORT FilterDialogMean : public FilterDialogBase<InputST, OutputST>
{
public:
	FilterDialogMean(const Image<InputST>& origImage,
                   wxWindow* parent,
                   wxWindowID id = BIAS::ID_WIN_MEAN,
                   const wxString& title = wxT("Mean Filter Dialog"),
                   const wxString& name = wxT("FilterDialogMean"));
	virtual ~FilterDialogMean();


protected:

  /**
   * concrete Implementation of the pure OnFilterButton() from the base class.
   */
  void OnFilterButton(wxCommandEvent& WXUNUSED(event));

  void OnResetButton(wxCommandEvent& event);

private:

	wxStaticText* SizeLabel;

	wxSpinCtrl* SizeSpinCtrl;
	wxSlider* SizeSlider;

	/**
	 * The Core: out Mean filter
	 */
	Mean<InputST, OutputST> MeanFilter;

	// paramvalue needed to avoid unnecessary filtering
	unsigned int OldParamVal;

	void OnSpinCtrl(wxSpinEvent& WXUNUSED(event));
	void OnSlider(wxScrollEvent& WXUNUSED(event));
	
	enum FilterDialogMean_IDs {
    ID_SPINCTRL_MEAN = 78921,
    ID_SLIDER_MEAN
	};

};

} // end namespace BIAS

#endif // __FILTER_DIALOG_MEAN_HH_
