/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Gui/FilterDialogMedian.hh>
#include <Base/Image/ImageConvert.hh>

using namespace BIAS;

template <class InputST, class OutputST>
FilterDialogMedian<InputST, OutputST>
::FilterDialogMedian(const Image<InputST>& origImage,
                     wxWindow* parent,
                     wxWindowID id,
                     const wxString& title,
                     const wxString& name)
: FilterDialogBase<InputST, OutputST>(origImage, parent, id, title, name),
  MedianFilter(Median<InputST, OutputST>()),
  NewSize(0),
  SecondSize(-1),
  Color(false),
  OldColor(false),
  MedParamSizer(new wxBoxSizer(wxVERTICAL)),
  NewSizeSizer(new wxBoxSizer(wxHORIZONTAL)),
  SecondSizeSizer(new wxBoxSizer(wxHORIZONTAL)),  
  ColorChoice(NULL)
{
	this->NewSizeSpinCtrl = new wxSpinCtrl(this,
																				 this->ID_SPINCTRL_MEDIAN_NS,
																				 wxEmptyString,
																				 wxDefaultPosition,
																				 wxDefaultSize,
																				 wxSP_ARROW_KEYS,
																				 0,   // min
																				 20,  // max
																				 0);  // initial
  this->SecondSizeSpinCtrl = new wxSpinCtrl(this,
																						this->ID_SPINCTRL_MEDIAN_SS,
																						wxEmptyString,
																						wxDefaultPosition,
																						wxDefaultSize,
																						wxSP_ARROW_KEYS,
																						-1,
																						20,
																						-1);
  this->NewSizeSlider = new wxSlider(this,
																		 this->ID_SLIDER_MEDIAN_NS,
																		 0,  // initial
																		 0,  // min
																		 20, // max
																		 wxDefaultPosition,
																		 wxSize(300, -1),
																		 wxSL_HORIZONTAL | wxSL_AUTOTICKS);
  this->SecondSizeSlider = new wxSlider(this,
																				this->ID_SLIDER_MEDIAN_SS,
																				-1,
																				-1,
																				20,
																				wxDefaultPosition,
																				wxSize(300, -1),
																				wxSL_HORIZONTAL | wxSL_AUTOTICKS),
  this->NewSizeLabel = new wxStaticText(this,
																				wxID_STATIC,
																				wxT("Newsize:"),
																				wxDefaultPosition,
																				wxDefaultSize,
																				0);
  this->SecondSizeLabel = new wxStaticText(this,
																					 wxID_STATIC,
																					 wxT("Secondsize:"),
																					 wxDefaultPosition,
																					 wxDefaultSize,
																					 0);

  // wxArrayString for ColorChoice
  wxArrayString str;
  str.Add(wxT("Grey Image Output"));
  str.Add(wxT("Colored Image Output"));

  // now ColorChoice takes concrete form
  this->ColorChoice = new wxChoice(this,
                                   this->ID_CHOICE_COLOR,
                                   wxDefaultPosition,
                                   wxDefaultSize,
                                   str);

  // add elements for the NewSize param input to sizer
  this->NewSizeSizer->Add(this->NewSizeLabel,
                          0,
                          wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL,
                          5);
  this->NewSizeSizer->AddSpacer(19);
  this->NewSizeSizer->Add(this->NewSizeSpinCtrl,
                          0,
                          wxALIGN_CENTER_VERTICAL
                            |wxALIGN_CENTER_HORIZONTAL
                            | wxALL,
                          5);
  this->NewSizeSizer->Add(this->NewSizeSlider,
                          0,
                          wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL,
                          5);

  // add elements for the SecondSize param input to sizer
  this->SecondSizeSizer->Add(this->SecondSizeLabel,
                             0,
                             wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL,
                             5);
  this->SecondSizeSizer->Add(this->SecondSizeSpinCtrl,
                             0,
                             wxALIGN_CENTER_VERTICAL
                               | wxALIGN_CENTER_HORIZONTAL
                               | wxALL,
                             5);
  this->SecondSizeSizer->Add(this->SecondSizeSlider,
                             0,
                             wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL,
                             5);

  // add above sizer and ColorChoice to this sizer
  this->MedParamSizer->Add(this->NewSizeSizer,
                           0,
                           wxALIGN_CENTER_HORIZONTAL | wxGROW | wxALL,
                           0);
  this->MedParamSizer->Add(this->SecondSizeSizer,
                           0,
                           wxALIGN_CENTER_HORIZONTAL | wxGROW | wxALL,
                           0);
  this->MedParamSizer->Add(this->ColorChoice,
                           0,
                           wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL,
                           5);

  // now MedParamSizer will be add to base class' ParamSizer
  wxBoxSizer* paramsizer = this->GetParamSizer();
  paramsizer->Add(this->MedParamSizer,
                  0,
                  wxALIGN_CENTER_VERTICAL | wxGROW | wxALL,
                  0);

  // connect events with handlers
  this->Connect(BIAS::ID_BTN_FILTER_FDB,
                wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(FilterDialogMedian::OnFilterButton));

  this->Connect(this->ID_SPINCTRL_MEDIAN_NS,
                wxEVT_COMMAND_SPINCTRL_UPDATED,
                wxSpinEventHandler(FilterDialogMedian::OnSpinCtrlNewSize));

  this->Connect(this->ID_SPINCTRL_MEDIAN_SS,
                wxEVT_COMMAND_SPINCTRL_UPDATED,
                wxSpinEventHandler(FilterDialogMedian::OnSpinCtrlSecondSize));

  this->Connect(this->ID_SLIDER_MEDIAN_NS,
                wxEVT_SCROLL_CHANGED,
                wxScrollEventHandler(FilterDialogMedian::OnSliderNewSize));

  this->Connect(this->ID_SLIDER_MEDIAN_SS,
                wxEVT_SCROLL_CHANGED,
                wxScrollEventHandler(FilterDialogMedian::OnSliderSecondSize));

  this->Connect(this->ID_CHOICE_COLOR,
                wxEVT_COMMAND_CHOICE_SELECTED,
                wxCommandEventHandler(FilterDialogMedian::OnChoice));

  this->GetParent()->Disable();
  this->CreateStatusBar(1);
  this->Fit();
  this->Centre();
}


template <class InputST, class OutputST>
FilterDialogMedian<InputST, OutputST>::~FilterDialogMedian()
{
  if (!this->GetParent()->IsEnabled()) {
      this->GetParent()->Enable();
  }
}


/******************************************************************************/
/* ============================== protected ================================= */
/******************************************************************************/

template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnFilterButton(wxCommandEvent& WXUNUSED(event))
{
  // disable some buttons to avoid interference, e.g. pressing preview during
  // filtering...
  this->EnableSaveButton(false);
  this->EnableFilterButton(false);
  this->EnablePreviewButton(false);
  this->EnableResetButton(false);
  this->EnableCancelButton(false);

  int nsize = this->NewSizeSpinCtrl->GetValue();
  int ssize = this->SecondSizeSpinCtrl->GetValue();

  bool dofilter = ((nsize == this->NewSize)
                   && (ssize == this->SecondSize)
                   && (this->Color == this->OldColor)
                   && this->GetImageIsFiltered());

  if (dofilter) {
    wxString msg(wxT("Nothing Done To The Filtered Image"));
    msg += wxT(" (Already Filtered With The Same Values?)");
    this->SetStatusText(msg);
  } else {

    this->MedianFilter.SetSize(nsize, ssize);
    Image<OutputST> dstImg;
    int notOk;

    if (Color) {

      notOk = this->MedianFilter.FilterColorImg(this->GetOriginalImage(),
                                                dstImg);

      if (notOk) {
        BIASERR("FilterDialogMedian::OnFilterButton: Filter Error");
        this->SetImageIsFiltered(false);
        this->SetStatusText(wxT("ERROR during FILTER"));
      } else {
        this->SetFilteredImage(dstImg);
        this->SetImageIsFiltered(true);
        this->NewSize = nsize;
        this->SecondSize = ssize;
        this->SetStatusText(wxT("Image SuccessFully Filtered"));
      }

    } else { // user wants grey image output

      Image<InputST> src = this->GetOriginalImage();
      Image<InputST> srcImg;
      bool planar = src.IsPlanar();
      notOk = ImageConvert::Convert(src, srcImg, ImageBase::CM_Grey, planar);

      if (notOk) {
        BIASERR("FilterDialogMedian::OnFilterButton: Error Converting To Grey");
        this->SetStatusText(wxT("ERROR during CONVERT"));
      } else {
        notOk = this->MedianFilter.Filter(srcImg, dstImg);
        if (notOk) {
          BIASERR("FilterDialogMedian::OnFilterButton: Filter Error");
          this->SetImageIsFiltered(false);
          this->SetStatusText(wxT("ERROR during FILTER"));
        } else {
          this->SetFilteredImage(dstImg);
          this->SetImageIsFiltered(true);
          this->NewSize = nsize;
          this->SecondSize = ssize;
          this->SetStatusText(wxT("Image SuccessFully Filtered"));

        }
      } // end if else convert

    } // end if else color

  }

  // enable the buttons again
  this->EnableSaveButton();
  this->EnableFilterButton();
  this->EnablePreviewButton();
  this->EnableResetButton();
  this->EnableCancelButton();
}


template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnResetButton(wxCommandEvent& event)
{
  // call the Base's class OnResetButton() method and pass the event
  this->FilterDialogBase<InputST, OutputST>::OnResetButton(event);
  this->SetImageIsFiltered(false);
  this->SetStatusText(wxT("Filtered Image Reseted"));
}


/******************************************************************************/
/* =============================== private ================================== */
/******************************************************************************/

template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnSpinCtrlNewSize(wxSpinEvent& WXUNUSED(event))
{
  this->NewSizeSlider->SetValue(this->NewSizeSpinCtrl->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnSpinCtrlSecondSize(wxSpinEvent& WXUNUSED(event))
{
  this->SecondSizeSlider->SetValue(this->SecondSizeSpinCtrl->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnSliderNewSize(wxScrollEvent& WXUNUSED(event))
{
  this->NewSizeSpinCtrl->SetValue(this->NewSizeSlider->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnSliderSecondSize(wxScrollEvent& WXUNUSED(event))
{
  this->SecondSizeSpinCtrl->SetValue(this->SecondSizeSlider->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogMedian<InputST, OutputST>
::OnChoice(wxCommandEvent& WXUNUSED(event))
{
  int choice = this->ColorChoice->GetCurrentSelection();
  this->OldColor = this->Color;
  choice == 1 ? this->Color = true : this->Color = false;
}


/* =================== supported storagetype combinations =================== */
template class FilterDialogMedian<unsigned char, unsigned char>;
template class FilterDialogMedian<unsigned char, float>;
//template class FilterDialogMedian<float, unsigned char>;
template class FilterDialogMedian<float, float>;

