/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Gui/FilterDialogPreviewWindow.hh>
#include <Gui/ImageCanvas.hh>
#include <Gui/ScaledImageCanvas.hh>

#include <Base/Common/BIASpragmaStart.hh>
#include <wx/gdicmn.h>
#include <Base/Common/BIASpragmaEnd.hh>

using namespace BIAS;

template <class StorageType>
FilterDialogPreviewWindow<StorageType>
  ::FilterDialogPreviewWindow(const ImageBase& img,
                              wxWindow* parent,
                              const wxString& title,
                              wxWindowID id,
                              const wxString& name,
                              const wxPoint& pos,
                              const wxSize& size,
                              long style)
: wxFrame(parent, id, title, pos, size, style, name),
  MainSizer(new wxBoxSizer(wxVERTICAL)),
  OrigImg(img)
{
  this->SetSizer(this->MainSizer);
  this->CreateStatusBar(2);

  ImageBase::EStorageType st = this->OrigImg.GetStorageType();
  if (ImageBase::ST_unsignedchar == st) {

    ImageCanvas* canvas = new ImageCanvas(this, this->GetStatusBar(), 1);
    this->MainSizer->Add(canvas, 1, wxALL| wxGROW |wxALIGN_CENTER, 5);

    Image<unsigned char> previewImg
      = static_cast<Image<unsigned char> > (this->OrigImg);

    canvas->Show(previewImg, "test");

  } else if (ImageBase::ST_float == st) {

    ScaledImageCanvas* canvas = new ScaledImageCanvas(this,
                                                      this->GetStatusBar(),
                                                      1);
    this->MainSizer->Add(canvas, 1, wxALL| wxGROW |wxALIGN_CENTER, 5);

    Image<float> previewImg = static_cast<Image<float> > (this->OrigImg);

    canvas->Show(previewImg, "test");

  }

  this->GetParent()->Disable();
  this->AdjustWindowDimension();
  this->Centre();

}


template <class StorageType>
FilterDialogPreviewWindow<StorageType>::~FilterDialogPreviewWindow()
{
    this->GetParent()->Enable();
}


template <class StorageType>
void FilterDialogPreviewWindow<StorageType>::AdjustWindowDimension()
{
  // dimension for the window
  wxRect rec = wxGetClientDisplayRect();
  unsigned int width = this->OrigImg.GetWidth() + 27;
  unsigned int height = this->OrigImg.GetHeight() + 35;
  const unsigned int maxW = rec.width;
  const unsigned int maxH = rec.height;

  // adjust dimensions if needed...with magic numbers
  if (width > maxW) {
    width = maxW;
  }

  if (typeid(StorageType) == typeid(float)) {
    height += 65;
  }

  if (height > maxH) {
    height = maxH;
  }

  this->SetSize(width, height);
}


template class FilterDialogPreviewWindow<unsigned char>;
template class FilterDialogPreviewWindow<float>;
