/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "bias_config.h"

#ifndef __GuiCV_hh__
#define __GuiCV_hh__

#ifndef BIAS_HAVE_OPENCV 	 
#  error BIAS_HAVE_OPENCV not defined. Please recompile BIAS with USE_OPENCV enabled if you want to use GuiCV.
#endif

#include "GuiBase.hh"
#include <string>


// forwrad declaration for cxtypes.h to avoid cv.h inclusion:
typedef struct _IplImage IplImage;

namespace BIAS{
  /** @class GuiCV
      @ingroup g_gui
      @brief Gui based on OpenCV is a simple windowing environment...
      
      Usage:
      - get an object

      - call ShowImage() to visualize an image

      - use CheckForKeyEvent() to have your programm react on pressed keys
  */
  class BIASGui_EXPORT GuiCV : public GuiBase
  {
  public:
    GuiCV(bool ShowMousePos = true);
    GuiCV(const std::string& Title, bool ShowMousePos = true);
    GuiCV(const GuiCV& gui);
    virtual ~GuiCV();

    // GuiBase functions
    virtual void SetTitle(std::string const& Title);
    virtual int DestroyWindow();
    virtual char CheckForKeyEvent();
    virtual char WaitForKeyEvent();
    virtual void WaitForMouseButtonEvent(bool& Pressed, int& Button, 
					 int& Xpos, int& Ypos);
 
   protected:
    IplImage* IplImage_;
    Image<unsigned char> CurrentImage_;

    // GuiBase functions
    virtual int ShowConvertedImage_(Image<unsigned char> &image);

    void MouseCallback(int event, int x, int y, int flags);
    static void StaticMouseCallback(int event, int x, int y,
                                    int flags, void* param);

    int CreateIplImageShared(const BIAS::ImageBase* p_src, IplImage* &p_dest);
  };

} // namespace BIAS

#endif // __GuiCV_hh__
