#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE
// fixed in wx 2.7.2
#  pragma warning(disable: 4005)
#endif

#include "VideoSource_Controller.hh"

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>
#endif

#if defined(WIN32) && defined(HAVE_TRIGGER_CLASSES)
#  include "TriggerIOControl.h"
#endif

#ifdef BIAS_HAVE_DCAM
#  include <VideoSource/VideoSource_DCAM.hh>
#endif


#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller, wxFrame)
  EVT_KEY_UP(VideoSource_Controller::OnKeyUp)
  EVT_CLOSE(VideoSource_Controller::OnClose)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller::OnHide)
  EVT_COMMAND_SCROLL(ID_GAIN,      VideoSource_Controller::OnGain)
  EVT_COMMAND_SCROLL(ID_SHUTTER,   VideoSource_Controller::OnShutter)
  EVT_COMMAND_SCROLL(ID_WBMANUAL,   VideoSource_Controller::OnWhiteBalanceManual)
  EVT_COMMAND_SCROLL(ID_BRIGHTNESS,      VideoSource_Controller::OnBrightness)
  EVT_COMMAND_SCROLL(ID_CONTRAST,   VideoSource_Controller::OnContrast)
  EVT_BUTTON(ID_BUTTON_AUTO_SHUTTER, VideoSource_Controller::OnAuto_Shutter)
  EVT_BUTTON(ID_BUTTON_AUTO_BRIGHTNESS, VideoSource_Controller::OnAuto_Brightness)
  EVT_BUTTON(ID_BUTTON_WHITEBALANCE, VideoSource_Controller::OnWhiteBalance)
  EVT_BUTTON(ID_BUTTON_AUTO_GAIN, VideoSource_Controller::OnAuto_Gain)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller::OnUpdate)
END_EVENT_TABLE()




VideoSource_Controller::
VideoSource_Controller(BIAS::VideoSource *cam, wxWindow *parent,
                       const string &title,
                       double minShutter, double maxShutter,
                       double minGain,double maxGain,
                       double minContrast,double maxContrast,
                       double minBrightness,double maxBrightness)
: wxFrame(parent, wxID_ANY, (AsciiToWx(title.c_str())),
		wxDefaultPosition, wxSize(400,500), wxSTAY_ON_TOP|wxCAPTION | wxRESIZE_BORDER)
		{
	Cam_ = cam;
	if(minShutter != -1 && maxShutter != -1){
		minShutter_=minShutter;
		maxShutter_=maxShutter;
	}
	else 
		Cam_->GetShutterRange(minShutter_, maxShutter_);

	if(minGain != -1 && maxGain != -1){
		minGain_= minGain;
		maxGain_= maxGain;
	}
	else 
		Cam_->GetGainRange(minGain_, maxGain_);

	if(minContrast != -1 && maxContrast != -1){
		minContrast_= minContrast;
		maxContrast_= maxContrast;
	}
	else 
		Cam_->GetContrastRange(minContrast_, maxContrast_);
	
	if(minBrightness != -1 && maxBrightness != -1){
		minBrightness_= minBrightness;
		maxBrightness_= maxBrightness;
	}
	else 
		Cam_->GetBrightnessRange(minBrightness_, maxBrightness_);

	Initialize_();
}



VideoSource_Controller::
~VideoSource_Controller()
{}

void VideoSource_Controller::
Initialize_()
{      
  SliderGain = NULL;
  SliderShutter= NULL;
  SliderWbRed= NULL;
  SliderWbBlue= NULL;
  SliderBrightness= NULL;
  SliderContrast= NULL;
  showButton = NULL;
  whiteBalanceButton = NULL;
  auto_sh_Button = NULL;
  auto_ga_Button = NULL;
  auto_br_Button = NULL;
  auto_co_Button = NULL;
  update_Button = NULL;
  sliderBox_ShGa = NULL;

  LetterAscii_ = WXK_F4;
  Ctrl_ = false;
  Shift_ = false;
  Alt_ = true;
  //Shut the Automode DOWN
  auto_gain = false;
  auto_shutter = false;
  auto_brightness = false;
  sliderBox_ShGa_Visible = true;
  vBoxVisible = true;
  
  SetShutterRange((unsigned)minShutter_,(unsigned)maxShutter_);
  SetGainRange((unsigned)minGain_,(unsigned)maxGain_);


  wxSize SlSize(70,200);
  wxSize BtSize(55,30);
  wxBoxSizer *sl = new wxBoxSizer(wxHORIZONTAL);
 
   if (Cam_->HasControlWhiteBalance()){
  ////////////// White Balance 
     whiteBalanceButton = new wxButton(this, ID_BUTTON_WHITEBALANCE, 
       wxT("WB"),wxDefaultPosition,BtSize);
     sl->Add(whiteBalanceButton,0,wxALL, 2);

     wxBoxSizer *slWbr = new wxBoxSizer(wxVERTICAL);
     wxStaticText *textWbr = new wxStaticText(this,ID_TEXT, wxT("red gain"));
     SliderWbRed = new wxSlider(this, ID_WBMANUAL, 50, 0, 100,
       wxDefaultPosition,SlSize,
       wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT| wxSL_AUTOTICKS |wxSL_SELRANGE,
       wxDefaultValidator, wxT("RedGain"));

     SliderWbRed->Enable(true);
     slWbr->Add(textWbr,0,wxALL,2);
     slWbr->Add(SliderWbRed,0,wxALL, 2);

     wxBoxSizer *slWbb = new wxBoxSizer(wxVERTICAL);
     wxStaticText *textWbb = new wxStaticText(this,ID_TEXT, wxT("blue gain"));
     SliderWbBlue = new wxSlider(this, ID_WBMANUAL, 50, 0, 100,
       wxDefaultPosition,SlSize,
       wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT| wxSL_AUTOTICKS |wxSL_SELRANGE,
       wxDefaultValidator, wxT("BlueGain"));

     SliderWbBlue->Enable(true);
     slWbb->Add(textWbb,0,wxALL,2);
     slWbb->Add(SliderWbBlue,0,wxALL, 2);


     sl->Add(slWbr,0,wxALL, 2);
     sl->Add(slWbb,0,wxALL, 2);
   }//end Cam_->HasControlWhiteBalance()
  /////////////////////////// Brightness //////////////////////
  if (Cam_->HasControlBrightness()){
    wxBoxSizer *slBr = new wxBoxSizer(wxVERTICAL);
    wxStaticText *textBr = new wxStaticText(this,ID_TEXT, wxT("Brightness"));
    SliderBrightness = new wxSlider(this, ID_BRIGHTNESS, 157, 0, 255,
				    wxDefaultPosition,SlSize,
				    wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT| wxSL_AUTOTICKS |wxSL_SELRANGE,
				    wxDefaultValidator, wxT("Brightness"));

    auto_br_Button = new wxButton(this, ID_BUTTON_AUTO_BRIGHTNESS, 
				  wxT("MAN"),wxDefaultPosition,BtSize);

    SliderBrightness->Enable(true);
    slBr->Add(textBr,0,wxALL,2);  
    slBr->Add(SliderBrightness,0, wxALL,2);
    slBr->Add(auto_br_Button,0,wxALL, 2);
    sl->Add(slBr,0,wxALL, 2);
  }
  ///////////////////////////// Gain /////////////////////
  if (Cam_->HasControlGain()){   
    wxBoxSizer *slGa = new wxBoxSizer(wxVERTICAL);
    wxStaticText *textGa = new wxStaticText(this,ID_TEXT, wxT("Gain [dB]"));
    SliderGain = new wxSlider(this, ID_GAIN, (maxGain_-minGain_)/2, minGain_, maxGain_,
			      wxDefaultPosition,SlSize,
			      wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT| wxSL_AUTOTICKS |wxSL_SELRANGE,
			      wxDefaultValidator, wxT("Gain"));

    auto_ga_Button = new wxButton(this, ID_BUTTON_AUTO_GAIN, 
				  wxT("MAN"),wxDefaultPosition,BtSize);

    SliderGain->Enable(true);
    slGa->Add(textGa,0,wxALL,2);  
    slGa->Add(SliderGain,0, wxALL,2);
    slGa->Add(auto_ga_Button,0,wxALL, 2);
    sl->Add(slGa,0,wxALL, 2);
  }
  ///////////////////////////// Contrast /////////////////////
  if (Cam_->HasControlContrast()){
    wxBoxSizer *slCo = new wxBoxSizer(wxVERTICAL);
    wxStaticText *textContrast = new wxStaticText(this,ID_TEXT, wxT("Contrast"));
    SliderContrast = new wxSlider(this, ID_CONTRAST, (maxContrast_ - minContrast_)/2, minContrast_, maxContrast_,
				  wxDefaultPosition,SlSize,
				  wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT| wxSL_AUTOTICKS |wxSL_SELRANGE,
				  wxDefaultValidator, wxT("Gain"));

    auto_co_Button = new wxButton(this, ID_BUTTON_AUTO_CONTRAST, 
				  wxT("MAN"),wxDefaultPosition,BtSize);

    SliderContrast->Enable(true);
    slCo->Add(textContrast,0,wxALL,2);  
    slCo->Add(SliderContrast,0, wxALL,2);
    slCo->Add(auto_co_Button,0,wxALL, 2);
    sl->Add(slCo,0,wxALL, 2);
  }
  /////////////////////////////// Shutter ///////////////////
  if (Cam_->HasControlShutter()){
    wxBoxSizer *slSh = new wxBoxSizer(wxVERTICAL);
    wxStaticText *textShutter = new wxStaticText(this,ID_TEXT, wxT("Shutter [ms]"));
    SliderShutter = new wxSlider(this, ID_SHUTTER, (maxShutter_-minShutter_)/2, minShutter_, maxShutter_,
				 wxDefaultPosition,SlSize,
				 wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT | wxSL_AUTOTICKS |wxSL_SELRANGE,
				 wxDefaultValidator, wxT("Shutter"));
  
    auto_sh_Button = new wxButton(this, ID_BUTTON_AUTO_SHUTTER, 
				  wxT("MAN"),wxDefaultPosition,BtSize);
    SliderShutter->Enable(true);
    slSh->Add(textShutter,0,wxALL,2);  
    slSh->Add(SliderShutter,0,wxALL, 2);
    slSh->Add(auto_sh_Button,0,wxALL, 2);
    sl->Add(slSh,0,wxALL,2);  
  }
  //////////////////////////////////////////////////

  
  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE,
			       wxT("UPDATE"),wxDefaultPosition,BtSize);

  vBox = new wxBoxSizer(wxHORIZONTAL);
  vBox->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  vBox->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM);


  sliderBox_ShGa = new wxBoxSizer(wxVERTICAL);
  sliderBox_ShGa->Add(sl,0,wxALL, 2);
  sliderBox_ShGa->Add(vBox,0,wxALL,2);

  vBox->SetSizeHints(this);
  this->SetSizer(sliderBox_ShGa);
  SetAutoLayout(true);
  Layout();
  Fit();
  wxCommandEvent fakeEvent;
  OnUpdate(fakeEvent);
  Show(true);
}

bool VideoSource_Controller::
Show(bool show)
{
  bool res = wxWindow::Show(show);
  if (show) {
    /// wait a moment to let wx sort its key and focus events
    wxMilliSleep(200);
    //SliderGain->SetFocus();
    Raise();
  }
  return res;

}

void VideoSource_Controller::
SetCloseShortcut(const int &asciiCode, 
		 bool ctrl, bool shift, bool alt)
{
  LetterAscii_ = asciiCode;
  Ctrl_ = ctrl;
  Shift_ = shift;
  Alt_ = alt;
}

bool VideoSource_Controller::
AreModifiersCorrect_(const wxKeyEvent &event)
{
  return ( (event.ShiftDown() == Shift_)
	   && (event.ControlDown() == Ctrl_)
	   && (event.AltDown() == Alt_) );
}

void VideoSource_Controller::
SetVideoSource(BIAS::VideoSource *cam)
{
  Cam_ = cam;
}

void VideoSource_Controller::
OnKeyUp(wxKeyEvent& event)
{
  int keyCode = event.GetKeyCode();

  /// hide window on shortcut 
  if ( AreModifiersCorrect_(event) ) {
    if ( keyCode == LetterAscii_ ) {
      Close();
    }
  }
  else {
    event.Skip();
  }
}

void VideoSource_Controller::
OnHide(wxCommandEvent& event)
{
  Close();
}

void VideoSource_Controller::
OnClose(wxCloseEvent& event) 
{
  Show(false);
  wxWindow* parent = GetParent();
  if ( parent != NULL ) parent->Refresh();
}

void VideoSource_Controller::
OnGain(wxScrollEvent& event) 
{
  double value = SliderGain->GetValue();
  if(Cam_ != NULL){
    auto_off(1);
    //cout<<"Set Gain:"<<value<<endl;
		Cam_->SetGain(value);
    //cout<<"Get Gain:"<<Cam_->GetGain()<<endl;
  }
}

void VideoSource_Controller::
OnShutter(wxScrollEvent& event) 
{ 
  if(Cam_ != NULL){
	  auto_off(0);
    double value = SliderShutter->GetValue();
   // cout<<"Set Shutter:"<<value/10000.0<<endl;
    Cam_->SetShutter(value/10000.0);
    //cout<<"Get Shutter:"<<Cam_->GetShutter()*01000.0<<endl;
  }
}

void VideoSource_Controller::
OnWhiteBalanceManual(wxScrollEvent& event) 
{
  double vred = SliderWbRed->GetValue() / 100.0 - 0.5;
  double vblue = SliderWbBlue->GetValue() / 100.0 - 0.5;
  if(Cam_ != NULL){
    Cam_->SetWhiteBalance(vred,vblue);
  }
}


void VideoSource_Controller::
OnWhiteBalance(wxCommandEvent& event) 
{
  if(Cam_!=NULL)
    Cam_->SoftwareWhiteBalance();
}

void VideoSource_Controller::
OnAuto_Shutter(wxCommandEvent& event) 
{
  if(!auto_shutter) {
    auto_on(0);
  }
  else {
    auto_off(0);
  }
}

void VideoSource_Controller::
OnAuto_Gain(wxCommandEvent& event) 
{
  if(!auto_gain) {
    auto_on(1);
  }
  else {
    auto_off(1);
  }
}

void VideoSource_Controller::
OnAuto_Brightness(wxCommandEvent& event) 
{
  if(!auto_brightness) {
    auto_on(2);
  }
  else {
    auto_off(2);
  }
}


void VideoSource_Controller::
OnBrightness(wxScrollEvent& event)
{
  if(Cam_ != NULL){
    auto_off(2);
    double value = SliderBrightness->GetValue();
    //cout<<"Brightness:"<<value<<endl;
    Cam_->SetBrightness(value); 
  }
}

void VideoSource_Controller::
OnContrast(wxScrollEvent& event)
{
  if(Cam_ != NULL){
    //	  auto_off(0);
    cout<<"SetContrast:"<<SliderContrast->GetValue()<<endl;    
    Cam_->SetContrast(SliderContrast->GetValue()); 
    cout<<"GetContrast:"<<Cam_->GetContrast()<<endl;
  }
}

void VideoSource_Controller::
Update() 
{
  if (Cam_!= NULL) {
    float current =0.0;
    if (Cam_->HasControlShutter()){
      current= Cam_->GetShutter()*10000.0;  
      if(current >=0)    
	      SliderShutter->SetValue(int(current));
      bool autoon = Cam_->GetAutoShutter();
      if (autoon)  auto_sh_Button->SetLabel(wxT("AUTO"));
      else  auto_sh_Button->SetLabel(wxT("MAN"));
    }
    if (Cam_->HasControlGain()){
      current = Cam_->GetGain();
      if (current >=0) 
	      SliderGain->SetValue(int(current));
      bool autoon = Cam_->GetAutoGain();
      if (autoon) auto_ga_Button->SetLabel(wxT("AUTO"));
      else  auto_ga_Button->SetLabel(wxT("MAN"));
    }
    if (Cam_->HasControlContrast()){
      current = Cam_->GetContrast();
      if (current >=0) 
	      SliderContrast->SetValue(int(current));
    }
    if (Cam_->HasControlBrightness()){
      current = Cam_->GetBrightness();
      if (current >=0) 
	      SliderBrightness->SetValue(int(current));
      bool autoon = Cam_->GetAutoBrightness();
      if (autoon) auto_br_Button->SetLabel(wxT("AUTO"));
      else  auto_br_Button->SetLabel(wxT("MAN"));
    }
  }
}

void VideoSource_Controller::
OnUpdate(wxCommandEvent& event) 
{
  Update();
}

void VideoSource_Controller::
auto_on(int param) 
{
  //SHUTTER
  if(param == 0){
    auto_shutter = true;
    Cam_->SetAutoShutter(true);
    auto_sh_Button->SetLabel(wxT("AUTO"));
  }
  //GAIN
  else if(param == 1){
    auto_gain = true;
    Cam_->SetAutoGain(true);
    auto_ga_Button->SetLabel(wxT("AUTO"));
  }
  //Brightness
  else if(param == 2){
    auto_brightness = true;
    Cam_->SetAutoBrightness(true);
    auto_br_Button->SetLabel(wxT("AUTO"));
  }
}

void VideoSource_Controller::
auto_off(int param) 
{
  //SHUTTER
  if(param == 0){
    auto_shutter = false;
    Cam_->SetAutoShutter(false);
    auto_sh_Button->SetLabel(wxT("MAN"));
  }
  //GAIN
  else if(param == 1){
    auto_gain = false;
    Cam_->SetAutoGain(false);
    auto_ga_Button->SetLabel(wxT("MAN"));
  }
  //Brightness
  else if(param == 2){
    auto_brightness = false;
    Cam_->SetAutoBrightness(false);
    auto_br_Button->SetLabel(wxT("MAN"));
  }
}

void  VideoSource_Controller::
SetShutterRange(unsigned int min,unsigned int max)
{
  if (Cam_->HasControlShutter() && SliderShutter != NULL)
    SliderShutter->SetRange(min,max);
}

void  VideoSource_Controller::
SetGainRange(unsigned int min,unsigned int max)
{
  if (Cam_->HasControlGain() && SliderGain != NULL)
    SliderGain->SetRange(min,max);
}

void VideoSource_Controller::
SetBrightnessRange(unsigned int min,unsigned int max)
{
  if (Cam_->HasControlBrightness()&& SliderShutter != NULL)
  	SliderBrightness->SetRange(min,max);
}
 
void VideoSource_Controller::
SetContrastRange(unsigned int min,unsigned int max)
{
  if (Cam_->HasControlContrast() && SliderContrast!= NULL)
      SliderContrast->SetRange(min,max);
}

