#ifndef __VideoSource_Controller_HH__
#define __VideoSource_Controller_HH__

#include <bias_config.h>

#include <Base/Common/W32Compat.hh>
#include <VideoSource/VideoSource_Base.hh>

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>

#ifndef BIAS_HAVE_WXWIDGETS
#  error You need BIAS_HAVE_WXWIDGETS. Recompile BIAS with USE_WXWIDGETS.
#endif

#include <wx/wx.h>


namespace BIAS {

  class BIASGui_EXPORT VideoSource_Controller : public wxFrame
  {

  public:

    VideoSource_Controller(BIAS::VideoSource *cam, wxWindow *parent, 
			   const std::string &title,
			   double minShutter=-1, double maxShutter=-1,
			   double minGain=-1,double maxGain=-1,
			   double minContrast=-1,double maxContrast=-1,
         double minBrightness=-1,double maxBrightness=-1);
  
    ~VideoSource_Controller();

    bool Show(bool show=true);

    

    void SetCloseShortcut(const int &asciiCode, 
			  bool ctrl, bool shift, bool alt);

    void SetVideoSource(BIAS::VideoSource *cam);
    void SetShutterRange(unsigned int min,unsigned int max);
    void SetGainRange(unsigned int min,unsigned int max);

    void SetBrightnessRange(unsigned int min,unsigned int max);
    void SetContrastRange(unsigned int min,unsigned int max);
    void Update(); // trigger an update, useful for auto-mode


    wxBoxSizer *vBox;

    enum {
      ID_HIDE,
      //All 16 FEATURES of DCAM
      //GAIN, SHUTTER PANEL
      ID_GAIN,
      ID_SHUTTER,
      ID_WBMANUAL,
      ID_BUTTON_AUTO_GAIN,
      ID_BUTTON_AUTO_SHUTTER,
      ID_BUTTON_UPDATE,
      ID_BUTTON_WHITEBALANCE,
      //HUE, SATURATION
      ID_HUE,
      ID_SATURATION,

      //GAMMA, BRIGHTNESS
      ID_GAMMA,
      ID_BRIGHTNESS,
      ID_BUTTON_AUTO_BRIGHTNESS,
      ID_BUTTON_AUTO_CONTRAST,
      ID_CONTRAST,
      ID_EXPOSURE,
      ID_SHARPNESS,
      ID_WHITE_BALANCE,
      ID_IRIS,
      ID_FOCUS,
      ID_TEMPERATURE,
      ID_TRIGGER,
      ID_TRIGGER_DELAY,
      ID_WHITE_SHADING,
      ID_FRAME_RATE,

      ID_TEXT
    };

  private:
    void Initialize_();
    void OnEscape(wxCommandEvent& event);
    void OnKeyUp(wxKeyEvent& event);

    void OnHide(wxCommandEvent& event);
    void OnClose(wxCloseEvent& event);
    void OnWhiteBalance(wxCommandEvent& event);
    void OnShutter(wxScrollEvent& event);
    void OnGain(wxScrollEvent& event);
    void OnWhiteBalanceManual(wxScrollEvent& event);
    void OnBrightness(wxScrollEvent& event);
    void OnContrast(wxScrollEvent& event);
    void OnAuto_Shutter(wxCommandEvent& event);
    void OnAuto_Brightness(wxCommandEvent& event);
    void OnAuto_Gain(wxCommandEvent& event);
    void OnUpdate(wxCommandEvent& event) ;

    void auto_on(int param);
    void auto_off(int param);

    bool AreModifiersCorrect_(const wxKeyEvent &event);

  protected:

    BIAS::VideoSource *Cam_;

    wxSlider          *SliderGain;
    wxSlider          *SliderShutter;
    wxSlider          *SliderWbRed;
    wxSlider          *SliderWbBlue;
    wxSlider          *SliderBrightness;
    wxSlider          *SliderContrast;
    wxBoxSizer        *sliderBox_ShGa;

    bool               sliderBox_ShGa_Visible;
    bool               auto_shutter;
    bool               auto_gain;
    bool               auto_brightness;

    bool               vBoxVisible;

    wxButton          *showButton;
    wxButton          *whiteBalanceButton;
    wxButton          *auto_sh_Button;
    wxButton          *auto_ga_Button;
    wxButton          *auto_br_Button;
    wxButton          *auto_co_Button;
    wxButton          *update_Button;

    double minShutter_, maxShutter_, minGain_,maxGain_;
    double minBrightness_, maxBrightness_, minContrast_, maxContrast_;

    bool Ctrl_, Shift_, Alt_;
    int LetterAscii_;

    DECLARE_EVENT_TABLE()
    
  }; // end class

} // end namespace BIAS

#endif // __VideoSource_Controller_HH__
