#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE 
// fixed in wx 2.7.2 
#  pragma warning(disable: 4005)
#endif

#include "VideoSource_Controller_SwissRanger.hh"

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>  
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller_SwissRanger, wxFrame)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller_SwissRanger::OnHide)
  EVT_COMMAND_SCROLL(ID_AMP,      VideoSource_Controller_SwissRanger::OnAmplitude)
  EVT_COMMAND_SCROLL(ID_SAT,      VideoSource_Controller_SwissRanger::OnSaturation)
  EVT_COMBOBOX(ID_MOD,            VideoSource_Controller_SwissRanger::OnModFreq)
  EVT_COMMAND_SCROLL(ID_DIST,     VideoSource_Controller_SwissRanger::OnDistance)
  EVT_COMMAND_SCROLL(ID_INT,      VideoSource_Controller_SwissRanger::OnIntegration)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller_SwissRanger::OnUpdate)
  EVT_CHECKBOX( ID_CHECK_ENSURELATEST, VideoSource_Controller_SwissRanger::OnCheckEnsureLatest )
  EVT_CHECKBOX( ID_CHECK_SHOWDEPTH, VideoSource_Controller_SwissRanger::OnCheckShowDepth )
END_EVENT_TABLE()




VideoSource_Controller_SwissRanger::
VideoSource_Controller_SwissRanger(BIAS::VideoSource_SwissRanger *cam, wxFrame *parent)
  : wxFrame(parent, -1, _T("VideoSourceController Swissranger")
  ,wxDefaultPosition,wxSize(500,500),wxSTAY_ON_TOP |wxCAPTION)
{
  Cam_ = cam;
  Initialize();

  min_amplitudeThreshold_=0; max_amplitudeThreshold_=2000;
  min_saturationThreshold_=0; max_saturationThreshold_=2000;
  min_integrationTime_=0; max_integrationTime_=0xFF;
  min_distanceOffset_=0; max_distanceOffset_=0xFF;
  
  //this is the enum used in /usr/inlcude/libMesaSR.h
  /* Set the Modulation frequency (from defines.h)
       MF_40MHz=0, //!<SR3k: maximal range 3.75m
       MF_30MHz,   //!<SR3k, SR4k: maximal range 5m
       MF_21MHz,   //!<SR3k: maximal range 7.14m
       MF_20MHz,   //!<SR3k: maximal range 7.5m
       MF_19MHz,   //!<SR3k: maximal range 7.89m
       MF_60MHz,   //!<SR4k: maximal range 2.5m \intDoc{frequency}
       MF_15MHz,   //!<SR4k: maximal range 10m \intDoc{frequency}
       MF_10MHz,   //!<SR4k: maximal range 15m \intDoc{frequency}
       MF_29MHz,   //!<SR4k: maximal range 5.17m
       MF_31MHz,   //!<SR4k: maximal range 4.84m
       MF_LAST};*/
  SetVideoSource(cam);

  SetAmplitudeThresholdRange(min_amplitudeThreshold_, max_amplitudeThreshold_);
  SetSaturationThresholdRange(min_saturationThreshold_, max_saturationThreshold_);
  SetDistanceOffsetRange(min_distanceOffset_, max_distanceOffset_);
  SetIntegrationTimeRange(min_integrationTime_, max_integrationTime_);
}



VideoSource_Controller_SwissRanger::
~VideoSource_Controller_SwissRanger()
{

}

void VideoSource_Controller_SwissRanger::
Initialize(){
  
  sliderBox_ShGa_Visible = true;    
  vBoxVisible = true;

  wxSize SlSize(100,200);
  wxSize BtSize(55,30);
  
  wxBoxSizer *slAmp = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tAmp = new wxStaticText(this,ID_TEXT, wxT("Amplitude Threshold"));

  SliderAmplitude = new wxSlider(this, ID_AMP, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Amp"));

  SliderAmplitude->Enable(true);
  slAmp->Add(tAmp,0,wxALL,2);  
  slAmp->Add(SliderAmplitude,0, wxALL,2);
  
  wxBoxSizer *slSat = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tSat = new wxStaticText(this,ID_TEXT, wxT("Saturation Threshold"));
  SliderSaturation = new wxSlider(this, ID_SAT, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Sat"));

  SliderSaturation->Enable(true);
  slSat->Add(tSat,0,wxALL,2);  
  slSat->Add(SliderSaturation,0, wxALL,2);
  
  wxBoxSizer *slInt = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tInt = new wxStaticText(this,ID_TEXT, wxT("Integration Time"));
  SliderIntegration = new wxSlider(this, ID_INT, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Int"));

  SliderIntegration->Enable(true);
  
  mIntTimeMs = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
  mIntTimeMs->SetMaxLength( 4 ); 
  mIntTimeMs->Enable( false );
  
  slInt->Add(tInt,0,wxALL,2);  
  slInt->Add(SliderIntegration,0, wxALL,2);
  slInt->Add( mIntTimeMs, 0, wxALL, 5 );
  
  wxBoxSizer *slMod = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tMod = new wxStaticText(this,ID_TEXT, wxT("Modulation Freq"));
  wxSize comboSize(150,25);
  wxString choices[10] = {wxT("MF_60MHz"),
                         wxT("MF_40MHz"), 
                         wxT("MF_31MHz"),
                         wxT("MF_30MHz"),
                         wxT("MF_29MHz"), 
                         wxT("MF_21MHz"),
                         wxT("MF_20MHz"),
                         wxT("MF_19MHz"), 
                         wxT("MF_15MHz"),
                         wxT("MF_10MHz")};
  comboBoxModFreq = new wxComboBox(this,ID_MOD,wxT("ModFreq"),wxDefaultPosition,
                                   comboSize,10,choices,wxCB_READONLY);
  comboBoxModFreq->SetSelection(6,6);
  comboBoxModFreq->SetValue(wxT("MF_20MHz"));

  comboBoxModFreq->Enable(true);
  slMod->Add(tMod,0,wxALL,2);  
  slMod->Add(comboBoxModFreq,0, wxALL,2);
  
  checkBoxEnsureLatest = new wxCheckBox( this, ID_CHECK_ENSURELATEST, wxT("Ensure Latest"), wxDefaultPosition, wxDefaultSize, 0 );
  checkBoxShowDepth = new wxCheckBox( this, ID_CHECK_SHOWDEPTH, wxT("Show Depth"), wxDefaultPosition, wxDefaultSize, 0 );

  slMod->Add( checkBoxEnsureLatest, 0, wxALL, 5 );
  slMod->Add( checkBoxShowDepth, 0, wxALL, 5 );
  
  wxBoxSizer *slDist = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tDist = new wxStaticText(this,ID_TEXT, wxT("Distance Offset"));
  SliderDistanceOffset = new wxSlider(this, ID_DIST, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Dist"));

  SliderDistanceOffset->Enable(true);
  slDist->Add(tDist,0,wxALL,2);  
  slDist->Add(SliderDistanceOffset,0, wxALL,2);
  

  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE, 
			       wxT("UPDATE"),wxDefaultPosition,BtSize);

  vBox = new wxBoxSizer(wxHORIZONTAL);
  vBox->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  vBox->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM); 

  wxBoxSizer *sl = new wxBoxSizer(wxHORIZONTAL);
  sl->Add(slAmp,0,wxALL, 2);
  sl->Add(slSat,0,wxALL,2); 
  sl->Add(slMod,0,wxALL,2); 
  sl->Add(slInt,0,wxALL,2); 
  sl->Add(slDist,0,wxALL,2); 
  
  sliderBox_ShGa = new wxBoxSizer(wxVERTICAL);
  sliderBox_ShGa->Add(sl,0,wxALL, 2);
  sliderBox_ShGa->Add(vBox,0,wxALL,2);

  vBox->SetSizeHints(this);
  this->SetSizer(sliderBox_ShGa);
  SetAutoLayout(true);
  Layout();
  Fit();
  Update();
  Show(TRUE);
}

void VideoSource_Controller_SwissRanger::
SetVideoSource(BIAS::VideoSource_SwissRanger *cam){
  Cam_ = cam;
}


void VideoSource_Controller_SwissRanger::
OnHide(wxCommandEvent& event){
  Show(false);
}

void VideoSource_Controller_SwissRanger::
OnClose(wxCloseEvent& event) {

  Show(false);
}

void VideoSource_Controller_SwissRanger::
OnAmplitude(wxScrollEvent& event) {
  double value = SliderAmplitude->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetAmplitudeThreshold(value);		
  }
}

void VideoSource_Controller_SwissRanger::
OnSaturation(wxScrollEvent& event) {
  double value = SliderSaturation->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetSaturationThreshold(value);		
  }
}

void VideoSource_Controller_SwissRanger::
OnIntegration(wxScrollEvent& event) {
  double value = SliderIntegration->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetIntegrationTime(value);		
  }
  wxString tval;
  float fval;
  Cam_->GetIntegrationTimeMS(fval);
  tval << fval;
  mIntTimeMs->SetValue(tval);
}

void VideoSource_Controller_SwissRanger::
OnModFreq(wxCommandEvent& event) {
 std::string text = WxToAscii(comboBoxModFreq->GetValue());
 ModulationFrq freq = MF_20MHz;

 if(text.compare("MF_60MHz") == 0)
   freq = MF_60MHz;
 else if(text.compare("MF_40MHz") == 0)
   freq = MF_40MHz;
 else if(text.compare("MF_31MHz") == 0)
   freq = MF_31MHz;
 else if(text.compare("MF_30MHz") == 0)
   freq = MF_30MHz;
 else if(text.compare("MF_29MHz") == 0)
   freq = MF_29MHz;
 else if(text.compare("MF_21MHz") == 0)
   freq = MF_21MHz;
 else if(text.compare("MF_20MHz") == 0)
   freq = MF_20MHz;
 else if(text.compare("MF_19MHz") == 0)
   freq = MF_19MHz;
 else if(text.compare("MF_15MHz") == 0)
   freq = MF_15MHz;
 else if(text.compare("MF_10MHz") == 0)
   freq = MF_10MHz;

  if (Cam_!= NULL) {
    Cam_->SetModulationFrequency(freq);		
  }
}

void VideoSource_Controller_SwissRanger::
OnDistance(wxScrollEvent& event) {
  double value = SliderDistanceOffset->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetDistanceOffset(value);		
  }
}



void VideoSource_Controller_SwissRanger::
Update() {
  if (Cam_!= NULL) {
    unsigned short currentShort =0;
    unsigned char currentChar = 0;
    unsigned int currentInt = 0;

		Cam_->GetAmplitudeThreshold(currentShort);
    SliderAmplitude->SetValue(int(currentShort));
		Cam_->GetSaturationThreshold(currentShort);
    SliderSaturation->SetValue(int(currentShort));
    Cam_->GetIntegrationTime(currentChar);
    SliderIntegration->SetValue(int(currentChar));
 		Cam_->GetDistanceOffset(currentChar);
    SliderDistanceOffset->SetValue(int(currentChar));
    unsigned int freq = MF_20MHz;
 		Cam_->GetModulationFrequency(freq);
    
    if(freq == MF_60MHz)
      currentInt = 0;
    else if(freq == MF_40MHz)
      currentInt = 1;
    else if(freq == MF_31MHz)
      currentInt = 2;
    else if(freq == MF_30MHz)
      currentInt = 3;
    else if(freq == MF_29MHz)
      currentInt = 4;
    else if(freq == MF_21MHz)
      currentInt = 5;
    else if(freq == MF_20MHz)
      currentInt = 6;
    else if(freq == MF_19MHz)
      currentInt = 7;
    else if(freq == MF_15MHz)
      currentInt = 8;
    else if(freq == MF_10MHz)
      currentInt = 9;

    comboBoxModFreq->SetSelection(currentInt,currentInt);
  }
   
}

void VideoSource_Controller_SwissRanger::
OnUpdate(wxCommandEvent& event) {
  Update();
}

void  VideoSource_Controller_SwissRanger::
SetAmplitudeThresholdRange(unsigned int min,unsigned int max)
{
  SliderAmplitude->SetRange(min,max);

}

void  VideoSource_Controller_SwissRanger::
SetSaturationThresholdRange(unsigned int min,unsigned int max)
{
  SliderSaturation->SetRange(min,max);

}


void  VideoSource_Controller_SwissRanger::
SetDistanceOffsetRange(unsigned int min,unsigned int max)
{
 SliderDistanceOffset->SetRange(min,max);
}

void  VideoSource_Controller_SwissRanger::
SetIntegrationTimeRange(unsigned int min,unsigned int max)
{
 SliderIntegration->SetRange(min,max);
}

void VideoSource_Controller_SwissRanger::
OnCheckEnsureLatest(wxCommandEvent& event) {
  Cam_->EnsureLatestFrame(event.IsChecked());
}

void VideoSource_Controller_SwissRanger::
OnCheckShowDepth(wxCommandEvent& event) {
  Cam_->GrabImageReturnsDepth(event.IsChecked());
}



