#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE 
// fixed in wx 2.7.2 
#  pragma warning(disable: 4005)
#endif

#include "VideoSource_Controller_XB3.hh"

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>  
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller_XB3, wxFrame)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller_XB3::OnHide)
  EVT_COMMAND_SCROLL(ID_GAIN,      VideoSource_Controller_XB3::OnGain)
  EVT_COMMAND_SCROLL(ID_SHUTTER,   VideoSource_Controller_XB3::OnShutter)
  EVT_BUTTON(ID_BUTTON_AUTO_SHUTTER, VideoSource_Controller_XB3::OnAuto_Shutter)
  EVT_BUTTON(ID_BUTTON_WHITEBALANCE, VideoSource_Controller_XB3::OnWhiteBalance)
  EVT_BUTTON(ID_BUTTON_AUTO_GAIN, VideoSource_Controller_XB3::OnAuto_Gain)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller_XB3::OnUpdate)
  EVT_RADIOBUTTON(ID_CHECKBOX_LEFT, VideoSource_Controller_XB3::OnRadioButton)
  EVT_RADIOBUTTON(ID_CHECKBOX_RIGHT, VideoSource_Controller_XB3::OnRadioButton)
  EVT_RADIOBUTTON(ID_CHECKBOX_CENTER, VideoSource_Controller_XB3::OnRadioButton)
  EVT_RADIOBUTTON(ID_CHECKBOX_ALL, VideoSource_Controller_XB3::OnRadioButton)
  EVT_RADIOBUTTON(ID_CHECKBOX_RAW, VideoSource_Controller_XB3::OnRadioButton)
END_EVENT_TABLE()


VideoSource_Controller_XB3::
VideoSource_Controller_XB3(BIAS::VideoSource_DCAM_XB3 *cam, wxFrame *parent,string title,unsigned int& imageToShow,
					   double minShutter, double maxShutter,
					   double minGain,double maxGain)
  : wxFrame(parent, -1 
        ,(AsciiToWx(title.c_str()))
        ,wxDefaultPosition,wxSize(400,500),wxSTAY_ON_TOP |wxCAPTION){
  Cam_ = cam;
  minShutter_=minShutter;
  maxShutter_=maxShutter;
  minGain_= minGain;
  maxGain_= maxGain;
  Initialize();

  ImageToShow_ = &imageToShow;

  SetShutterRange((unsigned)minShutter_,(unsigned)maxShutter_);
  SetGainRange((unsigned)minGain_,(unsigned)maxGain_);
}



VideoSource_Controller_XB3::
~VideoSource_Controller_XB3()
{

}

void VideoSource_Controller_XB3::
Initialize(){
  //Shut the Automode DOWN
  auto_gain = false;
  auto_shutter = false;

  sliderBox_ShGa_Visible = true;    
  vBoxVisible = true;

  wxSize SlSize(70,200);
  wxSize BtSize(55,30);
  

  wxBoxSizer *slGa = new wxBoxSizer(wxVERTICAL);
  wxStaticText *textGain = new wxStaticText(this,ID_TEXT, wxT("Gain\n[dB]"));
  SliderGain = new wxSlider(this, ID_GAIN, 1500, 0, 3000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Gain"));

  auto_ga_Button = new wxButton(this, ID_BUTTON_AUTO_GAIN, 
                                wxT("MAN"),wxDefaultPosition,BtSize);

  SliderGain->Enable(true);
  slGa->Add(textGain,0,wxALL,2);  
  slGa->Add(SliderGain,0, wxALL,2);
  slGa->Add(auto_ga_Button,0,wxALL, 2);

  wxBoxSizer *slSh = new wxBoxSizer(wxVERTICAL);
  wxStaticText *textShutter = new wxStaticText(this,ID_TEXT, wxT("Shutter\n[ms]"));
  SliderShutter = new wxSlider(this, ID_SHUTTER, 20, 1, 80,
                               wxDefaultPosition,SlSize,
                               wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                               wxDefaultValidator, wxT("Shutter"));
  
  auto_sh_Button = new wxButton(this, ID_BUTTON_AUTO_SHUTTER, 
                                wxT("MAN"),wxDefaultPosition,BtSize);
  SliderShutter->Enable(true);
  slSh->Add(textShutter,0,wxALL,2);  
  slSh->Add(SliderShutter,0,wxALL, 2);
  slSh->Add(auto_sh_Button,0,wxALL, 2);

  wxBoxSizer *sl = new wxBoxSizer(wxHORIZONTAL);
  sl->Add(slSh,0,wxALL,2);  
  sl->Add(slGa,0,wxALL, 2);

  whiteBalanceButton = new wxButton(this, ID_BUTTON_WHITEBALANCE, 
                                wxT("WB"),wxDefaultPosition,BtSize);
  sl->Add(whiteBalanceButton,0,wxALL, 2);

  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE, 
			       wxT("UPDATE"),wxDefaultPosition,BtSize);

  vBox = new wxBoxSizer(wxHORIZONTAL);
  vBox->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  vBox->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM); 

  
  sliderBox_ShGa = new wxBoxSizer(wxVERTICAL);
  sliderBox_ShGa->Add(sl,0,wxALL, 2);
  sliderBox_ShGa->Add(vBox,0,wxALL,2);

  wxBoxSizer *checkBoxes = new wxBoxSizer(wxVERTICAL);

  all_CheckBox = new wxRadioButton(this,ID_CHECKBOX_ALL,wxT("Alle Bilder anzeigen"),wxDefaultPosition, wxDefaultSize,wxRB_GROUP);
  all_CheckBox->SetValue(true);
  checkBoxes->Add(all_CheckBox,0,wxALL,2);
  
  left_CheckBox = new wxRadioButton(this,ID_CHECKBOX_LEFT,wxT("Linkes Bild anzeigen"));
  checkBoxes->Add(left_CheckBox,0,wxALL,2);

  center_CheckBox = new wxRadioButton(this,ID_CHECKBOX_CENTER,wxT("Mittleres Bild anzeigen"));
  checkBoxes->Add(center_CheckBox,0,wxALL,2);

  right_CheckBox = new wxRadioButton(this,ID_CHECKBOX_RIGHT,wxT("Rechtes Bild anzeigen"));
  checkBoxes->Add(right_CheckBox,0,wxALL,2);

  raw_CheckBox = new wxRadioButton(this,ID_CHECKBOX_RAW,wxT("RAW Bild anzeigen"));
  checkBoxes->Add(raw_CheckBox,0,wxALL,2);

  layoutBox = new wxBoxSizer(wxHORIZONTAL);
  layoutBox->Add(sliderBox_ShGa,0,wxALL,2);
  layoutBox->Add(checkBoxes,0,wxALL,2);

   vBox->SetSizeHints(this);
  this->SetSizer(layoutBox);
  SetAutoLayout(true);
  Layout();
  Fit();
  wxCommandEvent fakeEvent;
  OnUpdate(fakeEvent);
  Show(TRUE);
}

void VideoSource_Controller_XB3::
SetVideoSource(BIAS::VideoSource_DCAM_XB3 *cam){
  Cam_ = cam;
}


void VideoSource_Controller_XB3::
OnHide(wxCommandEvent& event){
  Show(false);
}

void VideoSource_Controller_XB3::
OnClose(wxCloseEvent& event) {
  Show(false);
}

void VideoSource_Controller_XB3::
OnGain(wxScrollEvent& event) {
  double value = SliderGain->GetValue();
  if(Cam_ != NULL){
    auto_off(1);
		Cam_->SetGain(value/100.0);
  }
}

void VideoSource_Controller_XB3::
OnShutter(wxScrollEvent& event) {
  double value = SliderShutter->GetValue();
  if(Cam_ != NULL){
	  auto_off(0);
		Cam_->SetShutter(value/1000.0); 
  }
}
void VideoSource_Controller_XB3::
OnWhiteBalance(wxCommandEvent& event) {
  if(Cam_!=NULL)
    Cam_->SoftwareWhiteBalance();
}

void VideoSource_Controller_XB3::
OnAuto_Shutter(wxCommandEvent& event) {
  if(!auto_shutter) {
    auto_on(0);
  }
  else {
    auto_off(0);
  }
}

void VideoSource_Controller_XB3::
OnAuto_Gain(wxCommandEvent& event) {
  if(!auto_gain) {
    auto_on(1);
  }
  else {
    auto_off(1);
  }
}


void VideoSource_Controller_XB3::
Update() {
  if (Cam_!= NULL) {
    float current =0.0;
    current= Cam_->GetShutter();
    SliderShutter->SetValue(int(current*1000.0f));
    current = Cam_->GetGain();

    if (current <0) 
     BIASERR("Got no gain value from camera")
	else
	  SliderGain->SetValue(int(current*100));

    bool autoon = Cam_->GetAutoShutter();
    if (autoon) auto_on(0);
    else auto_off(0);
    autoon = Cam_->GetAutoGain();
    if (autoon) auto_on(1);
    else auto_off(1);
  }
}

void VideoSource_Controller_XB3::
OnUpdate(wxCommandEvent& event) {
  Update();
}

void VideoSource_Controller_XB3::
OnRadioButton(wxCommandEvent& event){
  switch(event.GetId()){
  case ID_CHECKBOX_LEFT: *ImageToShow_ = 1;
    break;
  case ID_CHECKBOX_RIGHT: *ImageToShow_ = 3;
    break;
  case ID_CHECKBOX_CENTER: *ImageToShow_ = 2;
    break;
  case ID_CHECKBOX_RAW: *ImageToShow_ = 4;
    break;
  case ID_CHECKBOX_ALL:  *ImageToShow_ = 0;
    break;
  default: cout << "Could not recognize sender of event" << endl;
    break;
  }
}



void VideoSource_Controller_XB3::
auto_on(int param) {
  //SHUTTER
  if(param == 0){
    auto_shutter = true;
    Cam_->SetAutoShutter(true);
    auto_sh_Button->SetLabel(wxT("AUTO"));
  }
  //GAIN
  else if(param == 1){
    auto_gain = true;
    Cam_->SetAutoGain(true);
    auto_ga_Button->SetLabel(wxT("AUTO"));
  }
}

void VideoSource_Controller_XB3::
auto_off(int param) {
  //SHUTTER
  if(param == 0){
    auto_shutter = false;
    Cam_->SetAutoShutter(false);
    auto_sh_Button->SetLabel(wxT("MAN"));
  }
  //GAIN
  else if(param == 1){
    auto_gain = false;
    Cam_->SetAutoGain(false);
    auto_ga_Button->SetLabel(wxT("MAN"));  
  }
}

void  VideoSource_Controller_XB3::
SetShutterRange(unsigned int min,unsigned int max)
{
  SliderShutter->SetRange(min,max);

}

void  VideoSource_Controller_XB3::
SetGainRange(unsigned int min,unsigned int max)
{
 SliderGain->SetRange(min,max);
}
