#ifndef __ZoomImageCanvas_hh__
#define __ZoomImageCanvas_hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error BIAS_HAVE_WXWIDGETS not defined but required by ZoomImageCanvas. Please enable BIAS_HAVE_WXWIDGETS in CMake configure step.
#endif

#include <Base/Image/Image.hh>
#include <Base/Common/BIASpragmaStart.hh>
#include <wx/wx.h>
#include <wx/dcbuffer.h>
#include <Base/Common/BIASpragmaEnd.hh>

namespace BIAS{

  /** @class ZoomImageCanvas
      @brief 
      @author woelk */
  class BIASGui_EXPORT ZoomImageCanvas : public wxPanel
  {
  public:
    ZoomImageCanvas(wxWindow* parent, wxStatusBar *sbar=NULL, int sbarid=0,
                    wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition,
                    const wxSize& size = wxSize(320, 240),
                    long style = wxTAB_TRAVERSAL,
                    const wxString& name = wxT("ZoomImageCanvas"));
    ~ZoomImageCanvas();

    void NewImage(BIAS::ImageBase& im, BIAS::Image<unsigned char>& ucim);
    void NewCoo(unsigned x, unsigned y);
    void NewZoom(int zoom);

    /**
     * Canvas now uses double buffering to avoid flickering, so it should be ok
     * to always clear the canvas.
     */
    inline void SetClearDrawingContext(bool clear)
    { ClearDrawingContext_=clear; }

  protected:
    void UpdateStatusBar(unsigned x, unsigned y);
    void GetZoomImage();

    void OnResize(wxSizeEvent & event);
    void OnPaint(wxPaintEvent & event);
    void OnMouseMove(wxMouseEvent &event);
    void OnMouseLeave(wxMouseEvent &event);
    void OnRightMouse(wxMouseEvent &event);
    void OnLeftMouse(wxMouseEvent &event);
    void OnKey(wxKeyEvent &event);
    void OnLeftMouseDoubleClicked(wxMouseEvent &event);

    wxBitmap *Bitmap_;
    wxImage *wximage_;
    BIAS::Image<unsigned char> ucim_, zim_;
    BIAS::ImageBase origim_;
    wxStatusBar *SBar_;
    int SBarID_;

    // Canvas now uses double buffering to avoid flickering, so it should be ok
    // to always clear the canvas.
    bool ClearDrawingContext_;

    unsigned ZoomFactor_, ZoomX_, ZoomY_; // zoom coo, set from NewCoo
    // upper left corner of ZoomX_, ZoomY_ in Window_
    unsigned ZoomXPos_, ZoomYPos_;
    int wwidth_, wheight_; // width & height of Window_
    int mousex_, mousey_;

    wxMutex wxbitmap_mutex_, origim_mutex_;

    DECLARE_EVENT_TABLE()
  }; // class ZoomImageCanvas

} // namesape

#endif // __ZoomImageCanvas_hh__
