/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __CONVERTHDR_HH__
#define __CONVERTHDR_HH__

#include <Base/Image/Image.hh>


namespace BIAS{


    /**
    @class ConvertHDR
    @ingroup g_image
    This class handles conversions of HDR images to for example unsigned char.
    Needed for some tools and displaying HDR images.
    In the future this class will be expanded to incorporate white balance and
    gamma functions
    @author Anne Sedlazeck
    @status untested
    */

    template <class StorageType>
    class BIASImage_EXPORT ConvertHDR
    {
    public:
        ///std Con-/Destructor
        ConvertHDR();
        ~ConvertHDR();

        // converts image to unsinged char image, assumes bit depth 16 -> all values are diveded by 65535
        int ToUnsignedChar(BIAS::Image<StorageType>& src, BIAS::Image<unsigned char>& dst);

        // converts to unsigned char - currentMaxVal is the highest possible value, gamma function supported
        int ToUnsignedCharGamma(BIAS::Image<StorageType>& src, BIAS::Image<unsigned char>& dst, float gamma, unsigned int currentMaxVal);


    };

} // namespace BIAS

#endif // __CONVERTHDR_HH__
