/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2007    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIASCorrespondenceMap_HH__
#define __BIASCorrespondenceMap_HH__
#include "bias_config.h" 

#include <Base/Image/Image.hh>

namespace BIAS {
  /** @class CorrespondenceMap
      @ingroup g_image
      @brief holds dense 2d correspondences (flow field) for an image
      @author koeser 11/2007
  */
  template <class StorageType> 
  class BIASImage_EXPORT CorrespondenceMap : public Image<StorageType> 
  {
  public: 
    CorrespondenceMap();
    
    CorrespondenceMap(const CorrespondenceMap<StorageType> &src) { 
      operator=(src);
    };    
    virtual ~CorrespondenceMap(){};

    /** @brief read from ascii file vector as two doubles each line, starting
        with first image column, then second */
    int Read(const std::string filename, 
             unsigned int width=720, unsigned int height=576);

    /** virtual covariant copy constructor which produces the same type of
        object as the template object (Image, CorrespondenceMap, ...).
        Must be implemented in all derived classes and since some people do
        work with ImageBase objects, it is also implemented there.
        The caller is responsible to delete the clone.
        @author koeser 11/2007 */
    virtual CorrespondenceMap<StorageType>* Clone() const 
    { return new CorrespondenceMap<StorageType>(*this); }

    CorrespondenceMap<StorageType> 
      operator=(const CorrespondenceMap<StorageType> &src);
  
  };
}
#endif

