/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/


/**
 * @example ExampleFFT2D_Tiles.cpp
   @class ExampleFFT2D_Tiles
   @relates FFT2D_Tiles
   @brief Example for using the fast fourier Transform (FFT) Tiles
   @ingroup g_examples
   @author  MIP
*/

#include <Image/FFT2D_Tiles.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>


using namespace std;
using namespace BIAS;


int main(int argc, char *argv[])
{

  if (argc!=2){
    cerr << argv[0] << "  <image>\n";
    return -1;
  }

  Image<unsigned char> im,imgrey, imgnew;
  Image<float> res,r1,r2;

  if (ImageIO::Load(argv[1], im)!=0){
    BIASERR("error loading image "<<argv[1]);
    return -2;
  } 
  
  ImageConvert::ToGrey(im,imgrey);
//   ImageIO::Save("grey",imgrey);
  FFT2D_Tiles<unsigned char, float> fft;

  fft.Init(imgrey.GetWidth(),imgrey.GetHeight(),32,32);

  // transform to 1 image with 1 channel, length of complex only
  fft.TransformAbs(imgrey,res);
//   ImageIO::Save("FFT2D-real",r1);
//   ImageIO::Save("FFT2D-img",r2);
//   fft.Normalize(res);
  //ImageIO::Save("FFT2D-abs",res);
  ImageIO::Save("FFT2D-abs",res);



  return 0;
}
