/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * @example ExampleUnvignette.cpp
 * @class ExampleUnvignette
 * @relates UnVignette
 * @brief shows usage of image unvignetting (radial light fall off)
 * @author ischiller
 * @date 2007
 */

#include <bias_config.h>

#include <Image/CameraParam.hh>
#include <Image/UnVignette.hh>
#include <Base/Image/ImageIO.hh>
#include <iostream>
#include <Base/Debug/TimeMeasure.hh>


using namespace BIAS;
using namespace std;



int main(int argc, char** argv )
{
  if (argc<3 ) {
    cout <<"Usage: ExampleUnVignette <image> <CameraParam.xml>"<<endl;
    exit(0);
  }

  BIAS::CameraParam cParam;

  BIAS::Image<unsigned char> Image;
  BIAS::ImageIO::Load(argv[1],Image);
  int ret = cParam.ReadCameraData(argv[2]);
  if(ret < 0)
  {
    cout<<"Reading of CameraParams failed!"<<endl;
    exit(-1);
  }
  //parameter: CameraParam , useLuT 
  BIAS::UnVignette unVig(cParam,LEAST_SQUARES);
  BIAS::TimeMeasure timer;
  timer.Start();
  unVig.Compute(Image);
  timer.Stop();
  timer.PrintUserTime();
  //BIAS::ImageIO::Save("ExampleUnVignetted.mip",Image);
  BIAS::ImageIO::Save("ExampleUnVignetted.mip",Image);
  cout<<"Output image written to ExampleUnVignetted.mip."<<endl;
  return 0;  
}
