/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __2DFFT_free_hh__
#define __2DFFT_free_hh__

#include <bias_config.h>
#include <Base/Image/Image.hh>


namespace BIAS {

  /* wrapper to the fftpack library from netlib (see fftpack/fft.c)


  @author evers 2009 */
  template <class StorageType>
  class BIASImage_EXPORT  FFT2D_free 
  {
  public:
    FFT2D_free();
    virtual ~FFT2D_free();

    void Init(int sizex,int sizey);
    /// apply FFT on @param in, result is 2-channel image with real an
    /// img. Result ist _not_normalized.
    void Forward(const Image<StorageType> &in, Image<float> &ComplexOut);


    /// apply normalisation on  spectrum image, which is float-type
    void Normalize(Image<float> &Image);

    void Reverse(const Image<float> &ComplexIn, Image<StorageType> &res);

    /// take the spectrum image (2-channel) and compute the absolute value for each pixel
    void GetMagnitude(const Image<float> &ComplexIn, Image<float> &value);

  protected:

    void ComplexColumnToWorkArray1_(const Image<float> &ComplexImg,
				   unsigned int col);
    void ComplexRowToWorkArray0_(const Image<float> &ComplexImg,
				unsigned int row);

    void WorkArray0ToComplexRow_(Image<float> &ComplexImg,unsigned int row);
    void WorkArray1ToComplexColumn_(Image<float> &ComplexImg,unsigned int col);

    // one set of work- and tmparray for each dimension
    long int  WorkSize0_,WorkSize1_;
    float *WorkArray0_, *WorkArray1_;
    float *TmpArray0_,*TmpArray1_;
    long int TmpArraySize0_,TmpArraySize1_;
    unsigned long long int Normalize_ ; 
  }; // class


} // namespace

#endif
