#include "HistogramEqualization.hh"
#include <Image/HistogramImage.hh>
#include <Base/Image/ImageIO.hh>
#include <limits>

using namespace BIAS;
using namespace std;

template <class InputStorageType, class OutputStorageType>
int HistogramEqualization<InputStorageType,OutputStorageType>::
Filter(const Image<InputStorageType>& src,Image<OutputStorageType>& dst)
{
  if(src.GetChannelCount()!=1)
  {
    BIASERR("HistogramEqualization: not for multiple channel images");
    return -1;
  }
  if (!dst.SamePixelAndChannelCount(src)){
    dst.Release();
    dst.Init(src.GetWidth(), src.GetHeight(), src.GetChannelCount());
  }

  //1. compute histogram
  HistogramImage hsrc;
  hsrc.Init(256,256,1);//we don't really need the drawing functions but we heave to initialize the class...
  unsigned bincount =(unsigned) (1.0 + std::numeric_limits<InputStorageType>::max() + 
                      fabs((double) std::numeric_limits<InputStorageType>::min()));
  hsrc.InitHist(bincount);
  hsrc.AddHist<InputStorageType>(src);
  
  unsigned binnum = hsrc.GetBinCount(); 
  BIASASSERT(binnum>0);
  unsigned pixnum = 0;
  map<unsigned,unsigned> filledbins; //(binnum,value)    
  //we allow only integer values. thus for every possible grey value a bin exists.   
  for(unsigned b=0; b<binnum; b++)
  {
    unsigned val = (unsigned) hsrc.GetBin(b);
    if(val>0)
    {
      filledbins[b]=val; 
      pixnum += val;
      InputStorageType greyvalue = (InputStorageType)0.0;
      greyvalue = (InputStorageType)
        (b -  fabs((double) std::numeric_limits<InputStorageType>::min())); //in case we have signed data     
    }       
  }

 map<unsigned,unsigned>::iterator it;
 unsigned numoffilledbins = 0; numoffilledbins = filledbins.size(); 
 map<InputStorageType,InputStorageType> NewGreyVal;   
 double sum = 0.0; 
 
 double maxval = std::numeric_limits<InputStorageType>::max();//TODO make this a parameter
 double minval = std::numeric_limits<InputStorageType>::min();  
 double valrange = (maxval-minval);
 for(it=filledbins.begin(); it != filledbins.end(); it++)
 {
   sum+= (double) it->second;
   InputStorageType oldgreyvalue =(InputStorageType) 
     (it->first -  fabs((double) std::numeric_limits<InputStorageType>::min()));
   NewGreyVal[oldgreyvalue] =(InputStorageType) 
                             (((sum/(double)pixnum)*valrange)+minval);
 }

 const InputStorageType** pDat = src.GetImageDataArray();
 OutputStorageType** pDstDat = dst.GetImageDataArray(); 
 const ROI *roi = src.GetROI();
        
 unsigned wid = src.GetWidth();
 unsigned hei = src.GetHeight();
 for(unsigned y=0; y<hei; y++)
 for(unsigned x=0; x<wid; x++)
 {    
   if (roi->IsInROI(x,y))
   {
    InputStorageType oldgreyval = pDat[y][x];  
    //get new value:
    pDstDat[y][x]= (OutputStorageType) NewGreyVal[oldgreyval];
   }
 }

  return 0;
}

#define INST(intype,outtype) \
template class BIASImage_EXPORT HistogramEqualization<intype,outtype>;

INST(char,char)
INST(unsigned char, unsigned char)

//won't work for float and double instances.
//even int and unsigned int aren't so good ideas.
