/* This file is part of the BIAS library (Basic ImageAlgorithmS).
   
   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel

   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __HistogramEqualization_hh__
#define __HistogramEqualization_hh__

#include <Base/Image/Image.hh> 
#include <Filter/FilterBase.hh>

namespace BIAS 
{  
  /**
    @brief perfoms a histogram equalization on a grey value image

    this improves the contrast of low contrast images by
    redistributing the intensities on the histrogram.
    this is done by reassigning the pixel intensities.

    the histogram equalization is done by the following steps:

    1. compute histogram (a bin for every possible grey value)
    2. loop through filled bins b[n]. start with bin of lowest grey value. b[0]
       for every bin b[i] compute sum s(i) of bin values so far ( b[0]+b[1]+ ... + b[i] )
    3. create a look up table which maps the old  intensity (grey) values to new ones.
       
       newgreyvalue(i) = (s(i)/NumOfAllPixelsInImage) * PossibleNumberOfGreyValues
    4. use lookup table to remap input image
    
        
    this template can be used for 8 or 16bit integer values.
    don't try this with float or double.
  */
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT HistogramEqualization:
  public BIAS::FilterBase<InputStorageType, OutputStorageType>
  {
    public:
      HistogramEqualization(){};
      ~HistogramEqualization(){};

     virtual int Filter(const Image<InputStorageType>& src, 
                        Image<OutputStorageType>& dst);
     
    protected:

     void GetBordersValid_(int& border_x, int& border_y) const {};
  };
}

#endif