/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Image/LogPolarMapping.hh"
#include <Base/Geometry/HomgPoint2D.hh>

using namespace BIAS;
using namespace std;


template <class InputStorageType, class OutputStorageType>
LogPolarMapping<InputStorageType, OutputStorageType>::
~LogPolarMapping() 
{}

template <class InputStorageType, class OutputStorageType>
LogPolarMapping<InputStorageType, OutputStorageType>::
LogPolarMapping()
{}

template <class InputStorageType, class OutputStorageType>
LogPolarMapping<InputStorageType, OutputStorageType>::
LogPolarMapping(const LogPolarMapping & src) 
{
  (*this) = src;
}

template <class InputStorageType, class OutputStorageType>
LogPolarMapping<InputStorageType, OutputStorageType> & 
LogPolarMapping<InputStorageType, OutputStorageType>::
operator=(const LogPolarMapping & src)
{    
  XCenter_ = src.XCenter_;
  YCenter_ = src.YCenter_;
  return *this;
}

template <class InputStorageType, class OutputStorageType>
void LogPolarMapping<InputStorageType, OutputStorageType>::
SetImageCenter(const int & x, const int & y) 
{
  double a,r;
 
  XCenter_ = x;
  YCenter_ = y;
  _XYMap = (HomgPoint2D*) calloc(2*x*2*y,sizeof(HomgPoint2D));
  for(int i=0;i<2*x;i++){
    for(int j=0;j<2*y;j++){
      //map image height to 0 - PI (180 Deg)
      a = 0.5*M_PI*(double)j/(double)YCenter_;
      //map image width to minimum of center coords
      r = (double) pow((double)XCenter_,(double)i/(double)(2*XCenter_));
      //calculate source pixle position
      _XYMap[i+j*2*x][0] = sin(a) * r;
      _XYMap[i+j*2*x][1] = YCenter_ + cos(a) * r;
    }
  }

}

template <class InputStorageType, class OutputStorageType>
int LogPolarMapping<InputStorageType, OutputStorageType>::
GetSourceCoordinates_(const HomgPoint2D & sink,
                      HomgPoint2D & source) const {
  /*
  double a,r;
  //map image height to 0 - PI (180 Deg)
  a = 0.5*M_PI*(double)sink[1]/(double)YCenter_;
  //map image width to minimum of center coords
  r = (double) pow((double)XCenter_,(double)sink[0]/(double)(2*XCenter_));
  //r=exp(log((double)XCenter_)*(double)sink[0]/(2.0*(double)XCenter_));
  //calculate source pixle position
  source[0] = sin(a) * r;
  source[1] = YCenter_ + cos(a) * r;
  */
  source[0] = _XYMap[(int)sink[0]+2*XCenter_*(int)sink[1]][0];
  source[1] = _XYMap[(int)sink[0]+2*XCenter_*(int)sink[1]][1];
  return 0;
}

// TODO FIXME win32 template instantiation problem (JW)
template class LogPolarMapping<unsigned char, unsigned char>;
template class LogPolarMapping<float, float>;
template class LogPolarMapping<unsigned char, float>;

#if defined(BUILD_IMAGE_CHAR)
// TODO FIXME win32 template instantiation problem (JW)
template class LogPolarMapping<unsigned char, char>;
template class LogPolarMapping<char, char>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class LogPolarMapping<unsigned short,unsigned short>;
#endif

#if defined(BUILD_IMAGE_SHORT)
template class LogPolarMapping<short, short>;
#endif

#if defined(BUILD_IMAGE_SHORT)&&defined(BUILD_IMAGE_USHORT)
template class LogPolarMapping<unsigned short, short>;
#endif

#if defined(BUILD_IMAGE_INT)
template class LogPolarMapping<int,int>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class LogPolarMapping<unsigned short,float>;
#endif

#if defined(BUILD_IMAGE_USHORT) && defined(BUILD_IMAGE_INT)
template class LogPolarMapping<unsigned short,int>;
#endif

#if defined(BUILD_IMAGE_DOUBLE)
template class LogPolarMapping<double,double>;
#endif
