#include "StereoRedGreen.hh"

#include <Base/Image/ImageConvert.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Debug/Exception.hh>

using namespace BIAS;
using namespace std;


StereoRedGreen::
StereoRedGreen()
{}

StereoRedGreen::
~StereoRedGreen()
{}

template <class StorageType>
void StereoRedGreen::
Combine(const Image<StorageType>& left, const Image<StorageType>& right,
        Image<StorageType>& result)
{
  static Image<StorageType> LeftGrey, RightGrey;
  if ( left.GetWidth() != right.GetWidth() ||
       left.GetHeight() != right.GetHeight() ){
    BEXCEPTION("invalid input image sizes");
  }
  if (ImageConvert::ToGrey(left, LeftGrey)!=0){
    BEXCEPTION("conversion failed");
  }
  if (ImageConvert::ToGrey(right, RightGrey)!=0){
    BEXCEPTION("conversion failed");
  }
  
  if ( result.GetWidth() !=left .GetWidth() || 
       result.GetHeight() != left.GetHeight() ||
       result.GetChannelCount() != 3 ){
    if (!result.IsEmpty()){
      result.Release();
    }
    result.Init(left.GetWidth(), left.GetHeight(), 3);
  }
  if (result.GetColorModel() != ImageBase::CM_RGB){
    result.SetColorModel(ImageBase::CM_RGB);
  }
  result.SetInterleaved(true);

  StorageType *l = LeftGrey.GetImageData();
  StorageType *r = RightGrey.GetImageData();
  StorageType *d = result.GetImageData();
  StorageType *e = d + result.GetPixelCount()*3;
  while(d<e){
    *d++ = *l++;
    *d++ = *r++;
    *d++ = 0;
  }

}

////////////////////////
// instantiation
////////////////////

#define INST(type) \
template void BIASImage_EXPORT StereoRedGreen::Combine<type>(const Image<type>& left, \
        const Image<type>& right, Image<type>& result);

INST(unsigned char)
INST(float)

#ifdef BUILD_IMAGE_INT
INST(int)
#endif
#ifdef BUILD_IMAGE_CHAR
INST(char)
//INST(char)
#endif
#ifdef BUILD_IMAGE_SHORT
INST(short)
#endif

#if defined(BUILD_IMAGE_USHORT) 
INST(unsigned short)
#endif

#ifdef BUILD_IMAGE_DOUBLE
INST(double)
#endif
#ifdef BUILD_IMAGE_UINT
INST(unsigned int)
#endif
