/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TextureTransformDisplacement__hh__
#define __TextureTransformDisplacement__hh__

#include <Base/Common/BIASpragmaStart.hh>
#include <Image/TextureTransform.hh>


namespace BIAS {

  /** @class TextureTransformDisplacement
   *  @brief analytic properties of 2D image displacement
   *
   *  See BIAS::TextureTransform for explanation of methods.
   *  @author koeser 01/2008
   **/
  class TextureTransformDisplacement: public TextureTransform {
  public:
    TextureTransformDisplacement() {
      P_.newsize(2);
      P_[0] = 0;
      P_[1] = 0;
    }

    virtual ~TextureTransformDisplacement(){};

    int MapForward(const HomgPoint2D& src, HomgPoint2D& sink) const {
      sink[0] = src[0] + P_[0];
      sink[1] = src[1] + P_[1];
      sink[2] = 1;
      return 0;
    }

    int MapBackward(const HomgPoint2D& sink, HomgPoint2D& src) const {
      src[0] = sink[0] - P_[0];
      src[1] = sink[1] - P_[1];
      src[2] = 1;
      return 0;
    } 

    int TextureJacobianForward(const HomgPoint2D& src, 
                               Matrix2x2<double>& Jac) const {
      Jac.SetIdentity();
      return 0;
    }

    int TextureJacobianBackward(const HomgPoint2D& sink, 
                                Matrix2x2<double>& Jac) const {
      Jac.SetIdentity();
      return 0;
    }

    virtual bool TextureJacobianIsConstant() const { return true;};

    int ParameterJacobianForward(Matrix<double>& Jac, const HomgPoint2D& src) {
      Jac.newsize(2,2);
      Jac[0][0] = 1; Jac[0][1] = 0;
      Jac[1][0] = 0; Jac[1][1] = 1;
      return 0;
    }
    
    int ParameterJacobianBackward(Matrix<double>& Jac, const HomgPoint2D& src){
      Jac.newsize(2,2);
      Jac[0][0] = -1; Jac[0][1] = 0;
      Jac[1][0] = 0; Jac[1][1] = -1;
      return 0;
    }

    bool ParameterJacobianIsConstant() const { return true;};

    /** @brief dx,dy */
    void SetParameters(const Vector<double>& p) {
      BIASASSERT(p.Size()==2);
      P_ = p;
    }

    void ComposeWithInverseDeltaP(const Vector<double>& deltaP) {
      P_[0] -= deltaP[0];
      P_[1] -= deltaP[1];
    }

    int ParameterInversionJacobian(Matrix<double>& Jac) const {
      // inverse is simply inverse angle:   alpha -> -alpha
      Jac.newsize(2,2);
      Jac[0][1] = Jac[1][0] =0;
      Jac[0][0] = Jac[1][1] = -1;
      return 0;
    }

    virtual TextureTransformDisplacement* Clone() const {
      return new TextureTransformDisplacement(*this);
    }
  protected:
  
  }; 

} // end namespace BIAS

#endif
