/* This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @example ExampleSVD3x3.cpp
    @class ExampleSVD3x3
    @relates SVD3x3, SVD
    @ingroup g_examples    
    @brief  this is an example for SVD3x3 usage   
    @author woelk 07/2004 
*/

#include <MathAlgo/SVD.hh>
#include <MathAlgo/SVD3x3.hh>
#include <Base/Math/Random.hh>
#include <Base/Math/Matrix3x3.hh>
#include <Geometry/FMatrix.hh>



using namespace BIAS;
using namespace std;

#define F_SVD_THRESH 0.1

void Print(ostream& os, SVD& svd, SVD3x3&svd3x3)
{
  os << "U: SVD: "   <<svd.GetU()<<endl;
  os << "   SVD3x3: "<<svd3x3.GetU()<<endl;
  os << "   diff: "<<svd.GetU()-svd3x3.GetU()<<endl;
      
  os << "S: SVD: "   <<svd.GetS()<<endl;
  os << "   SVD3x3: "<<svd3x3.GetS()<<endl;
  os << "   diff: "<<svd.GetS()-svd3x3.GetS()<<endl;
      
  os << "V: SVD: "   <<svd.GetV()<<endl;
  os << "   SVD3x3: "<<svd3x3.GetV()<<endl;
  os << "   diff: "<<svd.GetV()-svd3x3.GetV()<<endl;
}

int main()
{
  FMatrix F;
  F[0][0] =0.00075284066373225704;    
  F[0][1] =-0.099906426438442303;      
  F[0][2] =0.16267766990357244;
  F[1][0] =0.10333163045064053;
  F[1][1] =0.0014727467665194025;
  F[1][2] =0.6803859609102173;
  F[2][0] =-0.16234020366691171;      
  F[2][1] =-0.68087420277635458;     
  F[2][2] =0.0021896762792443826;

  SVD3x3 svdF3x3 ( (Matrix<double>)F, F_SVD_THRESH );

  cout << "F: " << F << endl;
  Matrix3x3<double> S;
  S.SetIdentity();
  for (int i=0; i<3; i++)
    S[i][i]=svdF3x3.GetS()[i];
   
  cout << "U:" << svdF3x3.GetU() << "S: " << svdF3x3.GetS() << "V: " 
       << svdF3x3.GetVT() << "\n U S Vt:"  
       << svdF3x3.GetU() * (Matrix<double>)S * svdF3x3.GetVT() 
       << "\nF:" << F << endl;
  cout << "NullspaceDim(): " << svdF3x3.NullspaceDim() << endl;
  
  SVD svdF ( (Matrix<double>)F, F_SVD_THRESH );

  cout << "F: " << F << endl;
  Matrix3x3<double> S1;
  S.SetIdentity();
  for (int i=0; i<3; i++)
    S1[i][i]=svdF.GetS()[i];
   
  cout << "U:" << svdF.GetU() << "S: " << svdF.GetS() << "V: " 
       << svdF.GetVT() << "\n U S Vt:"  
       << svdF.GetU() * (Matrix<double>)S1 * svdF.GetVT() 
       << "\nF:" << F << endl;
  cout << "NullspaceDim(): " << svdF.NullspaceDim() << endl;
  
  return 0;
}

