#ifndef __Laguerre_hh__
#define __Laguerre_hh__

#include "bias_config.h" 
#include <complex>
#include <vector>

namespace BIAS{
/**
  @class LaguerreSolver
  @test tested with TestLaguerre.cpp
  @ingroup g_mathalgo
  @brief class encapsulating a laguerre solver for polynomials
*/
class BIASMathAlgo_EXPORT LaguerreSolver
{
  public:
    LaguerreSolver(){_doPolish = true;}
    /**
     @brief compute roots of a polynomial using the laguerre method
     @param[in] coeffs the coefficients in in ascending order
     @param[out] roots the roots of the polynomial
    
     p(x) = coeffs[0] + coeffs[1] * x + ... + coeffs[n]*(x^n)
    
    */
    int Solve(const std::vector< std::complex<double> >& coeffs,
                    std::vector< std::complex<double> >& roots,
                    const double eps = 1.0e-14);
    
    int Solve(const std::vector< double >& coeffs,
              std::vector< std::complex<double> >& roots,
              const double eps = 1.0e-14);
    
    void SetPolish(bool b){_doPolish=b;};
  private:
    //returns number of iterations
    int _Laguerre(const std::vector< std::complex<double> >& coeffs,
                  std::complex<double>& x);
    
    bool _doPolish;
};

} //namespace

#endif
