/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef NURBSSURFACE_HH_
#define NURBSSURFACE_HH_

#include <vector>
#include "Base/Math/Vector3.hh"
#include "Base/Debug/Debug.hh"

namespace BIAS {

/** @class NurbsSurface
    @ingroup g_mathalgo
    @brief this class represents a NURBS surface
    @author bruenger
*/
class BIASMathAlgo_EXPORT NurbsSurface : public Debug
{
public:
	NurbsSurface();
	virtual ~NurbsSurface();
	void Init(int dimu, int dimv, int order);
	void SetCtrlPoint(int u, int v, Vector3<float> &pos);
	Vector3<float> GetCtrlPoint(int u, int v);
	void SetKnotu(int i, float &val);
	float GetKnotu(int i);
	void SetKnotv(int i, float &val);
	float GetKnotv(int i);
	void SetKnotsToDefault();
	void SetCTRLPointsToRandom();
	Vector3<float> GetSurfacePoint(float u, float v);
	std::vector<std::vector<Vector3<float> > > GetSurfaceDerivs(float u, float v, int d);

#ifdef BIAS_HAVE_XML2
	void LoadFromXML(std::string xmlfile);
#endif

	std::vector<float> GetKnotsu() const
    {
        return knotsu_;
    }

	std::vector<float> GetKnotsv() const
    {
        return knotsv_;
    }

    int GetDimu() const
    {
        return dimu_;
    }

    void SetDimu(int dimu_)
    {
        this->dimu_ = dimu_;
    }

    int GetDimv() const
    {
        return dimv_;
    }

    void SetDimv(int dimv_)
    {
        this->dimv_ = dimv_;
    }

    int GetOrder() const
    {
        return order_;
    }

    void SetOrder(int order_)
    {
        this->order_ = order_;
    }

    bool GetInitialized() const
    {
        return initialized_;
    }

    void SetInitialized(bool initialized_)
    {
        this->initialized_ = initialized_;
    }

protected:
	int FindSpan(int n, int p, float u, std::vector<float> U);
	std::vector<float> BasisFuns(int i, float u, int p, std::vector<float> U);
	std::vector<std::vector<float> > DersBasisFuns(int i, float u, int p, int n, std::vector<float> U);
	std::vector<std::vector<Vector3<float> > > ctrlpoints_;
	std::vector<float> knotsu_;
	std::vector<float> knotsv_;
	int dimu_;
	int dimv_;
	int order_;
	bool initialized_;
};

}

#endif /* NURBSSURFACE_HH_ */
