/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "StandardDeviation.hh"


//template <class T>
BIAS::StandardDeviation::StandardDeviation(vector<double> &values)
{
  val = values;
}

//template <class T>
BIAS::StandardDeviation::StandardDeviation()
{
}



double BIAS::StandardDeviation::CalcAverage()
{
  if (val.size() == 0)
    return 0.0;
  
  double avg = 0.0;
  for (unsigned int i = 0; i < val.size(); i++)
  {
    avg += val[i];
  }
  avg = avg / val.size();
  return avg;
}


double BIAS::StandardDeviation::CalcStandardDeviation()
{
  double sum = 0.0;
  double avg = CalcAverage();
  
  for (unsigned int i = 0; i < val.size(); i++)
  {
    sum += (val[i] - avg) * (val[i] - avg);
  }
  sum /= val.size() - 1;
  return sqrt(sum);
}


void BIAS::StandardDeviation::AddValue(double v)
{
  val.push_back(v);
}

void BIAS::StandardDeviation::Clear()
{
  val.clear();
}

