/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfBlendMode_hh__
#define __glfBlendMode_hh__

#include "glfCommon.hh"

namespace BIAS {

  /**
   * \brief Defines the function that blends rendered geometry with its background.
   * \ingroup g_openglframework	
   * \author jkollmann
   */
  class BIASOpenGLFramework_EXPORT glfBlendMode {
  public:
    glfBlendMode();

    /**
     * Sets whether to enable blending. If this is false (the default), all
     * other values of the blend mode have no effect.
     */
    void EnableBlending(bool enable);

    /**
     * Sets the blend equation. Valid values are: GL_FUNC_ADD, GL_FUNC_SUBTRACT,
     * GL_FUNC_REVERSE_SUBTRACT, GL_FUNC_MIN, GL_FUNC_MAX.
     * For a description of these, see:
     * http://opengl.org/sdk/docs/man/xhtml/glBlendEquation.xml
     */
    void SetBlendEquation(GLenum equation);

    /**
     * Sets the blending factors for the source and destination color.
     * For a list of valid values and a description, see:
     * http://opengl.org/sdk/docs/man/xhtml/glBlendFunc.xml
     */
    void SetBlendFunc(GLenum sfactor, GLenum dfactor);

    /**
     * Binds the blend mode.
     */
    void Bind() const;

    /**
     * The default blend mode.
     */
    static const glfBlendMode DEFAULT;

  private:
    GLboolean enable_;
    GLenum equation_;
    GLenum sfactor_, dfactor_;
  };

} // namespace BIAS

#endif // __glfBlendMode_hh__
