/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfFormatDB_hh__
#define __glfFormatDB_hh__

#include "glfCommon.hh"
#include <Base/Image/ImageBase.hh>


namespace BIAS {

  /** \brief Helper methods for format conversions in GL and between GL and BIAS.
   * Class originates from glfTexture methods which from now on are deprecated and will be removed.
   * \ingroup g_openglframework
   * \author bartczak 12/2009
   **/
  class BIASOpenGLFramework_EXPORT glfFormatDB {
  public:
    /**
     * Computes a pixel format that can be used with <code>glGetTexImage</code>
     * and <code>glReadPixels</code>
     * for an internal format of a texture. Also returns the number of channels.
     * For example: If internalFormat = GL_R3_G3_B2, then format = GL_RGB
     * and numChannels = 3.
     */
    static void GetFormatForInternalFormat(GLenum internalFormat,
                                           GLenum& format,
                                           int& numChannels);
    
    /**
     * Returns the OpenGL pixel type (GL_UNSIGNED_BYTE etc.) for a BIAS storage type.
     */
    static GLenum GetPixelTypeForStorageType(ImageBase::EStorageType storageType);

    /**
     * Returns the OpenGL pixel format (GL_RGB etc.) for a BIAS color model.
     */
    static GLenum GetFormatForColorModel(ImageBase::EColorModel colorModel);

    /**
     * Returns an OpenGL internal pixel format for a BIAS storage type and color model.
     */
    static GLenum ProposeInternalFormat(ImageBase::EStorageType storageType,
                                        ImageBase::EColorModel colorModel);

    /** Taking the enum as string and converting to the appropriate numerical constant.
     **/
    static GLenum GLenumFromString(const std::string& enumString);
    
  };

}

#endif
