/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfRenderTarget.hh"
#include "glfException.hh"

using namespace BIAS;
using namespace std;

void glfRenderTarget::ClearColorBuffer(float red, float green, float blue, float alpha)
{
  Bind();
  glClearColor(red, green, blue, alpha);
 // cout<<"Clear with :"<<red<<","<<green<<","<<blue<<","<<alpha<<endl<<flush;
  glClear(GL_COLOR_BUFFER_BIT);

  GLF_THROW_ON_OPENGL_ERROR;
}

void glfRenderTarget::ClearDepthBuffer(float depth)
{
  Bind();
  glClearDepth(depth);
  glClear(GL_DEPTH_BUFFER_BIT);

  GLF_THROW_ON_OPENGL_ERROR;
}

void glfRenderTarget::ClearAccumBuffer(float red, float green, float blue, float alpha)
{
  Bind();
  glClearAccum(red, green, blue, alpha);
  glClear(GL_ACCUM_BUFFER_BIT);

  GLF_THROW_ON_OPENGL_ERROR;
}

void glfRenderTarget::ClearStencilBuffer(int s)
{
  Bind();
  glClearStencil(s);
  glClear(GL_STENCIL_BUFFER_BIT);

  GLF_THROW_ON_OPENGL_ERROR;
}

void glfRenderTarget::ReadBuffer(ImageBase& image, BufferType bufferType, int x, int y, int width, int height)
{
  Bind();

  // choose format for glReadPixels and numChannels for image.
  GLenum format;
  int numChannels;
  switch (bufferType) {
  case BUFFER_COLOR:
    format = GL_RGB;
    numChannels = 3;
    break;
  case BUFFER_DEPTH:
    format = GL_DEPTH_COMPONENT;
    numChannels = 1;
    break;
  case BUFFER_STENCIL:
    format = GL_STENCIL_INDEX;
    numChannels = 1;
    break;
  default:
    GLF_THROW_EXCEPTION("Invalid grab source");
    break;
  };

  // get pixel type
  GLenum type;
  ImageBase::EStorageType storageType = image.GetStorageType();
  switch (storageType) {
  case ImageBase::ST_unsignedchar:
    type = GL_UNSIGNED_BYTE;
    break;
  case ImageBase::ST_char:
    type = GL_BYTE;
    break;
  case ImageBase::ST_unsignedshortint:
    type = GL_UNSIGNED_SHORT;
    break;
  case ImageBase::ST_shortint:
    type = GL_SHORT;
    break;
  case ImageBase::ST_unsignedint:
    type = GL_UNSIGNED_INT;
    break;
  case ImageBase::ST_int:
    type = GL_INT;
    break;
  case ImageBase::ST_float:
    type = GL_FLOAT;
    break;
  default:
    GLF_THROW_EXCEPTION("Invalid storage type for ReadPixels");
    break;
  };

  // create image
  if (!image.IsEmpty()) {
    image.Release();
  }
  image.Init(width, height, numChannels, storageType);

  // copy data to image
  glReadPixels(x, y, width, height, format, type, image.GetImageData());
}
