/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfRenderTarget_hh__
#define __glfRenderTarget_hh__

#include <Base/Image/ImageBase.hh>
#include "glfCommon.hh"

namespace BIAS {

  /**
   * \brief Interface for render targets.
   * \ingroup g_openglframework
   * \author jkollmann
   */
  class BIASOpenGLFramework_EXPORT glfRenderTarget {
  public:
    virtual ~glfRenderTarget() {}

    /**
     * Clears the color buffer of the render target with the given color.
     * The color (0,0,0,0) is the default clear color in OpenGL.
     */
    void ClearColorBuffer(float red = 0.0f, float green = 0.0f, float blue = 0.0f, float alpha = 0.0f);

    /**
     * Clears the depth buffer of the render target with the given value.
     * The value 1 is the default clear value for depth buffers in OpenGL.
     */
    void ClearDepthBuffer(float depth = 1.0f);

    /**
     * Clears the accumulation buffer of the render target with the given color.
     * The color (0,0,0,0) is the default clear color in OpenGL.
     */
    void ClearAccumBuffer(float red = 0.0f, float green = 0.0f, float blue = 0.0f, float alpha = 0.0f);

    /**
     * Clears the stencil buffer of the render target with the given value.
     * The value 0 is the default clear value for stencil buffers in OpenGL.
     */
    void ClearStencilBuffer(int s = 0);

    enum BufferType {
      BUFFER_COLOR,
      BUFFER_DEPTH,
      BUFFER_STENCIL
    };

    /**
     * Copies the contents of a buffer to an image.
     */
    void ReadBuffer(BIAS::ImageBase& image, BufferType bufferType, int x, int y, int width, int height);

    /**
     * Makes this render target the currently used render target.
     * This must call <code>glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, ?)</code>.
     */
    virtual void Bind() const = 0;
  };
  
} // namespace BIAS

#endif // __glfRenderTarget_hh__
