/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfRenderbuffer_hh__
#define __glfRenderbuffer_hh__

#include "glfCommon.hh"

namespace BIAS {

  /**\brief The renderbuffer
   * \author jkollmann
   * \ingroup g_openglframework
   */
  class BIASOpenGLFramework_EXPORT glfRenderbuffer {
  public:
    glfRenderbuffer();
    ~glfRenderbuffer();

    /**
     * Creates the renderbuffer with the given format and size.
     * For a list of allowed internal formats see
     * http://opengl.org/registry/specs/EXT/framebuffer_object.txt.
     * \param samples is used to generate a multisample render buffer, if supported, requires 
     * the EXT_framebuffer_multisample extension
     */
    void Create(GLenum internalFormat, int width, int height, GLsizei samples=0);

    /**
     * Binds the renderbuffer.
     * \attention For internal usage inside OpenGLFramework Base library only.
     */
    void Bind() const;

    /**
     * Returns the OpenGL id of the renderbuffer.
     * \attention For internal usage inside OpenGLFramework Base library only.
     */
    GLuint GetRenderbufferID() const;    

    void CopyToImage(ImageBase& image);

    void GetSize(GLint& width, GLint& height);
    
  private:
    GLuint id_;
    glfRenderbuffer(const glfRenderbuffer&) {}
    glfRenderbuffer& operator=(const glfRenderbuffer&) {return (*this);}
  };
  
} // namespace BIAS

#endif // __glfRenderbuffer_hh__
