/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfShader.hh"
#include "glfException.hh"
#include <fstream>

using namespace BIAS;
using namespace std;

glfShader::glfShader()
{
  id_ = 0;
}

glfShader::~glfShader()
{
  if (id_ != 0) {
    glDeleteShader(id_);
  }
}

glfShader&
glfShader::operator=(const glfShader& b)
{
  return (*this);
}


glfShader::glfShader(const glfShader& b)
{}

void glfShader::
Create(GLenum type)
{
   if (id_ == 0) {
    id_ = glCreateShader(type);
    type_ = type;
  }
  GLF_THROW_ON_OPENGL_ERROR;
}

void glfShader::Create(GLenum type, const std::string& sourceCode)
{
  if (id_ == 0) {
    id_ = glCreateShader(type);
    type_ = type;
  }
  // compile shader
  const char* sourceString = sourceCode.c_str();
  glShaderSource(id_, 1, &sourceString, NULL);
  glCompileShader(id_);

  // check success
  GLint compilationSuccess;
  glGetShaderiv(id_, GL_COMPILE_STATUS, &compilationSuccess);

  if (compilationSuccess != GL_TRUE) {
    GLF_THROW_EXCEPTION("Failed to compile shader: " <<
                        endl << GetInfoLog());
  }
  GLF_THROW_ON_OPENGL_ERROR;
}

void glfShader::CreateFromFile(GLenum type, const std::string& fileName)
{
  ifstream ifs(fileName.c_str());
  if (!ifs) {
    GLF_THROW_EXCEPTION("Failed to load shader source file: " << fileName);
  }

  string source;
  while (!ifs.eof()) {
    string line;
    getline(ifs, line);
    source += line+"\r\n";
  }

  return Create(type, source);
}

string glfShader::GetInfoLog() const
{
  BIASASSERT(id_ != 0);

  GLint logLength;
  glGetShaderiv(id_, GL_INFO_LOG_LENGTH, &logLength);
  char* infoLog = new char[logLength];
  glGetShaderInfoLog(id_, logLength, NULL, infoLog);
  std::string out = infoLog;
  delete [] infoLog;

  GLF_THROW_ON_OPENGL_ERROR;

  return out;
}

GLuint glfShader::GetShaderID() const
{
  return id_;
}
