/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfStencilBufferMode.hh"
#include "glfException.hh"

#include <limits>

#ifdef WIN32
#undef max
#endif

using namespace BIAS;
using namespace std;

const glfStencilBufferMode glfStencilBufferMode::DEFAULT;

glfStencilBufferMode::glfStencilBufferMode()
{
  enableTest_ = GL_FALSE;
  stencilMask_ = numeric_limits<GLuint>::max();
  stencilFunc_ = GL_ALWAYS;
  referenceValue_ = 0;

  stencilFail_ = stencilPassZFail_ = stencilPassZPass_ = GL_KEEP;
}

void glfStencilBufferMode::SetStencilTest(bool enable)
{
  enableTest_ = enable ? GL_TRUE : GL_FALSE;
}

void glfStencilBufferMode::SetStencilMask(GLuint mask)
{
  stencilMask_ = mask;
}

void glfStencilBufferMode::SetStencilFunc(GLenum comparisonFunc)
{
  stencilFunc_ = comparisonFunc;
}

void glfStencilBufferMode::SetReferenceValue(GLint referenceValue)
{
  referenceValue_ = referenceValue;
}

void glfStencilBufferMode::
SetStencilOperation(GLenum stencilFail, GLenum stencilPassZFail, GLenum stencilPassZPass)
{
  stencilFail_ = stencilFail;
  stencilPassZFail_ = stencilPassZFail;
  stencilPassZPass_ = stencilPassZPass;
}

void glfStencilBufferMode::Bind() const
{
  if (enableTest_) {
    glEnable(GL_STENCIL_TEST);
    glStencilFunc(stencilFunc_, referenceValue_, stencilMask_);
    glStencilOp(stencilFail_, stencilPassZFail_, stencilPassZPass_);
  } else {
    glDisable(GL_STENCIL_TEST);
  }  

  GLF_THROW_ON_OPENGL_ERROR;
}
