/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfStencilBufferMode_hh__
#define __glfStencilBufferMode_hh__

#include "glfCommon.hh"

namespace BIAS {

  /**
   * \brief Defines the usage of the stencil buffer.
   * \ingroup g_openglframework
   * \author bartczak 02/2009
   */
  class BIASOpenGLFramework_EXPORT glfStencilBufferMode {
  public:
    glfStencilBufferMode();
    
    virtual ~glfStencilBufferMode() {}

    /**
     * Sets whether to use stencil buffer tests.
     * \note: This must be enabled, even if you only want
     * to write to the stencil buffer. 
     * \attention: Per default the test is disabled even if you 
     * create an instance and bind it.
     */
    void SetStencilTest(bool enable);

    /**
     * Sets the mask of bits that shall be considered for stencil test from the
     * stencil mask and the stencil buffer.
     * The default is a the whole range of values.
     */
    void SetStencilMask(GLuint mask);

    /**
     * Sets the stencil buffer comparison function.
     * Valid values are: GL_NEVER, GL_LESS, GL_LEQUAL, GL_EQUAL, GL_GEQUAL, GL_GREATER,
     * GL_NOTEQUAL, GL_ALWAYS.
     */
    void SetStencilFunc(GLenum comparisonFunc);

    /**
     * Sets the reference value for the comprison function.
     * Default is 0.
     */
    void SetReferenceValue(GLint referenceValue);

    /** Defines the operation of the stencil test. 
    * For each argument the valid values are: GL_KEEP, GL_ZERO, GL_REPLACE, 
    * GL_INCR, GL_INCR_WRAP, GL_DECR, GL_DECR_WRAP, GL_INVERT.
    * The default is GL_KEEP for all arguments.
    *
    * \param stencilFail what shall to be done when the stencil compare test fails.
    * \param stencilPassZFail what shall to be done when the stencil compare test passes but the z-buffer test fails.
    * \param stencilPassZpass what shall to be done when the stencil compare test passes and the z-buffer test passes.
    * 
    **/
    void SetStencilOperation(GLenum stencilFail, GLenum stencilPassZFail, GLenum stencilPassZPass);

    /**
     * Binds the stencil buffer mode.
     */
    void Bind() const;

    /**
     * The default stencil buffer mode.
     */
    static const glfStencilBufferMode DEFAULT;

  private:
    GLboolean enableTest_;
    GLuint stencilMask_;
    GLenum stencilFunc_;
    GLint referenceValue_;

    GLenum stencilFail_;
    GLenum stencilPassZFail_;
    GLenum stencilPassZPass_;

  };

} // namespace BIAS

#endif // __glfStencilBufferMode_hh__
