/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfVertexArrayObject_hh__
#define __glfVertexArrayObject_hh__

#include "glfCommon.hh"
#include "glfArrayBufferObject.hh"
#include <map>

#ifdef GL_VERSION_3_0

namespace BIAS {

/** \brief 
 * \ingroup g_openglframework
 *  \author bartczak 10/2009
	*/
  class BIASOpenGLFramework_EXPORT glfVertexArrayObject {
  public:
    glfVertexArrayObject();
    ~glfVertexArrayObject();

    /* 
       \attention Call this after context is available and before any other member function ! 
    */
    void Create();
    
    void Bind();
    void BindRevertable();
    void Revert();

    /** Creates or reuses an existing BufferObject to be bound to the respective attribute slot. */
    void UploadData(GLuint slotIndex,                     
                    GLint numValuesPerVertex,
                    float* data, unsigned int numValues,                     
                    GLenum usageHint=GL_STATIC_DRAW);


  private:
    GLuint vertexArrayObjID_;

    //store for each slot which objects is stored there, in case we want to update or similar
    std::map<GLuint, glfArrayBufferObject> internalArrayBufferObjects_;
    
    bool boundRevertably_;
    GLint previousBinding_;
    
    
    glfVertexArrayObject& operator=(const glfVertexArrayObject& b);
  };
}

#endif

#endif
