/* Naive 1D gauss filter implementation without precalculated weights.
 * Alpha texture values of 0 are treated as invalid.
 *
 * \author bartczak 03/2010
 */


uniform sampler2D Img; //input image
uniform float sigma; //standard deviation related to DISTANCES on the NORMALIZED pixel raster

uniform int hw; //half win size
uniform vec2 filterDir;//filtering along this direction! Should connect pixel centers!

//define part of the normalization, which is 1.0/sqrt(2*pi)
//#define normFac0 0.39894228

float Gauss(float distance) {
  float arg = distance/sigma;
  arg *= arg;
  arg/=2.0;
  return exp(-arg);//*normFac0/sigma; //either use normalization or weighted sum!
}

void main() {

  vec3 sum = vec3(0,0,0);
  float sumWeights = 0.0;


  for(int step=-hw; step<=hw; step++) {
    vec2 offset = float(step) * filterDir;
    vec2 access = gl_TexCoord[0].st + offset;
    float valid = texture2D(Img, access).a;
    if(valid == 1.0) {
      float weight = Gauss(length(offset)); 
      sumWeights += weight;
      sum += weight * texture2D(Img, access).rgb ;
    }
  }

  if(sumWeights == 0.0) {
    gl_FragColor = vec4(0,0,0,0);
  } else {
    gl_FragColor.rgb = (sum/sumWeights);
    gl_FragColor.a = 1.0;
  }

}
