/*
 * DisparityInterpolator.cpp
 *
 *  Created on: Jan 29, 2010
 *      Author: africk
 */

#include <OpenGLFramework/SpecializedBatches/DisparityInterpolator.hh>

using namespace std;
using namespace BIAS;

DisparityInterpolator::DisparityInterpolator() {
	dispMapP_ = NULL;
	interp_type_ = BG;

	width_ = 0;
	height_ = 0;

	fragmentShaderCode_ = ""
		"uniform sampler2D dispMap;"
		"uniform float step;"
		"uniform int interpType;"
		""
		"float Interpolate(float left_val, float left_x, "
		"float right_val, float right_x, float x) {"
		"if (left_val == right_val) {"
		"return left_val;"
		"} else {"
		"return ((x - left_x) / (right_x - left_x)) * right_val + "
		"((right_x - x) / (right_x - left_x))*left_val;"
		"}"
		"}"
		""
		"void main () {"
		""
		"vec2 coord = gl_TexCoord[0];"
		"float disp = texture2D(dispMap, coord).r;"
		""
		"if (disp == 0 && step > 0) {"
		""
		"vec2 coord_left = coord;"
		"vec2 coord_right = coord;"
		"float left_val = 0;"
		"float right_val = 0;"
		""
		"while (coord_right.x < 1.0 && right_val == 0) {"
		""
		"coord_right.x = coord_right.x+step;"
		"right_val = texture2D(dispMap, coord_right).r;"
		""
		"}"
		""
		"while (coord_left.x > step && left_val == 0) {"
		""
		"coord_left.x = coord_left.x - step;"
		"left_val = texture2D(dispMap, coord_left).r;"
		""
		"}"
		""
		"if (left_val!=0 && right_val!=0) {"
		"if (interpType == 0) {"
		"disp = Interpolate(left_val,coord_left.x,right_val,coord_right.x,coord.x);"
		"} else {"
		"disp = min(left_val,right_val);"
		"}"
		""
		"} else {"
		"disp = max(left_val,right_val);"
		"}"
		""
		"}"
		"gl_FragColor.r = disp;"
		"}"
		"";
}

DisparityInterpolator::~DisparityInterpolator() {

}

void DisparityInterpolator::SetDisparityMap(glfTexture2D* dispMap) {
	dispMapP_ = dispMap;
}

void DisparityInterpolator::Init(unsigned int width, unsigned int height) {
	width_ = width;
	height_ = height;
	SetViewPort(width_, height_);
	SetDefaultFloatRenderTarget(width_, height_);
	InitBatch_();
	CreateFragmentShaderProgram(fragmentShaderCode_);
}

void DisparityInterpolator::Draw() {

	if (dispMapP_ != NULL) {
		GetRenderTarget()->ClearDepthBuffer(1.0);
		GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
		GetBatch_()->SetTexture(dispMapP_, 0);
		GetShaderProgram_()->SetUniform("dispMap", (int) 0);
		GetShaderProgram_()->SetUniform("step", 1.0f / (float) width_);
		GetShaderProgram_()->SetUniform("interpType", (int) interp_type_);
		GetBatch_()->Draw();

	} else {
		BIASERR("set disparity map first !");
		BIASABORT;
	}
}
