/*
 * GpuImageFilter.hh
 *
 *  Created on: Mar 10, 2010
 *      Author: djung
 */

#ifndef __GPU_IMAGE_FILTER_HH__
#define __GPU_IMAGE_FILTER_HH__

#include <OpenGLFramework/Base/glfTexture2D.hh>
#include <OpenGLFramework/Base/glfBatch.hh>
#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/SpecializedBatches/PerPixelProcessingBase.hh>

#include <bias_config.h>

namespace BIAS {
  /**
   * @class GpuImageFilter
   * @brief basic image filter on the gpu
   *
   * @author djung
   **/
  class BIASOpenGLFramework_EXPORT GpuImageFilter: public PerPixelProcessingBase {
    public:

      enum filtertype {
        none, mean, median, gauss
      };

      GpuImageFilter();
      virtual ~GpuImageFilter();

      void FilterMedian(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad, float threshold,
          bool planar);
      void FilterMean(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad, float threshold);
      void FilterGauss(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad, float threshold);
      void FilterCp(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad, float threshold);

    private:
      void DrawMean_(int rad);
      void DrawMedian_(int rad, float threshold);
      void DrawGauss_(int rad);
      void DrawCp_();
      void Init_(int rad);
      void Filter_(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad, float threshold);
      int Binomial_(int n, int k);
      inline double Exp_(double dist, double sigma) {
        return factor_ / sigma * exp(-dist * dist / 2 / sigma / sigma);
      }
      inline double ExpS1_(double dist) {
        return factor_ * exp(-dist * dist / 2);
      }

      unsigned int width_;
      unsigned int height_;

      glfTexture2D* pTextureA_;
      glfTexture2D* pTextureB_;

      GLenum internalFormat_;

      std::string fragmentShaderCodeMean1_;
      std::string fragmentShaderCodeMean2_;
      std::string fragmentShaderCodeMedian3x3planar_;
      std::string fragmentShaderCodeMedian3x3grey_;
      std::string fragmentShaderCodeMedianPlanar_;
      std::string fragmentShaderCodeMedianGrey_;
      std::string fragmentShaderCodeGauss1_;
      std::string fragmentShaderCodeGauss2_;
      std::string fragmentShaderCodeCp_;
      float* gaussArray_;
      enum filtertype ft_;
      double factor_;
      int exp_;
      bool planar_;
  };
}

#endif /* __GPU_IMAGE_FILTER_HH__ */
