/*
 * MapFusion.cpp
 *
 *  Created on: Feb 4, 2010
 *      Author: africk
 */

#include <OpenGLFramework/SpecializedBatches/MapFusion.hh>
#include <Base/Image/ImageIO.hh>

using namespace std;
using namespace BIAS;

MapFusion::MapFusion() {
	width_ = 0;
	height_ = 0;
	fusion_type_ = PIXEL;
	dispMap1P_ = NULL;
	dispMap2P_ = NULL;
	filtered1P_ = NULL;
	filtered2P_ = NULL;
	initialized_ = false;
	filterWidth_ = 1;
	filterHeight_ = 1;
	fragmentShaderCode_ = ""
		"uniform sampler2D dispMap1;"
		"uniform sampler2D dispMap2;"
		"uniform sampler2D filtered1;"
		"uniform sampler2D filtered2;"
		"uniform int fusionType;"
		"uniform float areaHalfWidth;"
		"uniform float areaHalfSize;"
		"uniform float width;"
		"uniform float height;"
		""
		"void main () {"
		""
		"float disp = 0;"
		"vec2 coord = gl_TexCoord[0];"
		""
		"float color1 = texture2D(dispMap1, coord).r;"
		"float color2 = texture2D(dispMap2, coord).r;"
		""
		""
		"if (color1 > 0.0 && color2 > 0.0) {"
		"if (fusionType == 0) {"
		"disp = (color1 + color2)/2.0;"
		"} else {"
		"float mean1 = texture2D(filtered1, coord).r;"
		"float mean2 = texture2D(filtered2, coord).r;"
		"if (fusionType == 1) {"
		""
		"float dist1 = abs(mean1 - color1);"
		"float dist2 = abs(mean2 - color2);"
		""
		"float diff = abs(dist1-dist2);"
		""
		"if (diff > 10) {"
		""
		"if (dist1 < dist2) {"
		"disp = color1;"
		"} else {"
		"disp = color2;"
		"}"
		""
		"} else {"
		"disp = (color1+color2)/2.0;"
		"}"
		""
		"} else {"
		"if (mean1 > mean2) {"
		"disp = color1;"
		"} else {"
		"disp = color2;"
		"}"
		"}"
		""
		"}"
		"}else {"
		"disp = max(color1,color2);"
		"}"
		""
		""
		""
		"gl_FragColor = disp;"
		""
		"}"
		""
		"";
}

MapFusion::~MapFusion() {
	if (filtered1P_ == NULL) {
		delete filtered1P_;
	}
	if (filtered2P_ == NULL) {
		delete filtered2P_;
	}
}

void MapFusion::SetDisparityMaps(glfTexture2D* dispMap1, glfTexture2D* dispMap2) {
	dispMap1P_ = dispMap1;
	dispMap2P_ = dispMap2;

	if (dispMap1P_ != NULL && dispMap2P_ != NULL) {
		BIASASSERT((int)width_ == dispMap1P_->GetWidth() && (int) height_ == dispMap2P_->GetHeight());
	}
}

void MapFusion::SetFusionType(FUSION_TYPE fusion_type) {
	fusion_type_ = fusion_type;
}

void MapFusion::Init(unsigned int width, unsigned int height) {
	width_ = width;
	height_ = height;
	SetViewPort(width_, height_);
	SetDefaultFloatRenderTarget(width_, height_);
	InitBatch_();
	CreateFragmentShaderProgram(fragmentShaderCode_);
	CreateFilteredTextures_();
	boxFilter_.Init();
	initialized_ = true;
}

void MapFusion::Draw() {
	if (initialized_) {
		if (dispMap1P_ != NULL && dispMap2P_ != NULL) {
			//			Image<float> result;
			if (fusion_type_ == AREA || fusion_type_ == FG) {
				boxFilter_.SetFilterSize(filterWidth_, filterHeight_);
				boxFilter_.SetSourceAndDest(dispMap1P_, filtered1P_);
				boxFilter_.Draw();
				boxFilter_.SetSourceAndDest(dispMap2P_, filtered2P_);
				boxFilter_.Draw();
			}

			GetRenderTarget()->ClearDepthBuffer(1.0);
			GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
			GetBatch_()->SetTexture(dispMap1P_, (int) 0);
			GetBatch_()->SetTexture(dispMap2P_, (int) 1);
			GetBatch_()->SetTexture(filtered1P_, (int) 2);
			GetBatch_()->SetTexture(filtered2P_, (int) 3);
			GetShaderProgram_()->SetUniform("dispMap1", (int) 0);
			GetShaderProgram_()->SetUniform("dispMap2", (int) 1);
			GetShaderProgram_()->SetUniform("filtered1", (int) 2);
			GetShaderProgram_()->SetUniform("filtered2", (int) 3);
			GetShaderProgram_()->SetUniform("fusionType", (int) fusion_type_);
			GetBatch_()->Draw();

		} else {
			BIASERR("set disparity map first !");
			BIASABORT;
		}
	} else {
		BIASERR("call init first !");
		BIASABORT;
	}
}
