/*
 * MapFusion.hh
 *
 *  Created on: Feb 4, 2010
 *      Author: africk
 */

#ifndef __MAP_FUSION_HH__
#define __MAP_FUSION_HH__

#include <OpenGLFramework/Base/glfTexture2D.hh>
#include <OpenGLFramework/Base/glfBatch.hh>
#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/SpecializedBatches/PerPixelProcessingBase.hh>
#include <OpenGLFramework/SpecializedBatches/SeparabelBoxFilter.hh>

#include <bias_config.h>

namespace BIAS {

class BIASOpenGLFramework_EXPORT MapFusion: public PerPixelProcessingBase {
public:

	enum FUSION_TYPE {
		PIXEL = 0, AREA = 1, FG = 2
	};

	MapFusion();
	virtual ~MapFusion();

	void SetDisparityMaps(glfTexture2D* dispMap1, glfTexture2D* dispMap2);

	void SetFusionType(FUSION_TYPE fusion_type);
	void SetFilterSize(unsigned int width, unsigned int height) {
		filterWidth_ = width;
		filterHeight_ = height;
	}

	void Init(unsigned int width, unsigned int height);

	void Draw();

private:

	bool initialized_;

	unsigned int width_, height_, filterWidth_, filterHeight_;

	FUSION_TYPE fusion_type_;

	glfTexture2D* dispMap1P_;
	glfTexture2D* dispMap2P_;

	glfTexture2D* filtered1P_;
	glfTexture2D* filtered2P_;

	std::string fragmentShaderCode_;
	SeparabelBoxFilter boxFilter_;

	void CreateFilteredTextures_() {
		filtered1P_ = new glfTexture2D();
		filtered1P_->Create();
		filtered1P_->SetMagFilter(GL_NEAREST);
		filtered1P_->SetMinFilter(GL_NEAREST);
		filtered1P_->SetWrapS(GL_CLAMP);
		filtered1P_->SetWrapT(GL_CLAMP);
		filtered1P_->Allocate(width_, height_, GL_LUMINANCE32F_ARB);

		filtered2P_ = new glfTexture2D();
		filtered2P_->Create();
		filtered2P_->SetMagFilter(GL_NEAREST);
		filtered2P_->SetMinFilter(GL_NEAREST);
		filtered2P_->SetWrapS(GL_CLAMP);
		filtered2P_->SetWrapT(GL_CLAMP);
		filtered2P_->Allocate(width_, height_, GL_LUMINANCE32F_ARB);
	}
};

}
#endif /* MAPFUSION_HH_ */
