/*
 * SeparableBilateralFilter.cpp
 *
 *  Created on: Feb 15, 2010
 *      Author: africk
 */

#include <OpenGLFramework/SpecializedBatches/SeparableBilateralFilter.hh>
#include <Utils/IOUtils.hh>

using namespace std;
using namespace BIAS;

SeparableBilateralFilter::SeparableBilateralFilter() :
	pixelOffset_(2) {
	destP_ = NULL;
	sourceP_ = NULL;
	filteredXP_ = NULL;
	colorP_ = NULL;
	renderTargetP_ = NULL;
	colorScale_ = 10;
	spatialScale_ = 10;

	inited_ = false;
	width_ = 0;
	height_ = 0;
	filterHalfSize_ = 3;
}

SeparableBilateralFilter::~SeparableBilateralFilter() {
	if (filteredXP_ != NULL) {
		delete filteredXP_;
	}

	if (renderTargetP_ != NULL) {
		delete renderTargetP_;
	}
}

void SeparableBilateralFilter::Init() {
	InitBatch_();
	GetDepthBufferMode_()->SetDepthTest(false);

	string shader_path;
	IOUtils::FindFileEnv("SHADERSRC", "separabelBilateralFilter.glsl",
			shader_path);

	CreateFragmentShaderProgramFromFile(shader_path);

	inited_ = true;
}

void SeparableBilateralFilter::SetColorAndSourceAndDest(glfTexture2D* color,
		glfTexture2D* source, glfTexture2D* dest) {
	BIASASSERT(destP_ != NULL && destP_ != NULL && color != NULL);
	sourceP_ = source;
	destP_ = dest;
	colorP_ = color;

	unsigned int oldWidth = width_;
	unsigned int oldHeight = height_;

	width_ = sourceP_->GetWidth();
	height_ = sourceP_->GetHeight();

	BIASASSERT(width_ > 0 && height_ > 0); BIASASSERT((int) width_ == destP_->GetWidth() && (int) height_
			== destP_->GetHeight() && (int) width_ == colorP_->GetWidth()
			&& (int) height_ == colorP_->GetHeight());
	if (width_ != oldWidth || height_ != oldHeight) {
		SetViewPort(width_, height_);
		CreateFilteredXTexture_(width_, height_);
		CreateRenderTarget_(width_, height_);
		renderTargetP_->AttachTexture(*filteredXP_,
							GL_COLOR_ATTACHMENT0_EXT);
		renderTargetP_->CheckComplete();
		SetRenderTarget(renderTargetP_);
		pixelOffset_[0] = 1.0f / (float) width_;
		pixelOffset_[1] = 1.0f / (float) height_;
		GetShaderProgram_()->SetUniform("pixelOffset", pixelOffset_);
	}
}

void SeparableBilateralFilter::Draw() {
	if (inited_) {
		if (colorP_ != NULL && sourceP_ != NULL && destP_ != NULL) {
			GetBatch_()->SetTexture(colorP_, 0);
			GetBatch_()->SetTexture(sourceP_, 1);
			GetShaderProgram_()->SetUniform("texture", (int) 0);
			GetShaderProgram_()->SetUniform("ocosts", (int) 1);
			GetShaderProgram_()->SetUniform("halfSize", (int) filterHalfSize_);
			GetShaderProgram_()->SetUniform("colorScale", colorScale_);
			GetShaderProgram_()->SetUniform("spatialScale", spatialScale_);
			GetShaderProgram_()->SetUniform("flag", (int) 0);

			renderTargetP_->AttachTexture(*filteredXP_,
					GL_COLOR_ATTACHMENT0_EXT);
			GetBatch_()->Draw();

			//------------
			GetBatch_()->SetTexture(colorP_, 0);
			GetBatch_()->SetTexture(filteredXP_, 1);
			GetShaderProgram_()->SetUniform("texture", (int) 0);
			GetShaderProgram_()->SetUniform("ocosts", (int) 1);
			GetShaderProgram_()->SetUniform("flag", (int) 1);
			renderTargetP_->AttachTexture(*destP_, GL_COLOR_ATTACHMENT0_EXT);
			GetBatch_()->Draw();

		} else {
			BIASERR("call SetColorAndSourceAndDest first !");
			BIASABORT;
		}
	} else {
		BIASERR("call init first !");
		BIASABORT;
	}
}
