/*
 * WoWDisplayShader.cpp
 *
 *  Created on: Feb 9, 2010
 *      Author: africk
 */

#include <OpenGLFramework/SpecializedBatches/WoWDisplayShader.hh>

using namespace std;
using namespace BIAS;

WoWDisplayShader::WoWDisplayShader() {
	dispImageP_ = NULL;
	colorImageP_ = NULL;
	width_ = 960;
	height_ = 540;
	wowFactor_ = 0x40;
	wowOffset_ = 0x80;
	flip_ = 0;
	scale_ = 1;

	fragmentShaderCode_ = ""
		"uniform sampler2D colorImage;"
		"uniform sampler2D dispImage;"
		"uniform int header [256];"
		"uniform float w;"
		"uniform float h;"
		"uniform float scale;"
		"uniform int flip;" // 0 no , 1 yes
			""
			""
			"void main() {"
			""
			"vec2 coord = gl_TexCoord[0];"
			""
			"vec3 color = vec3(0.0,0.0,0.0);"
			""
			"if (mod(coord.y*h,2.0) >= 0.5) {"
			""
			"if ((coord.x) < 0.5) {"
			"vec2 new_coord;"
			"new_coord[0]=coord[0]*2;\n"
			"new_coord[1]=abs(flip - coord[1]);\n"
			"color = texture2D(colorImage, new_coord);\n"
			"} else {"
			"vec2 new_coord;"
			"new_coord[0]=(coord[0] - 0.5)*2;\n"
			"new_coord[1]=coord[1];\n"
			"color = (((max(0.0,min(255.0,texture2D(dispImage, new_coord))) - 128.0) * scale) + 128.0)/255.0;\n"
			"}"
			""
			""
			"}"
			""
			"float x = (coord.x*w);\n"
			"float y = (coord.y*h);\n"
			""
			"if (y >= h-1 && x < 256) {\n"
			"color[2]=header[int(x)]/255.0;\n"
			"}\n"
			""
			"gl_FragColor.rgb = color;"
			""
			"}";
}

WoWDisplayShader::~WoWDisplayShader() {
	// TODO Auto-generated destructor stub
}

void WoWDisplayShader::SetDisparityImage(glfTexture2D* dispImage) {
	dispImageP_ = dispImage;
}
void WoWDisplayShader::SetColorImage(glfTexture2D* colorImage) {
	colorImageP_ = colorImage;
}

void WoWDisplayShader::Init() {
	InitBatch_();
	CreateFragmentShaderProgram(fragmentShaderCode_);
	CreateHeader_(fullheader_, wowFactor_, wowOffset_);
}

void WoWDisplayShader::Draw() {
	if (dispImageP_ != NULL && colorImageP_ != NULL) {
		//		if (GetRenderTarget() == NULL) {
		//			glClearColor(0, 0, 0, 0);
		//			glClear(GL_COLOR_BUFFER_BIT);
		//		} else {
		//			GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
		//			GetRenderTarget()->ClearDepthBuffer(1.0);
		//		}
		GetDepthBufferMode_()->SetDepthTest(false);
		width_ = GetViewport_()->GetWidth();
		height_ = GetViewport_()->GetHeight();
		GetShaderProgram_()->SetUniform("flip", flip_);
		GetShaderProgram_()->SetUniform("scale", scale_);
		GetShaderProgram_()->SetUniform("w", (float) width_);
		GetShaderProgram_()->SetUniform("h", (float) height_);
		GetShaderProgram_()->SetUniform("header", 256, fullheader_);
		GetShaderProgram_()->SetUniform("dispImage", (int) 0);
		GetShaderProgram_()->SetUniform("colorImage", (int) 1);

		GetBatch_()->SetTexture(dispImageP_, 0);
		GetBatch_()->SetTexture(colorImageP_, 1);
		GetBatch_()->Draw();
	}

}

//#####################################################

void WoWDisplayShader::CreateHeader_(int* fullheader, int wowFactor,
		int wowOffset) {

	int wow_content = 0x02;
	int wow_use_factor = 0x80;
	int wow_use_offset = 0x40;

	unsigned char header[] = { 0xF1, wow_content, wowFactor, wowOffset,
			wow_use_factor + wow_use_offset, 0x00, 0x00, 0x00, 0x00, 0x00 };
	{
		unsigned long _register = 0;
		for (int i = 0; i < 10; ++i) {
			unsigned char mask = 0x80;
			unsigned char byte = header[i];
			for (int j = 0; j < 8; ++j) {
				bool topBit = (_register & 0x80000000) != 0;
				_register <<= 1;
				_register ^= ((byte & mask) != 0 ? 0x1 : 0x0);
				if (topBit) {
					_register ^= 0x04c11db7;
				}
				mask >>= 1;
			}
		}
		unsigned char *p = (unsigned char*) &_register;
		for (size_t i = 0; i < 4; ++i) {
			header[i + 6] = p[3 - i];
		}
	}

	{
		for (int x = 0; x < 256; ++x) {
			fullheader[x] = 0;
		}
		for (int x = 0; x <= 9; ++x) {
			for (int y = 7; y >= 0; --y) {
				int i = 2 * (7 - y) + 16 * x;
				fullheader[i] = (((1 << (y)) & (header[x])) << (7 - (y)));
			}
		}
	}
}
