/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FramebufferSetup_hh__
#define __FramebufferSetup_hh__

#include <bias_config.h>

#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/Base/glfTexture2D.hh>

#include <vector>


namespace BIAS {

  /*
  * Class shall support the framebuffer object organization.
  * It is usefull in a szenario where certain texture to fbo bindings
  * are (frequently) reoccuring.
  * \author bartczak 09/2008
  */
  class BIASOpenGLFramework_EXPORT FramebufferSetup {
  public:
    FramebufferSetup();

    void LinkFBO(BIAS::glfFramebufferObject* fbo);
    void ClearColorAttachments();
    void SetColorAttachment(BIAS::glfTexture* texture, int AttachmentPoint=0);
    /**
     * replaces all previous color attachments and through given
     */
    void SetColorAttachments(const std::vector<BIAS::glfTexture*>& textures);
    void SetDepthAttachment(BIAS::glfTexture* texture);
    void EnableColorClearance(float red = 0.0f, float green = 0.0f, float blue = 0.0f, float alpha = 0.0f);
    void EnableDepthClearance(float depth = 1.0f);
    void DisableColorClearance();
    void DisableDepthClearance();

    void Execute();

    /** Returns texture attached to AttachmentPoint if it is a glfTexture2D otherwise NULL.
    * Also returns NULL if not texture is attached to this attachmentpoint.
    **/
    glfTexture2D* GetColorAttachment2D(int AttachmentPoint);

  private:
    BIAS::glfFramebufferObject* fbo_;

    bool clearColorBuffer_;
    float clearColor_[4];

    bool clearDepthBuffer_;
    float clearDepth_;

    std::vector<BIAS::glfTexture*> colorAttachments_;
    BIAS::glfTexture* depthAttachment_;

  };

}


#endif
