/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <OpenGLFramework/Utils/Primitives.hh>
#include <OpenGLFramework/Base/glfVertexFormat.hh>

using namespace BIAS;
using namespace std;

#define VERTEX2D 2
#define VERTEX3D 3
#define VERTEX4D 4

#define TEXTURE2D 2

void Primitives::
AddRelativeQuadPatches(const unsigned int referenceWidth, 
                       const unsigned int referenceHeight,
                       const std::vector<unsigned int>& posX,
                       const std::vector<unsigned int>& posY,
                       const unsigned int hw,                        
                       BIAS::glfVertexBuffer& vb, BIAS::glfElementBuffer& eb,
                       const bool flip) 
{


  unsigned int numPatches = posX.size();
  BIASASSERT(numPatches == posY.size());

  glfVertexFormat vertexFrmt;
  vertexFrmt.AddAttribute(glfVertexFormat::ATTRIB_POSITION, GL_FLOAT, VERTEX2D);
  vertexFrmt.AddAttribute(glfVertexFormat::ATTRIB_TEXCOORD, 0, GL_FLOAT, TEXTURE2D);
  vertexFrmt.AddAttribute(glfVertexFormat::ATTRIB_TEXCOORD, 1, GL_FLOAT, TEXTURE2D);

  unsigned int numVertices = 4 * numPatches;
  float* vertexData = new float[numVertices * (VERTEX2D + 2* TEXTURE2D) ]; // four vertices a two pos. coords and two times two texture coords

  


  float W = static_cast<float>(referenceWidth);
  float H = static_cast<float>(referenceHeight);

  cout<<"W x H = "<<W<<"x"<<H<<endl;

  //here is no flip since we upload a flipped image, and render flipped then ?!a  
  float scaleToSpace[2] = { 2.0/W, 2.0/H}; //mx, my
  float shiftToSpace[2] = {1.0/W - 1.0, 1.0/H-1.0};//px, py
  cout<<"scaleToSpace = "<<scaleToSpace[0]<<", "<<scaleToSpace[1]<<endl;
  cout<<"shiftToSpace = "<<shiftToSpace[0]<<", "<<shiftToSpace[1]<<endl;

  float scaleToTex[2] = {1.0/W, -1.0/H}; //ms, mt
  float shiftToTex[2] = {1.0/(2.0*W), 1.0 - 1.0/(2.0*H)};//ps, pt

  float scaleToTexFlipped[2] = {1.0/W, 1.0/H};
  float shiftToTexFlipped[2] = {1.0/(2.0*W), 1.0/(2.0*H)};


  unsigned int offset=0;
  for(unsigned int p=0; p<numPatches; p++) {
    int ul[2] = {static_cast<int>(posX[p])-static_cast<int>(hw),
                 static_cast<int>(posY[p])-static_cast<int>(hw)};
    int lr[2] = {posX[p]+hw, posY[p]+hw};
    cout<<lr[0]<<", "<<lr[1]<<endl;
    cout<<ul[0]<<", "<<ul[1]<<endl;
    AddRelativeQuadPatch_(scaleToSpace, shiftToSpace, 
                          scaleToTex, shiftToTex, 
                          scaleToTexFlipped, shiftToTexFlipped,
                          ul, lr, 
                          vertexData, offset, !flip);
  }



  vb.Create(numVertices, vertexFrmt, vertexData);  
  delete [] vertexData;

  GLuint* indices = new GLuint[numVertices];
  for(unsigned int i=0; i<numVertices; i++) indices[i] = i;
  eb.Create(numVertices, GL_UNSIGNED_INT, GL_QUADS, indices); 
  delete [] indices; 
  
}


void Primitives::
AddRelativeQuadPatch_(const float scaleToSpace[2],
                      const float shiftToSpace[2],
                      const float scaleToTex[2],
                      const float shiftToTex[2],
                      const float scaleToTexFlipped[2],
                      const float shiftToTexFlipped[2],
                      const int ul[2],
                      const int lr[2],//included
                      float* supplementedArray,
                      unsigned int& offset,//incremented while adding
                      const bool flip) 
{

  float spaceUL[2];
  float spaceLR[2];
  float texUL[2];
  float texLR[2];
  float texFlippedUL[2];
  float texFlippedLR[2];
  float tmpTexFlippedUL[2];
  float tmpTexFlippedLR[2];


//   cout<<"scaleToSpace = "<<scaleToSpace[0]<<", "<<scaleToSpace[1]<<endl;
//   cout<<"shiftToSpace = "<<shiftToSpace[0]<<", "<<shiftToSpace[1]<<endl;

  for(unsigned int i=0; i<2; i++) {
    spaceUL[i] = scaleToSpace[i]*(static_cast<float>(ul[i])-0.5)+shiftToSpace[i];    
    spaceLR[i] = scaleToSpace[i]*(static_cast<float>(lr[i])+0.5)+shiftToSpace[i];
    texUL[i] = scaleToTex[i]*(static_cast<float>(ul[i])-0.5)+shiftToTex[i];
    texLR[i] = scaleToTex[i]*(static_cast<float>(lr[i])+0.5)+shiftToTex[i];
    tmpTexFlippedUL[i] = scaleToTexFlipped[i]*(static_cast<float>(ul[i])-0.5)+shiftToTexFlipped[i];
    tmpTexFlippedLR[i] = scaleToTexFlipped[i]*(static_cast<float>(lr[i])+0.5)+shiftToTexFlipped[i];
    if(!flip) {
      texFlippedUL[i] = tmpTexFlippedUL[i];
      texFlippedLR[i] = tmpTexFlippedLR[i];
    } else {
      texFlippedUL[i] = texUL[i];
      texFlippedLR[i] = texLR[i];
      texUL[i] = tmpTexFlippedUL[i];
      texLR[i] = tmpTexFlippedLR[i];
    }
  }

  /* GL_QUADS
   *   3--2
   *   |  |
   *   0--1
   */
  //0
  supplementedArray[offset++] = spaceUL[0];
  supplementedArray[offset++] = spaceLR[1];
//   cout<<spaceUL[0]<<","<<spaceLR[1]<<endl;
  supplementedArray[offset++] = texUL[0];
  supplementedArray[offset++] = texLR[1];
  supplementedArray[offset++] = texFlippedUL[0];
  supplementedArray[offset++] = texFlippedLR[1];
  //1 is LR
  supplementedArray[offset++] = spaceLR[0];
  supplementedArray[offset++] = spaceLR[1];
//   cout<<spaceLR[0]<<","<<spaceLR[1]<<endl;
  supplementedArray[offset++] = texLR[0];
  supplementedArray[offset++] = texLR[1];
  supplementedArray[offset++] = texFlippedLR[0];
  supplementedArray[offset++] = texFlippedLR[1];
  //2 
  supplementedArray[offset++] = spaceLR[0];
  supplementedArray[offset++] = spaceUL[1];
//   cout<<spaceLR[0]<<","<<spaceUL[1]<<endl;
  supplementedArray[offset++] = texLR[0];
  supplementedArray[offset++] = texUL[1];
  supplementedArray[offset++] = texFlippedLR[0];
  supplementedArray[offset++] = texFlippedUL[1];
  //23 is UL
  supplementedArray[offset++] = spaceUL[0];
  supplementedArray[offset++] = spaceUL[1];
//   cout<<spaceUL[0]<<","<<spaceUL[1]<<endl;
  supplementedArray[offset++] = texUL[0];
  supplementedArray[offset++] = texUL[1];
  supplementedArray[offset++] = texFlippedUL[0];
  supplementedArray[offset++] = texFlippedUL[1];
}


void Primitives::
PlainQuad2DWithTexture2D(glfVertexBuffer& vb, glfElementBuffer& eb, bool flip)
{
  glfVertexFormat vertexFrmt;
  vertexFrmt.AddAttribute(glfVertexFormat::ATTRIB_POSITION, GL_FLOAT, VERTEX2D);
  vertexFrmt.AddAttribute(glfVertexFormat::ATTRIB_TEXCOORD, 0, GL_FLOAT, TEXTURE2D);
  vertexFrmt.AddAttribute(glfVertexFormat::ATTRIB_TEXCOORD, 1, GL_FLOAT, TEXTURE2D);

  float* vertexData = new float[4 * (VERTEX2D + 2* TEXTURE2D) ]; // four vertices a two pos. coords and two times two texture coords
  AddPlain2DQuadVertices(vertexData, 0, 4);
  AddPlain2DTextureCoordinates(vertexData, 2, 4, flip);
  AddPlain2DTextureCoordinates(vertexData, 4, 4, !flip);

  vb.Create(4, vertexFrmt, vertexData);  
  delete [] vertexData;

  GLuint* indices = new GLuint[4];
  for(unsigned int i=0; i<4; i++) indices[i] = i;
  eb.Create(4, GL_UNSIGNED_INT, GL_QUADS, indices); 
  delete [] indices; 
  
}


void Primitives::
AddPlain2DQuadVertices(float* supplementedArray, unsigned int offset, unsigned int stride)
{
  /* GL_QUADS
   *   3--2
   *   |  |
   *   0--1
   */

  unsigned int i=offset;
  supplementedArray[i++] = -1.0;
  supplementedArray[i++] = -1.0; //vertex
  i+=stride;
     
  supplementedArray[i++] = 1.0;
  supplementedArray[i++] = -1.0; //vertex
  i+=stride;
  
  supplementedArray[i++] = 1.0;
  supplementedArray[i++] = 1.0; //vertex
  i+=stride;
  
  supplementedArray[i++] = -1.0;
  supplementedArray[i++] = 1.0;  //vertex
  
}


void Primitives::
AddPlain2DTextureCoordinates(float* supplementedArray, unsigned int offset, unsigned int stride, bool flip)
{
/* GL_QUADS
   *   3--2
   *   |  |
   *   0--1
   */

  unsigned int i=offset;
  supplementedArray[i++] = 0.0;
    supplementedArray[i++] = (!flip)? 0.0 : 1.0; //tex
  i+=stride;

  supplementedArray[i++] = 1.0;
  supplementedArray[i++] = (!flip)? 0.0 : 1.0; //tex
  i+=stride;

  supplementedArray[i++] = 1.0;
  supplementedArray[i++] = (!flip)? 1.0 : 0.0; //tex
  i+=stride;

  supplementedArray[i++] = 0.0;
  supplementedArray[i++] = (!flip)? 1.0 : 0.0; //tex
  
   
}

void Primitives::
LocalPerspectiveQuad(const BIAS::ProjectionParametersPerspective& ppp, 
                     glfVertexBuffer& vb, glfElementBuffer& eb)
{

  unsigned int width, height;
  ppp.GetImageSize(width, height);

  glfVertexFormat vertexFormat;
  vertexFormat.AddAttribute(glfVertexFormat::ATTRIB_POSITION, GL_FLOAT, VERTEX4D);

  float* vertexData = new float [4 * VERTEX4D];//4 vertices a 4 components

  HomgPoint2D imgPoint;
  enum {ul =0, ll, lr, ur};//names relate to the 3D position in cam coordinates.
  Vector3<double> ray[4];

  imgPoint.Set(-0.5, -0.5);
  ray[ul] = ppp.UnProjectLocal(imgPoint);

  imgPoint.Set(-0.5, float(height)-0.5);
  ray[ll] = ppp.UnProjectLocal(imgPoint);

  imgPoint.Set(float(width)-0.5, float(height)-0.5);
  ray[lr] = ppp.UnProjectLocal(imgPoint);

  imgPoint.Set(float(width)-0.5, -0.5);
  ray[ur] = ppp.UnProjectLocal(imgPoint);

  unsigned int count = 0;
  Vector4<double> in, out;
  for(unsigned int i =0; i< 4; i++) {  
    /*cout<<"\n\n v"<<i<<" = "<<ray[i]<<endl;*/
    vertexData[count++] = 
      static_cast<float>(ray[i][0]);
    vertexData[count++] = 
      static_cast<float>(ray[i][1]);
    vertexData[count++] = 
      static_cast<float>(ray[i][2]);
    vertexData[count++] = 1.0;

  }

  vb.Create(4, vertexFormat, vertexData); 
  delete [] vertexData;

  GLuint* indices = new GLuint[4];
  for(unsigned int i=0; i<4; i++) indices[i] = i;
  eb.Create(4, GL_UNSIGNED_INT, GL_QUADS, indices); 
  delete [] indices; 


}

void Primitives::
LocalPerspectivePatches(const BIAS::ProjectionParametersPerspective& ppp, 
                        const std::vector<unsigned int>& posX,
                        const std::vector<unsigned int>& posY,
                        const unsigned int hw,
                        glfVertexBuffer& vb, glfElementBuffer& eb)
{

  unsigned int numPatches = posX.size();
  BIASASSERT(numPatches == posY.size());
  

  unsigned int width, height;
  ppp.GetImageSize(width, height);

  glfVertexFormat vertexFormat;
  vertexFormat.AddAttribute(glfVertexFormat::ATTRIB_POSITION, GL_FLOAT, VERTEX4D);

  float* vertexData = new float [numPatches * 4 * VERTEX4D];//4 vertices a 4 components for each patch.

  HomgPoint2D imgPoint;
  enum {ul =0, ll, lr, ur};//names relate to the 3D position in cam coordinates.
  Vector3<double> ray[4];

  unsigned int count = 0;
  for(unsigned int p=0; p<numPatches; p++) {

    //find the four corners of the patch 
    
    imgPoint.Set(posX[p]-hw-0.5, posY[p]-hw-0.5);
    ray[ul] = ppp.UnProjectLocal(imgPoint);

    imgPoint.Set(posX[p]-hw-0.5, posY[p]+hw+0.5);
    ray[ll] = ppp.UnProjectLocal(imgPoint);

    imgPoint.Set(posX[p]+hw+0.5, posY[p]+hw+0.5);
    ray[lr] = ppp.UnProjectLocal(imgPoint);

    imgPoint.Set(posX[p]+hw+0.5, posY[p]-hw-0.5);
    ray[ur] = ppp.UnProjectLocal(imgPoint);


    Vector4<double> in, out;
    for(unsigned int i =0; i< 4; i++) {  
      /*cout<<"\n\n v"<<i<<" = "<<ray[i]<<endl;*/
      vertexData[count++] = 
        static_cast<float>(ray[i][0]);
      vertexData[count++] = 
        static_cast<float>(ray[i][1]);
      vertexData[count++] = 
        static_cast<float>(ray[i][2]);
      vertexData[count++] = 1.0;

    }
  }
  vb.Create(4 * numPatches, vertexFormat, vertexData); 
  delete [] vertexData;

  GLuint* indices = new GLuint[4 * numPatches];
  for(unsigned int i=0; i< 4 * numPatches; i++) indices[i] = i;
  eb.Create(4*numPatches, GL_UNSIGNED_INT, GL_QUADS, indices); 
  delete [] indices; 


}


void Primitives::
LocalNormalizedVertexCloud(const BIAS::ProjectionParametersPerspective& ppp, 
                           glfVertexBuffer& vb, 
                           bool normalizeZ, bool invertS)
{
  unsigned int width, height;
  ppp.GetImageSize(width, height);

  glfVertexFormat vertexFormat;
  vertexFormat.AddAttribute(glfVertexFormat::ATTRIB_POSITION, GL_FLOAT, VERTEX3D);
  vertexFormat.AddAttribute(glfVertexFormat::ATTRIB_TEXCOORD, 0, GL_FLOAT, TEXTURE2D);

  float* vertexData = new float[width*height * (VERTEX3D + TEXTURE2D)];

  unsigned int pos = 0;
  for(unsigned int y=0; y<height; y++) {
    for(unsigned int x=0; x<width; x++) {
      //calculate vertex
      Vector3<double> v = ppp.UnProjectLocal(HomgPoint2D(x,y));

      if(normalizeZ)
        v /= v[2];

      vertexData[pos++] = static_cast<float>(v[0]);
      vertexData[pos++] = static_cast<float>(v[1]);
      vertexData[pos++] = static_cast<float>(v[2]);

      //calculate texture coord
      vertexData[pos++] = (static_cast<float>(x)+0.5)/static_cast<float>(width);
      if(invertS) {
        vertexData[pos++] = 1.0 - (static_cast<float>(y)+0.5)/static_cast<float>(height);
      } else {
        vertexData[pos++] = (static_cast<float>(y)+0.5)/static_cast<float>(height);
      }
    }
  }


  vb.Create(width*height, vertexFormat, vertexData); 
  delete [] vertexData;
}
                           


void Primitives::
LocalNormalizedPointCloud(const BIAS::ProjectionParametersPerspective& ppp, 
      glfVertexBuffer& vb, glfElementBuffer& eb, bool normalizeZ, bool invertS)
{

  unsigned int width, height;
  ppp.GetImageSize(width, height);

  LocalNormalizedVertexCloud(ppp, vb, normalizeZ, invertS);

  GLuint* indices = new GLuint[width*height];
  for(unsigned int i=0; i<width*height; i++) indices[i] = i;
  eb.Create(width*height, GL_UNSIGNED_INT, GL_POINTS, indices); 
  delete [] indices;


}

void Primitives::
LocalNormalizedTriangleStrip(const BIAS::ProjectionParametersPerspective& pppSource,
                             glfVertexBuffer& vb, glfElementBuffer& eb,
                             bool normalizeZ) 
{
  unsigned int width, height;
  pppSource.GetImageSize( width, height);

  LocalNormalizedVertexCloud(pppSource, vb, normalizeZ, true);

  //************** INDICES ***********************
  //creating coordinates for triangle strips!
  //each image rows, except for the first and last, are contained twice
  //a triangle strip consists of 2*width vertices
  // and height-1 strips have to be rendered
  GLuint* triangleIndices = 
    new GLuint [width*(height-1)*2];
  unsigned int numTriangleIndex = 0;
  //
  for(unsigned int y=0; y<height-1; y++) {
    for(unsigned int x=0; x<width; x++) {
      /*         v1
      *      i0--- i2
      *       |   / |
      *       |  /  |
      *       i1    i3
      */
      triangleIndices[numTriangleIndex++] = y*width+x;//up
      triangleIndices[numTriangleIndex++] = (y+1)*width+x;//below
    }
  }
  eb.Create(width*(height-1)*2, 
    GL_UNSIGNED_INT, 
    GL_TRIANGLE_STRIP, 
    triangleIndices);
 
  delete [] triangleIndices;
}


void Primitives::
LocalNormalizedTriangleMesh(const BIAS::ProjectionParametersPerspective& pppSource,
                             glfVertexBuffer& vb, glfElementBuffer& eb,
                             bool normalizeZ, bool invertS)
{
  unsigned int width, height;
  pppSource.GetImageSize( width, height);

  LocalNormalizedVertexCloud(pppSource, vb, normalizeZ, invertS);

  //************** INDICES ***********************
  /*      Type A     Type B
  *      i0--- i2      i0
  *       |   /       / |
  *       |  /       /  |
  *       i1       i1-- i2
  */
  /*  
   * Assume that each image points belongs to 6 triangles : width*height*6
   * two corners belong to only one triangle              : - 2*6+2   = -2*5 = -10
   * two corners belong to two triangles                  : - 2*6+2*2 = -2*4 = -8
   * edges (without corners) belong to 3 triangles        : - 2*(width-2+height-2)*6 + 2*(width-2+height-2)*3
   *                                                          = -2*(width-2+height-2)*3
   *                                                          = -(width+height-4)*6 = -(width+height)*6 + 24
   * sum = (width*height)*6 - (width + height)*6 + 6 = (width*height - width - height + 1)*6
   * 
   */
  const int numIndices = (width*height - width - height + 1)*6;
  GLuint* triangleIndices = new GLuint [numIndices];

  unsigned int cIdx = 0;//current element array position
  for(unsigned int y=0; y<height-1; y++) {
    for(unsigned int x=0; x<width-1; x++) {
      //build Type A
      triangleIndices[cIdx++] = y*width+x;    // i0
      triangleIndices[cIdx++] =(y+1)*width+x; // i1
      triangleIndices[cIdx++] = y * width+x+1;// i2

      //build TypeB
      triangleIndices[cIdx++] = y*width+ x+1;   // i0
      triangleIndices[cIdx++] =(y+1)*width+ x;  // i1
      triangleIndices[cIdx++] =(y+1)*width+ x+1;// i2

    }
  }
  eb.Create(numIndices, 
    GL_UNSIGNED_INT, 
    GL_TRIANGLES, 
    triangleIndices);
 
  delete [] triangleIndices;
}

#ifdef GL_VERSION_3_0
void Primitives::
CalculateLocalRayField(const ProjectionParametersBase& ppb, glfVertexArrayObject& vao, 
                       GLsizei& count,
                       GLint VertexCoordSlot, GLint TexCoordSlot, bool normalizeZ)
{
#ifdef BIAS_DEBUG
  if(VertexCoordSlot != -1 || TexCoordSlot != -1) {
    BIASASSERT(VertexCoordSlot!=TexCoordSlot)
  }
#endif
  count = 0;
  if(VertexCoordSlot == -1 && TexCoordSlot == -1) {
    BIASWARN("This function call has no effect!");
    return;
  }
  
  unsigned int width, height;
  ppb.GetImageSize(width, height);
  float* vertexData = (VertexCoordSlot != -1) ? new float[width*height*3] : NULL;
  float* texCoordData = (TexCoordSlot != -1) ? new float[width*height*3] : NULL; // there will be two sets of texture coordinates which share the s component
   

  unsigned int posVertexCoord = 0;
  unsigned int posTexCoord = 0;
  for(unsigned int y=0; y<height; y++) {
    for(unsigned int x=0; x<width; x++) {
      Vector3<double> v = ppb.UnProjectLocal(HomgPoint2D(x,y));

      if(v[0] == 0 && v[1] == 0 && v[2] == 0) continue;
      if(normalizeZ) { 
        if(v[2]==0) {
          BIASERR("ray is valid but z-component is zero!");
          BIASABORT;
        }
        BIASASSERT(v[2]>0)        
      }
      count++;
      if(VertexCoordSlot != -1) {
        if(normalizeZ)
          v /= v[2];
        vertexData[posVertexCoord++] = static_cast<float>(v[0]);
        vertexData[posVertexCoord++] = static_cast<float>(v[1]);
        vertexData[posVertexCoord++] = static_cast<float>(v[2]);
      }

      if(TexCoordSlot != -1) {
        texCoordData[posTexCoord++] = (static_cast<float>(x)+0.5)/static_cast<float>(width);
        texCoordData[posTexCoord++] = (static_cast<float>(y)+0.5)/static_cast<float>(height);
        texCoordData[posTexCoord++] = 1.0 - (static_cast<float>(y)+0.5)/static_cast<float>(height);
      }
    }
  }
  unsigned int numValues = width*height;
  if(VertexCoordSlot != -1)
    vao.UploadData(VertexCoordSlot, 3, vertexData, numValues);
  if(TexCoordSlot != -1)
    vao.UploadData(TexCoordSlot, 3, texCoordData, numValues);

  delete [] vertexData;
  delete [] texCoordData;
}
#endif

