/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __Texture2DPool_hh__
#define __Texture2DPool_hh__

#include <OpenGLFramework/Base/glfTexture2D.hh>
#include <map>

#include <bias_config.h>

namespace BIAS
{

  /** Convenience container for managing 2D textures.
   * \author bartczak 03/2009
   **/
  class BIASOpenGLFramework_EXPORT Texture2DPool
  {
  public:

    /** Terminates the lifecycle of all textures created in the pool! **/
    ~Texture2DPool();
    Texture2DPool()
    {
    }

    /** Returns NULL if name is not found.
     **/
    BIAS::glfTexture2D*
    operator[](const std::string& name);

    /** Allocates an empty texture **/
    BIAS::glfTexture2D*
    Create(const std::string& name, int width, int height,
        GLenum internalFormat, GLenum minFilter = GL_NEAREST, GLenum magFilter =
            GL_NEAREST, GLenum wrapS = GL_CLAMP, GLenum wrapT = GL_CLAMP,
        int mipmap = 0);

    /** Uploads BIAS image to texture space **/
    BIAS::glfTexture2D*
    Create(const std::string& name, const BIAS::ImageBase& img,
        GLenum minFilter = GL_NEAREST, GLenum magFilter = GL_NEAREST,
        GLenum wrapS = GL_CLAMP, GLenum wrapT = GL_CLAMP,
        GLenum internalFormat = 0, int mipmap = 0);

    BIAS::glfTexture2D*
    Create(const std::string& name, int width, int height,
        ImageBase::EStorageType storageType, ImageBase::EColorModel colorModel,
        GLenum minFilter = GL_NEAREST, GLenum magFilter = GL_NEAREST,
        GLenum wrapS = GL_CLAMP, GLenum wrapT = GL_CLAMP, int mipmap = 0);

    BIAS::glfTexture2D*
    UploadImage(const std::string& name, const BIAS::ImageBase& img,
        GLenum minFilter = GL_NEAREST, GLenum magFilter = GL_NEAREST,
        GLenum wrapS = GL_CLAMP, GLenum wrapT = GL_CLAMP,
        GLenum internalFormat = 0, int mipmap = 0);

    void
    ClearAll();
    void
    Clear(const std::string& name);

  private:

    std::map<std::string, BIAS::glfTexture2D*> texturePool_;

    Texture2DPool&
    operator=(const Texture2DPool&)
    {
      return (*this);
    }
    Texture2DPool(const Texture2DPool&)
    {
    }

  };

}

#endif
