#ifndef CONTROL_HH_
#define CONTROL_HH_

#include <PanTilt/DPPanTiltControl.hh>

using namespace std;

namespace BIAS{

class BIASPanTilt_EXPORT Control {
public:
	// starts controlling the pan/tilt
	virtual void startControl() = 0;
	// stops/pauses control
	virtual void stopControl() = 0;
	// resumes control
	virtual void resumeControl() = 0;
	virtual void startMoveTo(const long int pan, const long int tilt) {

	}
	// initialize ptu
	Control() {
#ifdef WIN32
	  string device_ = "COM1";
#else
    string device_ = "/dev/ttyUSB0";
#endif
		bool bRet = ptControl.InitPanTiltUnit(device_.c_str());
		if(bRet!=true){
			cout<<"Initialising PanTiltUnit failed!"<<endl;
		}
		// next line sets heavy duty mode
		ptControl.SendRawCommand("PU2000 PA2000 PB60 TU2000 TA2000 TB60 PMR TMR %%1R600 DS\n");
		string s;
		while(ptControl.ReadRawLine(s)!=0) {
			cout << "PTU Controller: " << s << endl;
		}
		ptControl.SetMode(ABSOLUTEMODE);
		await_completion();
		ptControl.SetSpeed(MAXSPEED,MAXSPEED);
		ptControl.SetPosition(0,0);
		await_completion();
		ptControl.GetCurrentPosition(cur_pan, cur_tilt);
		ptControl.GetCurrentPosition(cur_pan, cur_tilt);
    ptControl.QueryResolution(presolution,tresolution);
	}
	// move to (0,0) position and release ptu
	virtual ~Control() {
		ptControl.SetMode(ABSOLUTEMODE);
		ptControl.SetSpeed(MAXSPEED/4,MAXSPEED/4);
		ptControl.SetPosition(0,0);
		await_completion();
		ptControl.ClosePanTiltUnit();
	}

	// get position, values should be updated by derived classes
	static void getPosition(long int &pan, long int &tilt) {
		pan = cur_pan;
		tilt = cur_tilt;
	}

	// get position, values should be updated by derived classes
	static void getPositionDegrees(double &panDegree, double &tiltDegree) {
		panDegree = -cur_pan * presolution * M_PI / 180.0f;
		tiltDegree = cur_tilt * tresolution * M_PI / 180.0f;
	}

	void setPosition(long int pan, long int tilt, bool waitComplete=true) {
		ptControl.SetMode(ABSOLUTEMODE);
		//ptControl.SetSpeed(MAXSPEED/4,MAXSPEED/4);
		ptControl.SetPosition(pan, tilt, waitComplete);
		cur_pan = pan;
		cur_tilt = tilt;
	}

	void awaitCompletion() {
    await_completion();
    ptControl.GetCurrentPosition(cur_pan, cur_tilt);
	}

protected:
	// some members
	BIAS::DPPanTiltControl ptControl;
	static long int cur_pan;
	static long int cur_tilt;
  static double presolution;
  static double tresolution;
};

// a controlfactory should be used to create all different kinds of controls
class ControlFactory {
public:
	virtual Control* create() = 0;
	virtual ~ControlFactory() {};
};

}

#endif /*CONTROL_HH_*/
