#include "DPPanTiltControl.hh"
#include "Base/Common/W32Compat.hh"
#include "Base/Common/BIASpragma.hh"
#include <cstring>

using namespace std;
using namespace BIAS;

#ifdef BIAS_HAVE_PTHREADS

pthread_mutex_t DPPanTiltControl::condition_mutex = PTHREAD_MUTEX_INITIALIZER;
long DPPanTiltControl::panThreaded_ =0;
long DPPanTiltControl::tiltThreaded_ =0;

#endif // BIAS_HAVE_PTHREADS

#define ROT_DEBUG false

DPPanTiltControl::DPPanTiltControl()
{
  xApertureAngle_ = 32.97; // ptr gey dragonfly 8mm
  yApertureAngle_ = 25.03;
  xPictureSize_ = 320;
  yPictureSize_ = 240;
  followObjectMaxDiff_=400;
  followObjectMinDiff_=30;
  bDefaultOrientation_=true;
  xPTSpeed_ = 0;
  yPTSpeed_ = 0;
  xPTDeltaSpeed_ = 0;
  yPTDeltaSpeed_ = 0;
  xPTDeltaSpeedTMinus1_ = 0;
  yPTDeltaSpeedTMinus1_ = 0;

  maxSpeed_ = 2902;
  KiX_=KiY_= 0.0;
  KdX_=KdY_ = 0.0;
  KpX_=KpY_= 1.0;
  ZiNiErrorX_=0; 
  ZiNiErrorY_=0; 
  ZiNiErrorTminus1X_=0;
  ZiNiErrorTminus1Y_=0;
  RPICounter_=0;

  //the compensationMatrizes
  ptuMatrix_.SetZYX(0,0,0);
  deltaMatrix_.SetZYX(0,0,0);
  sensorMatrixMinus1_.SetZYX(0,0,0);
  HandEyeMatrix_.SetZYX(0.0, 0.0, 0.0);
  bInitComp_ = true;
  panSpeedComp_=0;
  tiltSpeedComp_=0;
 
}

DPPanTiltControl::~DPPanTiltControl() {}

/**
   here define the mode in which the camera is to operate
   0 is SpeedControlMode
   1 is AbsolutePositioning mode 
   @returns 0 on no error
   @returns !=0 on error
*/
int DPPanTiltControl::SetMode(int nMode)
{
  char status;
  int ret = 0;
  set_mode(POSITION_LIMITS_MODE, ON_MODE);
  if(nMode == SPEEDMODE)
  {    
    if((status=set_mode(SPEED_CONTROL_MODE,PTU_PURE_VELOCITY_SPEED_CONTROL_MODE))==TRUE)
    {
      BIASDOUT(D_DP_MODE,"SPEED_CONTROL_MODE failed");
      ret = 1;
    }
    else  BIASDOUT(D_DP_MODE,"SPEED_CONTROL_MODE executed");
  }
  else if(nMode == ABSOLUTEMODE)
  {
    if ((status=set_mode(COMMAND_EXECUTION_MODE,EXECUTE_IMMEDIATELY))==TRUE)
    {
      BIASDOUT(D_DP_MODE,"COMMAND_EXECUTION_MODE/EXECUTE IMMEDIATELY failed");
        ret = 1;
   }
   else 
     BIASDOUT(D_DP_MODE,"COMMAND_EXECUTION_MODE/EXECUTE IMMEDIATELY executed");
  }
  else
    BIASDOUT(D_DP_MODE,"Wrong mode set for DPPanTiltControl!");
  
  return ret;
}




bool DPPanTiltControl::ResetPanTiltUnit()
{
  Vector<int> steps(2);
  steps[0]=0;
  steps[1]=0;
  bool bRet=false;
     
  xPTSpeed_ = 0;
  yPTSpeed_ = 0;
  xPTDeltaSpeed_ =0;
  yPTDeltaSpeed_ =0;
  xPTDeltaSpeedTMinus1_ =0;
  yPTDeltaSpeedTMinus1_ =0;
  
  RPICounter_=0;

 switch ( reset_PTU_parser(5000) ) 
   {
   case PTU_OK:	
     SetMode(SPEEDMODE);
     set_desired(PAN,SPEED, (PTU_PARM_PTR *) &maxSpeed_,ABSOLUTE);
     set_desired(TILT,SPEED, (PTU_PARM_PTR *) &maxSpeed_,ABSOLUTE);
     await_completion();
     SetMode(ABSOLUTEMODE);
     set_desired(PAN, POSITION, (PTU_PARM_PTR *) &steps[0],ABSOLUTE);
     set_desired(TILT, POSITION, (PTU_PARM_PTR *) &steps[1],ABSOLUTE);
     await_completion();
     
     bRet = true;
     break;
   case PTU_FIRMWARE_VERSION_TOO_LOW:  
		 bRet = false;
		 return bRet;
   case PTU_NOT_RESPONDING:
		 bRet = false;
		 return bRet;
   }


  return bRet;
}

void DPPanTiltControl::StopPanTiltUnit()
{
  int speed = 0;
  set_desired(PAN,SPEED, (PTU_PARM_PTR *) &speed,ABSOLUTE); 
  set_desired(TILT,SPEED, (PTU_PARM_PTR *) &speed,ABSOLUTE);
}

bool DPPanTiltControl::InitPanTiltUnit(int nCOMportNumber)
{
    char COMportPrefix[10] = "/dev/ttyS"; 
  //char COMportPrefix[12] = "/dev/ttyUSB"; 
  char COMportName[17];
  sprintf(COMportName, "%s%d", COMportPrefix, nCOMportNumber);
  return InitPanTiltUnit(COMportName);
}

bool DPPanTiltControl::InitPanTiltUnit(const char *COMportName)
{ 
  //reset_ptu();
  bool bRet = false;    
  
  while(!bRet){    
    cout << "Opening port " << COMportName << " for DPControl."<<endl;
    /* initialize the serial port */  
    sleep(2);    
    COMstream_ = open_host_port((char*)COMportName);
    if ( COMstream_ == PORT_NOT_OPENED )
      { 
        return bRet;
      }  
    set_mode(DEFAULTS, RESTORE_FACTORY_SETTINGS);
    switch ( reset_PTU_parser(5000) ) 
      {
      case PTU_OK:	
        bRet = true;
        break;
      case PTU_FIRMWARE_VERSION_TOO_LOW: 
        BIASERR("PTU firmware version too low"); 
        bRet = false;
        break;
      case PTU_NOT_RESPONDING:
        BIASERR("PTU not responding");
        bRet = false;
        break;
      }
    if(bRet==true){
      char *fw = firmware_version();
      char ok = firmware_version_OK();
      bRet=(ok==TRUE);
      cout << "("<<fw<<") firmeware ok? "<<boolalpha<<bRet<<endl;
    }
    if (!bRet){
      close_host_port(COMstream_);
    } else {
      double a,b;
      QueryResolution(a,b,true);
    }
  }
  return bRet;
}

void DPPanTiltControl::ClosePanTiltUnit()
{
  int nPanPos = 0;
  int nTiltPos =0;
  set_desired(PAN,POSITION,(PTU_PARM_PTR *) &nPanPos,ABSOLUTE);
  set_desired(TILT,POSITION,(PTU_PARM_PTR *) &nTiltPos,ABSOLUTE);
  set_desired(PAN,SPEED,(PTU_PARM_PTR *) &nPanPos,ABSOLUTE);
  set_desired(TILT,SPEED,(PTU_PARM_PTR *) &nTiltPos,ABSOLUTE);
  await_completion();
  close_host_port(COMstream_);
}


void DPPanTiltControl::SetKp( Vector<int> & kp)
{
  KpX_ = kp[0];
  KpY_ = kp[1];
  KpX_ /=100;
  KpY_ /=100;
}

void DPPanTiltControl::SetKi( Vector<int> & ki)
{
  KiX_ = ki[0];
  KiY_ = ki[1];
  KiX_ /=100; 
  KiY_ /=100;
}

void DPPanTiltControl::SetKd( Vector<int> & kd)
{
  KdX_ = kd[0];
  KdY_ = kd[1];
  KdX_ /=100;
  KdY_ /=100;
}

char DPPanTiltControl::SetSpeed(int nPanSpeed, int nTiltSpeed)
{   
  char status;
  status = set_desired(PAN,SPEED,(PTU_PARM_PTR *) &nPanSpeed,ABSOLUTE);
  status = set_desired(TILT,SPEED,(PTU_PARM_PTR *) &nTiltSpeed,ABSOLUTE);
  return status;
}

void DPPanTiltControl::GetSpeed(long &panSpeed, long &tiltSpeed)
{  
  panSpeed = get_current(PAN,SPEED);
  tiltSpeed = get_current(TILT,SPEED);
}

char DPPanTiltControl::SetAcceleration(int nPanAcc, int nTiltAcc)
{
    char status = set_desired(PAN,ACCELERATION,(PTU_PARM_PTR *) &nPanAcc,ABSOLUTE);
    if(status == PTU_OK)
	   status = set_desired(TILT,ACCELERATION,(PTU_PARM_PTR *) &nTiltAcc,ABSOLUTE);
    return status;
}

void DPPanTiltControl::GetAcceleration(long &panAcc, long &tiltAcc)
{
  panAcc = get_current(PAN,ACCELERATION);
  tiltAcc = get_current(TILT,ACCELERATION);
}

void DPPanTiltControl::MoveUp(int nValue)
{ 
  if(!bDefaultOrientation_)    
    nValue = -nValue;
  set_desired(TILT,POSITION,(PTU_PARM_PTR *) &nValue,RELATIVE);
  await_completion();
}
 


void DPPanTiltControl::MoveDown(int nValue)
{
  if(bDefaultOrientation_)
	 nValue = -nValue;
  set_desired(TILT,POSITION,(PTU_PARM_PTR *) &nValue,RELATIVE);
  await_completion();
}


 void DPPanTiltControl::MoveLeft(int nValue)
{
 if(!bDefaultOrientation_)
	 nValue = -nValue;
  set_desired(PAN,POSITION,(PTU_PARM_PTR *) &nValue,RELATIVE); 
  await_completion();
}
 
void DPPanTiltControl::MoveRight(int nValue)
{
  if(bDefaultOrientation_)    
	 nValue = -nValue;
  set_desired(PAN,POSITION,(PTU_PARM_PTR *) &nValue,RELATIVE);
  await_completion(); 
}

void DPPanTiltControl::GetCurrentPositionDegrees(double &panDegree, double &tiltDegree) {
  long lPanPos = get_current(PAN,POSITION);    
  long lTiltPos = get_current(TILT,POSITION);	
  panDegree = -lPanPos * panResolution_ * M_PI / 180.0f;
	tiltDegree = lTiltPos * tiltResolution_ * M_PI / 180.0f;
}


void DPPanTiltControl::GetCurrentPosition(long &lPanPos, long &lTiltPos)
{
    lPanPos = get_current(PAN,POSITION);    
    lTiltPos = get_current(TILT,POSITION);
}

void DPPanTiltControl::SetPosition(int nPanPos, int nTiltPos, bool bWaitComplete)
{
  char status =  set_desired(PAN,POSITION,(PTU_PARM_PTR *) &nPanPos,ABSOLUTE);
  if(status != PTU_OK )
    cout<<"Setting Pan failed"<<endl;

  status = set_desired(TILT,POSITION,(PTU_PARM_PTR *) &nTiltPos,ABSOLUTE);
  if(status != PTU_OK )
    cout<<"Setting Tilt failed"<<endl;
  if (bWaitComplete)
    await_completion();
}

void DPPanTiltControl::SetSensorInitOrientation(RMatrix& initMatrix)
{  
  //  initMatrix_ = initMatrix;
  //initMatrix_.transposeIP();
  BIASERR("dont use");
}


void DPPanTiltControl::CompensateHeliMovement(RMatrix& matrix)
{  
  
  Vector<double> anglesPTU(3);
  Vector<int> steps(2);
  long int pan, tilt; 
  //double dummy;
  RMatrix compMatrix;

  if(!bInitComp_) {
    GetCurrentPosition(pan, tilt);
    if (ROT_DEBUG) cout<<"Pan "<<pan<<" Tilt :"<<tilt<<endl;

    //convert to radians and set
    anglesPTU[2] = ((double)pan * panResolution_)* M_PI/180.0;
    anglesPTU[1] = ((double)tilt * tiltResolution_)* M_PI/180.0;
    anglesPTU[0] = 0.0;
    //if (ROT_DEBUG) cout<<"AnglesPTU: "<<anglesPTU<<endl;
    
    ptuMatrix_.SetZYX(anglesPTU[0],anglesPTU[1],anglesPTU[2]);
    ptuMatrix_.GetRotationAnglesZYX(anglesPTU[0],anglesPTU[1],anglesPTU[2]);
    if (ROT_DEBUG) cout<<"PTUMatrix :"<<anglesPTU<<endl; 

    matrix.GetRotationAnglesZYX(anglesPTU[0],anglesPTU[1],anglesPTU[2]);
    if (ROT_DEBUG) cout<<"sensorMatrix :"<<anglesPTU*(180/M_PI)<<endl;
  
    deltaMatrix_ = matrix * sensorMatrixMinus1_.Transpose();

    sensorMatrixMinus1_.GetRotationAnglesZYX(anglesPTU[0],anglesPTU[1],anglesPTU[2]);
    if (ROT_DEBUG) cout<<"sensorMatrixMinus1_ :"<<anglesPTU<<endl;

    deltaMatrix_.GetRotationAnglesZYX(anglesPTU[0],anglesPTU[1],anglesPTU[2]);
    if (ROT_DEBUG) cout<<"deltaMatrix :"<<anglesPTU<<endl;
    
    compMatrix = ptuMatrix_ * HandEyeMatrix_ * deltaMatrix_ *
      HandEyeMatrix_.Transpose() * ptuMatrix_.Transpose();
  
    compMatrix.GetRotationAnglesZYX(anglesPTU[0],anglesPTU[1],anglesPTU[2]);
    if (ROT_DEBUG) cout<<"CompMatrix :"<<anglesPTU<<endl;

    steps[0] = (int)(anglesPTU[2] * 180.0/M_PI / panResolution_);
    steps[1] = (int)(anglesPTU[1] * 180.0/M_PI / tiltResolution_);
    
    panSpeedComp_ = (short int)(steps[0]/T_*1e6);
    tiltSpeedComp_ =(short int)(steps[1]/T_*1e6);

    /*
    if(!bDefaultOrientation_)
      {
	panSpeedComp_ = -panSpeedComp_;
	tiltSpeedComp_ = -tiltSpeedComp_;
      }
  
    */
   if(!bDefaultOrientation_)
      {
        panSpeedComp_ = -panSpeedComp_;
        tiltSpeedComp_ = tiltSpeedComp_;
      }
   else{
     panSpeedComp_ = panSpeedComp_;
     tiltSpeedComp_ = tiltSpeedComp_;
   }
  } else {
    bInitComp_ = false;
  }
    

  //save the matrix in compMatrixMinus1_
  sensorMatrixMinus1_ = matrix;
}


void DPPanTiltControl::StartCircularMovement(){
  #ifdef BIAS_HAVE_PTHREADS
  pthread_create(&control_thread, NULL, this->threadFuncCircle_, this);
  #endif
}

#ifdef BIAS_HAVE_PTHREADS

void DPPanTiltControl::GetCurrentPositionDegreesThreaded(double &panDegree, double& tiltDegree){
  pthread_mutex_lock(&condition_mutex); 
  panDegree = -panThreaded_ * panResolution_ * M_PI / 180.0f;
	tiltDegree = tiltThreaded_ * tiltResolution_ * M_PI / 180.0f;
  pthread_mutex_unlock(&condition_mutex); 
}


void* DPPanTiltControl::threadFuncCircle_(void *ptr) {
  DPPanTiltControl *ptc = (DPPanTiltControl*)ptr;
  ptc->SetMode(ABSOLUTEMODE);
  ptc->SetSpeed(50,50);
  //first set to 0,0
  ptc->SetPosition(0,0);
  await_completion();

  //oben
  ptc->SetPosition(0,200);
  pthread_mutex_lock(&condition_mutex); 
  ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
  pthread_mutex_unlock(&condition_mutex);
  while(panThreaded_ != 0 && tiltThreaded_ != 200){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }
  //oben und links
  ptc->SetPosition(200,200);
  while(panThreaded_ != 200 && tiltThreaded_ != 200){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }

  //unten links
  ptc->SetPosition(200,-200);
  while(panThreaded_ != 200 && tiltThreaded_ != -200){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }
 
  //unten rechts
  ptc->SetPosition(-200,-200);
  while(panThreaded_ != -200 && tiltThreaded_ != -200){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }
 
   //oben rechts
  ptc->SetPosition(-200,200);
  while(panThreaded_ != -200 && tiltThreaded_ != 200){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }
 
  //oben
  ptc->SetPosition(0,200);
  while(panThreaded_ != 0 && tiltThreaded_ != 200){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }

  ptc->SetPosition(0,0);
  while(panThreaded_ != 0 && tiltThreaded_ != 0){
    sleep(0.01);
    pthread_mutex_lock(&condition_mutex); 
    ptc->GetCurrentPosition(panThreaded_,tiltThreaded_);
    pthread_mutex_unlock(&condition_mutex);
  }

  return NULL;
}

#endif // BIAS_HAVE_PTHREADS

void DPPanTiltControl::FollowMidPos(Vector<double> &newMean)
{  
  Vector<float>  beta(2);
  Vector<int> d(2);
  Vector<int> steps(2);
  //DISTANCE ON THE IMAGE PLANE
  d[0] = -((int)xPictureSize_/2)+(int)newMean[0];
  d[1] = -((int)yPictureSize_/2)+(int)newMean[1];

  //the angle which the camera has to turn  
  beta[0] = ((float)d[0]/(float)xPictureSize_) * xApertureAngle_;
  beta[1] = ((float)d[1]/(float)yPictureSize_) * yApertureAngle_;
  
  //now calculate positions from angle
  int stepsPerDegreePan = (int)(1.0/panResolution_);
  int stepsPerDegreeTilt = (int)(1.0/tiltResolution_);
  steps[0] = (int)(beta[0] * stepsPerDegreePan);
  steps[1] = (int)(beta[1] * stepsPerDegreeTilt);

  //switch sign
  if(bDefaultOrientation_)
	 steps[0] = -steps[0]; 
  else
	 steps[1] = steps[1];

  // cout<<"steps x,y "<<steps[0]<<","<<steps[1]<<endl;

  if(steps[0]>followObjectMinDiff_ && steps[0]<followObjectMaxDiff_)
    {
      set_desired(PAN, POSITION, (PTU_PARM_PTR *) &steps[0],RELATIVE);
      await_completion();
      // cout<<"now following mid pos , steps x: "<<steps[0]<<endl;
    }
 if(steps[1] > followObjectMinDiff_ && steps[1]<followObjectMaxDiff_)
   {
     set_desired(TILT, POSITION, (PTU_PARM_PTR *) &steps[1],RELATIVE);
     await_completion();
     //cout<<" now following mid pos , steps y: "<<steps[1]<<endl;
   }
}

void DPPanTiltControl::PIDSpeedAdjustFollow(Vector<double> &newMean)
{
  Vector<double> error(2);
  //DISTANCE ON THE IMAGE PLANE
  error[0] = -((double)xPictureSize_/2)+newMean[0];
  error[1] = -((double)yPictureSize_/2)+newMean[1];
  BIASDOUT(D_DP_PID,"Error x:"<<error[0]); 
  BIASDOUT(D_DP_PID,"Error Y:"<<error[1]);

  ZiNiErrorX_+=(int)error[0];
  ZiNiErrorY_+=(int)error[1];
  
  double maxSpeedScaleX = (error[0]/xPictureSize_);
  double maxSpeedScaleY = (error[1]/yPictureSize_);
  
  BIASDOUT(D_DP_PID,"scale x:"<<maxSpeedScaleX); 
  BIASDOUT(D_DP_PID,"scale y:"<<maxSpeedScaleY);
  //now calculate new speeds
  xPTSpeed_ = (short int)(maxSpeed_ * maxSpeedScaleX
                          * KpX_);// * error[0]);// /xPictureSize_;
  yPTSpeed_ =  (short int)(maxSpeed_ * maxSpeedScaleY 
                           * KpY_);// * error[1]);// /yPictureSize_;
  BIASDOUT(D_DP_PID,"KpX_, xPTSpeed :"<<KpX_<<","<<xPTSpeed_); 
  BIASDOUT(D_DP_PID,"KpY_, yPTSpeed :"<<KpY_<<","<<yPTSpeed_);

  //Add the integrative part to the Speed (sum of all errors over time); 
  xPTSpeed_ +=  (short int)KiX_ * ZiNiErrorX_;
  yPTSpeed_ +=  (short int)KiY_ * ZiNiErrorY_;

  //Add the derivative part to the Speed (d e(t)/dt)
  xPTSpeed_ +=  (short int)(KdX_ * (error[0]- ZiNiErrorTminus1X_));
  yPTSpeed_ +=  (short int)(KdY_ * (error[1]- ZiNiErrorTminus1Y_));
  
  //now save actual error as tminus1 error
  ZiNiErrorTminus1X_=(int)error[0];
  ZiNiErrorTminus1Y_=(int)error[1];

  //switch speeds if camera upside down
  if(bDefaultOrientation_)
   {
     xPTSpeed_ = -xPTSpeed_;
   }
  else{
      yPTSpeed_ = yPTSpeed_;
     
    //    yPTSpeed_ = -yPTSpeed_;
  }
  // add the speeds from the helicopter movement compensation 
  // calculation to speeds.
  xPTSpeed_ += panSpeedComp_;
  yPTSpeed_ += tiltSpeedComp_;

  //and set the new speeds
  set_desired(PAN, SPEED, (PTU_PARM_PTR *) &xPTSpeed_,ABSOLUTE);
  set_desired(TILT, SPEED, (PTU_PARM_PTR *) &yPTSpeed_,ABSOLUTE);

  BIASDOUT(D_DP_PID,"Speed set to: "<<xPTSpeed_<<","<<yPTSpeed_);
}

void DPPanTiltControl::SetSamplingTime(float t)
{
  T_ = t;
}

//Nachstellzeit?? My english is very baaaad
void DPPanTiltControl::SetResetTime(float tn)
{
  Tn_ = tn;
}

void DPPanTiltControl::SendRawCommand(string cmd) {
   SerialStringOut(COMstream_, (unsigned char*) cmd.c_str());
}

int DPPanTiltControl::ReadRawLine(string &cmd) {
	char result[0xFF];
	int len = 0;
	ReadSerialLine(COMstream_, (unsigned char*)result,1,&len);
	cmd = string(result,len);
	return len;
}

int DPPanTiltControl::SetMotorStepping(int panStepping, int tiltStepping) {

  int ret = 0;
  std::string command="";
  string s;
  SendRawCommand("PR\n");
  ReadRawLine(s);
  ReadRawLine(s);
  switch (panStepping) {
    case AUTO:
      if (s.find("A")!=string::npos) {
        std::cout << "Pan Tilt Control: pan stepping already set" << std::endl;
        panStepping = -1;
      } else {
        command="WPA\n";
      }
      break;
    case FULL:
      if (s.find("F")!=string::npos) {
        std::cout << "Pan Tilt Control: pan stepping already set" << std::endl;
        panStepping = -1;
      } else {
        command="WPF\n";
      }
      break;
    case HALF:
      if (s.find("H")!=string::npos) {
        std::cout << "Pan Tilt Control: pan stepping already set" << std::endl;
        panStepping = -1;
      } else {
        command="WPH\n";
      }
      break;
    case QUARTER:
      if (s.find("Q")!=string::npos) {
        std::cout << "Pan Tilt Control: pan stepping already set" << std::endl;
        panStepping = -1;
      } else {
        command="WPQ\n";
      }
      break;
    case EIGHTH:
      if (s.find("E")!=string::npos) {
        std::cout << "Pan Tilt Control: pan stepping already set" << std::endl;
        panStepping = -1;
      } else {
        ret = -1;
        command="WPE\n";
      }
      break;
    default:
      ret = -1;
      break;
  }

  if (ret != 0) 
    return ret;

  if (panStepping != -1) {
    SendRawCommand(command);
    // todo! when switching stepping mode, pan tilt recalibrates. "await completion" does not work here. we have to wait for finish.
    while (1) { 
      if (ReadRawLine(s) != 0) {
        std::cout << "Pan Tilt Control >> " << s << std::endl;
        if (s.find("*") != string::npos) {
          break;
        }
      }
      std::cout << "Pan Tilt Control sleeping for 1 second" << std::endl;
      sleep(1);
    }
  }

  SendRawCommand("TR\n");
  ReadRawLine(s);
  ReadRawLine(s);
  switch (tiltStepping) {
    case AUTO:
      if (s.find("A")!=string::npos) {
        std::cout << "Pan Tilt Control: tilt stepping already set" << std::endl;
        tiltStepping = -1;
      } else {
        command="WTA\n";
      }
      break;
    case FULL:
      if (s.find("F")!=string::npos) {
        std::cout << "Pan Tilt Control: tilt stepping already set" << std::endl;
        tiltStepping = -1;
      } else {
        command="WTF\n";
      }
      break;
    case HALF:
      if (s.find("H")!=string::npos) {
        std::cout << "Pan Tilt Control: tilt stepping already set" << std::endl;
        tiltStepping = -1;
      } else {
        command="WTH\n";
      }
      break;
    case QUARTER:
      if (s.find("Q")!=string::npos) {
        std::cout << "Pan Tilt Control: tilt stepping already set" << std::endl;
        tiltStepping = -1;
      } else {
        command="WTQ\n";
      }
      break;
    case EIGHTH:
      if (s.find("E")!=string::npos) {
        std::cout << "Pan Tilt Control: tilt stepping already set" << std::endl;
        tiltStepping = -1;
      } else {
        command="WTE\n";
      }
      break;
    default:
      ret = -1;
      break;

  }
  if (ret != 0) 
    return ret;

  if (tiltStepping != -1) {
    SendRawCommand(command);
    while (1) { // todo! see above
      if (ReadRawLine(s) != 0) {
        std::cout << "Pan Tilt Control >> " << s << std::endl;
        if (s.find("*") != string::npos) {
          break;
        }
      }
      std::cout << "Pan Tilt Control sleeping for 1 second" << std::endl;
      sleep(1);
    }
  }

  double a,b;
  std::cout << "Pan Tilt Control sleeping for 1 second" << std::endl;
  sleep(1);
  QueryResolution (a,b,true);
  return ret;
}

void DPPanTiltControl::QueryResolution(double &panResolution, double &tiltResolution, bool updateInternal) {
  if (updateInternal) {
 		string s, tmp;
    double argsec;
    SendRawCommand("PR\n");
    ReadRawLine(s); // this is the command
    ReadRawLine(s); // this is the answer
    stringstream ss1;
    ss1 << s;
    ss1 >> tmp;
    ss1 >> argsec;
    std::cout << argsec << std::endl;
    panResolution_ = argsec / 3600.0;
    SendRawCommand("TR\n");
    ReadRawLine(s); // this is the command
    ReadRawLine(s); // this is the answer
    stringstream ss2;
    ss2 << s;
    ss2 >> tmp;
    ss2 >> argsec;
    std::cout << argsec << std::endl;
    tiltResolution_ = argsec / 3600.0;
  }
  panResolution = panResolution_;
  tiltResolution = tiltResolution_;
}


void DPPanTiltControl::RecursivePIControl(Vector<double> &newMean)
{
  BIASDOUT(D_DP_RECURSIVE,"NewMean "<<newMean[0]<<" "<<newMean[1]);
  Vector<int> error(2);
  //DISTANCE ON THE IMAGE PLANE 
  error[0] = (int)(newMean[0]-xPictureSize_/2);
  error[1] = (int)(newMean[1]-yPictureSize_/2);

  BIASDOUT(D_DP_RECURSIVE,"Error "<<error[0]<<","<<error[1]);

  if(RPICounter_==0)
    {
      //u(0) (e(k-1) and u(k-1) are zero here!)
      //scale maxspeed with picturesize/error = speed relative to error
      xPTDeltaSpeed_=(short int)(KpX_*(1+T_/(2*Tn_))*error[0]);
      yPTDeltaSpeed_=(short int)(KpY_*(1+T_/(2*Tn_))*error[1]);
    
      //switch speeds if camera upside down
      if(bDefaultOrientation_)
        xPTDeltaSpeed_ = -xPTDeltaSpeed_;
      else
        yPTDeltaSpeed_ = -yPTDeltaSpeed_;

      //and set the new speeds
      set_desired(PAN,SPEED,(PTU_PARM_PTR *) &xPTDeltaSpeed_,RELATIVE);
      set_desired(TILT,SPEED,(PTU_PARM_PTR *) &yPTDeltaSpeed_,RELATIVE);
      BIASDOUT(D_DP_RECURSIVE,"The counter is 0 in RecursivePIControl");
      //save the speed for next iteration
      xPTDeltaSpeedTMinus1_ = xPTDeltaSpeed_;
      yPTDeltaSpeedTMinus1_ = yPTDeltaSpeed_;
      // save the error here
      ErrorTMinus1_.push_back(error[0]);
      ErrorTMinus1_.push_back(error[1]);

      RPICounter_++;
    }    
  else
    {
      //u(1) (u(k-2) is zero here)
      xPTDeltaSpeed_ = (short int)(KpX_*(1+(T_/(2*Tn_)))
        *error[0]+KpX_*(-1+(T_/(2*Tn_)))*ErrorTMinus1_[0]
        +0.1*xPTDeltaSpeedTMinus1_);
         
      yPTDeltaSpeed_ = (short int)(KpY_*(1+(T_/(2*Tn_)))
        *error[1]+KpY_*(-1+(T_/(2*Tn_)))*ErrorTMinus1_[1]
        +yPTDeltaSpeedTMinus1_);

    
      //switch speeds if camera upside down
      if(bDefaultOrientation_)
        {
          xPTDeltaSpeed_ = -xPTDeltaSpeed_;
        }
      else{
          yPTDeltaSpeed_ = -yPTDeltaSpeed_;
      }

      BIASDOUT(D_DP_RECURSIVE,"T_ "<<T_<<" Tn_ "<<Tn_);
      BIASDOUT(D_DP_RECURSIVE,"SpeedTminus1x "<<xPTDeltaSpeedTMinus1_);
      BIASDOUT(D_DP_RECURSIVE,"SpeedTminus1y "<<yPTDeltaSpeedTMinus1_);
      BIASDOUT(D_DP_RECURSIVE,"KpX_ "<<KpX_); 
      BIASDOUT(D_DP_RECURSIVE,"KpY_ "<<KpY_);

      BIASDOUT(D_DP_RECURSIVE,"Rel. Speedx "<<xPTDeltaSpeed_);
      BIASDOUT(D_DP_RECURSIVE,"Rel. Speedy "<<yPTDeltaSpeed_);
    
      //and set the new speeds
      set_desired(PAN,SPEED,(PTU_PARM_PTR *) &xPTDeltaSpeed_,RELATIVE);
      set_desired(TILT,SPEED,(PTU_PARM_PTR *) &yPTDeltaSpeed_,RELATIVE);
      cout<<"delta speedx: "<<xPTDeltaSpeed_<<endl;
      cout<<"delta speedy: "<<yPTDeltaSpeed_<<endl;
      //now save speeds and errors for the next iteration
      xPTDeltaSpeedTMinus1_ = xPTDeltaSpeed_;
      yPTDeltaSpeedTMinus1_ = yPTDeltaSpeed_;
  
      ErrorTMinus1_[0] = error[0];
      ErrorTMinus1_[1] = error[1];
      BIASDOUT(D_DP_RECURSIVE,"\n\n\n");
    }
}
