#ifndef __GLUTGLVIEWERBASE_HH__
#define __GLUTGLVIEWERBASE_HH__
/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @class  biasGLviewerGLUT
   @ingroup g_tools
   @relates ContextGLUT,DistanceMeasureControl,SceneOpenSceneGraph,GLProjectionParametersPerspective
   @brief  BIAS OpenGLRendering Tool. Simple viewer for 3D models, executable biasOpenGLviewerGlut
   @author Ingo Schiller, Finn Schulte
   @date 02/2010
*/


#include <GLviewer/ContextGLUT.hh>
#include <GLviewer/Controller/DistanceMeasureControl.hh>
#include <GLviewer/Controller/FlyControl.hh>
#include <GLviewer/Controller/OnlyRotationControl.hh>
#include <GLviewer/GLProjectionParametersPerspective.hh>
#include <GLviewer/ScreenShotListener.hh>
#include <GLviewer/Scenes/SceneBGImage.hh>
#ifdef BIAS_HAVE_OPENSCENEGRAPH
#include <GLviewer/Scenes/SceneOpenSceneGraph.hh>
#endif
#include <GLviewer/Scenes/SceneGlutPrimitives.hh>
#include <GLviewer/Scenes/PointLightScene.hh>

#include <Utils/Param.hh>

namespace BIAS{

  class biasGLviewerGLUT : public BIAS::ControlInterface , public BIAS::Param
  {

public:
  	biasGLviewerGLUT(int argc, char* argv[]);
  //virtual ~GlutGLviewerBase();
  virtual void Init();
  virtual void Run();

  bool StandardKeyPressed(unsigned char key, int, int);

  void AnimationFunc();

protected:
  
  void toggleStoreToDisc_();

  bool initialized_;
  bool animate_;
  bool storeToDisc_;
  bool renderBoundingBoxes_;
  BIAS::ScreenShotListener screenShotControl_;
  BIAS::GLProjectionParametersPerspective camera_;
  unsigned int slotCounter_;
  unsigned int viewingSlot_;
  
  BIAS::ContextGLUT* contextPointer_;
  
#ifdef BIAS_HAVE_OPENSCENEGRAPH
  BIAS::SceneOpenSceneGraph scene_;
#endif
  BIAS::SceneGlutPrimitives sceneGlut_;
  BIAS::PointLightScene sceneLight_;
  
  BIAS::DistanceMeasureControl control_;
  BIAS::FlyControl flyControl_;
  BIAS::OnlyRotationControl rotControl_;

  int firstUnkownParam_;
  int argc_;
  char** argv_;

  // Animation / Control Params
  std::string* animationFile_;
  double* fixedFPS_;
  bool* useFlyMode_;
  bool* useRotMode_;
  
  // View Params
  double* zNear_;
  double* zFar_;
  double* fov_;
  bool* autoClipping_;
  bool* backfaceCulling_;
  bool* teapot_;
  bool* cube_;
  bool* tetrahedron_;
  bool* icosahedron_;
  // Camera Params  
  std::string* projectionFile_;
  bool* useSceneCenter_;
  bool* useJustIntrinsics_;
  int* autoReshapeBehaviour_;

  // Misc Params
  int* winwidth_;
  int* winheight_;
  std::string* screenShotName_;
  bool* greyScreenShot_; 
  BIAS::Vector<double>* backGroundColor_;
  bool* help_;
};

}

#endif //__GLUTGLVIEWERBASE_HH__

