/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <wx/wxprec.h>

#include "ShowCamWxFormat7Frame.hh"
#include "ShowCamWxFrame.hh"
#include <Gui/StringConv.hh>
#include <Base/Common/FileHandling.hh>
#include <wx/wx.h>
#include <wx/utils.h>
#include <wx/choicdlg.h>

using namespace BIAS;
using namespace std;

IMPLEMENT_CLASS( ShowCamWxFormat7Frame, wxFrame )

BEGIN_EVENT_TABLE( ShowCamWxFormat7Frame, wxDialog )
  EVT_CLOSE( ShowCamWxFormat7Frame::OnCloseWindow )
  EVT_BUTTON( ID_BUTTON_OK,ShowCamWxFormat7Frame::OnButtonOK )
  EVT_BUTTON( ID_BUTTON_CANCEL,ShowCamWxFormat7Frame::OnButtonCancel)
  EVT_BUTTON( ID_BUTTON_CHECK,ShowCamWxFormat7Frame::OnButtonCheck)
  EVT_SCROLL_CHANGED(ShowCamWxFormat7Frame::OnScrollSlider)
  EVT_SCROLL(ShowCamWxFormat7Frame::OnSlide)
  EVT_TEXT(ID_TEXT_DA_XOFFSET, ShowCamWxFormat7Frame::OnTextChange)
  EVT_TEXT(ID_TEXT_DA_YOFFSET, ShowCamWxFormat7Frame::OnTextChange)
  EVT_TEXT(ID_TEXT_DA_XSIZE, ShowCamWxFormat7Frame::OnTextChange)
  EVT_TEXT(ID_TEXT_DA_YSIZE, ShowCamWxFormat7Frame::OnTextChange)
END_EVENT_TABLE()


//had to declare here because of conflicts
BIAS::ShowCamWxFrame *parentFrame_;

/*!
 *  ShowCamWxFormat7Frame constructors
 */

ShowCamWxFormat7Frame::
ShowCamWxFormat7Frame(wxFrame* parent,const wxString& title, const wxPoint& pos,
                      const wxSize& size)
  : wxDialog(parent, -1, title, pos, size, wxSTAY_ON_TOP|wxCAPTION|wxRESIZE_BORDER)
{
  parentFrame_= NULL;
  if(dynamic_cast< ShowCamWxFrame*>(parent) != NULL){
    parentFrame_ = (ShowCamWxFrame *)parent;
  }

  Init();
  CreateControls();
  Centre();
#ifdef BIAS_HAVE_DCAM
  dcam = new VideoSource_DCAM();
#endif
}


/*!
 *  ShowCamWxFormat7Frame destructor
 */
ShowCamWxFormat7Frame::~ShowCamWxFormat7Frame()
{

}

/*!
 * Member initialisation
 */
void ShowCamWxFormat7Frame::Init()
{

  yOffsetMin_ = xSizeMin_ = ySizeMin_ =  xOffsetMin_=0;
  yOffsetMax_ = xSizeMax_ = ySizeMax_ = xOffsetMax_ = 1024;
  bppMin_ =  100;
  bppMax_= 7000;
  
}

/*!
 * Control creation for ProjectDefinitionFrame
 */

void ShowCamWxFormat7Frame::CreateControls()
{
  // ccd button
  itemButtonOK=new wxButton( this, ID_BUTTON_OK,
                             _("OK"), wxDefaultPosition,
                             wxDefaultSize, 0 );

  itemButtonCancel=new wxButton( this, ID_BUTTON_CANCEL,
                               _("Cancel"), wxDefaultPosition,
                               wxDefaultSize, 0 );itemButtonCheck=new wxButton( this, ID_BUTTON_CHECK,
                                _("Check"), wxDefaultPosition,
                                wxDefaultSize, 0 );
  itemButtonCheck->Disable();

  // the sizer
  wxFlexGridSizer* itemFlexGridSizer = new wxFlexGridSizer(11, 3, 0, 0);
  this->SetSizer(itemFlexGridSizer);

  wxSize sliderSize(300,45);

  xSizeSB = new  wxSlider(this, ID_SB_XSIZE, 20, xSizeMin_, xSizeMax_,
                          wxDefaultPosition,sliderSize,
                          wxSL_HORIZONTAL | wxSL_LABELS | wxSL_AUTOTICKS,
                          wxDefaultValidator, wxT("XSize"));

  ySizeSB = new  wxSlider(this, ID_SB_YSIZE,20, ySizeMin_, ySizeMax_,
                          wxDefaultPosition,sliderSize,
                          wxSL_HORIZONTAL | wxSL_LABELS | wxSL_AUTOTICKS,
                          wxDefaultValidator, wxT("YSize"));

  xOffsetSB = new  wxSlider(this, ID_SB_XOFFSET,20, xOffsetMin_, xOffsetMax_,
                            wxDefaultPosition,sliderSize,
                            wxSL_HORIZONTAL | wxSL_LABELS | wxSL_AUTOTICKS,
                            wxDefaultValidator, wxT("XOffset"));

  yOffsetSB = new  wxSlider(this, ID_SB_YOFFSET,20, yOffsetMin_, yOffsetMax_,
                            wxDefaultPosition,sliderSize,
                            wxSL_HORIZONTAL | wxSL_LABELS |wxSL_AUTOTICKS,
                            wxDefaultValidator, wxT("YOffset"));

  wxString cm_choices[8];
  cm_choices[0] = wxT("Mono 8bpp");
  cm_choices[1] = wxT("YUV 4:1:1");
  cm_choices[2] = wxT("YUV 4:2:2");
  cm_choices[3] = wxT("YUV 4:4:4");
  cm_choices[4] = wxT("RGB 24bpp");
  cm_choices[5] = wxT("Mono 16bpp");
  cm_choices[6] = wxT("Raw 8bpp");
  cm_choices[7] = wxT("Raw 16bpp");

  colorModeCB = new  wxChoice(this, ID_CB_COLORMODE, wxDefaultPosition, wxDefaultSize, 8, cm_choices, 0, wxDefaultValidator, wxT("Color Mode for Cam"));

  wxString cmv_choices[6];
  cmv_choices[0] = wxT("Same as Cam");
  cmv_choices[1] = wxT("Grey");
  cmv_choices[2] = wxT("Bayer BGGR");
  cmv_choices[3] = wxT("Bayer GBRG");
  cmv_choices[4] = wxT("Bayer GRBG");
  cmv_choices[5] = wxT("Bayer RGGB");

  colorModeViewCB = new wxChoice(this,ID_CB_COLORMODEVIEW, wxDefaultPosition, wxDefaultSize, 6, cmv_choices, 0, wxDefaultValidator, wxT("Color Mode for Viewing"));
  BpPSB = new  wxSlider(this, ID_SB_BPP,20, bppMin_, bppMax_,
                               wxDefaultPosition,sliderSize,
                               wxSL_HORIZONTAL | wxSL_LABELS | wxSL_AUTOTICKS,
                               wxDefaultValidator, wxT("Bits per packet"));

  directAccessXSize = new wxTextCtrl(this,ID_TEXT_DA_XSIZE, _("0"),
                          wxDefaultPosition,wxDefaultSize, 0);
  directAccessYSize = new wxTextCtrl(this,ID_TEXT_DA_YSIZE, _("0"),
                          wxDefaultPosition,wxDefaultSize, 0);
  directAccessXOffset = new wxTextCtrl(this,ID_TEXT_DA_XOFFSET, _("0"),
                          wxDefaultPosition,wxDefaultSize, 0);
  directAccessYOffset = new wxTextCtrl(this,ID_TEXT_DA_YOFFSET, _("0"),
                          wxDefaultPosition,wxDefaultSize, 0);
  

  wxStaticText* itemStaticText1 =
    new wxStaticText( this, wxID_STATIC, _("XSize"),
                      wxDefaultPosition, wxDefaultSize, 0 );
  itemFlexGridSizer->Add(itemStaticText1, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(xSizeSB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(directAccessXSize,0,wxALIGN_CENTER|wxALL,5);

  wxStaticText* itemStaticText2 =
    new wxStaticText( this, wxID_STATIC, _("YSize"),
                      wxDefaultPosition, wxDefaultSize, 0 );
  itemFlexGridSizer->Add(itemStaticText2, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(ySizeSB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(directAccessYSize,0,wxALIGN_CENTER|wxALL,5);

  wxStaticText* itemStaticText3 =
    new wxStaticText( this, wxID_STATIC, _("XOffset"),
                      wxDefaultPosition, wxDefaultSize, 0 );

  itemFlexGridSizer->Add(itemStaticText3, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(xOffsetSB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

  itemFlexGridSizer->Add(directAccessXOffset,0,wxALIGN_CENTER|wxALL,5);

  wxStaticText* itemStaticText4 =
    new wxStaticText( this, wxID_STATIC, _("YOffset"),
                      wxDefaultPosition, wxDefaultSize, 0 );
  itemFlexGridSizer->Add(itemStaticText4, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(yOffsetSB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(directAccessYOffset,0,wxALIGN_CENTER|wxALL,5);

  wxStaticText* itemStaticText5 =
    new wxStaticText( this, wxID_STATIC, _("Color Mode for cam"),
                      wxDefaultPosition, wxDefaultSize, 0 );
  itemFlexGridSizer->Add(itemStaticText5, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(colorModeCB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(itemButtonCheck,0,wxALIGN_CENTER|wxALL,5);

  wxStaticText* itemStaticText51 =
    new wxStaticText( this, wxID_STATIC, _("Color Mode for view"),
                      wxDefaultPosition, wxDefaultSize, 0 );
  itemFlexGridSizer->Add(itemStaticText51, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(colorModeViewCB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->AddSpacer(10);

  wxStaticText* itemStaticText6 =
    new wxStaticText( this, wxID_STATIC, _("Bits per packet"),
                      wxDefaultPosition, wxDefaultSize, 0 );
  itemFlexGridSizer->Add(itemStaticText6, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(BpPSB, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemStaticTextFPS = new wxTextCtrl(this,ID_TEXT_FPS, _("= 0.0 FPS"),
                          wxDefaultPosition,wxDefaultSize, 0);
  itemFlexGridSizer->Add(itemStaticTextFPS,0,wxALIGN_CENTER|wxALL,5);


  

  wxBoxSizer *boxSizerCCD = new wxBoxSizer(wxHORIZONTAL);
  boxSizerCCD->Add(itemButtonOK, 1,wxEXPAND|wxALL,2);
  boxSizerCCD->Add(itemButtonCancel, 1,wxEXPAND|wxALL,2);
  itemFlexGridSizer->Add(5, 5, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);
  itemFlexGridSizer->Add(boxSizerCCD, 0,
                         wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

}

/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_PROJECTDEFINITIONFRAME
 */
void ShowCamWxFormat7Frame::OnCloseWindow( wxCloseEvent& event )
{
  // event.skip();
  this->Destroy();
}

/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_OK
 */

void ShowCamWxFormat7Frame::OnButtonOK( wxCommandEvent& event )
{
  // check if y value is even
  if((ySizeSB->GetValue() % 2) != 0){
    wxMessageDialog* dialog = new wxMessageDialog(this, wxT("Y-Size has to be an even value"),wxT("Message box"),wxOK,wxDefaultPosition);
    dialog->ShowModal();
  }else{

  //  cout << "button OK! "
  //      << xSizeSB->GetValue() << endl;
  xSize = xSizeSB->GetValue();
  ySize = ySizeSB->GetValue();
  xOffset = xOffsetSB->GetValue();
  yOffset = yOffsetSB->GetValue();
  bpp = BpPSB->GetValue();
#ifdef BIAS_HAVE_DCAM
  switch (colorModeCB->GetSelection()) {
    case 0:
    colorMode = DC1394_COLOR_CODING_MONO8;
    break;
    case 1:
    colorMode = DC1394_COLOR_CODING_YUV411;
    break;
    case 2:
    colorMode = DC1394_COLOR_CODING_YUV422;
    break;
    case 3:
    colorMode = DC1394_COLOR_CODING_YUV444;
    break;
    case 4:
    colorMode = DC1394_COLOR_CODING_RGB8;
    break;
    case 5:
    colorMode = DC1394_COLOR_CODING_MONO16;
    break;
    case 6:
    colorMode = DC1394_COLOR_CODING_RAW8;
    break;
    case 7:
    colorMode = DC1394_COLOR_CODING_RAW16;
    break;
    default:
    colorMode = DC1394_COLOR_CODING_MONO8;
    break;
  }

  switch(colorModeViewCB->GetSelection()){
    case 0: colorModeView = ImageBase::CM_invalid;
        break;
    case 1: colorModeView = ImageBase::CM_Grey;
        break;
    case 2: colorModeView = ImageBase::CM_Bayer_BGGR;
        break;
    case 3: colorModeView = ImageBase::CM_Bayer_GBRG;
        break;
    case 4: colorModeView = ImageBase::CM_Bayer_GRBG;
        break;
    case 5: colorModeView = ImageBase::CM_Bayer_RGGB;
        break;
    default: colorModeView = ImageBase::CM_invalid;
        break;
  }

#endif // BIAS_HAVE_DCAM
  this->MakeModal(false);
  this->Close();
  this->SetReturnCode(wxID_OK);
  }
}

void ShowCamWxFormat7Frame::OnButtonCancel( wxCommandEvent& event ){
  this->MakeModal(false);
  this->Close();
}

void ShowCamWxFormat7Frame::OnButtonCheck( wxCommandEvent& event )
{
  // Feld 1 pruefen
  if(wxAtoi(directAccessXSize->GetValue()) != xSizeSB->GetValue()){
    xSizeSB->SetValue(wxAtoi(directAccessXSize->GetValue()));
    ScrollSliderHelper(xSizeSB,xSize,xSizeMin_);
  }

  if(wxAtoi(directAccessYSize->GetValue()) != ySizeSB->GetValue()){
    ySizeSB->SetValue(wxAtoi(directAccessYSize->GetValue()));
    ScrollSliderHelper(ySizeSB,ySize,ySizeMin_);
  }

  if(wxAtoi(directAccessXOffset->GetValue()) != xOffsetSB->GetValue()){
    xOffsetSB->SetValue(wxAtoi(directAccessXOffset->GetValue()));
    ScrollSliderHelper(xOffsetSB,xSize,xSizeMin_,xSizeMax_,xSizeSB,true);
  }

  if(wxAtoi(directAccessYOffset->GetValue()) != yOffsetSB->GetValue()){
      yOffsetSB->SetValue(wxAtoi(directAccessYOffset->GetValue()));
      ScrollSliderHelper(yOffsetSB,ySize,ySizeMin_,ySizeMax_,ySizeSB,true);
    }

  AdjustBytesPerPacket();
  CalcFPS();

 // itemButtonCheck->Disable();
  itemButtonOK->Enable();

}

void ShowCamWxFormat7Frame::OnTextChange( wxCommandEvent& event )
{
  itemButtonOK->Disable();
  itemButtonCheck->Enable();

}

void ShowCamWxFormat7Frame::ScrollSliderHelper(wxSlider* slider, int& size, int& sizeMin, int sizeMax, wxSlider* sliderSize, bool isOffset){

  int value = slider->GetValue();
  int diff = abs(size -value);

  if(diff > sizeMin/2){
    int mod = value % sizeMin;
    if(mod != 0){
      if(mod > (sizeMin/2))
        value = value + (sizeMin - mod);
      else
        value = value - mod;

      slider->SetValue(value);
      size = value;
    }
  }else if(diff < sizeMin/2){
    if(size -value > 0){
      value = size - sizeMin;
    }else if(size -value < 0){
      value = size + sizeMin;
    }
    slider->SetValue(value);
    size = value;
    if(isOffset)
      sliderSize->SetMax(sizeMax - slider->GetValue());
  }

  if(slider->GetId() == ID_SB_XSIZE) directAccessXSize->SetValue(wxString::Format(wxT("%d"), value));
  if(slider->GetId() == ID_SB_YSIZE) directAccessYSize->SetValue(wxString::Format(wxT("%d"), value));
  if(slider->GetId() == ID_SB_XOFFSET) directAccessXOffset->SetValue(wxString::Format(wxT("%d"), value));
  if(slider->GetId() == ID_SB_YOFFSET) directAccessYOffset->SetValue(wxString::Format(wxT("%d"), value));

  CalcFPS();
}
void ShowCamWxFormat7Frame::AdjustBytesPerPacket(){
#ifdef BIAS_HAVE_DCAM
  unsigned int minbpp, maxbpp;
  dcam->GetBytesPerPacketForFormat7(CamID_, Mode_,xSizeSB->GetValue(), ySizeSB->GetValue(), minbpp, maxbpp);
  //cout << "Min: " << minbpp << " Max: " << maxbpp << endl;
  BpPSB->SetRange(minbpp,maxbpp);
  bppMin_ = minbpp;
  bppMax_ = maxbpp;
#endif
}

void ShowCamWxFormat7Frame::OnScrollSlider( wxScrollEvent& event ){
  int id = event.GetId();
  //cout << "slide..." << id << endl;

  // check if the slider has a multiple of unit size
  if(id == ID_SB_XSIZE){ // X value
    ScrollSliderHelper(xSizeSB,xSize,xSizeMin_);
    AdjustBytesPerPacket();
  }

  if(id == ID_SB_XOFFSET){ // X value
    ScrollSliderHelper(xOffsetSB,xSize,xSizeMin_,xSizeMax_,xSizeSB,true);
  }

  if(id == ID_SB_YSIZE){ // Y Value
    ScrollSliderHelper(ySizeSB,ySize,ySizeMin_);
    AdjustBytesPerPacket();
  }

  if(id == ID_SB_YOFFSET){ // Y Value
    ScrollSliderHelper(yOffsetSB,ySize,ySizeMin_,ySizeMax_,ySizeSB,true);
  }

  if(id == ID_SB_BPP){ // Y Value
        int value = BpPSB->GetValue();
        int mod = value % bppMin_;
        if(mod != 0){
          if(mod > (bppMin_/2))
              value = value + (bppMin_ - mod);
          else
            value = value - mod;

          BpPSB->SetValue(value);

        }
      }
  CalcFPS();
}

void ShowCamWxFormat7Frame::OnSlide( wxScrollEvent& event ){
  if(event.GetId() == ID_SB_BPP){
    CalcFPS();
  }

  if(event.GetId() == ID_SB_XOFFSET){
    // we have to adjust the max of sliders x and y
    xSizeSB->SetMax(xSizeMax_ - xOffsetSB->GetValue());
    xSizeSB->Update();
  }

  if (event.GetId() == ID_SB_YOFFSET){
    ySizeSB->SetMax(ySizeMax_ - yOffsetSB->GetValue());
    //ySizeSB->Update();
    ySizeSB->Validate();
  }
}

void ShowCamWxFormat7Frame::CalcFPS(){

  int depth = 8;

  switch (colorModeCB->GetSelection()) {
      case 0:
      case 4:
      case 6:
        depth = 8;
        break;
      case 1:
        depth = 12;
        break;
      case 3:
        depth = 24;
        break;
      case 2:
      case 5:
      case 7:
        depth = 16;
        break;
    }

  int denom = 8 * BpPSB->GetValue();
  //cout << "denom" << denom << endl;

  int num_packets = (xSizeSB->GetValue() * ySizeSB->GetValue() * depth + denom -1)/denom;
  //cout << "num_packets" << num_packets << endl;
  int fps = 1.0/((double)0.000125 * (double)num_packets);
  //cout << fps << endl;
  wxString stringnumber = wxString::Format(wxT(" = %d fps"), (int)fps);
  //itemStaticTextFPS->ChangeValue(stringnumber);
  itemStaticTextFPS->SetValue(stringnumber);

}
