/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _FORMAT7FRAME_H_
#define _FORMAT7FRAME_H_

#include <Base/Common/BIASpragmaStart.hh>
#include <bias_config.h>
#include <Base/Image/ImageBase.hh>

#ifdef BIAS_HAVE_DCAM
 #include <VideoSource/VideoSource_DCAM.hh>
#endif

#include <wx/wx.h>
#include <wx/frame.h>
#include <wx/listctrl.h>
#include <vector>
#include <Base/Common/W32Compat.hh>

enum {
  ID_FORMAT7FRAME = wxID_HIGHEST+255,
  ID_BUTTON_OK,
  ID_BUTTON_CANCEL,
  ID_BUTTON_CHECK,
  ID_SB_XSIZE,
  ID_SB_YSIZE,
  ID_SB_XOFFSET,
  ID_SB_YOFFSET,
  ID_CB_COLORMODE,
  ID_CB_COLORMODEVIEW,
  ID_SB_BPP,
  ID_TEXT_FPS,
  ID_TEXT_DA_XOFFSET,
  ID_TEXT_DA_YOFFSET,
  ID_TEXT_DA_XSIZE,
  ID_TEXT_DA_YSIZE
};

namespace BIAS{

  class ShowCamWxFormat7Frame: public wxDialog
  {

  public:

    ShowCamWxFormat7Frame(wxFrame* parent,
                          const wxString& title,
                          const wxPoint& pos,
                          const wxSize& size);

    /// Destructor
    ~ShowCamWxFormat7Frame();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    /// wxEVT_CLOSE_WINDOW event handler for ID_PROJECTDEFINITIONFRAME
    void OnCloseWindow( wxCloseEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_ADDCCD
    void OnButtonOK( wxCommandEvent& event );
    void OnButtonExit( wxCommandEvent& event );
    void OnButtonApply( wxCommandEvent& event );
    void OnButtonCancel( wxCommandEvent& event );
    void OnButtonCheck( wxCommandEvent& event );
    void OnTextChange(wxCommandEvent& event);
    void OnScrollSlider( wxScrollEvent& event );
    void OnSlide( wxScrollEvent& event );

    void SetXSize(int xmin,int xmax){
      xSizeMin_=xmin; xSizeMax_=xmax;
      xSizeSB->SetRange(xmin,xmax);
      xSizeSB->SetValue(xmax-xmin);
      xSize = xmax;
      wxString stringnumber = wxString::Format(wxT("%d"), xSizeSB->GetValue());
      directAccessXSize->SetValue(stringnumber);
      CalcFPS();
    };
    void SetYSize(int ymin,int ymax){
      ySizeMin_ = ymin; ySizeMax_ = ymax;
      ySizeSB->SetRange(ymin,ymax);
      ySizeSB->SetValue(ymax-xSizeMin_);
      wxString stringnumber = wxString::Format(wxT("%d"), ySizeSB->GetValue());
      directAccessYSize->SetValue(stringnumber);
      CalcFPS();
    };
    void SetXOffset(int xoffmin,int xoffmax){
      xOffsetMin_ = xoffmin; xOffsetMax_ = xoffmax;
      xOffsetSB->SetRange(xoffmin,xoffmax);
      xOffsetSB->SetValue(xoffmin);
      CalcFPS();
    };
    void SetYOffset(int yoffmin,int yoffmax){
      yOffsetMin_ = yoffmin; yOffsetMax_ = yoffmax;
      yOffsetSB->SetRange(yoffmin,yoffmax);
      yOffsetSB->SetValue(yoffmin);
      CalcFPS();
    };

    void SetBpP(int min,int max,int value=-1){
      bppMin_ = min; bppMax_ = max;
      BpPSB->SetRange(min,max);
      if(value == -1)
        value = min;
      BpPSB->SetValue(value);
      CalcFPS();
    };
#ifdef BIAS_HAVE_DCAM
    void SetCameraIDandMode(int camid, dc1394video_mode_t mode ){
      CamID_ = camid;
      Mode_ = mode;
    };
#endif

    int xOffset,yOffset,xSize,ySize,colorMode,bpp;
    ImageBase::EColorModel colorModeView;
    int xSizeMin_,ySizeMin_, xSizeMax_,ySizeMax_,xOffsetMin_,xOffsetMax_;
    int yOffsetMin_,yOffsetMax_;
    int bppMin_, bppMax_;

  protected:
    void CalcFPS();
    void ScrollSliderHelper(wxSlider* slider, int& size, int& sizeMin, int sizeMax = 0, wxSlider* sliderSize = NULL, bool isOffset = false);
    void AdjustBytesPerPacket();
    wxButton* itemButtonOK;
    wxButton* itemButtonCancel;
    wxButton* itemButtonCheck;

    wxTextCtrl* itemTextCtrlXSize;
    wxTextCtrl* itemTextCtrlYSize;
    wxTextCtrl* itemTextCtrlXOffset;
    wxTextCtrl* itemTextCtrlYOffset;
    wxTextCtrl* itemTextCtrlPacketsize;
    wxTextCtrl* itemTextCtrlBpP;

    wxTextCtrl* itemStaticTextFPS;


    wxTextCtrl* directAccessXSize;
    wxTextCtrl* directAccessYSize;
    wxTextCtrl* directAccessXOffset;
    wxTextCtrl* directAccessYOffset;

    wxSlider* xSizeSB;
    wxSlider* ySizeSB;
    wxSlider* xOffsetSB;
    wxSlider* yOffsetSB;
    wxChoice* colorModeCB; // mode for fetching the image from cam
    wxChoice* colorModeViewCB;  // mode for showing image in showcam
    wxSlider* BpPSB;

    int CamID_;
#ifdef BIAS_HAVE_DCAM
    dc1394video_mode_t Mode_;
    VideoSource_DCAM* dcam;
#endif
    DECLARE_CLASS( ShowCamWxFormat7Frame )
    DECLARE_EVENT_TABLE()
  };
}
#endif
