/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Base/Common/W32Compat.hh"
#include "ShowCamWxFrame.hh"
#include "ShowCamWxThreads.hh"
#include "ShowCamWxFormat7Frame.hh"
//logos
//#include <img/miplogo32x32.xpm>
#include <img/miplogochars64x64.xpm>
#include <wx/artprov.h>
#include <wx/aboutdlg.h>

using namespace BIAS;
using namespace std;

#define PARAM_FILE_NAME "ShowCamWx.param"
#define BUFFER_SIZE 50

//#define AUTOSCAN

BEGIN_EVENT_TABLE(ShowCamWxFrame, wxFrame)

  EVT_CLOSE     (                       ShowCamWxFrame::OnWindowClose)
  EVT_KEY_DOWN  (                       ShowCamWxFrame::OnKey)
  EVT_MENU      (wxID_ABOUT,            ShowCamWxFrame::OnAbout)
  EVT_MENU      (ID_Quit,               ShowCamWxFrame::OnQuit)
  EVT_MENU      (ID_Grab,               ShowCamWxFrame::OnGrab)
  EVT_MENU      (ID_GrabContinuous,     ShowCamWxFrame::OnGrabContinuous)
  EVT_MENU      (ID_GrabToStream,       ShowCamWxFrame::OnGrabToStream)
  EVT_MENU      (ID_Show1,              ShowCamWxFrame::OnShowImage1)
  EVT_MENU      (ID_Show2,              ShowCamWxFrame::OnShowImage2)
  EVT_MENU      (ID_Show3,              ShowCamWxFrame::OnShowImage3)
  EVT_MENU      (ID_Show4,              ShowCamWxFrame::OnShowImage4)
  EVT_MENU      (ID_Show5,              ShowCamWxFrame::OnShowImage5)
  EVT_MENU      (ID_Show6,              ShowCamWxFrame::OnShowImage6)
  EVT_MENU      (ID_Show7,              ShowCamWxFrame::OnShowImage7)
  EVT_MENU      (ID_Show8,              ShowCamWxFrame::OnShowImage8)
  EVT_MENU      (ID_Show9,              ShowCamWxFrame::OnShowImage9)
  EVT_MENU      (ID_Controller,         ShowCamWxFrame::OnShowController)
  EVT_MENU      (ID_SaveParameters,     ShowCamWxFrame::OnSaveParameters)
  EVT_MENU      (ID_ShowParameters,     ShowCamWxFrame::OnShowParameters)
  EVT_MENU      (ID_FindIEEE1394,       ShowCamWxFrame::OnFindIEEE1394)
  EVT_MENU      (ID_FindUEYE,           ShowCamWxFrame::OnFindUEye)
  EVT_MENU      (ID_FindZess,           ShowCamWxFrame::OnFindPMDZess)
  EVT_MENU      (ID_FindPMDTec,         ShowCamWxFrame::OnFindPMDTec)
  EVT_MENU      (ID_FindSwissranger,    ShowCamWxFrame::OnFindSwissRanger)
  EVT_MENU      (ID_FindWebcams,        ShowCamWxFrame::OnFindWebcams)
  
  EVT_TIMER     (ID_Timer,              ShowCamWxFrame::OnTimer)
  EVT_CHECKBOX  (ID_Fit,                ShowCamWxFrame::OnFit)
  EVT_CHECKBOX  (ID_KeepScroll,         ShowCamWxFrame::OnKeepScrollPosition)
  
  EVT_BUTTON      (ID_Grab,               ShowCamWxFrame::OnGrab)
  EVT_BUTTON      (ID_GrabContinuous,     ShowCamWxFrame::OnGrabContinuous)
  EVT_BUTTON      (ID_GrabToStream,       ShowCamWxFrame::OnGrabToStream)
  EVT_BUTTON      (ID_FindWebcams,        ShowCamWxFrame::OnFindWebcams)
  EVT_BUTTON      (ID_FindIEEE1394,       ShowCamWxFrame::OnFindIEEE1394)
  EVT_BUTTON      (ID_FindPMDTec,         ShowCamWxFrame::OnFindPMDTec)
  EVT_BUTTON      (ID_FindSwissranger,    ShowCamWxFrame::OnFindSwissRanger)


  END_EVENT_TABLE()


/****************************************************
   @brief


*****************************************************/
ShowCamWxFrame::
ShowCamWxFrame(const wxString& title,
                               const wxPoint& pos, const wxSize& size)
  : wxFrame(NULL, -1, title, pos, size) ,
    ParamGUI ()   // disable param desctructor warning

{
#ifdef WIN32
  //  SetIcon(wxIcon(wxT(miplogo32x32_xpm)));
  SetIcon(wxIcon(wxT(miplogochars64x64_xpm)));
#else
  //  SetIcon(wxICON(miplogo32x32));
  SetIcon(wxICON(miplogochars64x64));
#endif
  // set default value for class variables
  bSingleSaveDirCreated_ = false;
  dSingleDirIndex_ = 0;
  dDirIndex_=0;
  paramFrame_ =NULL;
  OpenedDevices_    = false;
  InitCameras_ = false;
  InitParams_  = false;
  bSaveImages_ = false;
  bWriteBuffered_ = false;
  EmptyDevice_ = false;
  FrameCounter_ = 0;
  GrabCounter_  = 0;
  LastWidth_    = 0;
  LastHeight_   = 0;
  NumCamSel_    = 0;
  Win32DCAMNumber_ = 0;
  ShowXB3_ = 0;
#ifdef BIAS_HAVE_DCAM
  format7Frame_=NULL;
#endif
  FirstCameraID_ = 10000;
  LastCameraID_ = FirstCameraID_;
  TimerActive_ = false;
  ActiveCameras_ = 0;

  NumOfCams_ = 0;
  grabMasterThread_ = NULL;
  menuShow = NULL;

  CleanedUp_ = false;

#ifdef BIAS_HAVE_PanTilt
  panTiltControl_=NULL;
  panTiltControlDialog_=NULL;
#endif //BIAS_HAVE_PanTilt

  // set timer delay to 5 ms
  Delay_ = 25;
  Timer_.SetOwner(this, ID_Timer);

  // init images for cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++)
  {
    // init Images
    CamImg_.push_back(NULL);
    CamImgMiddle_.push_back(NULL);
    CamImgRight_.push_back(NULL);

    CamImgPMDDepth_.push_back(NULL);
    CamImgPMDAmp_.push_back(NULL);
    CamImgPMDInt_.push_back(NULL);
    XB3AllInOne_.push_back(NULL);

    ThreadInited_.push_back(false);
    CameraActive_.push_back(false);
    CameraInitialized_.push_back(false);
    SetFirewireB_.push_back(false);
    SetTrigger_.push_back(false);
    SetUserMode_.push_back(true);
    SelectedFps_.push_back(15.0);
    Format7LeftTop_.push_back(Vector2<int>(0,0));
    Format7WidthHeight_.push_back(Vector2<int>(0,0));
    Format7BpP_.push_back(0);
    Format7ColorMode_.push_back(0);

    grabSingleThread_.push_back(NULL);
    saveThreads_.push_back(NULL);

	  grabMasterThread_ = NULL;

    // Ringbuffer for writing images
    BufferAccess_.push_back(NULL);
    WriteIndex_.push_back(0);
    ReadIndex_.push_back(BUFFER_SIZE-1);

    ImageBuffer_.push_back(new vector<Camera<unsigned char>* > );
    ImageBufferMiddle_.push_back(new vector<Camera<unsigned char>* >);
    ImageBufferRight_.push_back(new vector<Camera<unsigned char>* >);
    ImageBufferPMDDepth_.push_back(new vector<Camera<float>* >);
    ImageBufferPMDAmp_.push_back(new vector<Camera<float>* >);
    ImageBufferPMDInt_.push_back(new vector<Camera<float>* >);

    for(int j=0;j<BUFFER_SIZE;j++){
      ImageBuffer_[i]->push_back(NULL);
      ImageBufferMiddle_[i]->push_back(NULL);
      ImageBufferRight_[i]->push_back(NULL);
      ImageBufferPMDDepth_[i]->push_back(NULL);
      ImageBufferPMDAmp_[i]->push_back(NULL);
      ImageBufferPMDInt_[i]->push_back(NULL);
    }

    ColorModels_.push_back(ImageBase::CM_invalid);
    ViewColorMode_.push_back(ImageBase::CM_invalid);
    VideoControllers_.push_back(NULL);

#ifdef BIAS_HAVE_DCAM
    VideoControllersXB3_.push_back(NULL);
#endif

#ifdef BIAS_HAVE_PMD_PMDZess
    VideoControllersUSBPMD_.push_back(NULL);
#endif //BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_PMD_PMDTec
    VideoControllersPMD_.push_back(NULL);
#endif //BIAS_HAVE_PMD_PMDTec

#ifdef BIAS_HAVE_SWISSRANGER
    VideoControllersSwissRanger_.push_back(NULL);
#endif // BIAS_HAVE_SWISSRANGER

  }
 // create tool bar
  wxToolBar* toolBar = CreateToolBar();
  toolBar->SetToolBitmapSize( wxSize( 24,24 ) );
  toolBar->SetToolSeparation( 0 );
  toolBar->SetToolPacking( 0 );
  
  pButtonGrab= new wxButton(toolBar, ID_Grab, wxT("Grab"),wxDefaultPosition);
  pButtonGrab->SetToolTip(wxT("Grab single image."));
  toolBar->AddControl(pButtonGrab);
  
  pButtonContinuous= new wxButton(toolBar, ID_GrabContinuous, wxT("Grab Cont."),wxDefaultPosition);
  pButtonContinuous->SetToolTip(wxT("Toggle grabbing continuously."));
  toolBar->AddControl(pButtonContinuous);
  
  // toolBar->AddCheckTool( ID_GrabContinuous, wxT("Toggle grabbing continuously."),
  //												wxArtProvider::GetIcon(wxART_ERROR ,wxART_TOOLBAR) );
  
  pButtonStream= new wxButton(toolBar, ID_GrabToStream, wxT("Grab To Stream"),wxDefaultPosition);
  pButtonStream->SetToolTip(wxT("Toggle grabbing to stream."));
  toolBar->AddControl(pButtonStream);
  toolBar->AddSeparator();
  
  pButtonIEEE1394= new wxButton(toolBar,ID_FindIEEE1394, wxT("Find IEEE1394"),wxDefaultPosition);
  pButtonIEEE1394->SetToolTip(wxT("Find Firewire IEEE1394 cameras."));
  toolBar->AddControl(pButtonIEEE1394);
#ifdef BIAS_HAVE_V4L
  pButtonWebCams= new wxButton(toolBar,ID_FindWebcams, wxT("Find WebCams"),wxDefaultPosition);
  pButtonWebCams->SetToolTip(wxT("Find V4L Cameras."));
  toolBar->AddControl(pButtonWebCams);
#endif
#ifdef BIAS_HAVE_DSHOW
  pButtonWebCams= new wxButton(toolBar,ID_FindWebcams, wxT("Find WebCams"),wxDefaultPosition);
  pButtonWebCams->SetToolTip(wxT("Find DSHOW Cameras."));
  toolBar->AddControl(pButtonWebCams);
#endif
  
  pButtonSR= new wxButton(toolBar,ID_FindSwissranger, wxT("Find SR"),wxDefaultPosition);
  pButtonSR->SetToolTip(wxT("Find Swissranger ToF cameras."));
  toolBar->AddControl(pButtonSR);
  pButtonPMDTec= new wxButton(toolBar,ID_FindPMDTec, wxT("Find PMDTec"),wxDefaultPosition);
  pButtonPMDTec->SetToolTip(wxT("Find PMDTec ToF cameras."));
  toolBar->AddControl(pButtonPMDTec);
  toolBar->AddSeparator();
  
  pCheckBoxFit = new wxCheckBox(toolBar, ID_Fit, wxT("Fit window"));
  toolBar->AddControl(pCheckBoxFit);
  pCheckBoxKeepScrollPosition = new wxCheckBox(toolBar, ID_KeepScroll, wxT("Keep Scroll"));
  toolBar->AddControl(pCheckBoxKeepScrollPosition);
   
  toolBar->Realize();


  CreateCameraMenu();
  // create status bar and add status text
  CreateStatusBar(3);
  wxString text;
  text.Printf(wxT("size: %4dx%4d"),0, 0);
  SetStatusText(text, 1);

  // init image canvas
  IC_ = new ImageCanvas(this,GetStatusBar(), 0,-1,wxDefaultPosition,wxDefaultSize);
  IC_->SetEvtHandlerEnabled(true);
  IC_->KeepScrollPosition(true);
  //ShowImage_.Init(640,480,3);
  string img = BIAS_IMG_DIR;
  img+= "TestbildMIP.gif"; 
  BIAS::ImageIO::Load(img,ShowImage_);
  //unsigned char color[3]={255,255,255};
  //ShowImage_.FillImageWithConstValue(color);
  IC_->Show(ShowImage_,"Camera Image");

   
  Timer_.Start(Delay_);
  
  // add image canvas to main window
  wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
  vbox->Add(IC_,1,  wxEXPAND |wxALL , 5); //|wxFIXED_MINSIZE , 5);
  SetMinSize(size);
  SetSizer(vbox);
  SetAutoLayout(true);
  Layout();
 
}

ShowCamWxFrame::
~ShowCamWxFrame()
{
  // close threads
  for(int i = 0; i < NUMBER_CAMERAS; i++){
    if(grabSingleThread_[i] != NULL){
      delete grabSingleThread_[i];
    }
    if(saveThreads_[i] != NULL){
      delete saveThreads_[i];
    }
  }
}
/****************************************************
   @brief


*****************************************************/

void ShowCamWxFrame::
InitParams(int &argc, char *argv[])
{
  // initialize camera names for params
  vector<string> cameras;
  cameras.push_back("NONE");
  cameras.push_back("DCAM");
  cameras.push_back("PMD");
  cameras.push_back("PMDUSB");
  cameras.push_back("PMDSHM");
  cameras.push_back("UEYE");
  cameras.push_back("SHM");
  cameras.push_back("NET");
  cameras.push_back("V4L");
  cameras.push_back("SR");
  cameras.push_back("XB3");
  cameras.push_back("DSHOW");

  // initialize color models for cameras
  vector<string> colorModels;
  colorModels.push_back("CM_Default");
  colorModels.push_back("CM_Grey");
  colorModels.push_back("CM_RGB");
  colorModels.push_back("CM_BGR");
  colorModels.push_back("CM_YUYV422");
  colorModels.push_back("CM_UYVY422");
  colorModels.push_back("CM_YUV420P");
  colorModels.push_back("CM_YUV444");
  colorModels.push_back("CM_YUV411");
  colorModels.push_back("CM_HSV");
  colorModels.push_back("CM_HSI_OBS");
  colorModels.push_back("CM_DV");
  colorModels.push_back("CM_RGBA");
  colorModels.push_back("CM_GreyA");
  colorModels.push_back("CM_Bayer_RGGB");
  colorModels.push_back("CM_Bayer_GBRG");
  colorModels.push_back("CM_Bayer_GRBG");
  colorModels.push_back("CM_Bayer_BGGR");
  colorModels.push_back("CM_HSL");
  colorModels.push_back("CM_hsL");
  colorModels.push_back("CM_SymTensor2x2");
  colorModels.push_back("CM_BGRA");
  colorModels.push_back("CM_RGBE");



  for (int i = 0; i < NUMBER_CAMERAS; i++)
    {
      // add nine different cameras to params and set group name
      stringstream number;
      number << i + 1;
      ParamCameras_[i] = AddParamEnum("CameraType" + number.str(),
                                      "Type of camera", cameras, 0, NULL, 0, i);
      ParamColorModel_[i] = AddParamEnum("CameraColorModel" + number.str(),
                                         "Color Model for camera", colorModels,
                                         0, NULL, 0, i);
      ParamFPS_[i]    = AddParamDouble("CameraFPS" + number.str(),
                                       "fps possibilities: 1.875 3.75 7.5 15.0 30.0 60.0 120.0 240.0",
                                       0.0, 0.0, 240.0, 0, i);
      ParamWidth_[i]  = AddParamInt("CameraWidth" + number.str(),
                                    "Width of camera", 0, 0, 10000, 0, i);
      ParamHeight_[i] = AddParamInt("CameraHeight" + number.str(),
                                    "Height of camera", 0, 0, 10000, 0, i);
      //ParamSetTop_[i]  = AddParamInt();
      //ParamSetLeft_[i] = AddParamInt();
      ParamDown_[i]   = AddParamInt("CameraDownsampling" + number.str(),
                                    "Downsampling of camera", 0, 0, 10, 0, i);
      ParamPort_[i]   = AddParamInt("CameraPort" + number.str(),
                                    "Port of NET camera", 0, 0, 100000, 0, i);
      ParamIP_[i]     = AddParamString("CameraIP" + number.str(),
                                       "IP of NET camera", "", 0, i);
      ParamBitFile_[i]= AddParamString("CameraBitFile" + number.str(),
                                       "Bit file for usb pmd cameras", "", 0, i);
      ParamDevice_[i] = AddParamString("CameraDevice" + number.str(),
                                       "Device/Interface name of camera",
                                       "", 0, i);
      // every camera gets own group number
      stringstream text;
      text<<number.str()<<". Camera";
      SetGroupName(i, text.str());
    }
#ifdef BIAS_HAVE_PanTilt
  bUsePanTiltUnit_ = AddParamBool("UsePanTiltUnit","Use the Pan-Tilt-Unit?",false,
                                  'p', NUMBER_CAMERAS);
  stringstream text;
  text<<"General";
  SetGroupName(NUMBER_CAMERAS, text.str());
#endif
  // check if param file exists, else create one
  ifstream ifs(PARAM_FILE_NAME);
  if (!ifs)
    WriteParameter(PARAM_FILE_NAME);
  else
    {
      // read param from file
      ReadParameter(PARAM_FILE_NAME);
      // update parameters from file with arguments from command line
      UpdateParameter(argc, argv, PARAM_FILE_NAME);
      ifs.close();
    }

  // get color models for cameras
  ParseColorModels();

  InitParams_ = true;
}

#ifdef BIAS_HAVE_PanTilt
/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
InitPanTiltUnit(){
  if(*bUsePanTiltUnit_){
    panTiltControl_= new DPPanTiltControl();
    bool bRet = false;
#ifdef WIN32
    bRet = panTiltControl_->InitPanTiltUnit("COM1");
#else
    bRet = panTiltControl_->InitPanTiltUnit("/dev/ttyUSB0");
#endif
    panTiltControlDialog_ = new DPPanTiltControlDialog(this, 7651,
                                  wxT("Pan-Tilt-Unit Control"));
    panTiltControlDialog_->SetDPControl(panTiltControl_);
    panTiltControlDialog_->Show(true);
  }
}
#endif //BIAS_HAVE_PanTilt

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
InitCameras()
{
  if (!InitParams_){
    BIASERR("Initialize params first");
    return;
  }

  bool foundCamera = false;
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    Cameras_.push_back(NULL);
    if (*ParamCameras_[i] != 0)
      foundCamera = true;
  }


  //if there were no cams found, exit and wait for a manual camera scan.
  if (!foundCamera) {
    return;
  }else{
#ifndef WIN32
#  ifdef BIAS_HAVE_DCAM
    int numDcams = 0;
#  endif //BIAS_HAVE_DCAM
    for(int i = 0; i < NUMBER_CAMERAS; i++){

      stringstream errorText;
      // scan for dcam camera parameters
      if(*ParamCameras_[i] == 1){
#ifdef BIAS_HAVE_DCAM
        ScanBus(IEEECameraInfo_);

        if(CameraMenu_ == NULL)
          CameraMenu_ = new wxMenu;
        // search for the cam
        for(unsigned int k=0; k< IEEECameraInfo_.size();k++){
          if(IEEECameraInfo_[k].camid == atoi((*ParamDevice_[i]).c_str())){
            // add to menu
            AddIEEECameraToMenu(i,k,true);
          }
        }
        numDcams++;
#else
        errorText << "DCAM Camera in param file, please enable support in BIAS\n";
        BIASERR("DCAM Camera in param file, please enable support in BIAS");

#endif
      }
#else
      int NumDcams = 0;
      for(int i = 0; i < NUMBER_CAMERAS; i++){
        stringstream errorText;
        // scan for dcam camera parameters
        if(*ParamCameras_[i] == 1){
#ifdef BIAS_HAVE_DCAM
          ScanBus(IEEECameraInfo_);

          if(CameraMenu_ == NULL)
            CameraMenu_ = new wxMenu;
          // search for the cam
          for(unsigned int k=0; k< IEEECameraInfo_.size();k++){
            if(IEEECameraInfo_[k].camid == NumDcams + 1){
              // add to menu
              stringstream ss;
              ss << NumDcams + 1;
              *ParamDevice_[i] = ss.str();
              AddIEEECameraToMenu(i,k,true);

            }
          }
          NumDcams++;
#else
          errorText << "DCAM Camera in param file, please enable support in BIAS\n";
          BIASERR("DCAM Camera in param file, please enable support in BIAS");

#endif
        }
#endif
        if(*ParamCameras_[i] == 2){
#ifdef BIAS_HAVE_PMD_PMDTec
          AddPMDTecCameraToMenu(i,true);
#else
          errorText << "PMDTec Camera in param file, please enable support in BIAS\n";
          BIASERR("PMDTec Camera in param file, please enable support in BIAS");
#endif
        }



        if(*ParamCameras_[i] == 3){
#ifdef BIAS_HAVE_PMD_PMDZess
          AddZessCameraToMenu(i,true,"");
#else
          errorText << "PMD Zess Camera in param file, please enable support in BIAS\n";
          BIASERR("PMD Zess Camera in param file, please enable support in BIAS");
#endif
        }

        if(*ParamCameras_[i] == 5){
#ifdef BIAS_HAVE_UEYE
          AddUEyeCameraToMenu(i,true);
#else
          errorText << "UEYE Camera in param file, please enable support in BIAS\n";
          BIASERR("UEYE Camera in param file, please enable support in BIAS");
#endif
        }

        if(*ParamCameras_[i] == 9){
#ifdef BIAS_HAVE_SWISSRANGER
          AddSwissRangerCameraToMenu(i,true);
#else
          errorText << "SwissRanger Camera in param file, please enable support in BIAS\n";
          BIASERR("SwissRanger Camera in param file, please enable support in BIAS");
#endif
        }


      if(*ParamCameras_[i] != 0) {
        cout <<"INIT CAM" << endl;
        InitCamera(i);
      }
    }
  }

  InitCameras_ = true;

  OpenDevices();
}

///////////////////////////////////////////////////////
void ShowCamWxFrame::InitCamera(int camindex){
    int i = camindex;
    stringstream errorText;
    bool bCameraError = false;

    BufferAccess_[camindex] = new wxMutex;
    WriteIndex_[camindex] = 0;
    ReadIndex_[camindex] = BUFFER_SIZE - 1;

    if(CamImg_[camindex] == NULL)
      CamImg_[camindex] = new Camera<unsigned char> ;

    //CamImgCombined_.Release();

    switch(*ParamCameras_[i]){
    case 0: // None camera
      // Cameras_.push_back(NULL);
      break;
    case 1: // DCAM
#ifdef BIAS_HAVE_DCAM
      Cameras_[i] = new VideoSource_DCAM;
      Cameras_[i]->SetDebugLevel(D_DCAM);
      dynamic_cast<VideoSource_DCAM*>(Cameras_[i])->SetFirewireB(SetFirewireB_[i]);
      
      if(SetUserMode_[i]){
        dc1394video_mode_t mode = SelectedModeFromMenu(i);

        // choose between normal mode and format 7
        if (mode >= DC1394_VIDEO_MODE_FORMAT7_0) {
          dynamic_cast<VideoSource_DCAM*> (Cameras_[i])-> SetFormat7(mode
              - DC1394_VIDEO_MODE_FORMAT7_MIN, Format7BpP_[i],
              (dc1394color_coding_t) Format7ColorMode_[i]);
          dynamic_cast<VideoSource_DCAM*> (Cameras_[i])-> SetLeftTop(
              Format7LeftTop_[i][0], Format7LeftTop_[i][1]);
          dynamic_cast<VideoSource_DCAM*> (Cameras_[i])-> SetSize(
              Format7WidthHeight_[i][0], Format7WidthHeight_[i][1]);
        }else{
          dynamic_cast<VideoSource_DCAM*>(Cameras_[i])->SetModeAndFramerate(mode,SelectedFps_[i]);
        }
      }
#else
      errorText << "DCAM Camera in param file, please enable support in BIAS\n";
      bCameraError = true;
      //Cameras_.push_back(NULL);
      BIASERR("DCAM Camera in param file, please enable support in BIAS");
#endif

      break;
    case 2: // PMD

#ifdef BIAS_HAVE_PMD_PMDTec
      Cameras_[i] = new VideoSource_PMD;
      
      if(CamImgPMDDepth_[camindex] == NULL)
        CamImgPMDDepth_[camindex] = new Camera<float> ;
      if(CamImgPMDAmp_[camindex] == NULL)
        CamImgPMDAmp_[camindex] = new Camera<float> ;
      if(CamImgPMDInt_[camindex] == NULL)
        CamImgPMDInt_[camindex] = new Camera<float> ;
#else
      bCameraError = true;
      errorText <<"PMDTec Camera in param file, please enable support in BIAS\n";
      //Cameras_[i]NULL);
      BIASERR("PMDTec Camera in param file, please enable support in BIAS");
#endif

      break;
    case 3: // PMDUSB

#ifdef BIAS_HAVE_PMD_PMDZess
      Cameras_[i] = new VideoSource_usbPMD;
      
      if (*ParamBitFile_[i] != "")
        dynamic_cast<VideoSource_usbPMD*>(Cameras_[i])->SetBitFile(*ParamBitFile_[i]);
      else
        {
          BIASERR("Specify bit file for usb pmd camera");
          bCameraError = true;
          errorText << "Specify bit file for ZESS PMD camera!\n";
        }
      if(CamImgPMDDepth_[camindex] == NULL)
        CamImgPMDDepth_[camindex] = new Camera<float> ;
      if(CamImgPMDAmp_[camindex] == NULL)
        CamImgPMDAmp_[camindex] = new Camera<float> ;
#else
      bCameraError = true;
      //Cameras_.push_back(NULL);
      errorText << "Zess PMD Camera in param file, please enable support in BIAS\n";
      BIASERR("Zess PMD Camera in param file, please enable support in BIAS");
#endif

      break;
    case 4: // PMDSHM

#ifdef BIAS_HAVE_TIFF
      Cameras_[i] = new VideoSource_ShmPMD;
      
#else
      bCameraError = true;
      //Cameras_.push_back(NULL);
      errorText << "SHM PMD Camera in param file, please enable support in BIAS\n";
      BIASERR("SHM PMD Camera in param file, please enable support in BIAS");
#endif

      break;
    case 5: // UEYE

#ifdef BIAS_HAVE_UEYE
      Cameras_[i] = new VideoSource_uEye;
      
      dynamic_cast<VideoSource_uEye*>(Cameras_[i])->SetLeftTop(56,0);
      dynamic_cast<VideoSource_uEye*>(Cameras_[i])->SetShutter(0.01f);
      dynamic_cast<VideoSource_uEye*>(Cameras_[i])->SetGain(0);
#else
      bCameraError = true;
      //Cameras_.push_back(NULL);
      errorText << "UEYE Camera in param file, please enable support in BIAS\n";
      BIASERR("UEYE Camera in param file, please enable support in BIAS");
#endif

      break;
    case 6: // SHM
      Cameras_[i] = new VideoSource_Shm;      
      break;
    case 7: // NET
      Cameras_[i] = new VideoSource_Net;      
      break;
    case 8: { // V4L
      bool defined = false;
#if defined  BIAS_HAVE_V4L      
    Cameras_[i] = new VideoSource_V4L;      
    defined = true;
#endif
#ifdef BIAS_HAVE_DSHOW
    Cameras_[i] = new VideoSource_DSHOW(false);  
    dynamic_cast<VideoSource_DSHOW*>(Cameras_[i])->SetNewImageCheck(true);
    BIASWARN("Checking DSHOW Camera for new image by image memory comparison.");
    defined = true;
#endif
    if(!defined){
      bCameraError = true;
      //   Cameras_.push_back(NULL);
      errorText << "V4L Camera in param file, please enable support in BIAS\n";
      BIASERR("V4L Camera in param file, please enable support in BIAS");
    }
    break;
    }
    case 9: // SR
#ifdef BIAS_HAVE_SWISSRANGER
      Cameras_[i] = new VideoSource_SwissRanger;
      
      if(CamImgPMDDepth_[camindex] == NULL)
        CamImgPMDDepth_[camindex] = new Camera<float> ;
      if(CamImgPMDAmp_[camindex] == NULL)
        CamImgPMDAmp_[camindex] = new Camera<float> ;
#else
      bCameraError = true;
      //Cameras_.push_back(NULL);
      errorText << "SwissRanger Camera in param file, please enable support in BIAS\n";
      BIASERR("SwissRanger in param file, please enable support in BIAS");
#endif
      break;
    case 10: // XB3
      //# ifndef WIN32
#ifdef BIAS_HAVE_DCAM
      cout << "i am a xb3" << endl;
      Cameras_[i] = new VideoSource_DCAM_XB3;
      
      Cameras_[i]->SetDebugLevel(D_DCAM|D_DCAM_F7);

      Cameras_[i]->SetColorModel(ImageBase::CM_PGR_XB3_F7M3_GBRG);
      ColorModels_[i] = ImageBase::CM_PGR_XB3_F7M3_GBRG;

      if(CamImgMiddle_[camindex] == NULL)
        CamImgMiddle_[camindex] = new Camera<unsigned char> ;
      if(CamImgRight_[camindex] == NULL)
        CamImgRight_[camindex] = new Camera<unsigned char> ;

      if( (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])) != NULL){
        dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->SetFormat7(3,7680,DC1394_COLOR_CODING_RGB8);
        dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->SetLeftTop(0,0);
        dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i])->SetSize(1280,960,3);
      }
#else
      bCameraError = true;
      //Cameras_.push_back(NULL);
      errorText << "DCAM (Bumblebee) Camera in param file, please enable support in BIAS\n";
      BIASERR("DCAM (Bumblebee) in param file, please enable support in BIAS");
#endif

      break;
    case 11: // DSHOW
#ifdef BIAS_HAVE_DSHOW
      cout <<"I am a DShow cam"<<endl;
      Cameras_[i] = new VideoSource_DSHOW;
      
#else
      bCameraError = true;
      errorText << "DSHOW Camera in param file, please enable support in BIAS\n";
      BIASERR("DShow  in param file, please enable support in BIAS");
#endif
      break;
    default: // None
      //Cameras_.push_back(NULL);
      break;
    }

    if(bCameraError){
      wxMessageDialog errorDialog1(this,
                                   AsciiToWx(errorText.str().c_str()),
                                   wxT("ShowCamWx - Error"),
                                   wxICON_ERROR|wxOK);
      errorDialog1.ShowModal();
    }

    if (*ParamHeight_[i] != 0 && *ParamWidth_[i] != 0 && Cameras_[i] != NULL) {
      Cameras_[i]->SetSize(*ParamWidth_[i], *ParamHeight_[i]);
       //cout<<"Setting size to :"<<*ParamWidth_[i]<<", "<< *ParamHeight_[i]<<endl;
    }

    if (*ParamFPS_[i] != 0.0 && Cameras_[i] != NULL) {
      cout<<"Setting "<<i<<". camera fps to "<<*ParamFPS_[i]<<endl;
      Cameras_[i]->SetFPS((float) *ParamFPS_[i]);
    }

    CameraInitialized_[i] = true;
  }//end Init Cameras



/****************************************************
   @brief


*****************************************************/
int ShowCamWxFrame::OpenDevice(int camindex) {

  // we have to check which camera type we have
  int ret = 0;
  int i = camindex;
  if(*ParamCameras_[i] == 0)
    return -1;
  
  if(!CameraInitialized_[i]){
   InitCamera(i);
  }

  Cameras_[i]->SetExternalTrigger(SetTrigger_[i]);
        
  switch(*ParamCameras_[i])
    {
#ifdef BIAS_HAVE_DCAM
    case 10: // XB3
    case 1: // DCAM
    
      if (*ParamDevice_[i] != "")  {
        Cameras_[i]->AddDebugLevel(D_DCAM);

//        dynamic_cast<VideoSource_DCAM*>(Cameras_[i])->SetLeftTop(12,12);
//        dynamic_cast<VideoSource_DCAM*>(Cameras_[i])->SetSize(1024,768);
//        dynamic_cast<VideoSource_DCAM*>(Cameras_[i])->SetFormat7(0,4096,0);

        if (Cameras_[i]->OpenDevice(atoi((*ParamDevice_[i]).c_str())) < 0)
          {
            /*
              Cameras_[i]->CloseDevice();
              delete Cameras_[i];
              Cameras_[i] = NULL;
              cout<<"Card 0 failed, trying card 1:"<<endl;
              Cameras_[i] = new VideoSource_DCAM(1);
              Cameras_[i]->AddDebugLevel(D_DCAM);

              if (Cameras_[i]->OpenDevice(atoi((*ParamDevice_[i]).c_str())) < 0){
              Cameras_[i]->CloseDevice();
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "DCAM Camera number " << i+1 <<" "<<(*ParamDevice_[i]).c_str()
              <<" on Card 1 could not be openend";
              wxMessageDialog errorDialog(this,
              AsciiToWx(text.str().c_str()),
              wxT("ShowCamWx - Error"),
              wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
              }
              else{
              cout<<"camera number " << i+1 << " "<<atoi((*ParamDevice_[i]).c_str())
              <<" successfully opened on card 1"<<endl;
              }
            */
            return -1;
        }
        else
          cout<<"Camera number " << i+1 << " successfully opened"<<endl;
      }
      else  {
#ifdef WIN32
        // this does not work if not all cameras are dcam!
        if (Cameras_[i]->OpenDevice(Win32DCAMNumber_++) < 0)
          {
            Cameras_[i]->CloseDevice();
            delete Cameras_[i];
            Cameras_[i] = NULL;
            stringstream text;
            text << "Camera number " << i << " could not be openend";
            wxMessageDialog errorDialog(this,
                                        AsciiToWx(text.str().c_str()),
                                        wxT("ShowCamWx - Error"),
                                        wxICON_ERROR|wxOK);
            errorDialog.ShowModal();
            return -1;
          }
#else
        // check if there are multiple cameras with no device name

        if (EmptyDevice_) {
          stringstream text;
          text << "Found more than one camera with no device name!\n";
          //   <<"Available cameras:\n"<<cameras.str().c_str();
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
        }
        // going here the first time sets on empty device
        EmptyDevice_ = true;
        if (Cameras_[i]->OpenDevice() < 0) {
          delete Cameras_[i];
          Cameras_[i] = NULL;
          stringstream text;
          text << "Test Camera number " << i+1 << " could not be openend";
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
        }
#endif // WIN32
      }
      break;
#endif // BIAS_HAVE_DCAM

#ifdef BIAS_HAVE_PMD_PMDTec
    case 2: // PMD
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }
      if (Cameras_[i]->OpenDevice()<0)
        {
          delete Cameras_[i];
          Cameras_[i] = NULL;
          stringstream text;
          text << "Camera PMDTec " << i << " could not be openend";
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
          return -1;
        }
      ((VideoSource_PMD*)Cameras_[i])->SetIntegrationTime(4000);
      ((VideoSource_PMD*)Cameras_[i])->SetFPNCalibrationOn(false);
      break;
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
    case 3: // PMDUSB
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }
      if (Cameras_[i]->OpenDevice()<0)
        {
          delete Cameras_[i];
          Cameras_[i] = NULL;
          stringstream text;
          text << "Camera PMD ZESS " << i << " could not be openend";
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
          return -1;
        }
      break;
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
    case 4: // PMDSHM
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }
      if ((Cameras_[i])->OpenDevice((*ParamDevice_[i]).c_str()) < 0)
        {
          delete Cameras_[i];
          Cameras_[i] = NULL;
          stringstream text;
          text << "Camera PMD Shared Memory " << i << " could not be openend";
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
          return -1;
        }
      break;
#endif // BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_UEYE
    case 5: // UEYE
#endif // BIAS_HAVE_UEYE
    case 6: // SHM
#ifdef BIAS_HAVE_DSHOW
    case 8: // DSHOW            
#endif // BIAS_HAVE_DSHOW
#ifdef BIAS_HAVE_V4L
    case 8: // V4L
#endif // BIAS_HAVE_V4L
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }
      if (*ParamDevice_[i] != "")
        {
          if (Cameras_[i]->OpenDevice((*ParamDevice_[i]).c_str()) < 0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Camera " << (*ParamDevice_[i]).c_str() << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
              return -1;
            }
        }
      else
        {
          if (Cameras_[i]->OpenDevice() < 0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Camera " << i << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
              return -1;
            }
        }
      break;

    case 7: // NET
      // open on default port specified by class
      if (*ParamPort_[i] == 0)
        {
          if (((VideoSource_Net*)Cameras_[i])->OpenDevice(*ParamIP_[i]) < 0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Net Camera " << i << " could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
              return -1;
            }
        }
      // open on defined port by param file
      else
        {
          if (((VideoSource_Net*)Cameras_[i])->OpenDevice(*ParamIP_[i],
                                                          *ParamPort_[i]) < 0)
            {
              delete Cameras_[i];
              Cameras_[i] = NULL;
              stringstream text;
              text << "Net Camera on:"<<*ParamIP_[i]<<" Port:"<<*ParamPort_[i]
                   <<" could not be openend";
              wxMessageDialog errorDialog(this,
                                          AsciiToWx(text.str().c_str()),
                                          wxT("ShowCamWx - Error"),
                                          wxICON_ERROR|wxOK);
              errorDialog.ShowModal();
              return -1;
            }
        }
      break;

#ifdef BIAS_HAVE_SWISSRANGER
    case 9: // SwissRanger
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }
      //(dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]))->SetModulationFrequency(MF_19MHz);      		      
      if (Cameras_[i]->OpenDevice()<0)
        {
          delete Cameras_[i];
          Cameras_[i] = NULL;
          stringstream text;
          text << "Swissranger PMD-Camera could not be openend";
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
          return -1;
        }
      else{
      	//(dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]))->SetModulationFrequency(MF_19MHz);
      	//(dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]))->SetIntegrationTime(100);
      }
      break;
#endif

    default: // NONE
      break;
    }


  // show the correct camera
  NumCamSel_ = camindex;

  if (Cameras_[i] != NULL)
    {
      // try to set color model from param file for camera
      if (ColorModels_[i] != ImageBase::CM_invalid)
        Cameras_[i]->SetColorModel(ColorModels_[i]);

      // initialize image for camera and grab
      Cameras_[i]->InitImage(*CamImg_[i]);
      cout<<"Inited Image with :"<<CamImg_[i]->GetWidth()<<" "<<CamImg_[i]->GetHeight()
      <<" and :"<<CamImg_[i]->GetChannelCount()<<" color channels"<<endl;
#ifdef BIAS_HAVE_DCAM
      if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]) != NULL)
        {
          CamImg_[i]->Release();
          CamImg_[i]->Init(1280, 960, 1);
          CamImg_[i]->SetColorModel(ImageBase::CM_Bayer_GBRG);
          CamImgMiddle_[i]->Init(1280, 960, 1);
          CamImgMiddle_[i]->SetColorModel(ImageBase::CM_Bayer_GBRG);
          CamImgRight_[i]->Init(1280, 960, 1);
          CamImgRight_[i]->SetColorModel(ImageBase::CM_Bayer_GBRG);
          XB3AllInOne_[i] = new Camera<unsigned char>;
          Cameras_[i]->InitImage(*XB3AllInOne_[i]);
        }
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
      // init 2D image for usb pmd cameras
      if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[i]) != NULL)
        {
          // dynamic_cast<VideoSource_usbPMD*>(Cameras_[i])->Init2DImage(*CamImg2D_[i]);
          //CamImgCombined is 3 times image in horizontal way
          CamImgCombined_.Init((CamImg_[NumCamSel_]->GetWidth()+1)*3,
                               CamImg_[NumCamSel_]->GetHeight()+1);
          CamImgCombined_.SetColorModel(ImageBase::CM_Grey);
        }
#endif // BIAS_HAVE_PMD_PMDZess

      ret = Cameras_[i]->PreGrab();
      Cameras_[i]->SetDebugLevel(1);
    }

  /************************************************/
  /******* now start the threads for grabbing *****/
  //check if camera is used
  if(Cameras_[i] != NULL){
#ifdef BIAS_HAVE_PMD_PMDTec
    if (dynamic_cast<VideoSource_PMD*>(Cameras_[i]) != NULL) {      
      if(Cameras_[i]!=NULL){
        dynamic_cast<VideoSource_PMD*>(Cameras_[i])->InitDepthImage(*CamImgPMDDepth_[i]);
        dynamic_cast<VideoSource_PMD*>(Cameras_[i])->InitModCoeffImage(*CamImgPMDAmp_[i]);
        dynamic_cast<VideoSource_PMD*>(Cameras_[i])->InitIntensityImage(*CamImgPMDInt_[i]);
      }
    }
    else if (dynamic_cast<VideoSource_ShmPMD*>(Cameras_[i]) != NULL) {    
      if(Cameras_[i]!=NULL){
        dynamic_cast<VideoSource_ShmPMD*>(Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
        dynamic_cast<VideoSource_ShmPMD*>(Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);
      }
    }
#endif // BIAS_HAVE_PMD_PMDTec

    if(!ThreadInited_[camindex])
    	ret = CreateGrabThreadForCamera_(i);
    if(ret < 0) return ret;

  }// end if Cameras_[i] != NULL


  // Set CameraActive
  CameraActive_[camindex] = true;
  OpenedDevices_ = true;


  // activate selection of camera
  if(menuShow != NULL)
    menuShow->Enable(ID_Show1 + camindex, true);

  // init and start timer
  if (!TimerActive_) {
    StopWatch_.Start();
    TimerActive_ = true;
  }

  ActiveCameras_++;
  return ret;
}

int ShowCamWxFrame::CloseDevice(int camindex) {
  // close device and reinitialize variables
  int ret =0;
  if(grabSingleThread_[camindex] != NULL){
    ThreadInited_[camindex] = false;
    grabSingleThread_[camindex]->Delete();
    //delete grabSingleThread_[camindex];
    grabSingleThread_[camindex] = NULL;
  }
  bool gmtLocked = false;
  bool noCamActive = false;
  if (grabMasterThread_ != NULL && Cameras_[camindex] != NULL) {
    ServerMutex_[camindex]->Lock();
    gmtLocked = true;
  }

  if(saveThreads_[camindex] != NULL){
    saveThreads_[camindex]->Delete();
    delete saveThreads_[camindex];
    saveThreads_[camindex] = NULL;
  }

  if(BufferAccess_[camindex] != NULL){
    delete BufferAccess_[camindex];
    BufferAccess_[camindex] = NULL;
  }

  WriteIndex_[camindex] = 0;
  ReadIndex_[camindex] = BUFFER_SIZE - 1;

  CameraActive_[camindex] = false;
  CameraInitialized_[camindex] = false;

  if(CamImg_[camindex]!= NULL){
    delete CamImg_[camindex];
    CamImg_[camindex] = NULL;
  }
  if(CamImgPMDDepth_[camindex]!= NULL){
    delete CamImgPMDDepth_[camindex];
    CamImgPMDDepth_[camindex] = NULL;
  }
  if(CamImgPMDAmp_[camindex]!= NULL){
    delete CamImgPMDAmp_[camindex];
    CamImgPMDAmp_[camindex] = NULL;
  }
  if(CamImgPMDInt_[camindex]!= NULL){
    delete CamImgPMDInt_[camindex];
    CamImgPMDInt_[camindex] = NULL;
  }
  if(CamImgMiddle_[camindex]!= NULL){
    delete CamImgMiddle_[camindex];
    CamImgMiddle_[camindex] = NULL;
  }
  if(CamImgRight_[camindex]!= NULL){
    delete CamImgRight_[camindex];
    CamImgRight_[camindex] = NULL;
  }
  if(XB3AllInOne_[camindex]!= NULL){
    delete XB3AllInOne_[camindex];
    XB3AllInOne_[camindex] = NULL;
  }

  if(ViewColorMode_[camindex] != ImageBase::CM_invalid){
    ViewColorMode_[camindex] = ImageBase::CM_invalid;
  }

  menuShow->Enable(ID_Show1 + camindex, false);

  
  if (Cameras_[camindex] != NULL) {
  	Cameras_[camindex]->CloseDevice();
  	delete Cameras_[camindex];
  	Cameras_[camindex] = NULL;
  	ActiveCameras_--;
  	noCamActive = (ActiveCameras_ == 0);
  }

  if (gmtLocked) {
    ServerCondition_[camindex]->Signal();
    ServerMutex_[camindex]->Unlock();
  }

  // maybe stop and reset timer
  if(noCamActive) {
    StopWatch_.Stop();
    TimerActive_ = false;
    if(grabMasterThread_ != NULL){
      grabMasterThread_->Delete();
      //delete grabMasterThread_;
      grabMasterThread_=NULL;
    }
    if(ServerCondition_[camindex] !=NULL){
      delete ServerCondition_[camindex];
      ServerCondition_[camindex]=NULL;
    }
    if(ServerMutex_[camindex] !=NULL){
      delete ServerMutex_[camindex];
      ServerMutex_[camindex]=NULL;
    }
    if(ClientsCondition_[camindex] !=NULL){
      delete ClientsCondition_[camindex];
      ClientsCondition_[camindex]=NULL;
    }
    if(ClientsMutex_[camindex] !=NULL){
      delete ClientsMutex_[camindex];
      ClientsMutex_[camindex]=NULL;
    }
  }
  cout<<"Closed Device :"<<camindex<<endl;
  return ret;
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::OpenDevices() {
  if (!InitCameras_) {
    BIASERR("Initialize cameras first");
    return;
  }

  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    OpenDevice(i);
  }
}

/****************************************************
   @brief


*****************************************************/
#ifdef BIAS_HAVE_DCAM
 dc1394video_mode_t ShowCamWxFrame::SelectedModeFromMenu(int camindex){

    int menuindex = (camindex + 1) * 10000;
    //    cout<<"Menuindex:"<<menuindex<<endl;
    dc1394video_mode_t mode = DC1394_VIDEO_MODE_640x480_MONO8;
    bool foundItem = false;

    //Never!! commit such things!!
    //return DC1394_VIDEO_MODE_640x480_RGB8;

    while(!foundItem){
      menuindex++;
      wxMenuItem* menuItem = CameraMenu_->FindItem(menuindex);
      if(menuItem!= NULL && menuItem->IsChecked()){
        foundItem = true;
      }
    }

    //cout << "Checked Item: " << (menuindex % 10000 - 1) << endl;
    int modeindex = (menuindex % 10000 - 1);

    for(unsigned int k=0; k< IEEECameraInfo_.size();k++){
      if(IEEECameraInfo_[k].camid == atoi((*ParamDevice_[camindex]).c_str())){
        // add to menu
        mode = IEEECameraInfo_[k].videoModes[modeindex];
      }
    }
    cout << "Found mode: " << mode << endl;
    return mode;
  }
#endif


/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::CreateMenu() {
  // create file menu
  wxMenu *menuFile = new wxMenu;
  menuFile->Append(ID_Grab, wxT("Grab &single image") );
  menuFile->AppendCheckItem(ID_GrabContinuous, wxT("Grab &continuously (Start/Stop)") );
  menuFile->AppendCheckItem(ID_GrabToStream, wxT("Grab to stream (Start/Stop)") );
  menuFile->Append(ID_Quit, wxT("E&xit") );

  menuShow = new wxMenu;

  menuShow->Append(ID_Show1, wxT("Camera 1") );
  menuShow->Append(ID_Show2, wxT("Camera 2") );
  menuShow->Append(ID_Show3, wxT("Camera 3") );
  menuShow->Append(ID_Show4, wxT("Camera 4") );
  menuShow->Append(ID_Show5, wxT("Camera 5") );
  menuShow->Append(ID_Show6, wxT("Camera 6") );
  menuShow->Append(ID_Show7, wxT("Camera 7") );
  menuShow->Append(ID_Show8, wxT("Camera 8") );
  menuShow->Append(ID_Show9, wxT("Camera 9") );
  menuShow->AppendSeparator();
  menuShow->Append(ID_Controller, wxT("Controller") );

  if(CameraMenu_ == NULL)
    CameraMenu_ = new wxMenu;


  wxMenu *menuSearchCameras = new wxMenu;
  menuSearchCameras->Append(ID_FindIEEE1394, wxT("Scan for IEEE1394 Camera") );
  menuSearchCameras->Append(ID_FindUEYE, wxT("Scan for UEYE Camera") );
  menuSearchCameras->Append(ID_FindWebcams, wxT("Scan for Webcams (USB cameras)") );
  menuSearchCameras->Append(ID_FindSwissranger, wxT("Scan for Swissranger ToF") );
  menuSearchCameras->Append(ID_FindZess, wxT("Scan for Zess ToF") );
  menuSearchCameras->Append(ID_FindPMDTec, wxT("Scan for PMDTec ToF") );
  //menuSearchCameras->Append(ID_FindV4L, wxT("Scan for Video for Linux Camera") );


  wxMenu *menuParameters = new wxMenu;
  menuParameters->Append(ID_SaveParameters, wxT("Save Parameters") );

  wxMenu *menuHelp = new wxMenu;
  menuHelp->Append(wxID_ABOUT);
   
  // add menues to menu bar
  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  menuBar->Append(CameraMenu_, wxT("&Cameras"));
  menuBar->Append(menuSearchCameras, wxT("&Camera Init"));
  menuBar->Append(menuParameters, wxT("&Parameters"));
  menuBar->Append(menuShow, wxT("&Show"));
  menuBar->Append(menuHelp, wxT("&Help"));

  SetMenuBar(menuBar);

  // deactivate uninitialized cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    if (Cameras_[i] == NULL) {
      menuShow->Enable(ID_Show1 + i, false);
    }
  }

  // fit all components to size of window
  Fit();
}

  void ShowCamWxFrame::CreateCameraMenu(){
    CameraMenu_ = new wxMenu;
  }

#ifdef BIAS_HAVE_PanTilt

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
GetProjectionFromPanTilt_(BIAS::Projection &projHeader){

  double pan=0.0, tilt=0.0;
  panTiltControl_->GetCurrentPositionDegreesThreaded(pan, tilt);
  std::cout<<"Current Position:"<<pan<<","<<tilt<<std::endl;

  // rotation matrix resulting from panning
  BIAS::RMatrix pan_matrix = BIAS::RMatrix(BIAS::Vector3<double>(0,1,0), pan);
  // rotation matrix resulting from tilting
  BIAS::RMatrix tilt_matrix = BIAS::RMatrix(BIAS::Vector3<double>(1,0,0), tilt);

  // new modelling of ptu pose:
  // first, pan ptu
  BIAS::Matrix4x4<double> pan_ptu = BIAS::Matrix4x4<double>(pan_matrix, BIAS::Vector3<double>(0,0,0));

  // tilt ptu
  BIAS::Matrix4x4<double> tilt_ptu = BIAS::Matrix4x4<double>(tilt_matrix, BIAS::Vector3<double>(0,0,0));

  // identity matrix
  BIAS::Matrix3x3<double> id33 = BIAS::Matrix3x3<double>(1,0,0, 0,1,0, 0,0,1);
  // translation from rotation joint to camera center
  BIAS::Vector3<double> translation(0.0, 0.0, 0.0);
  BIAS::Matrix4x4<double> trans_to_camera_center = BIAS::Matrix4x4<double>(id33, translation);

  // calculate complete ptu transformation matrix
  BIAS::Matrix4x4<double> ptu_result = tilt_ptu * pan_ptu * trans_to_camera_center;

  // do some hackish access to the translation part of resulting matrix
  BIAS::Vector4<double> translation_rotated4 = ptu_result.GetCol(3);
  BIAS::Vector3<double> translation_rotated3;
  double *translation_rotated4_ptr = translation_rotated4.GetData();
  for (int i=0;i<3;i++)
    translation_rotated3.GetData()[i] = translation_rotated4_ptr[i];

  // store transformation in dynamic data pool entry
  Pose p;
  Quaternion<double> panq;
  pan_matrix.GetQuaternion(panq);
  Quaternion<double> tiltq;
  tilt_matrix.GetQuaternion(tiltq);
  Quaternion<double> temp = panq * tiltq;
  p.SetC(translation_rotated3);
  p.SetQ(temp);
  KMatrix K;
  projHeader.CreatePerspective(p,K,640,480);
  cout<<"Projection:"<<projHeader<<endl;
}
#endif


/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
StopSavingImagesToDisk()
{
  bWriteBuffered_=false;

  for (int i = 0; i < NUMBER_CAMERAS; i++){
    if(saveThreads_[i]!=NULL && Cameras_[i] != NULL){
      //      cout<<"Pausing camera :"<<i<<endl;
	    saveThreads_[i]->Delete();
      saveThreads_[i] = NULL;
      //cout << "\n\nSaveThread for cam: " << i << " paused\n\n" << endl;
    }
  }
  //for (int i = 0; i < NUMBER_CAMERAS; i++){
  //  if(saveThreads_[i]!=NULL && Cameras_[i] != NULL){
  //    saveThreads_[i]->FinishSaving();
  //  }
  //}
  cout << "leave StopSavingThreads" << endl;
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
StartSavingImagesToDisk()
{

  //try to create a new directory, skip of already exists
  int ret =-1;
  while(ret != 0){
    stringstream dir;
    dir <<"Take"<<FileHandling::LeadingZeroString(dDirIndex_,5)<<"/";
    currentSaveDir_ = dir.str().c_str();
    #ifdef WIN32
    ret = _mkdir(dir.str().c_str());
    #else //WIN32
    ret = mkdir(dir.str().c_str(), 0755);
    #endif //WIN32
    dDirIndex_++;
  }

  // important!! This triggers the write to buffer in threads!
  bWriteBuffered_ = true;

  //create a dummy Projection to save Pan Tilt angles in Image headers
  BIAS::Projection projHeader;

  #ifdef BIAS_HAVE_PanTilt
  if(*bUsePanTiltUnit_){
    GetProjectionFromPanTilt_(projHeader);
  }
  #endif //BIAS_HAVE_PanTilt

  for (int i = 0; i < NUMBER_CAMERAS; i++){
    // check if camera is opened
    if(Cameras_[i] != NULL){

      #ifdef BIAS_HAVE_PanTilt
      if(*bUsePanTiltUnit_){
        (*CamImg_[i]).SetProj(projHeader);
        (*CamImg_[i]).UpdateMetaData();
      }
      #endif //BIAS_HAVE_PanTilt

      if(saveThreads_[i] == NULL){
        //Here create ImageSaveThreadForCamera
        CreateImageSaveThreadForCamera_(i);
        saveThreads_[i]->SetSaveDirectory(currentSaveDir_);
      }
      else{
        // signal thread to save a single image
        saveThreads_[i]->SetSaveDirectory(currentSaveDir_);
		    saveThreads_[i]->ResetCounter();
        saveThreads_[i]->Resume();
      }
    }//end else

  }//end for all cameras
}

/****************************************************
  @brief


*****************************************************/
void ShowCamWxFrame::
ParseColorModels()
{
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    switch(*ParamColorModel_[i])  {
    case 0:  ColorModels_[i] = ImageBase::CM_invalid; break;
    case 1:  ColorModels_[i] = ImageBase::CM_Grey; break;
    case 2:  ColorModels_[i] = ImageBase::CM_RGB; break;
    case 3:  ColorModels_[i] = ImageBase::CM_BGR; break;
    case 4:  ColorModels_[i] = ImageBase::CM_YUYV422; break;
    case 5:  ColorModels_[i] = ImageBase::CM_UYVY422; break;
    case 6:  ColorModels_[i] = ImageBase::CM_YUV420P; break;
    case 7:  ColorModels_[i] = ImageBase::CM_YUV444; break;
    case 8:  ColorModels_[i] = ImageBase::CM_YUV411; break;
    case 9:  ColorModels_[i] = ImageBase::CM_HSV; break;
    case 10: ColorModels_[i] = ImageBase::CM_HSI_OBS; break;
    case 11: ColorModels_[i] = ImageBase::CM_DV; break;
    case 12: ColorModels_[i] = ImageBase::CM_RGBA; break;
    case 13: ColorModels_[i] = ImageBase::CM_GreyA; break;
    case 14: ColorModels_[i] = ImageBase::CM_Bayer_RGGB; break;
    case 15: ColorModels_[i] = ImageBase::CM_Bayer_GBRG; break;
    case 16: ColorModels_[i] = ImageBase::CM_Bayer_GRBG; break;
    case 17: ColorModels_[i] = ImageBase::CM_Bayer_BGGR; break;
    case 18: ColorModels_[i] = ImageBase::CM_HSL; break;
    case 19: ColorModels_[i] = ImageBase::CM_hsL; break;
    case 20: ColorModels_[i] = ImageBase::CM_SymTensor2x2; break;
    case 21: ColorModels_[i] = ImageBase::CM_BGRA; break;
    case 22: ColorModels_[i] = ImageBase::CM_RGBE; break;
    default: ColorModels_[i] = ImageBase::CM_invalid; break;
    }
  }
}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
InitCombinedImage()
{
  if(!CamImgCombined_.IsEmpty())
    CamImgCombined_.Release();
  CamImgCombined_.Init((CamImg_[NumCamSel_]->GetWidth()+1)*3,
                       CamImg_[NumCamSel_]->GetHeight()+1);
  CamImgCombined_.SetColorModel(ImageBase::CM_Grey);
  // initialize combined image from pmd usb camera images
  CamImgCombined_.SetROICorners(0, 0,
                                CamImg_[NumCamSel_]->GetWidth(),
                                CamImg_[NumCamSel_]->GetHeight());
  CamImgCombined_.Paste2ROI(*CamImg_[NumCamSel_]);
  CamImgCombined_.SetROICorners(CamImg_[NumCamSel_]->GetWidth(),
                                0,
                                CamImg_[NumCamSel_]->GetWidth() * 2,
                                CamImg_[NumCamSel_]->GetHeight());


  // add Amp image from pmd usb camera
  BIAS::Camera<float> tmpImg;
  BIAS::Camera<unsigned char> tmpImgUC;
  if(CamImgPMDAmp_[NumCamSel_] != NULL){
    tmpImg = *CamImgPMDAmp_[NumCamSel_];
    tmpImg.ScaleShiftBetween(0,255);

    BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
    CamImgCombined_.Paste2ROI(tmpImgUC);
    CamImgCombined_.SetROICorners(CamImg_[NumCamSel_]->GetWidth() * 2,
                                  0,
                                  CamImg_[NumCamSel_]->GetWidth() * 3,
                                  CamImg_[NumCamSel_]->GetHeight());
  }
  // add Depth image from pmd usb camera
  if(CamImgPMDDepth_[NumCamSel_]!= NULL){
    tmpImg = *CamImgPMDDepth_[NumCamSel_];
    tmpImg.ScaleShiftBetween(0,255);

    BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
    CamImgCombined_.Paste2ROI(tmpImgUC);
  }
}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnSaveParameters(wxCommandEvent& event){

  wxString fileName;
  wxFileDialog dialog(this,wxT("Save Parameters"),wxT(""),wxT(""),
                      wxT("MIP Parameter files (*.param)|*.param"),
                      wxSAVE|wxOVERWRITE_PROMPT);
  int ret = dialog.ShowModal();
  if(ret == wxID_OK){
    fileName = dialog.GetPath();
    WriteParameter(WxToAscii(fileName));
  }
}
  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnWindowClose(wxCloseEvent& event){
  //cout << "window close event!!!!!!!!" << endl;
  CleanUpAndExit();
  //Close(true);
  Destroy();
}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
CleanUpAndExit()
{
  if(!CleanedUp_){
    // stop timer
    Timer_.Stop();
    // first delete threads while timer is running
    //cout << "start killing threads" << endl;
    for (int i = 0; i < NUMBER_CAMERAS; i++) {
      CloseDevice(i);
    }

    for (unsigned int i = 0; i < ServerMutex_.size();i++) {
      if(ServerMutex_[i] != NULL)
        delete ServerMutex_[i];
      ServerMutex_[i]=NULL;
      if(ServerCondition_[i] != NULL)
        delete ServerCondition_[i];
      ServerCondition_[i] = NULL;
      if(ClientsMutex_[i] != NULL)
        delete ClientsMutex_[i];
      ClientsMutex_[i]=NULL;
      if(ClientsCondition_[i] != NULL)
        delete ClientsCondition_[i];
      ClientsCondition_[i] = NULL;
    }

    #ifdef BIAS_HAVE_PanTilt
    if(*bUsePanTiltUnit_){
      if(panTiltControlDialog_->IsShown())
        panTiltControlDialog_->Hide();
      delete panTiltControlDialog_;

      if(panTiltControl_ != NULL){
        panTiltControl_->SetPosition(0,0);
        panTiltControl_->ClosePanTiltUnit();
        delete panTiltControl_;
      }
    }
    #endif //BIAS_HAVE_PanTilt

    CleanedUp_ = true;
  }

  cout << "Cleaned up successfully" << endl;
}

/****************************************************
  @brief


 *****************************************************/
void ShowCamWxFrame::OnFit(wxCommandEvent& WXUNUSED(event))
{
	if (pCheckBoxKeepScrollPosition != NULL)
	{
		if (IC_!=NULL){
			IC_->KeepScrollPosition(pCheckBoxKeepScrollPosition->IsChecked());
		}         
	}
}



/****************************************************
  @brief


 *****************************************************/
void ShowCamWxFrame::OnKeepScrollPosition(wxCommandEvent& WXUNUSED(event))
{
	if (pCheckBoxKeepScrollPosition != NULL)
	{
		if (IC_!=NULL){
			IC_->KeepScrollPosition(pCheckBoxKeepScrollPosition->IsChecked());
		}         
	}
}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnQuit(wxCommandEvent& event)
{
  CleanUpAndExit();
  //Close(true);
  Destroy();
}

  /****************************************************
   @Brief


  *****************************************************/
void ShowCamWxFrame::
OnGrab(wxCommandEvent& event)
{
  if(!bSingleSaveDirCreated_){
    int ret =-1;

    while(ret != 0){
      stringstream dir;
      dir <<"SingleTake"<<FileHandling::LeadingZeroString(dSingleDirIndex_,5)<<"/";
      currentSingleSaveDir_ = dir.str().c_str();

#ifdef WIN32
      ret = _mkdir(dir.str().c_str());
#else //WIN32
      ret = mkdir(dir.str().c_str(), 0755);
#endif //WIN32

      dSingleDirIndex_++;
    }
    bSingleSaveDirCreated_ = true;
  }

  for (int i = 0; i < NUMBER_CAMERAS; i++){
    if(Cameras_[i] != NULL){
      SaveAllImagesUnthreaded_(i);
    }
  }
  GrabCounter_++;
}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnGrabContinuous(wxCommandEvent& event)
{
  // reset counter and enable/disable continuous saving of images
  FrameCounter_ = 0;
  bSaveImages_ = !bSaveImages_;
  bSaveToStream_ = false;
  if (bSaveImages_){
    StartSavingImagesToDisk();
    pButtonContinuous->SetLabel(_T("Stop"));
  }
  else{
    StopSavingImagesToDisk();
    pButtonContinuous->SetLabel(_T("Grab Cont."));
  }
}

void ShowCamWxFrame::
OnGrabToStream(wxCommandEvent& event)
{
  // reset counter and enable/disable continuous saving of images
  FrameCounter_ = 0;
  bSaveImages_ = !bSaveImages_;
  bSaveToStream_ = true;
  if (bSaveImages_){
    StartSavingImagesToDisk();
    pButtonStream->SetLabel(_T("Stop"));
  }
  else{
    StopSavingImagesToDisk();
    pButtonStream->SetLabel(_T("Grab to Stream"));
  }
}


  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage1(wxCommandEvent& event)
{
  NumCamSel_ = 0;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage2(wxCommandEvent& event)
{
  NumCamSel_ = 1;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage3(wxCommandEvent& event)
{
  NumCamSel_ = 2;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage4(wxCommandEvent& event)
{
  NumCamSel_ = 3;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage5(wxCommandEvent& event)
{
  NumCamSel_ = 4;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage6(wxCommandEvent& event)
{
  NumCamSel_ = 5;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage7(wxCommandEvent& event)
{
  NumCamSel_ = 6;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage8(wxCommandEvent& event)
{
  NumCamSel_ = 7;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowImage9(wxCommandEvent& event)
{
  NumCamSel_ = 8;

}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnShowController(wxCommandEvent& event)
{
  // check if camera is initialized
  if (Cameras_[NumCamSel_] != NULL) {

#ifdef BIAS_HAVE_PMD_PMDZess
    // check if it is a usb pmd camera
    if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]) != NULL) {
      // initialize controller if it was not before
      if(VideoControllersUSBPMD_[NumCamSel_] == NULL) {
        VideoControllersUSBPMD_[NumCamSel_] =
          new VideoSource_Controller_usbPMD(
                                            dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]),
                                            this,
                                            0,3000,1002,8000,100,2000);
      }
      // show controller
      VideoControllersUSBPMD_[NumCamSel_]->Show(true);
    } else
#endif // BIAS_HAVE_PMD_PMDZess
#ifdef BIAS_HAVE_PMD_PMDTec
    // check if it is a PMDTec camera
    if (dynamic_cast<VideoSource_PMD*>(Cameras_[NumCamSel_]) != NULL) {
      // initialize controller if it was not before
      if(VideoControllersPMD_[NumCamSel_] == NULL) {
        VideoControllersPMD_[NumCamSel_] =
          new VideoSource_Controller_PMD(
                                            dynamic_cast<VideoSource_PMD*>(Cameras_[NumCamSel_]),
                                            this);
      }
      // show controller
      VideoControllersPMD_[NumCamSel_]->Show(true);
    } else
#endif // BIAS_HAVE_PMD_PMDTec
#ifdef BIAS_HAVE_SWISSRANGER
      if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[NumCamSel_]) != NULL) {
        // initialize controller if it was not before
        if(VideoControllersSwissRanger_[NumCamSel_] == NULL) {
          VideoControllersSwissRanger_[NumCamSel_] =
            new VideoSource_Controller_SwissRanger(
                                                   dynamic_cast<VideoSource_SwissRanger*>(Cameras_[NumCamSel_]),
                                                   this);
        }
        // show controller
        VideoControllersSwissRanger_[NumCamSel_]->Show(true);
      } else
#endif // BIAS_HAVE_SWISSRANGER
#ifdef BIAS_HAVE_DCAM
        if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[NumCamSel_]) != NULL) {
          // initialize controller if it was not before
          if(VideoControllersXB3_[NumCamSel_] == NULL) {
            VideoControllersXB3_[NumCamSel_] =
              new VideoSource_Controller_XB3(
                                             dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[NumCamSel_]),
                                             this,"VideoController for XB3",ShowXB3_);
          }
          // show controller
          VideoControllersXB3_[NumCamSel_]->Show(true);
        } else
#endif
          // initialize controller if it was not before
          if (VideoControllers_[NumCamSel_] == NULL)
            {              
							stringstream text; text << "DCAM: "<<NumCamSel_;
							VideoControllers_[NumCamSel_] =
                new VideoSource_Controller(Cameras_[NumCamSel_],
                                           this,text.str());
              // show controller
              VideoControllers_[NumCamSel_]->Show(true);
            } else {
            // show controller
            VideoControllers_[NumCamSel_]->Show(true);
          }

  }

  // camera was not initalized
  else
    {
      wxMessageDialog errorDialog(this,
                                  wxT("No correct camera was selected"),
                                  wxT("ShowCamWx - Error"),
                                  wxICON_ERROR|wxOK);
      errorDialog.ShowModal();
    }
}

  /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::
OnTimer(wxTimerEvent& event)
{
  //check if we opened all devices and started the grabthreads
  if(!OpenedDevices_){
    //BIASERR("Devices not open!");
    return;
  }
  if(grabMasterThread_ != NULL){
    double fps = grabMasterThread_->fps_;
    FrameCounter_ = grabMasterThread_->FrameCounter_;
    wxString text;
    text.Printf(wxT("%05d %2.1f fps "),FrameCounter_, fps);
    SetStatusText(text, 2);
    SelectAndShowCameraImage_();
  }
  Timer_.Start(Delay_);
}
 /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::OnFindIEEE1394(wxCommandEvent& event) {
#ifdef BIAS_HAVE_DCAM
  NumIEEECams_ = ScanBus(IEEECameraInfo_);

  if (NumIEEECams_ > 0) {
    stringstream text;

    text << "Found " << NumIEEECams_ << " IEEE1394 Cameras: " << endl << endl;

    for (int i = 0; i < NumIEEECams_; i++) {
      text << "Camid: " << IEEECameraInfo_[i].camid << " "
           << IEEECameraInfo_[i].vendor << " " << IEEECameraInfo_[i].model
           << endl;
    }

    text<<"\nTrigger them?"<<endl;
    
    wxMessageDialog messageDialog(this, AsciiToWx(text.str().c_str()), wxT("Trigger- IEEE Cameras?"), 
    		wxYES_NO | wxNO_DEFAULT| wxICON_QUESTION);   
    bool trigger = false;
    if(messageDialog.ShowModal() == wxID_YES){
    	trigger = true;
    	//cout<<"Trigger!"<<endl;
    }
    //else
    	//cout<<"No Trigger!"<<endl;
    
    for (int i = 0; i < NumIEEECams_; i++) {
    	// check if camera already in Menu...
    	bool inUse = false;
    	for(int j = 0; j < NUMBER_CAMERAS; j++){
    		if(IEEECameraInfo_[i].camid == atoi((*ParamDevice_[j]).c_str())){
    			inUse = true;
    		}
    	}
    	SetTrigger_[NumOfCams_] = trigger;
    	// here we have to add the cameras to the menu.
    	if(!inUse)
    		AddIEEECameraToMenu(NumOfCams_,i,false);    		    
    }


    
  } else {
    wxMessageDialog messageDialog(this, wxT("Sorry, no IEEE Cameras found!!"), wxT("ShowCamWx - IEEE Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();

  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, DCAM support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}

/****************************************************
  @brief


 *****************************************************/
void ShowCamWxFrame::OnFindWebcams(wxCommandEvent& event) {
	int ret =0;
	VideoSource* vs=NULL;
#ifdef BIAS_HAVE_V4L
	vs = new VideoSource_V4L;  
  ret = vs->OpenDevice();  
    if (ret != 0) {
  	//Dialogbox
  	wxMessageDialog messageDialog(this, wxT("Sorry, Webcam not found!!"), wxT("Webcam Scan"), wxICON_ERROR | wxOK);
  	messageDialog.ShowModal();
  	delete vs;
  } else {
  	vs->CloseDevice();
  	delete vs;

  	// Add camera and open
  	AddWebcamToMenu("Webcam",NumOfCams_, false);
  }
#endif

#ifdef BIAS_HAVE_DSHOW
  vs = new VideoSource_DSHOW;
  vector<string> devices;
  vs->GetAllDevices(devices);
  stringstream text;

  if (devices.size() > 0) {  
    text << "Found " << devices.size() << "DirectShow Cameras: " << endl << endl;
    for (unsigned int i = 0; i < devices.size(); i++) {
      text << "Identifier: " << devices[i]<<endl;
    }
   }
   else{
     text <<"Sorry no DirectShow Cameras found."<<endl;
   }

  wxMessageDialog messageDialog(this, AsciiToWx(text.str().c_str()), 
      wxT("DShow Cameras"), wxOK);   
    messageDialog.ShowModal();

  /*wxArrayString devStrings;
  for(unsigned i=0;i<devices.size();i++){
    cout<<"Found device:"<<devices[i]<<endl;
    devStrings.Add(AsciiToWx(devices[i]));
  }

  string camera;
  if(devices.size() > 1){  
    // ask for depth error model
	  wxSingleChoiceDialog wcbDlg(this,
			  wxT("Select camera."),
			  wxT("DShow Cameras"),
			  devStrings);
	  wcbDlg.SetSelection(0);
	  int ret  = wcbDlg.ShowModal();
	  if(ret == wxID_OK){
		  camera = WxToAscii(devStrings.Item(wcbDlg.GetSelection()));
	  } 
    else 
      return;
  }
  else if(devices.size() > 0){
    camera = devices[0];
  }
  else 
    return;
*/
  for(unsigned i=0;i<devices.size();i++){
    *ParamDevice_[NumOfCams_]=devices[i];
    AddWebcamToMenu(devices[i], NumOfCams_, false);
  }

#endif  


}

 /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::OnFindUEye(wxCommandEvent& event){
  // simply try to open ueye
#ifdef BIAS_HAVE_UEYE

  VideoSource_uEye* vs = new VideoSource_uEye;
  int ret = vs->OpenDevice();

  if (ret != 0) {
    //Dialogbox
    wxMessageDialog messageDialog(this, wxT("Sorry, UEYE not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete vs;
  } else {
    vs->CloseDevice();
    delete vs;

    // Add camera and open
    AddUEyeCameraToMenu(NumOfCams_, false);
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, UEYE support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}

void ShowCamWxFrame::OnFindPMDTec(wxCommandEvent& event){
  // simply try to open pmdtec
#ifdef BIAS_HAVE_PMD_PMDTec
  Cameras_[NumOfCams_] = new VideoSource_PMD;
  *ParamCameras_[NumOfCams_] = 2;
  int ret = OpenDevice(NumOfCams_);

  if (ret != 0) {
    //Dialogbox
    wxMessageDialog messageDialog(this, wxT("Sorry, PMDTec Camera not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete Cameras_[NumOfCams_];
  } else {
    // Add camera and open
    AddPMDTecCameraToMenu(NumOfCams_, true);
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, PMDTec support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}


 /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::OnFindPMDZess(wxCommandEvent& event){
#ifdef BIAS_HAVE_PMD_PMDZess

  VideoSource_usbPMD* vs = new VideoSource_usbPMD;
  //ask for bitfile
  wxTextEntryDialog dialog (this,wxT("Please insert path to bitfile,\n else ShowCamWx will try to open default file"),wxT("Searching for PMD Zess"),wxT(""));
  dialog.ShowModal();

  wxString wxStr = dialog.GetValue();
  if(wxStr.empty())
    wxStr = "C:\\ExternLibs\\PMDZess\\Bitfiles\\default.rbt";

  vs->SetBitFile(wxStr.c_str());

  int ret = vs->OpenDevice();

  if(ret != 0){
    wxMessageDialog messageDialog(this, wxT("Sorry, PMD Zess not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete vs;
  }else{
    vs->CloseDevice();
    delete vs;

    // Add camera and open
    AddZessCameraToMenu(NumOfCams_, false,wxStr.c_str());
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, PMD Zess support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif //BIAS_HAVE_PMD_PMDZess
}


 /****************************************************
   @brief


  *****************************************************/
void ShowCamWxFrame::OnFindSwissRanger(wxCommandEvent& event){
#ifdef BIAS_HAVE_SWISSRANGER
  Cameras_[NumOfCams_] = new VideoSource_SwissRanger;
  *ParamCameras_[NumOfCams_] = 9;
  stringstream message;
  message<<"Trigger Swissranger?";
  wxMessageDialog dlg (this, wxT("Trigger Swissranger?"),
  		AsciiToWx(message.str().c_str()),
  		wxYES_NO | wxNO_DEFAULT| wxICON_QUESTION);
  if(dlg.ShowModal() == wxID_YES){  	
  	SetTrigger_[NumOfCams_] = true;
  }
  else{
  	SetTrigger_[NumOfCams_] = false;
  }
  	
  
  int ret = OpenDevice(NumOfCams_);

  if (ret != 0) {
    //Dialogbox
    wxMessageDialog messageDialog(this, wxT("Sorry, SwissRanger not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete Cameras_[NumOfCams_];
  } else {
    // Add camera and open
    AddSwissRangerCameraToMenu(NumOfCams_, true);
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, SwissRanger support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}


/****************************************************
   @brief


*****************************************************/
#ifdef BIAS_HAVE_SWISSRANGER
void ShowCamWxFrame::AddSwissRangerCameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);
  
  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
    
  CameraMenu_->Append(LastCameraID_,wxT("SwissRanger"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 9;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif

#ifdef BIAS_HAVE_PMD_PMDTec
void ShowCamWxFrame::AddPMDTecCameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));


  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);

  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
  
  CameraMenu_->Append(LastCameraID_,wxT("PMDTec Camera"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 2;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif

/****************************************************
   @brief


*****************************************************/
#ifdef BIAS_HAVE_PMD_PMDZess
void ShowCamWxFrame::AddZessCameraToMenu(int camindex, bool fromFile, std::string paramFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));


  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);

  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
  
  CameraMenu_->Append(LastCameraID_,wxT("PMD Zess"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 3;
    *ParamBitFile_[camindex] = paramFile;
    OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif //BIAS_HAVE_PMD_PMDZess


/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::AddUEyeCameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;
  // int ckid = 0;

  // add videomodes
  for (int i = 0; i < 6; i++) {
    stringstream ss;

    id++;
    switch (i) {
    case 0:
      ss << "1.875 FPS";
      break;

    case 1:
      ss << "3.75 FPS";
      break;

    case 2:
      ss << "7.5 FPS";
      break;

    case 3:
      ss << "15 FPS";
      break;

    case 4:
      ss << "30 FPS";
      break;

    case 5:
      ss << "60 FPS";
      break;

    default:
      break;
    }

    menuItem->AppendRadioItem(id,AsciiToWx(ss.str().c_str()));
    // connnect with eventhandler
    Connect(id, wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeVideoMode));

  }
  menuItem->Check(id, true);

  id++;
  menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));


  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));

  menuItem->Check(id, true);

  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, false);
  
  CameraMenu_->Append(LastCameraID_,wxT("UEYE"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 5;
    OpenDevice(camindex);
  }

  NumOfCams_++;
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::AddWebcamToMenu(string name, int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;
  // int ckid = 0;

  //add activate Camera to menu
  id++;
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  menuItem->Check(id, false);

  //add Trigger to menu
  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, false);
  
  //append camera to menu
  CameraMenu_->Append(LastCameraID_,AsciiToWx(name),menuItem);
  // now call opendevice
  
  if(!fromFile){
    *ParamCameras_[camindex] = 8;
   // OpenDevice(camindex);
  }
  NumOfCams_++;
}


/****************************************************
   @brief


*****************************************************/
#ifdef BIAS_HAVE_DCAM
void ShowCamWxFrame::AddIEEECameraToMenu(int camindex,int IEEEIndex, bool fromFile) {
  stringstream camtext;
  camtext << IEEECameraInfo_[IEEEIndex].vendor << " " << IEEECameraInfo_[IEEEIndex].model;

  wxMenu* menuItem = new wxMenu;
  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;
  int ckid = 0;

    // add "normal" formats
  for (unsigned int j = 0; j < IEEECameraInfo_[IEEEIndex].videoModes.size(); j++) {
    id++;

    // get all possible modis...
    dc1394video_mode_t mode = IEEECameraInfo_[IEEEIndex].videoModes[j];

    string s;
    s = VideoModeToString(mode);

    if (IEEECameraInfo_[IEEEIndex].bestMode == mode) {
      ckid = id;
    }

    // check possible framerates
    stringstream helptext;
    if(mode < DC1394_VIDEO_MODE_EXIF){
    helptext << "Supported Fps: ";
    for(unsigned int k = 0; k < IEEECameraInfo_[IEEEIndex].framerates[j].size(); k++){
      helptext << IEEECameraInfo_[IEEEIndex].framerates[j][k] << ", ";
    }
    }else{
      helptext << "Format 7";
    }

    menuItem->AppendRadioItem(id,AsciiToWx(s.c_str()),AsciiToWx(helptext.str().c_str()));

    // connnect with eventhandler
    Connect(id, wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeVideoMode));
  }

  menuItem->Check(ckid, true);
  menuItem->AppendSeparator();
  if (IEEECameraInfo_[IEEEIndex].b_capable) {
    id++;

    Connect(id,wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeFirewireB));

    menuItem->AppendCheckItem(id,wxT("Activate Firewire B"));
    menuItem->Check(id, true);
    SetFirewireB_[camindex] = true;
  }

  id++;

  menuItem->AppendCheckItem(id,wxT("Activate Camera"));

  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));

  if(fromFile)
    menuItem->Check(id, true);

  
  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);

  CameraMenu_->Append(LastCameraID_, AsciiToWx(camtext.str().c_str()), menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 1;
    if (IEEECameraInfo_[IEEEIndex].model.find("XB3") != string::npos)
      *ParamCameras_[camindex] = 10;
    std::ostringstream buffer;
    buffer << IEEECameraInfo_[IEEEIndex].camid;
    *ParamDevice_[camindex] = buffer.str();
  }

  SelectedFps_[camindex] = IEEECameraInfo_[IEEEIndex].bestFramerate;

  NumOfCams_++;
}
#endif


/****************************************************
   @brief


*****************************************************/
#ifdef BIAS_HAVE_DCAM
int ShowCamWxFrame::ShowFormat7Selector(int camindex, dc1394video_mode_t mode ){

  //dc1394video_mode_t mode = SelectedModeFromMenu(camindex);
  //int maxWidth, maxHeight,minBpP,maxBpP;
  //cout<<"Cameras_[camindex]:"<<Cameras_[camindex]<<endl;
  cout<<"Camindex:"<<camindex<<endl;

  //if format 7 mode was selected open GUI for parameters
	//if(format7Frame_ == NULL){
    format7Frame_ = new ShowCamWxFormat7Frame(this,
                  wxT("Select Format"),wxDefaultPosition,wxSize(550,400));
  //}

	// first get
	unsigned int IEEEIndex = 0;

    for (unsigned int j = 0; j < IEEECameraInfo_.size(); j++) {
      if (IEEECameraInfo_[j].camid == atoi((*ParamDevice_[camindex]).c_str())) {
        IEEEIndex = j;
      }
    }

    unsigned int modeindex = 0;
	for(unsigned int j = 0; j < IEEECameraInfo_[IEEEIndex].f7_modes.size();j++){
		if(IEEECameraInfo_[IEEEIndex].f7_modes[j] == mode)
        modeindex = j;
    }

    // set min and max settings
  format7Frame_->SetXSize(IEEECameraInfo_[IEEEIndex].f7_unitWidth[modeindex],
                          IEEECameraInfo_[IEEEIndex].f7_maxWidth[modeindex]);
  format7Frame_->SetXOffset(0,IEEECameraInfo_[IEEEIndex].f7_maxWidth[modeindex]);
  format7Frame_->SetYSize(IEEECameraInfo_[IEEEIndex].f7_unitHeight[modeindex],
                          IEEECameraInfo_[IEEEIndex].f7_maxHeight[modeindex]);
  format7Frame_->SetYOffset(0,IEEECameraInfo_[IEEEIndex].f7_maxHeight[modeindex]);
  format7Frame_->SetBpP(IEEECameraInfo_[IEEEIndex].f7_MinBpP[modeindex],
                        IEEECameraInfo_[IEEEIndex].f7_MaxBpP[modeindex],
                        IEEECameraInfo_[IEEEIndex].f7_bestBpP[modeindex]);

  format7Frame_->SetCameraIDandMode(IEEECameraInfo_[IEEEIndex].camid, mode);


  int ret = format7Frame_->ShowModal();
  if (ret == wxID_CANCEL){
      ret = -1;
  }
  if(ret == wxID_OK/* || ret==wxID_CANCEL*/){
    Format7LeftTop_[camindex] = Vector2<int>(format7Frame_->xOffset,format7Frame_->yOffset);
    Format7WidthHeight_[camindex] = Vector2<int>(format7Frame_->xSize,format7Frame_->ySize);
    Format7BpP_[camindex] = format7Frame_->bpp;
    Format7ColorMode_[camindex] = format7Frame_->colorMode;
    ViewColorMode_[camindex] = format7Frame_->colorModeView;
    cout<<"Selection:"<<Format7LeftTop_[camindex]<< " , Colormode: " << ViewColorMode_[camindex] << " " << (int)ViewColorMode_[camindex] << endl;    
    //format7Frame_ = NULL;
  }

  return ret;
}
#endif
/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::OnChangeVideoMode(wxCommandEvent& event) {

  // probieren den Mode zu setzen und die Kamera neu zu starten...
  int camindex = event.GetId() / 10000 - 1;
  int modeindex = (event.GetId() % 10000) - 1;
  SetUserMode_[camindex] = true;

  switch (*ParamCameras_[camindex]) {
  case 1:{ // DCAM
#ifdef BIAS_HAVE_DCAM
    unsigned int IEEEIndex = 0;

    for (unsigned int j = 0; j < IEEECameraInfo_.size(); j++) {
      if (IEEECameraInfo_[j].camid == atoi((*ParamDevice_[camindex]).c_str())) {
        IEEEIndex = j;
      }
    }

    if(IEEECameraInfo_[IEEEIndex].videoModes[modeindex] < DC1394_VIDEO_MODE_EXIF){

      //user has to select framerate from dropdown menu
      wxArrayString array;
      stringstream helptext;
      helptext << "Supported Fps: ";
      for (unsigned int k = 0; k
             < IEEECameraInfo_[IEEEIndex].framerates[modeindex].size(); k++) {
        stringstream fpss;
        fpss << IEEECameraInfo_[IEEEIndex].framerates[modeindex][k];
        array.Add(AsciiToWx(fpss.str().c_str()));
      }

      wxSingleChoiceDialog choiceDialog(this, wxT("Please choose framerate"),
                                        wxT("Choose Framerate"), array, NULL,
                                        wxDEFAULT_DIALOG_STYLE
                                        | wxRESIZE_BORDER | wxOK | wxCANCEL| wxCENTRE);


      choiceDialog.ShowModal();
      int selectedFps = choiceDialog.GetSelection();
      SelectedFps_[camindex]
        = IEEECameraInfo_[IEEEIndex].framerates[modeindex][selectedFps];

    }
    else{
      ///////////////////////
      //cout<<"Showing selector"<<endl;
      int ret  = ShowFormat7Selector(camindex,IEEECameraInfo_[IEEEIndex].videoModes[modeindex]);
      if(ret < 0)
        return;
    }

    // if camera is active then restart camera
    if (CameraActive_[camindex]) {
      CloseDevice(camindex);
      // wait some time for the cam to reinitialize
      sleep(1);

      OpenDevice(camindex);
    } else {
      OnChangeActivate(event);
      int id = IEEECameraInfo_[IEEEIndex].videoModes.size();
      id += (camindex + 1) * 10000 + 1;
      if (IEEECameraInfo_[IEEEIndex].b_capable)
        id++;
      wxMenuItem* item;
      item = CameraMenu_->FindItem(id);
      item->Check(true);
#ifdef WIN32
      CloseDevice(camindex);
      usleep(10);
      OpenDevice(camindex);
#endif
    }
#endif
  }
    break;

  case 5:{ // UEYE
    // just change framerate
    double fps = 0.0;
    switch (modeindex) {
    case 0:
      fps = 1.875;
      break;
    case 1:
      fps = 3.75;
      break;
    case 2:
      fps = 7.5;
      break;
    case 3:
      fps = 15;
      break;
    case 4:
      fps = 30;
      break;
    case 5:
      fps = 60;
      break;
    default:
      break;
    }
    *ParamFPS_[camindex] = fps;
    //dynamic_cast<VideoSource_uEye*>(Cameras_[camindex])->SetFPS(fps);
    // restart camera
    if (CameraActive_[camindex]) {
      CloseDevice(camindex);
      // wait some time for the cam to reinitialize
      sleep(1);
      OpenDevice(camindex);
    }
  }
    break;
  default:
    break;
  }
}
/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::OnChangeActivate(wxCommandEvent& event){

  int camindex = event.GetId()/10000 -1;
  if(CameraActive_[camindex])
    CloseDevice(camindex);
  else{
   //wxCommandEvent e;
   // OnChangeVideoMode(e);
   OpenDevice(camindex);
  }

}
/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::OnChangeFirewireB(wxCommandEvent& event) {

  int camindex = event.GetId() / 10000 - 1;

  if (SetFirewireB_[camindex]) {
    cout<<"Setting 1394b false."<<endl;
    SetFirewireB_[camindex] = false;
  } else {
    cout<<"Setting 1394b true."<<endl;
    SetFirewireB_[camindex] = true;
  }

  // if camera is active then restart camera
  if (CameraActive_[camindex]) {
    CloseDevice(camindex);
    // wait some time for the cam to reinitialize
    sleep(1);

    OpenDevice(camindex);
  }

}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::OnChangeTrigger(wxCommandEvent& event){
	int camindex = event.GetId() / 10000 - 1;

	  if (SetTrigger_[camindex]) {
	    cout<<"Setting Trigger false."<<endl;
	    SetTrigger_[camindex] = false;
	  } else {
	    cout<<"Setting Trigger true."<<endl;
	    SetTrigger_[camindex] = true;
	  }

	  // if camera is active then restart camera
	  if (CameraActive_[camindex]) {
	    CloseDevice(camindex);
	    // wait some time for the cam to reinitialize
	    sleep(2);
	    OpenDevice(camindex);
	}
}

#ifdef BIAS_HAVE_DCAM
/****************************************************
   @brief


*****************************************************/
std::string ShowCamWxFrame::
VideoModeToString(dc1394video_mode_t mode){
  string s;
  switch(mode){
  case DC1394_VIDEO_MODE_160x120_YUV444:
    s = "160 x 120, YUV444";
    break;
  case DC1394_VIDEO_MODE_320x240_YUV422:
    s = "320 x 240, YUV422";
    break;
  case DC1394_VIDEO_MODE_640x480_YUV411:
    s = "640 x 480, YUV411";
    break;
  case DC1394_VIDEO_MODE_640x480_YUV422:
    s = "640 x 480, YUV422";
    break;
  case   DC1394_VIDEO_MODE_640x480_RGB8:
    s = "640 x 480, RGB";
    break;
  case      DC1394_VIDEO_MODE_640x480_MONO8:
    s = "640 x 480, Mono8";
    break;
  case      DC1394_VIDEO_MODE_640x480_MONO16:
    s = "640 x 480, Mono16";
    break;
  case DC1394_VIDEO_MODE_800x600_YUV422:
    s = "800 x 600, YUV422";
    break;
  case    DC1394_VIDEO_MODE_800x600_RGB8:
    s = "800 x 600, RGB";
    break;
  case    DC1394_VIDEO_MODE_800x600_MONO8:
    s = "800 x 600, Mono8";
    break;
  case DC1394_VIDEO_MODE_800x600_MONO16:
    s = "800 x 600, Mono16";
    break;
  case DC1394_VIDEO_MODE_1024x768_YUV422:
    s = "1024 x 768, YUV442";
    break;
  case DC1394_VIDEO_MODE_1024x768_RGB8:
    s = "1024 x 768, RGB";
    break;
  case DC1394_VIDEO_MODE_1024x768_MONO8:
    s = "1024 x 768, Mono8";
    break;
  case DC1394_VIDEO_MODE_1024x768_MONO16:
    s = "1024 x 768, Mono16";
    break;
  case DC1394_VIDEO_MODE_1280x960_YUV422:
    s = "1280 x 960, YUV422";
    break;
  case DC1394_VIDEO_MODE_1280x960_RGB8:
    s = "1280 x 960, RGB";
    break;
  case DC1394_VIDEO_MODE_1280x960_MONO8:
    s = "1280 x 960, Mono8";
    break;
  case DC1394_VIDEO_MODE_1280x960_MONO16:
    s = "1280 x 960, Mono16";
    break;
  case DC1394_VIDEO_MODE_1600x1200_YUV422:
    s = "1600 x 1200, YUV422";
    break;
  case DC1394_VIDEO_MODE_1600x1200_RGB8:
    s = "1600 x 1200, RGB";
    break;
  case DC1394_VIDEO_MODE_1600x1200_MONO8:
    s = "1600 x 1200, Mono8";
    break;
  case DC1394_VIDEO_MODE_1600x1200_MONO16:
    s = "1600 x 1200, Mono16";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_0:
    s = "Format7 0";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_1:
    s = "Format7 1";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_2:
    s = "Format7 2";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_3:
    s = "Format7 3";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_4:
    s = "Format7 4";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_5:
    s = "Format7 5";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_6:
    s = "Format7 6";
    break;
  case DC1394_VIDEO_MODE_FORMAT7_7:
    s = "Format7 7";
    break;
  default:
    s = "unknown";
  }

  return s;
}

#endif

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::
SelectAndShowCameraImage_(){
  // show camera image in main window if a camera was selected
  unsigned int w=0;
  unsigned int h=0;

  if (Cameras_[NumCamSel_] != NULL)
    {
#ifdef BIAS_HAVE_PMD_PMDZess
      if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]) != NULL)
        {
          // get width and height from combined image
          w = CamImg_[NumCamSel_]->GetWidth()*3;
          h = CamImg_[NumCamSel_]->GetHeight();
        }
      else
        {
#endif // BIAS_HAVE_PMD_PMDZess
          // get width and height from image
          w = CamImg_[NumCamSel_]->GetWidth();
          h = CamImg_[NumCamSel_]->GetHeight();

#ifdef BIAS_HAVE_PMD_PMDZess
        }
#endif // BIAS_HAVE_PMD_PMDZess

      wxString text;
      text.Printf(wxT("size: %4dx%4d"),w,h);
      SetStatusText(text, 1);

      if (CamImg_[NumCamSel_] != NULL)
        {
#ifdef BIAS_HAVE_PMD_PMDZess
          if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[NumCamSel_]) != NULL)
            {
              InitCombinedImage();
              ShowImage_ = CamImgCombined_;
            }
          else
#endif // BIAS_HAVE_PMD_ZESS
            // show grabbed image on main frame
            if(*ParamCameras_[NumCamSel_] == 10){
              switch(ShowXB3_){
              case 1: ShowImage_ = *CamImg_[NumCamSel_]; break;
              case 2: ShowImage_ = *CamImgMiddle_[NumCamSel_]; break;
              case 3: ShowImage_ = *CamImgRight_[NumCamSel_]; break;
              case 4: ShowImage_ = *XB3AllInOne_[NumCamSel_]; break;
              case 0: ShowImage_ = *XB3AllInOne_[NumCamSel_];
                BIAS::ImageBase tmp = ShowImage_;
                ImageConvert::DeinterleaveHorizontal(tmp, ShowImage_);
                break;
              }
            }else{

              ShowImage_ = *CamImg_[NumCamSel_];
              if(ViewColorMode_[NumCamSel_] != ImageBase::CM_invalid){

                //cout << "Integer of ViewMode " <<  (int)ViewColorMode_[NumCamSel_] << endl;
                ShowImage_.SetColorModel((ImageBase::EColorModel)ViewColorMode_[NumCamSel_]);
                //cout << "Colormodel: " << ViewColorMode_[NumCamSel_] << endl;
              }
              //BIAS::ImageBase tmp = ShowImage_;
              //ImageConvert::ToRGB(*CamImg_[NumCamSel_],ShowImage_);
              //cout<<"OT ColorMode:"<<ShowImage_.GetColorModel()<<endl;
            }


          // downsampling of actual camera image if set by param file
          int downSample = *ParamDown_[NumCamSel_];
          if (downSample != 0) {
            Camera<unsigned char> rescaledImage;
            Rescale<unsigned char, unsigned char> rescale;
            rescale.Downsample(ShowImage_, rescaledImage,
                               (int)(ShowImage_.GetWidth() / downSample),
                                 (int)(ShowImage_.GetHeight() / downSample));
            ShowImage_ = rescaledImage;
          }

          /////////////////////////////////////
          // get size available for image canvas
          int width = (int)ShowImage_.GetWidth();
          int height = (int)ShowImage_.GetHeight();
          int widthClient=0, heightClient=0;
          GetClientSize(&widthClient, &heightClient);
          heightClient -= 30;

          if(ShowXB3_ == 1){
            w = 960;
            h = 768;
          }          
          else if (pCheckBoxFit->IsChecked()) {
          
            // compute scale that keeps the aspect ratio and the image is fully visible
            float xscale = (float)widthClient / width;
            float yscale = (float)heightClient / height;
            float scale = xscale < yscale ? xscale : yscale;

            IC_->SetScaledImageSize(width * scale, height* scale);
            IC_->SetShowScaled(true);
          }//fitcheckbox
          else{
            IC_->SetShowScaled(false);
          }
                          
          // set new height or width if changed
          if (w != LastWidth_ || h != LastHeight_){
            LastWidth_  = w;
            LastHeight_ = h;
          }


          // show actual camera image on main frame
          //cout << "Show Image " << ShowImage_.GetColorModel() << endl;
        if (ShowImage_.GetColorModel() != ImageBase::CM_invalid && ShowImage_.GetStorageType() != ImageBase::ST_invalid) {
         IC_->Show(ShowImage_, "Camera Image");
         }
        }
    }
}

/****************************************************
   @brief


*****************************************************/
void ShowCamWxFrame::OnKey(wxKeyEvent& event)
{
  int code = event.GetKeyCode();

  //cout << "Recieved keycode: " << code << endl;
  // grab single shot
  wxCommandEvent e;
  if(code == 32){
    OnGrab(e);
  }
  // grab continious
  if(code == 67){
    OnGrabContinuous(e);
  }
}


/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
OnShowParameters(wxCommandEvent& event){
  ShowParamGUI();
}


void ShowCamWxFrame::
OnAbout(wxCommandEvent &event)
{
  wxString desc;
  desc << wxT(" This tool ShowCamWX is part of the\n");
  desc << wxT(" BIAS library (Basic ImageAlgorithmS)\n");
  desc << wxT(" It is used to open cameras and grab images to disk.\n");
    
  wxString lic = wxT("BIAS is free software; you can redistribute it and/or modify\n \
it under the terms of the GNU Lesser General Public Licence as published by\n \
the Free Software Foundation; either version 2.1 of the Licence, or\n \
(at your option) any later version.\n \
\n \
BIAS is distributed in the hope that it will be useful,\n \
but WITHOUT ANY WARRANTY; without even the implied warranty of\n \
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n \
GNU Lesser General Public Licence for more details.\n \
\n \
You should have received a copy of the GNU Lesser General Public Licence\n \
along with BIAS; if not, write to the Free Software\n \
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");

  wxAboutDialogInfo aboutdlg;
  aboutdlg.SetDescription( desc );
  aboutdlg.SetLicence( lic );
  aboutdlg.SetCopyright( wxT("Copyright (C) 2003-2009\n \
  Multimediale Systeme der Informationsverarbeitung\n \
  Institut fuer Informatik\n \
  Christian-Albrechts-Universitaet Kiel") );
  aboutdlg.SetWebSite( wxT("http://www.mip.informatik.uni-kiel.de/") );

  wxAboutBox( aboutdlg );
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
ShowParamGUI(){
  if(paramFrame_ == NULL){
    paramFrame_ = new wxFrame(this,ID_Paramframe,wxT("Parameters"),
                              wxPoint(500,200),wxSize(550,400));
    paramNotebook_ = GetNotebook(paramFrame_);
    wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
    vbox->Add(paramFrame_,1,  wxEXPAND |wxALL |wxFIXED_MINSIZE , 5);
    //paramFrame_->SetMinSize(wxSize(450,400));
    paramFrame_->SetSizer(vbox);
    paramFrame_->Show(true);
    paramFrame_->Move( wxPoint(500,200));
  }
}


/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveAllImagesUnthreaded_(int cameraIndex){
  //HACK!!!!
#ifdef BIAS_HAVE_PMD_PMDZess
  if (dynamic_cast<VideoSource_usbPMD*>(Cameras_[cameraIndex]) != NULL){
    SaveImagePMDZessUnthreaded_(cameraIndex);
  }
#endif // BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_PMD_PMDTec
  if (dynamic_cast<VideoSource_PMD*>(Cameras_[cameraIndex]) != NULL){
    SaveImagePMDUnthreaded_(cameraIndex);
  }
#endif // BIAS_HAVE_SWISSRANGER


#ifdef BIAS_HAVE_SWISSRANGER
  if (dynamic_cast<VideoSource_SwissRanger*>(Cameras_[cameraIndex]) != NULL){
    SaveImagePMDSRUnthreaded_(cameraIndex);
  }
#endif // BIAS_HAVE_SWISSRANGER


#ifdef BIAS_HAVE_DCAM
  //#ifndef WIN32
  if (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[cameraIndex]) != NULL){
    SaveImageXB3Unthreaded_(cameraIndex);
  }
  else if (dynamic_cast<VideoSource_DCAM*>(Cameras_[cameraIndex]) != NULL){
    SaveImageUnthreaded_(cameraIndex);
  }
#endif //DCAM

#ifdef BIAS_HAVE_UEYE
  if (dynamic_cast<VideoSource_uEye*>(Cameras_[cameraIndex]) != NULL){
    SaveImageUnthreaded_(cameraIndex);
  }
#endif
  //  SaveImageUnthreaded_(cameraIndex);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImagePMDZessUnthreaded_(int cameraIndex){
  wxString text;
  text.Printf(wxT("SINGLE-CAM%02d-%05d-grey.mip"), cameraIndex, GrabCounter_);
  CamImg_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  text.Printf(wxT("SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
  CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
  text.Printf(wxT("SINGLE-CAM%02d-%05d-amp.mip"), cameraIndex, GrabCounter_);
  CamImgPMDAmp_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
}


/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImagePMDSRUnthreaded_(int cameraIndex){
  wxString text;
  CamImg_[cameraIndex]->UpdateMetaData();
  text.Printf(wxT("SINGLE-CAM%02d-%05d-grey.mip"), cameraIndex, GrabCounter_);
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
  CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-%05d-amp.mip"), cameraIndex, GrabCounter_);
  CamImgPMDAmp_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
}

void  ShowCamWxFrame::
SaveImagePMDUnthreaded_(int cameraIndex){
  wxString text;
  CamImg_[cameraIndex]->UpdateMetaData();
  text.Printf(wxT("SINGLE-CAM%02d-%05d-grey.mip"), cameraIndex, GrabCounter_);
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
  CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-%05d-amp.mip"), cameraIndex, GrabCounter_);
  CamImgPMDAmp_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-%05d-int.mip"), cameraIndex, GrabCounter_);
  CamImgPMDInt_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDInt_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDInt_[cameraIndex]);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImageUnthreaded_(int cameraIndex){
  wxString text;
  CamImg_[cameraIndex]->UpdateMetaData();
  text.Printf(wxT("SINGLE-CAM%02d-%05d.mip"), cameraIndex, GrabCounter_);
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
SaveImageXB3Unthreaded_(int cameraIndex){
  wxString text;
  text.Printf(wxT("SINGLE-CAM%02d-left-%05d.mip"), cameraIndex, GrabCounter_);
  CamImg_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-center-%05d.mip"), cameraIndex, GrabCounter_);
  CamImgMiddle_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgMiddle_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgMiddle_[cameraIndex]);

  text.Printf(wxT("SINGLE-CAM%02d-right-%05d.mip"), cameraIndex, GrabCounter_);
  CamImgRight_[cameraIndex]->UpdateMetaData();
  //ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgRight_[cameraIndex]);
  ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgRight_[cameraIndex]);
}

/****************************************************
   @brief


*****************************************************/
void  ShowCamWxFrame::
GrabSingleImageUnthreadedFromCamera_(int cameraIndex){
  int i=0;
  i = cameraIndex;
#ifdef BIAS_HAVE_SWISSRANGER
  if(Cameras_[i] != NULL && dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]) != NULL){
    Cameras_[i]->GrabSingle(*CamImg_[i]);
    dynamic_cast<VideoSource_SwissRanger*>
      (Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
    dynamic_cast<VideoSource_SwissRanger*>
      (Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);
  }else

#endif // BIAS_HAVE_SWISSRANGER
#ifdef BIAS_HAVE_PMD_PMDTec
  if(Cameras_[i] != NULL && dynamic_cast<VideoSource_PMD*>(Cameras_[i]) != NULL){
    Cameras_[i]->GrabSingle(*CamImg_[i]);
    dynamic_cast<VideoSource_PMD*>
      (Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
    dynamic_cast<VideoSource_PMD*>
      (Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);
    dynamic_cast<VideoSource_PMD*>
      (Cameras_[i])->GrabSingleIntensity(*CamImgPMDInt_[i]);
  }else

#endif // BIAS_HAVE_SWISSRANGER

    //#ifndef WIN32
#ifdef BIAS_HAVE_DCAM
    if(dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]) != NULL){
      //cout << "Before Casting" << endl;
      (dynamic_cast<VideoSource_DCAM_XB3*>(Cameras_[i]))->
        GrabSingle(*XB3AllInOne_[i], *CamImg_[i],*CamImgMiddle_[i],*CamImgRight_[i]);
    }else
      //#endif
#endif //not WIN32
      {
        Cameras_[i]->GrabSingle(*CamImg_[i]);
      }
}


/****************************************************
   @brief


*****************************************************/
int ShowCamWxFrame::
CreateMasterThread_(){
	ServerMutex_.resize(NUMBER_CAMERAS);
	ServerCondition_.resize(NUMBER_CAMERAS);
	ClientsMutex_.resize(NUMBER_CAMERAS);
	ClientsCondition_.resize(NUMBER_CAMERAS);
	for (unsigned int i=0;i<NUMBER_CAMERAS;i++) {
		ServerMutex_[i]= new wxMutex();
		ServerCondition_[i] = new wxCondition(*(ServerMutex_[i]));
		ClientsMutex_[i]= new wxMutex();
		ClientsCondition_[i] = new wxCondition(*(ClientsMutex_[i]));
	}

	grabMasterThread_ = new GrabMasterThread(&Cameras_,
			&ClientsCondition_,
			&ServerCondition_,
			&ClientsMutex_,
			&ServerMutex_);
	grabMasterThread_->Create();
	grabMasterThread_->Run();
	usleep(5);
	return 0;
}


/****************************************************
   @brief


*****************************************************/
int ShowCamWxFrame::
CreateGrabThreadForCamera_(int cameraIndex){
  // create new masterthread if none present
  bool gmtLock = false;
  if (grabMasterThread_ == NULL) {
  	CreateMasterThread_();
  } else {
    gmtLock = true;
    ServerMutex_[cameraIndex]->Lock();
    ClientsMutex_[cameraIndex]->Lock();
  }

  //delete thread before creating new one
  if(grabSingleThread_[cameraIndex] != NULL){
    delete grabSingleThread_[cameraIndex];
    grabSingleThread_[cameraIndex] = NULL;
  }

  //create new camera thread
  grabSingleThread_[cameraIndex] =
            new GrabSingleThread(Cameras_[cameraIndex],
                                 ClientsCondition_[cameraIndex],
                                 ServerCondition_[cameraIndex],
                                 ClientsMutex_[cameraIndex],
                                 ServerMutex_[cameraIndex],
                                 cameraIndex,
                                 BufferAccess_[cameraIndex],
                                 bWriteBuffered_,
                                 ReadIndex_[cameraIndex],
                                 WriteIndex_[cameraIndex],
                                 CamImg_[cameraIndex],
                                 CamImgMiddle_[cameraIndex],
                                 CamImgRight_[cameraIndex],
                                 CamImgPMDDepth_[cameraIndex],
                                 CamImgPMDAmp_[cameraIndex],
                                 CamImgPMDInt_[cameraIndex],
                                 XB3AllInOne_[cameraIndex],
                                 ImageBuffer_[cameraIndex],
                                 ImageBufferMiddle_[cameraIndex],
                                 ImageBufferRight_[cameraIndex],
                                 ImageBufferPMDDepth_[cameraIndex],
                                 ImageBufferPMDAmp_[cameraIndex],
                                 ImageBufferPMDInt_[cameraIndex]);



  // try to create and start thread
  if(grabSingleThread_[cameraIndex]!= NULL && !grabSingleThread_[cameraIndex]->IsAlive()){
    if(grabSingleThread_[cameraIndex]->Create()!=wxTHREAD_NO_ERROR) {
      BIASERR("Cannot create GrabSingleThread, saving unthreaded");
    }
    else{
      std::cout<<"Created GrabSingleThread: "<<cameraIndex<<endl;
      grabSingleThread_[cameraIndex]->Run();
      ThreadInited_[cameraIndex] = true;
      // Thread ID ausgeben
      cout << "Created Thread with ID: " << grabSingleThread_[cameraIndex]->GetId()
           << " for CAM: " << cameraIndex << endl;
    }
  }
  else if(grabSingleThread_[cameraIndex]== NULL){
    BIASERR("Thread:"<<cameraIndex<<" is NULL!");
  }
  else if(grabSingleThread_[cameraIndex]->IsAlive()){
    BIASERR("Thread:"<<cameraIndex<<" is already Alive!");
  }

  if (gmtLock) {
    ServerMutex_[cameraIndex]->Unlock();
    ClientsMutex_[cameraIndex]->Unlock();
  }

  return 0;
}



/****************************************************
   @brief


*****************************************************/
int ShowCamWxFrame::
CreateImageSaveThreadForCamera_(int cameraIndex){
  int i = 0;
  i = cameraIndex;
  stringstream text;
  text<<"CAM"<<FileHandling::LeadingZeroString(i,2);

  // #ifdef BIAS_HAVE_PanTilt
  //     if(*bUsePanTiltUnit_){
  //       (*CamImgPMDDepth_[i]).SetProj(projHeader);
  //       (*CamImgPMDAmp_[i]).SetProj(projHeader);
  //       (*CamImgPMDAmp_[i]).UpdateMetaData();
  //       (*CamImgPMDDepth_[i]).UpdateMetaData();
  //     }
  // #endif

  // create new thread
  saveThreads_[i] = new ImageSaveThread(text.str(),
                                        *ParamDown_[i],
                                        BufferAccess_[i],
                                        bWriteBuffered_,
                                        ReadIndex_[i],
                                        WriteIndex_[i],
                                        CamImg_[i],
                                        CamImgMiddle_[i],
                                        CamImgRight_[i],
                                        CamImgPMDDepth_[i],
                                        CamImgPMDAmp_[i],
                                        CamImgPMDInt_[i],
                                        ImageBuffer_[i],
                                        ImageBufferMiddle_[i],
                                        ImageBufferRight_[i],
                                        ImageBufferPMDDepth_[i],
                                        ImageBufferPMDAmp_[i],
                                        ImageBufferPMDInt_[i],
                                        bSaveToStream_);

  // init the thread or capture single image
  if(saveThreads_[i] != NULL && saveThreads_[i]->Create() != wxTHREAD_NO_ERROR){
    BIASERR("Cannot create ImageSaveThread, saving unthreaded");
  }//saveThreads_[i] != NULL && saveThreads_[i]->Create()
  else{
    cout << "Run Thread" << endl;
    saveThreads_[i]->SetPriority(100);
    saveThreads_[i]->Run();
  }
  return 0;
}
