/*
 * getPmdData.cpp
 *
 *  Created on: Jan 22, 2010
 *      Author: africk
 */

#include <Base/Image/ImageIO.hh>
#include <Utils/Param.hh>
#include <VideoSource/VideoSource_PMD.hh>

using namespace std;
using namespace BIAS;

/**
   @class biasgetpmddata
   @ingroup g_tools
   @brief program for calculating pmd data from source images
          with pmd source descriptor as meta data
   @author africk
   */
int main(int argc, char** argv) {
	Param *p = new Param();

	p->AddParamString("pmdImagesList",
			"list of pmd images with discriptor data saved as meta data", "");

	p->ParseCommandLine(argc, argv);

	string imageListName = *(p->GetParamString("pmdImagesList"));

	vector<string> imageNames;
	if (Param::ParseListFile(imageListName, imageNames) != 0) {
		BIASERR("error: loading " << imageListName << endl);
		return -1;
	} else {
		cout << "loaded : " << imageListName << endl;
	}

	VideoSource_PMD vsPMD;
	vsPMD.OpenProcessingPlugin();
	Camera<char> camera;
	Camera<float> newCamera;
//	Camera<float> tmpCamera;
//	tmpCamera.Init(204,204,1);
	for (unsigned int i = 0; i < imageNames.size(); i++) {
		if (ImageIO::Load(imageNames[i], camera) != 0) {
			BIASERR("error loading "<< imageNames[i]);
		} else {
			camera.ParseMetaData();
		}

		string imageName = imageNames[i];

		MetaData* metaData = camera.GetMetaData();

		if (metaData != NULL) {
			AppData appData;
			int res = metaData->Find(AppData::MD_PMD_DataDesc, appData);
			if (res >= 0) {
				vsPMD.CalcIntensitiesFromSourceData(camera, newCamera);
				newCamera.FlipHorizontal();
				newCamera.SetMetaData(*metaData);
				ImageIO::Save("int_" + imageName, newCamera);
				vsPMD.CalcDistancesFromSourceData(camera, newCamera);
				newCamera.FlipHorizontal();
				newCamera.ScaleShift(1000.0,0);
				newCamera.SetMetaData(*metaData);
				ImageIO::Save("depth_" + imageName, newCamera);
				vsPMD.CalcAmplitudesFromSourceData(camera, newCamera);
				newCamera.FlipHorizontal();
				newCamera.SetMetaData(*metaData);
				ImageIO::Save("amp_" + imageName, newCamera);
				cout << imageName << " done ..." << endl;
			} else {
				BIASERR("No MD_PMD_DataDesc meta data found for " << imageName);
			}
		} else {
			BIASERR("  no meta data exists for " << imageName);
		}

	}

	vsPMD.CloseProcessingPlugin();

	return 0;

}
