/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Image/Image.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Utils/IOUtils.hh>
#include <Utils/Param.hh>
#include <Base/Common/FileHandling.hh>

using namespace std;
using namespace BIAS;


/**
   @class biasrgb2luv
   @ingroup g_tools
   @brief convert a rgb image into float luv color space image
   @author bartczak 12/2008
 */
int main(int argc, char* argv[]) {

  Param params(true);
  int fup;
  if(!IOUtils::ParseCommandLineEvalHelp(params, argc, argv, fup)) {
    cout<<"Usage:\n";
    cout<<"\t"<<argv[0]<<" <rgb_image.###> \n";
    cout<<"result is delivered into a float image named '<rgb_image>_LUV.mip'.\n\n";
  }

  ImageBase in;
  if(ImageIO::Load(argv[fup], in)!=0) {
    BIASERR("error loading input image named :"<<argv[fup]);
    return -1;
  }

  Image<float> out;
  switch(in.GetStorageType()) {
    case ImageBase::ST_unsignedchar :
      if(ImageConvert::RGBToLUV(Image<unsigned char>(in), out)!=0) {
        BIASERR("error converting to LUV!");
        return -1;
      }
      break;
    case ImageBase::ST_float :
      if(ImageConvert::RGBToLUV(Image<float>(in), out)!=0) {
        BIASERR("error converting to LUV!");
        return -1;
      }
      break;
    default:
      BIASERR("not implemented for storage type :"<<in.GetStorageType());
      return -1;
      break;
  }


  out.SetMetaData(*in.GetMetaData());

  string basenameIn = FileHandling::Basename(argv[fup]);
  //if(ImageIO::Save(basenameIn+"_LUV.mip", out)!=0) {
  if(ImageIO::Save(basenameIn+"_LUV.mip", out)!=0) {
    BIASERR("error saving output image named :"<<basenameIn+"_LUV.mip");
    return -1;
  }
  cout<<"\ndone!\n";
  return 0;
}
