#ifndef __GlutInitWrapper_hh__
#define __GlutInitWrapper_hh__

#include <Base/Common/Singleton.hh>

namespace BIAS {

  /**
   * @class GlutInitWrapper 
     @brief To call glutInit() at the right time and only once is not easy in
     complex applications. This wrapper helps to avoid calling it
     twice or more times.  At the eraliest possible place right after
     the gl-context is ready, just call:

     GlutInitWrapper::GetInstance()->Init();
     No need to protect it agains multiple calls, this is checked internally.
		 @ingroup g_openglframework	
		 @author ischiller, adopted from glewinitwrapper
   */
  class BIASUtils_EXPORT GlutInitWrapper: public Singleton<GlutInitWrapper>
  {
    
  public:
    int Init();
    int Init(int argc, char* argv[]);
    bool IsInitalised() {return IsInitialised_;} ;

    friend class BIAS::Singleton<GlutInitWrapper>;

  protected:

    GlutInitWrapper();

    bool IsInitialised_;
  }; //class


} //namespace


#endif
