#ifndef __STRING_UTILS_HH__
#define __STRING_UTILS_HH__

// STL
#include <string>
#include <vector>

#include <BIAS_DeclSpec.hh>

namespace BIAS {

  /**
   * Contains some useful methods for string manipulation.
   *
   * @author rwulff 04/2010
   */
  class BIASUtils_EXPORT StringUtils {

  public:

    /**
     * @brief Explodes the given string into tokens which are sperated by the
     *        given seperator.
     *
     * Usage example:
     *
     * std::vector<std::string> retVec;
     * StringUtils.Explode("token1 token2 token3", " ", retVec);
     *
     * retVec will now contain the three elements "token1", "token2" and
     * "token3"
     *
     * @param str
     *        the string to explode
     * @param seperator
     *        the string that seperates each token from the next
     * @param ret
     *        a vector that contains the single tokens
     */
    static void Explode(const std::string &str,
                        const std::string &seperator,
                        std::vector<std::string> &ret);


    /**
     * @brief Trims the given string, i.e. removes all permutations of the
     *        given charsToTrim from the front and the end of the string.
     *
     * Usage example:
     *
     * std::string retStr;
     * StringUtils::Trim("_/_string_/_", retStr, "/_");
     *
     * retStr will now be "string"
     *
     * @param str
     *        the string to trim
     * @param ret
     *        the trimmed string
     * @param charsToTrim
     *        the characters to remove from the front and the end of the string.
     *        the default is to remove all whitespace charakters (spaces, tabs
     *        and newlines). note that the order of characters is irrelevant
     */
    static void Trim(const std::string &str,
                     std::string &ret,
                     const std::string &charsToTrim = " \t\n");

  }; // end of class
} // end of namespace
#endif // __STRING_UTILS_HH__