#include "DShowVideo.hh"
#include <BIAS_DeclSpec.hh>
#include <Base/Debug/Exception.hh>
#include "SampleGrabberCallback.hh"
#include <Base/Debug/Debug.hh>
//#include <Base/Helper.hh>



#ifdef BIAS_HAVE_DSHOW
# include <windows.h>
# include <atlbase.h>
# include <dshow.h>
////see: http://social.msdn.microsoft.com/Forums/en-US/windowssdk/thread/ed097d2c-3d68-4f48-8448-277eaaf68252/
//# pragma include_alias( "dxtrans.h", "qedit.h" )
# define __IDxtCompositor_INTERFACE_DEFINED__
# define __IDxtAlphaSetter_INTERFACE_DEFINED__
# define __IDxtJpeg_INTERFACE_DEFINED__
# define __IDxtKey_INTERFACE_DEFINED__
# include <qedit.h>

#endif

namespace BIAS 
{
	class DShowStuff {
	public:
		DShowStuff(){};
		~DShowStuff(){};
#ifdef BIAS_HAVE_DSHOW
		CComPtr< IMediaControl >    pMediaControl_;
		CComPtr< IMediaEventEx >	  pMediaEvent_;    
		CComPtr< IMediaSeeking >    pMediaSeeking_;
		CComPtr< ISampleGrabber >   pSampleGrabber_;
		CComPtr< IVideoFrameStep>   pVideoFrameStep_;  //test
		CComPtr< IMediaFilter>      pMediaFilter_;
		CComPtr< IBaseFilter >	    pNullRenderer_; //stepping doesnt seem to work with nullrenderer
		CComPtr< IGraphBuilder >          pGraphBuilder_;
		CComPtr< IBaseFilter >            pSrcFilter_  ;
		CComPtr< IPin >		pSrcOutPin_;
		CComPtr< IPin >		pSampleGrabberOutPin_;
		CComPtr< IPin >		pSampleGrabberInPin_;
		CComPtr< IPin >     pNullRendererInPin_;
#endif
	};
}

using namespace BIAS;
using namespace std;

#define RAISE(arg)\
  { \
    cout << "exception: " << arg << endl;  \
    throw BaseException(arg);\
  }
#define CHECK_HR(hr,msg) { if (hr!=S_OK) { RAISE(msg); } }

DShowVideo::
DShowVideo(SampleGrabberCallback *sgcb)
{
  SGCB_ = sgcb;
  DShowStuff_ = new DShowStuff;
  Initialised_ = false;
  Open_ = false;
  FrameNum_ = 0;
  IsRunning_ = false;
  IsPaused_ = false;
  FastNLazy_ = true;  //TODO set to true as default
  Width_ = Height_ = ChannelCount_ = 0;
}


DShowVideo::
~DShowVideo()
{
	Close();
  if (Initialised_){
#ifdef BIAS_HAVE_DSHOW
    CoUninitialize();
#endif
  }
}

bool DShowVideo::
Init_()
{
#ifdef BIAS_HAVE_DSHOW
  //sample code 
  //http://www.codeproject.com/KB/audio-video/framegrabber.aspx?fid=2584&df=90&mpp=25&noise=3&sort=Position&view=Quick&fr=101&select=193093
  //und
  //BIAS::VideoSource_Dshow

  if (!Initialised_){
    //	cout<<"DShowVideo::initializing"<<endl;
    HRESULT hr;  // initialize the COM library.    
    CoInitialize(0);
    // Get DirectShow interfaces
    // hr = GetInterfaces_();
    //------------------------------
    // Create the filter graph
    hr = CoCreateInstance(CLSID_FilterGraph, NULL, CLSCTX_INPROC_SERVER,
                          IID_IGraphBuilder, 
                          (void **) &DShowStuff_->pGraphBuilder_);
    CHECK_HR(hr,"CoCreateInstance GraphBuilder failed");

    // Obtain interfaces for media control and Video Window
    hr = DShowStuff_->pGraphBuilder_->QueryInterface(IID_IMediaControl,
                                                     (LPVOID *) &DShowStuff_->pMediaControl_);
    CHECK_HR(hr,"QueryInterface MediaControl failed");
 

    hr = DShowStuff_->pGraphBuilder_->QueryInterface(IID_IMediaFilter,
                                                     (LPVOID *) &DShowStuff_->pMediaFilter_);
    CHECK_HR(hr,"QueryInterface IMediaFilter failed");
 

    //use IMediaSeeking for positioning 
    
    hr = DShowStuff_->pGraphBuilder_->QueryInterface(IID_IMediaSeeking, 
                                                     (LPVOID *) &DShowStuff_->pMediaSeeking_);
    CHECK_HR(hr,"QueryInterface MediaSeeking failed");

    hr = DShowStuff_->pGraphBuilder_->QueryInterface(IID_IMediaEvent,
                                                     (LPVOID *) &DShowStuff_->pMediaEvent_);
    CHECK_HR(hr,"QueryInterface MediaEvent failed");
 
    // Create NullRenderer Instance
    hr = DShowStuff_->pNullRenderer_.CoCreateInstance(CLSID_NullRenderer);
    CHECK_HR(hr,"CoCreateInstance NullRenderer failed");
    hr = DShowStuff_->pGraphBuilder_->AddFilter(DShowStuff_->pNullRenderer_,
                                                L"NullRenderer");
    CHECK_HR(hr,"AddFilter(NullRenderer) failed");

    // Create SampleGrabber Instance and register callback for each frame
    hr = DShowStuff_->pSampleGrabber_.CoCreateInstance(CLSID_SampleGrabber);
    CHECK_HR(hr,"CoCreateInstance SampleGrabber failed");
    {
      CComQIPtr< IBaseFilter, &IID_IBaseFilter > 
        pGrabberBase( DShowStuff_->pSampleGrabber_ );
      hr = DShowStuff_->pGraphBuilder_->AddFilter(pGrabberBase,L"SampleGrabber");
      CHECK_HR(hr,"AddFilter(NullRenderer) failed");
      pGrabberBase.Release();
    }
    // configure SampleGrabber
    hr = DShowStuff_->pSampleGrabber_->SetBufferSamples(false);
    CHECK_HR(hr,"SetBufferSample failed");
    hr = DShowStuff_->pSampleGrabber_->SetOneShot(false); //was false
    CHECK_HR(hr,"SetOneShot failed");
    // TODO register callback 
    DShowStuff_->pSampleGrabber_->SetCallback(SGCB_,0);
 
    //  
    // can not connect OutPin of SampleGrabber with InPin of
    // NullRenderer here First, they have to be configured according to
    // media type of pSrcFilter which is creatd in Open()

    AM_MEDIA_TYPE mt;
    ZeroMemory(&mt,sizeof(AM_MEDIA_TYPE));
    mt.majortype = MEDIATYPE_Video;
    mt.subtype = MEDIASUBTYPE_RGB24;
    mt.formattype = FORMAT_VideoInfo; 
    hr = DShowStuff_->pSampleGrabber_->SetMediaType(&mt);
    CHECK_HR(hr,"SetMediaType failed");
    Initialised_ = true;
  } else {
    // this should not happen
  }
#endif
  
  return true;
}



bool DShowVideo::
Open(const string &filename)
{
  cout<<"opening avi "<<filename<<endl;
  Init_();

#ifdef BIAS_HAVE_DSHOW
  HRESULT hr;

  int length = MultiByteToWideChar(CP_ACP,0,filename.c_str() ,
                                   filename.size()+1,NULL,0);
  LPWSTR wc_filename = new WCHAR[length];
  MultiByteToWideChar(CP_ACP,0,filename.c_str() ,filename.size()+1, 
                      wc_filename,length);
 
  // Automatically generate graph nodes to decode file
  hr = DShowStuff_->pGraphBuilder_->AddSourceFilter(wc_filename,L"SourceFilter",
                                                    &DShowStuff_->pSrcFilter_); 
  CHECK_HR(hr,"AddSourceFilter failed");
  
  // connect first out-pin of SrcFilter with in-pin of SampleGrabber
  CComQIPtr< IBaseFilter, &IID_IBaseFilter > pGrabberBase( DShowStuff_->pSampleGrabber_ );
  hr = pGrabberBase->FindPin(L"In",&DShowStuff_->pSampleGrabberInPin_);
  CHECK_HR(hr,"Failed to find in-pin of SampleGrabber");
    
  IEnumPins  *pEnum = NULL;
  IPin       *pPin  = NULL;
  hr = DShowStuff_->pSrcFilter_->EnumPins(&pEnum);
  CHECK_HR(hr,"EnumPins failed on SrcFilter");
  while(pEnum->Next(1, &pPin, 0) == S_OK)  {
    PIN_DIRECTION PinDirThis;
    hr = pPin->QueryDirection(&PinDirThis);
    if (FAILED(hr))    {
      pPin->Release();
      pEnum->Release();
      pGrabberBase.Release();
      return false;
    }
		LPWSTR pinName;
		pPin->QueryId(&pinName);
    if (PINDIR_OUTPUT == PinDirThis)    {
      DShowStuff_->pSrcOutPin_ = pPin;
      pPin->Release();
      break;
    }
    // Release the pin for the next time through the loop.
    pPin->Release();
  }
  pEnum->Release();

  hr = DShowStuff_->pGraphBuilder_->Connect(DShowStuff_->pSrcOutPin_,
                                            DShowStuff_->pSampleGrabberInPin_);
  CHECK_HR(hr,"Failed to connect SrcFilter with SampleGrabber");
  //cout<<"Succesfully connected SrcFilter and SampleGrabber Pins" << endl;

  // connect out-pin of SampleGrabber with in-pin of NullRenderer
  hr = pGrabberBase->FindPin(L"Out",&DShowStuff_->pSampleGrabberOutPin_);
  CHECK_HR(hr,"Failed to find out-pin of SampleGrabber");

  hr = DShowStuff_->pNullRenderer_->FindPin(L"In",
                                            &DShowStuff_->pNullRendererInPin_);
  CHECK_HR(hr,"Failed to find out-pin of SampleGrabber");

  hr = DShowStuff_->pGraphBuilder_->Connect(DShowStuff_->pSampleGrabberOutPin_,
                                            DShowStuff_->pNullRendererInPin_);
  CHECK_HR(hr,"Failed to connect SampleGrabber with NullRenderer");
  //cout<<"Succesfully connected SampleGrabber and NullRenderer Pins" << endl;

  AM_MEDIA_TYPE MediaType; 
  ZeroMemory(&MediaType,sizeof(MediaType)); 
  hr = DShowStuff_->pSampleGrabber_->GetConnectedMediaType(&MediaType);   
  CHECK_HR(hr,"GetConnectedMediaType failed");

  // Get a pointer to the video header. 
  VIDEOINFOHEADER *pVideoHeader = (VIDEOINFOHEADER*)MediaType.pbFormat; 
  if (pVideoHeader == NULL) {
    pGrabberBase.Release();
    return false; 
  }
  // The video header contains the bitmap information. 

  // Copy it into a BITMAPINFO structure. 
  BITMAPINFO BitmapInfo; 
  ZeroMemory(&BitmapInfo, sizeof(BitmapInfo)); 
  CopyMemory(&BitmapInfo.bmiHeader, &(pVideoHeader->bmiHeader), 
             sizeof(BITMAPINFOHEADER)); 
  
  Width_ = BitmapInfo.bmiHeader.biWidth;
  Height_ = BitmapInfo.bmiHeader.biHeight;
  ChannelCount_ = BitmapInfo.bmiHeader.biBitCount/8;

  DShowStuff_->pMediaSeeking_->SetTimeFormat(&TIME_FORMAT_FRAME);
  DShowStuff_->pMediaSeeking_->SetRate(128); 
  DShowStuff_->pSrcFilter_->SetSyncSource(NULL);  //<-------------------
  pGrabberBase->SetSyncSource(NULL);
  pGrabberBase.Release();
  DShowStuff_->pNullRenderer_->SetSyncSource(NULL);
  //render as fast as possible dont sync graph
  DShowStuff_->pMediaFilter_->SetSyncSource(NULL); 

  long long currentPos,stopPos;  
  currentPos=0;
  stopPos = 1;  
  DShowStuff_->pMediaSeeking_->SetPositions(&currentPos,
                                            AM_SEEKING_AbsolutePositioning,
                                            &stopPos,
                                            AM_SEEKING_AbsolutePositioning );
	
  DShowStuff_->pMediaControl_->Pause(); 
#endif
  Open_ = true;
  IsRunning_ = false;
  IsPaused_ = false;
  FrameNum_=0;
  return true;
}


void DShowVideo::
PlaySingleFrame()
{
  //cout << "frame "<<FrameNum_<<" requested\n";

#ifdef BIAS_HAVE_DSHOW
  long long currentPos,stopPos;  
  currentPos=FrameNum_;
  stopPos = currentPos;  
  DShowStuff_->pMediaSeeking_->SetPositions(&currentPos,AM_SEEKING_AbsolutePositioning,
                                            &stopPos,AM_SEEKING_AbsolutePositioning );
	
  SGCB_->SampleTriggerArm(); 
  DShowStuff_->pMediaControl_->Run(); 
  //while(DShowStuff_->pMediaControl_->Run()!=S_OK){USleep(2000);}; //ignore the result....
  //OAFilterState  fs;
  //DShowStuff_->pMediaControl_->GetState(4 /*msecs timout*/, &fs);  

  if (SGCB_->HasCallbackFunction()){ 
    FrameNum_++; 
    return; 
  }

  unsigned timeoutcount=0;  //counts loops without sampletrigger signal
  unsigned paniclevel  =0;  //counter timeouts in nonfastnlazymode.
  while (!SGCB_->SampleTriggered()) //wait until grabbercallback has been executed
  {
	  Sleep(1000/1000);
	  timeoutcount++;
	  if(timeoutcount>30) //samplegrabbercallback timed out
	  {
		  //cout<<"sampling failed"<<endl;		      
      if(FastNLazy_)
      {
        return; //timeout suggests that play has failed for some reason              
      }
      else
      {
        timeoutcount = 0;   //resest timeout counter
        if(paniclevel>100)  //read avi really really failed! we are doomed!
        {
          cout<<"avi read panic!"<<endl;
          //TODO throw exception
          return;
        }
        //retry reading
        //cout<<"retrying"<<endl;
        DShowStuff_->pMediaSeeking_->
        SetPositions(&currentPos,AM_SEEKING_AbsolutePositioning,
        &stopPos,AM_SEEKING_AbsolutePositioning );
        DShowStuff_->pMediaControl_->Run(); 
        paniclevel++;
      }      		  
    }
  }  

  FrameNum_++;
#endif

}

void DShowVideo::
PlayContinously()
{
#ifdef BIAS_HAVE_DSHOW
  HRESULT hr;
  int tries = 0;
  do {
	  hr = DShowStuff_->pMediaControl_->Run();
	  tries++;
	  if (hr != S_OK) Sleep(10);
	  else  IsRunning_ = true;
  }  while (hr != S_OK && tries <10);
  
  CHECK_HR(hr,"DirectShow Run failed");
#endif
}



void DShowVideo::
Pause()
{
	//cout <<"ReadAvi::Pause()"<<endl;
#ifdef BIAS_HAVE_DSHOW
  HRESULT hr;
  int tries = 0;
  do {
	  hr = DShowStuff_->pMediaControl_->Stop();
	  tries++;
	  if (hr != S_OK) Sleep(1);
	  else  IsRunning_ = true;
  }  while (hr != S_OK && tries <10);
  CHECK_HR(hr,"Stop failed");
    IsPaused_ = true;
  long evCode;
  hr = DShowStuff_->pMediaEvent_->WaitForCompletion(INFINITE, &evCode); 
  CHECK_HR(hr,"WaitForCompletion failed");
#endif  

}

void DShowVideo::
GotoFrame(unsigned long long frameNo)
{
#ifdef BIAS_HAVE_DSHOW
  //HRESULT hr;
  int tries=0;
	// TODO check pMedisControl for running
  FrameNum_ = frameNo;
	//do {
	//  hr = DShowStuff_->pMediaControl_->Stop();
	//  tries++;
	//  if (hr != S_OK) Sleep(1);
	//  else  IsRunning_ = true;
 // }  while (hr != S_OK && tries <10);
 // CHECK_HR(hr,"Stop failed");
 // long long seekPos = frameNo;
 // DShowStuff_->pMediaSeeking_->SetPositions(&seekPos,AM_SEEKING_AbsolutePositioning,
 //   NULL, AM_SEEKING_NoPositioning);
#endif
}


void DShowVideo::
Close()
{
	if (Open_ && DShowStuff_) {
#ifdef BIAS_HAVE_DSHOW
    HRESULT hr;
    cout << "ReadAvi::Close()\n";
    // Release all COM objects
    if (DShowStuff_->pMediaControl_) { 
      DShowStuff_->pMediaControl_->Stop(); 
    }
    if (DShowStuff_->pGraphBuilder_){
      // Automagically generate graph nodes to decode file
      if (DShowStuff_->pSrcFilter_){
        hr = DShowStuff_->pGraphBuilder_->
          RemoveFilter(DShowStuff_->pSrcFilter_); 
        CHECK_HR(hr,"RemoveFilter failed");
      }
      if (DShowStuff_->pNullRenderer_){
        hr = DShowStuff_->pGraphBuilder_->
          RemoveFilter(DShowStuff_->pNullRenderer_); 
        CHECK_HR(hr,"RemoveFilter failed");
      }
      if (DShowStuff_->pSampleGrabber_){
        CComQIPtr< IBaseFilter, &IID_IBaseFilter > 
          pGrabberBase( DShowStuff_->pSampleGrabber_ );
        hr = DShowStuff_->pGraphBuilder_->RemoveFilter(pGrabberBase); 
        CHECK_HR(hr,"RemoveFilter failed");
        pGrabberBase.Release();
      }
    }

    if (DShowStuff_->pMediaControl_){
      DShowStuff_->pMediaControl_.Release();
    }
    if (DShowStuff_->pMediaEvent_){
      DShowStuff_->pMediaEvent_.Release();
    }
    if (DShowStuff_->pMediaSeeking_){
      DShowStuff_->pMediaSeeking_.Release();
    }
    if (DShowStuff_->pSrcOutPin_){
      DShowStuff_->pSrcOutPin_.Release();
    }
    if (DShowStuff_->pSampleGrabberInPin_){
      DShowStuff_->pSampleGrabberInPin_.Release();
    }
    if (DShowStuff_->pSampleGrabberOutPin_){
      DShowStuff_->pSampleGrabberOutPin_.Release();
    }
    if (DShowStuff_->pNullRendererInPin_){
      DShowStuff_->pNullRendererInPin_.Release();
    }
    if (DShowStuff_->pSrcFilter_){
      DShowStuff_->pSrcFilter_.Release();
    }
    if (DShowStuff_->pSampleGrabber_){
      DShowStuff_->pSampleGrabber_.Release();
    }
    if (DShowStuff_->pNullRenderer_){
      DShowStuff_->pNullRenderer_.Release();
    }
    if (DShowStuff_->pMediaFilter_){
      DShowStuff_->pMediaFilter_.Release();
    }
    if (DShowStuff_->pGraphBuilder_){
      DShowStuff_->pGraphBuilder_.Release();
    }

#endif
    Open_ = false;
    Width_ = Height_ = ChannelCount_ = 0;
  }
}

bool DShowVideo::
GetImageSize(unsigned &width, unsigned &height,
             unsigned &channel_count)
{
  if (Width_==0 || Height_==0 || ChannelCount_==0) { return false; }
  width = Width_;
  height = Height_;
  channel_count = ChannelCount_;
  return true;
}
