/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __READAVI_HH__
#define __READAVI_HH__
#include <BIAS_DeclSpec.hh>
#include <string>

namespace BIAS {
/**
 *   \test tested with TestDShowVideo.cpp
 *   \class DShowVideo
		 \brief Die Klasse ReadAvi ist zentraler Bestandteil der Infrastruktur zum
				Dekodieren von AVI-Videocontainern. Weitere Bestandteile sind:
				SampleGrabberCallback.
				
				ReadAvi baut einen DShow-Graphen auf aus drei Filtern: pSrcFilter_ zum
				oeffnen und dekodieren des Videos, pSampleGrabber um einzelne Samples
				abzugreifen und pNullRenderer umd die Vollstaendigkeit des Graphs mit
				einer Senke zu gewaehrleisten. Die Filter werden ueber ihre jeweiligen
				In und Out-Pins verbunden.  Die Klasse SampleGrabberCallback muss dem
				Konstruktor von ReadAvi mitgegeben werden. Fuer jedes Frame wird dann
				von DShow die Methode SampleGrabberCallback::SampleCB(..)
				aufgerufen. Diese kann dann die Bilddaten herausgeben.
				
				Um im Code der Klassen die ReadAvi u.a. benutzen keine #ifdef ...DSHOW...
				einbauen zu muessen, sollen die Klassen und alle Interfaces _immer_ 
				mitkompiliert werden. Falls DShow nicht zur Verfuegung steht, sind die 
				Methoden der Klassen ReadAvi und SampleGrabberCallback eben leer.
				\author Max Hollmann 
 */

  class SampleGrabberCallback;
  class DShowStuff;

  class BIASVideoSource_EXPORT DShowVideo 
  {
  public:
    DShowVideo(SampleGrabberCallback *sgcb);

    ~DShowVideo();

    /// open file, if not possible throw exception
    bool Open(const std::string &filename);

    void Close();

    // to allow SampleGrabberCallback to stop immedeatly after one frame
    void Pause();

	  void GotoFrame(unsigned long long frameNo);

    void PlaySingleFrame();
	  
    void PlayContinously();

    SampleGrabberCallback *GetSampleGrabberCallback()
    {return SGCB_;};

    /** If grabber is in realibale mode and
        frame grabbing failed for some reason
        the reader retrys to read the current frame.
        This mode can slow down frame reading so
        don't use it for realtime playback. */
    void EnableReliableFrameRead(const bool enabled)
    {FastNLazy_ = !enabled;};

    /** returns the image size contained in the stream, only meaningfull
        results after call to Open() function */
    bool GetImageSize(unsigned &width, unsigned &height,
                      unsigned &channel_count);

  protected:

    bool Init_();
    bool IsRunning_;
    bool IsPaused_;

    unsigned long long FrameNum_;	
    SampleGrabberCallback *SGCB_;

    DShowStuff *DShowStuff_;
    bool Initialised_;
    bool Open_;
    bool FastNLazy_;
    unsigned Width_, Height_, ChannelCount_;
  };


} // namespace

#endif
