#include <Base/Common/W32Compat.hh> // sprintf wrn
#include <VideoSource/VideoSource_Base.hh>
#include <VideoSource/VideoSource_DCAM.hh>

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Image/Camera.hh>
#include <Gui/ImageCanvas.hh>
#include <Gui/ConsoleRedirectIO.hh>


using namespace BIAS;
using namespace std;

bool done = false;
bool save = false;

#define GUI
#ifdef GUI

class MyApp : public wxApp
{
  virtual bool OnInit();
};

class MyFrame: public wxFrame
{
public:

  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void OnQuit(wxCommandEvent& event);
  void OnTimer(wxTimerEvent& event);

 DECLARE_EVENT_TABLE()
protected:
  wxButton *QuitButton_, *WhiteBalanceButton_;
  ImageCanvas *ic_;
  wxTimer      Timer_;
  VideoSource_DCAM cam;
  Camera<unsigned char> pic,rgb;
  const string camfile;
  int framecount;
};


enum
{
  ID_Quit = 1,
  ID_BUTTON_QUIT,
  ID_TIMER = 1001
};


BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(ID_Quit, MyFrame::OnQuit)
  EVT_BUTTON(ID_BUTTON_QUIT, MyFrame::OnQuit)
  EVT_TIMER(ID_TIMER, MyFrame::OnTimer)
END_EVENT_TABLE()

IMPLEMENT_APP(MyApp)

bool MyApp::OnInit()
{
//  ConsoleRedirectIO();
//  int channel = 0;
//  int res = ioctl(open("/dev/video1394-0",O_RDONLY),VIDEO1394_IOC_UNLISTEN_CHANNEL,&channel)
//cout <<res;
  MyFrame *frame =
      new MyFrame( wxT("MDCam2ShmWx"), wxPoint(50,50), wxSize(800,600) );
SetTopWindow(frame);

  return TRUE;
}

MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
  wxMenu *menuFile = new wxMenu;
  menuFile->Append( ID_Quit, wxT("E&xit") );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append( menuFile, wxT("&File") );

  SetMenuBar( menuBar );
  wxStatusBar* s = CreateStatusBar();
  SetStatusText( wxT("") );

  wxSizer *Sizer = new wxBoxSizer( wxHORIZONTAL );


  ic_ = new ImageCanvas(this,s,0);
  ic_->SetSize(1024,768);
  Sizer->Add(ic_,1, wxEXPAND |wxALL , 5);
  rgb.Init(1024,768,3);
  rgb.FillImageWithConstValue(255);
  SetSizer( Sizer );

  Timer_.SetOwner(this, ID_TIMER);

  cam.SetDebugLevel(D_DCAM );
  //cam.SetDebugLevelDcam(D_DCAM_F7 |D_DCAM );

  cam.SetSize(1024,768);
  cam.SetColorModel(ImageBase::CM_UYVY422);
cam.SetFPS(7.5);
//cam.SetFormat7(0,1536,DC1394_COLOR_CODING_YUV422);


  int  res = cam.OpenDevice();
  if (res!=0) {
      cout<<"Failed to open device"<<endl;
      exit(-1);
  }

  cam.InitImage(pic);

  cam.PreGrab();

  framecount = 0;
  ic_->Show(rgb,"foo");
 SetAutoLayout(true);
  Layout();
  Show();

  Timer_.Start(100, true);
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{
  Timer_.Stop();
  sleep(1);
  cam.PostGrab();
  cam.CloseDevice();
  Destroy();
}


void MyFrame::OnTimer(wxTimerEvent& event)
{
  cout <<"OnTimer()"<<endl;
 cam.GrabSingle(pic);
 framecount++;
  ImageConvert::ToRGB(pic,rgb);

  unsigned char **ida = rgb.GetImageDataArray();
  unsigned x = framecount % rgb.GetWidth();
  unsigned y = framecount / rgb.GetHeight();
  for (unsigned int c=0;c<rgb.GetChannelCount(); c++)
  ida[y][x*rgb.GetChannelCount()+c] = 0;

  ic_->Show(rgb,"foo");



//  sprintf(name,"image_%04i",imgNo);
    // BIAS::ImageIO::Save(name,rgb,ImageIO::FF_jpg);
    //  cout<<"Written image:"<<name<<" to disk"<<endl;
  Refresh();

  Timer_.Start(20, true); // grab new images every 20 ms = resp 50 Hz
}



#else //ifdef GUI

int main(int argc,char *argv[])
{
cout <<"Nun-gui mode"<<endl;

 VideoSource_DCAM cam;
Camera<unsigned char> img;
// cam.SetModeAndFramerate(DC1394_VIDEO_MODE_1024x768_YUV422,7.5);
int res = cam.OpenDevice();
if (res<0) return 1;
cam.InitImage(img);
cam.PreGrab();
for (unsigned int i=0;i<10;i++) {
cout <<"Awaiting image"<<endl;
cam.GrabSingle(img);
cout <<"Got one image"<<endl;
}
cam.PostGrab();
cam.CloseDevice();

 return 0;
}
#endif



