#include <stdio.h>
#include <stdlib.h>
#include <VideoSource/VideoSource_PMD.hh>
#include <Base/Image/ImageIO.hh>
#include <pmdsdk2.h>

void
doStuff(PMDHandle hnd);
void
checkError(PMDHandle hnd, int code);

int
main(int argc, char** argv)
{
  /* The handle is used to identify a connection */
  PMDHandle hnd1;
  PMDHandle hnd2;
  /* Result code for PMDSDK2 calls */
  int res;


//  res = pmdOpen(&hnd1, BIAS_PMD_PMDTec_SOURCEPLUGIN, "0035.0952",
//      BIAS_PMD_PMDTec_PROCPLUGIN, "");
//
///* Check if the call succeeded */
//if (res != PMD_OK)
//  {
//    std::cout << "result 1 = " << res << std::endl;
//    char err[256];
//    /* The handle parameter in pmdGetLastError is 0
//     because no connection was established */
//    pmdGetLastError(0, err, 256);
//    fprintf(stderr, "Could not connect: %s\n", err);
//    /* Close the connection */
//    pmdClose(hnd1);
//    return 1;
//  }


  res = pmdOpen(&hnd2, BIAS_PMD_PMDTec_SOURCEPLUGIN, "0000.0952",
      BIAS_PMD_PMDTec_PROCPLUGIN, "");

  /* Check if the call succeeded */
  if (res != PMD_OK)
    {
      std::cout << "result 2 = " << res << std::endl;
      char err[256];
      /* The handle parameter in pmdGetLastError is 0
       because no connection was established */
      pmdGetLastError(0, err, 256);
      fprintf(stderr, "Could not connect: %s\n", err);
      /* Close the connection */
      pmdClose(hnd2);
      return 1;
    }



//  doStuff(hnd);

  /* Close the connection */
  pmdClose(hnd1);
  pmdClose(hnd2);

  return 0;
}

/* We put error handling in an extra function to keep
 the interesting code short
 */
void
checkError(PMDHandle hnd, int code)
{
  /* Only do something if the result code is not PMD_OK */
  if (code != PMD_OK)
    {
      char err[256];

      /* Get an error description from the PMDSDK2 */
      pmdGetLastError(hnd, err, 256);
      /* Display the error message */
      fprintf(stderr, "Error: %s\n", err);
      /* Close the connection */
      pmdClose(hnd);
      /* For our example programs, we just quit */
      exit(1);
    }
}

/* This is the interesting part */
void
doStuff(PMDHandle hnd)
{
  int res;
  unsigned int integrationTime;
  unsigned int modFreq;

  res = pmdGetValidModulationFrequency(hnd, &modFreq, 0, CloseTo, 20000000);
  checkError(hnd, res);
  std::cout << "valid modulation frequency" << modFreq << std::endl;
  res = pmdSetModulationFrequency(hnd, 0, modFreq);
  checkError(hnd, res);
  res = pmdGetValidIntegrationTime(hnd, &integrationTime, 0, CloseTo, 8000);
  checkError(hnd, res);
  std::cout << "valid integration time " << integrationTime << std::endl;

  res = pmdSetIntegrationTime(hnd, 0, integrationTime);
  checkError(hnd, res);
  std::cout << integrationTime << std::endl;
  std::cout << modFreq << std::endl;
  /* We know how much memory we need for a distance image
   In a real application, it is better to query this
   information with pmdGetSourceDataDescription() */
  float dist[204 * 204];

  BIAS::Image<float> image;
  image.InitWithForeignData(204, 204, 1, &(dist[0]));

  /* Take a picture */
  res = pmdUpdate(hnd);
  checkError(hnd, res);

  /* Get a distance image */
  res = pmdGetDistances(hnd, dist, sizeof(dist));
  checkError(hnd, res);

  //  /* Get information about the frame */
  //  PMDDataDescription dd;
  //  res = pmdGetSourceDataDescription(hnd, &dd);
  //  checkError(hnd, res);
  //
  //  std::cout << "dd->size() = " << dd.size << std::endl;
  //
  //  char * sourceData;
  //  sourceData = new char[dd.size];
  //  /* Get the frame raw data */
  //  res = pmdGetSourceData(hnd, sourceData, dd.size);
  //  checkError(hnd, res);
  //
  //  std::cout << "retrieved source data " << std::endl;
  //
  //  /* Calculate distances from the raw data */
  //  res = pmdCalcDistances(hnd, dist, 204 * 204 * sizeof(float), dd, sourceData);
  //  checkError(hnd, res);

  BIAS::ImageIO::Save("pmdTest.mip", image);
  std::cout << "written pmd image" << std::endl;
  image.ClearDataPointer();

  printf("A distance from the middle: %f m\n", dist[204 * 102 + 102]);
}

