/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file TestFFMPEGVideo.cpp
   @brief Test FFMPEGVideo  
   @relates FFMPEGVideo
   @ingroup g_tests
   @author MIP
*/


#include <iostream>
#include <VideoSource/FFMPEGVideo.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Common/FileHandling.hh>

using namespace std;
using namespace BIAS;


// FIXME
void av_free_packet(AVPacket *pkt) 
{ 
  if (pkt)
  {
    if (pkt->destruct) pkt->destruct(pkt);
    pkt->data = NULL; pkt->size = 0;
  }
}



int main(int argc, char* argv[])
{
  FFMPEGVideo vid;
  ImageBase::EColorModel cm = ImageBase::CM_RGB;
  ImageBase img;
  string videoFile;
  if(argc < 2) videoFile = "test.avi";
  else videoFile = argv[1];
  cout << "Opening video: "<<videoFile<<endl;
  if (!vid.Open(videoFile.c_str()))
  {
    cout << vid.ErrMsg() << endl;
    return -1;
  }

  cout << "Initializing codec..." << endl;
  if (!vid.Init(0, 0, FFMPEGVideo::Colormodel2PixFmt(cm)))
  {
    cout << vid.ErrMsg() << endl;
    return -1;
  }
  
  cout << "Initializing image..." << endl;
  img.Init(vid.Width(), vid.Height(),
	   ImageBase::GetSizeUnits(cm));
  img.SetColorModel(cm);
  
  cout << "Getting frame..." << endl;
  if (!vid.GetFrame(img))
  {
    cout << "Could not get frame" << endl;
    return -1;
  }

  
  
  bool ret = true;
  int i = 0;
  while (ret)
  {
    cout << "Getting frame " << i << "..." << endl;
    if ((ret = vid.GetFrame(img)) && i % 1 == 0)
    {
      cout << "Writing frame "<<i<<" to disk..." << endl;    
      if(!ImageIO::Save("frame-"+BIAS::FileHandling::LeadingZeroString(i,5)+".mip", img))      
      	cout << "Error writing frame!" << endl;
    }
    i++;
  }
  

  cout << "Closing video..." << endl;
  vid.Close();
  
  return 0;
}
