/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file TestVideoSource_Disk.cpp
   @brief Test VideoSource_Disk
   @relates VideoSource_Disk
   @ingroup g_tests
   @author MIP
*/

#include <VideoSource/VideoSource_Disk.hh>
#include <Image/Camera.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

int main( int argc, char *argv[])
{
  VideoSource *cam;
  
  Camera<unsigned char> pic;
  string img = BIAS_TESTS_DATA;
  img+= "r4.jpg"; 
  BIAS::ImageIO::Load(img,pic);
 
  cout << "Import OK\n";

  cam = new VideoSource_Disk;

  cam->SetColorModel(ImageBase::CM_Grey);
  
  // build string to tests data
  string imgs = BIAS_TESTS_DATA;
  imgs += "chess7x4_";
  
  //opendevice with 5 images, 3 digits, start at count 1
  cam->OpenDevice(imgs,".jpg",3,5,1);
 
  pic.SetDebugLevel(D_COPYIN | D_CONVERT);
  pic.Init(cam->GetWidth(), cam->GetHeight(), cam->GetColorChannels());
  pic.FillImageWithConstValue((unsigned char)128);
  
  cout << "pic.Init() done with: "<<cam->GetWidth()<<" "<<cam->GetHeight()<<" "<< cam->GetColorChannels()<<endl;
  
  cam->PreGrab();
  for (int i=0; i<10; i++){
    cam->GrabSingle(pic);
    cout <<i<< " Grab() done"<<endl;
  }
  cam->PostGrab();
  cam->CloseDevice();
  
  // pic.IP_ToGrey();
  BIAS::ImageIO::Save("cam",pic);

  delete cam;
  return 0;
}

