#ifndef __BIAS_VIDEOSERVER_HH__
#define __BIAS_VIDEOSERVER_HH__

#include <iostream>
#include <sstream>
#include <string>

#include <bias_config.h>
// BIAS
#include <Base/Debug/Debug.hh>
#include <Base/Image/ImageIO.hh>
#include <Filter/Rescale.hh>
#include <Image/Camera.hh>
#include <VideoSource/VideoSource_Base.hh>
#include <NetworkComm/CScommServer.hh>


#ifdef  BIAS_HAVE_LIBJPEG
#include <Base/Image/CompressJpeg.hh>
#include <Base/Image/ImageConvert.hh>
#endif // BIAS_HAVE_LIBJPEG


namespace BIAS {

/** @class VideoServer
    @ingroup g_videotransmission
    @ingroup network
    @brief class to send images from a camera or disk to a client
    
    uses TCP or UDPServer,  so far only for D_CAM input.
    Catch images with the VideoSourve_Net class. <br>
    VideoSourceDisk takes the all images in the current dir.
    @autor grest Nov. 2005
*/
class BIASVideoSource_EXPORT VideoServer :public BIAS::Debug {
public:
  VideoServer();
  ~VideoServer();
  
  int Init(int port=D_CS_DEFAULT_PORT);

  int ProcessOneImage();  

  VideoSource *GetCamera() {return camera_;}

  double GetFPS(){return camParam_[0];}

  BIAS::Camera<unsigned char> *GetSendImage(){return &SendImage_;}

  inline unsigned int GetMessageSize() { return msgSize_; }

protected:
  int port_;
  std::string targetAddress_;
  BIAS::Vector<double> camParam_;
  int camType_, imgWidth_, imgHeight_, jpeg_;
  double resizeFactor_;
  int UDPServerThrottle_;
  bool useUDP_;


  VideoSource *camera_;
  BIAS::Camera<unsigned char> CamImage_;
  BIAS::Camera<unsigned char> SendImage_;
  bool Initialized_;
  std::ostringstream SendBuffer_;
  BIAS::Rescale<unsigned char,unsigned char> RescaleFilter_;

  unsigned int msgSize_;
  //  bool *CompressedMode_;
  bool compressionColor_;
  //  int *CompressionQuality_;

  CScommServer server_;

#ifdef BIAS_HAVE_LIBJPEG
  JpegHandler Compressor_;
#endif //BIAS_HAVE_LIBJPEG

private:
  // disabled copy-constructor
  VideoServer(const VideoServer &){};
   VideoServer &operator=(const VideoServer &) {return *this;};

}; // class VideoServer

} // namespace BIAS


#endif

 
