/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __VIDEOSHMFEEDER_HH__
#define __VIDEOSHMFEEDER_HH__




#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#  include <windows.h>
#else
# include <sys/types.h>
# include <sys/mman.h>
#include <fcntl.h>
#endif


#include <sstream>
#include <string>

// BIAS
#include <Base/Debug/Debug.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>

// MIP
#include <VideoSource/VideoSource_Shm.hh>


namespace BIAS {

/** @class VideoShMFeeder
    @ingroup g_videosource
    @brief class to save Images to a Sharerd memory 
 
    @autor ischiller 2006
*/
class BIASVideoSource_EXPORT VideoShMFeeder :public BIAS::Debug {
public:
  VideoShMFeeder();
  VideoShMFeeder(const std::string &filename );
  ~VideoShMFeeder();

  bool ProcessImage(BIAS::Camera<unsigned char> *Image);

protected:
  bool Init_();

#ifdef WIN32
  HANDLE                            ShmContext_; // _hFile
  HANDLE                            Mapping_;//! header describing the memory layout
  long int                          Granularity_;
#else
  int                               ShmContext_;
#endif
  
  int                               MemoryLengthTotal_;
  int                               DataLength_;
  void                              *Shm_;
  VideoSource_Shm::ShmImageHeader   *Header_;
  std::stringstream                 ImgBuffer_; 
  unsigned char                     *Data_;
  bool                              bIsInitialized_;
  std::string                       FileName_;

protected:
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoShMFeeder(const VideoShMFeeder &source);
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoShMFeeder& operator =(const VideoShMFeeder &source);

}; // class VideoShMFeeder

} // namespace MIP


#endif

 
