/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __VIDEOSHMFEEDERPMD_HH__
#define __VIDEOSHMFEEDERPMD_HH__




#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#  include <windows.h>
#else
# include <sys/types.h>
# include <sys/mman.h>
#include <fcntl.h>
#endif


#include <sstream>
#include <string>

// BIAS
#include <Base/Debug/Debug.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>

// MIP
#include <VideoSource/VideoSource_ShmPMD.hh>
#include <VideoSource/VideoShMFeeder.hh>


namespace BIAS {

/** @class VideoShMFeederPMD
	  @ingroup g_videosource
    @brief class to save Images to a Sharerd memory. It also transfers Depth 
    and Amplitude data coming from a PMD camera.
 
    @autor streckel 08/2006
*/
class BIASVideoSource_EXPORT VideoShMFeederPMD : public VideoShMFeeder {
public:
  VideoShMFeederPMD();
  VideoShMFeederPMD(const std::string &filename );
  ~VideoShMFeederPMD();

  bool ProcessImage(BIAS::Camera<unsigned char> *Img2D,
                    BIAS::Camera<float> *ImgDepth,
                    BIAS::Camera<float> *ImgModCoeff);

protected:
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoShMFeederPMD(const VideoShMFeederPMD &source);
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoShMFeederPMD& operator =(const VideoShMFeederPMD &source);

}; // class VideoShMFeederPMD

} // namespace BIAS


#endif

 
