#include "VideoSourceCapabilities.hh"

using namespace BIAS;
using namespace std;

VideoSourceCapabilities::
VideoSourceCapabilities()
{

}


VideoSourceCapabilities::~
VideoSourceCapabilities()
{

}


void VideoSourceCapabilities::Reset()
{
  Inputs_.clear();
  AllResolutions_.clear();
}



void VideoSourceCapabilities::
AddMode(unsigned int width, unsigned int height, float fps,  
	ImageBase::EColorModel colormodel, bool interlaced)
{
  if (!HasMode(width,height,fps,colormodel,interlaced)) {
    ResolutionEntry e;
    e.width = width;
    e.height = height;
    e.fps = fps;
    e.colormodel = colormodel;
    e.interlaced = interlaced;
    AllResolutions_.push_back(e);
  }
}

void VideoSourceCapabilities::
AddInput(InputDescriptor id)
{
  //  cout <<"VideoSourceCabilities::AddInput()"<<endl;;
  Inputs_.push_back(id);
}



bool VideoSourceCapabilities::
HasMode(unsigned int width, unsigned int height, float fps,  
	ImageBase::EColorModel colormodel,bool interlaced)
{

  for (unsigned int i=0;i<AllResolutions_.size();i++ ) {
    ResolutionEntry &e = AllResolutions_[i];
    if (e.width == width && e.height == height &&e.fps==fps 
	      && e.colormodel == colormodel) 
      return true;
  }
  return false;
}


void VideoSourceCapabilities::
Show()
{
  cout <<"VideoSourceCapabilities for device: "<<endl;
  cout <<"  "<<Name_<<endl;
  for (unsigned int i=0;i<Inputs_.size();i++ ) {
    InputDescriptor &id = Inputs_[i];
    cout <<"Input "<<id.number<<endl;
    cout <<"  Name: "<<id.name<<endl;
  }

  for (unsigned int i=0;i<AllResolutions_.size();i++ ) {
    ResolutionEntry &e = AllResolutions_[i];
    cout <<"Mode "<<i<<" "<<e.width<<"x"<<e.height<<" fps: "<<e.fps
	  <<" colormodel: "<<e.colormodel<<endl;      
  }
}

void VideoSourceCapabilities::
GetBestForEachResolution(std::vector<ResolutionEntry> &res)
{
  res.clear();
  for (unsigned int i=0;i<AllResolutions_.size();i++ ) {
    ResolutionEntry &e = AllResolutions_[i];
    bool found = false;
    for (unsigned int j=0;j<res.size();j++ ) {
      if (e.width == res[j].width && e.height== res[j].height) {
	      // if resoltion is equal and fps is better, update entry
	      if (e.fps >res[j].fps) res[j] = e;
	      found = true;
	      break;
      }
    }
    if (!found) res.push_back(e);
  }
}
