/*
 * VideoSource_SonyX300.cpp
 *
 *  Created on: Dec 8, 2009
 *      Author: africk
 */

#include <VideoSource/VideoSource_Centaurus.hh>

using namespace std;
using namespace BIAS;

char VideoSource_Centaurus::setup = 0;

VideoSource_Centaurus::VideoSource_Centaurus() {
	cam_handleP_ = NULL;
	inputFifoP_ = NULL;
}

VideoSource_Centaurus::~VideoSource_Centaurus() {
	CloseDevice();
}

int VideoSource_Centaurus::OpenDevice() {
	char setup = 0;
	int res = OpenDevice(&setup, SV_OPENPROGRAM_DEFAULT, SV_OPENTYPE_DEFAULT, 0, 0);

	return res;
}

int VideoSource_Centaurus::OpenDevice(char * setup, int openprogram,
		int opentype, int timeout, int spare) {
	int res = sv_openex(&cam_handleP_, setup, openprogram, opentype, timeout,
			spare);
	if (res != SV_OK) {
		BIASERR("sv_openex : " << sv_geterrortext(res) << endl);
	}
	return res;
}

int VideoSource_Centaurus::CloseDevice() {
	int res = 0;
	if (cam_handleP_ != NULL) {
		res = sv_close(cam_handleP_);
		if (res != SV_OK) {
			BIASERR("sv_close : " << sv_geterrortext(res) << endl);
		} else {
			cam_handleP_ = NULL;
		}
	}

	return res;
}

int VideoSource_Centaurus::SetVideoMode(int mode) {
	int res = sv_videomode(cam_handleP_, mode);
	if (res != SV_OK) {
		cout << "sv_videomode : " << sv_geterrortext(res) << endl;
	}

	return res;
}

int VideoSource_Centaurus::InitFiFo() {
	int res = sv_fifo_init(cam_handleP_, &inputFifoP_, 1, 0, 1,
			SV_FIFO_FLAG_VIDEOONLY, 0);
	if (res != SV_OK) {
		cout << "sv_fifo_init : " << sv_geterrortext(res) << endl;
	}

	res = sv_fifo_start(cam_handleP_, inputFifoP_);
	if (res != SV_OK) {
		cout << "sv_fifo_start " << sv_geterrortext(res) << endl;
	}

	return res;
}

int VideoSource_Centaurus::GrabSingle(Camera<unsigned char> &image) {

	int res = -1;

	if (cam_handleP_ != NULL) {
		if (inputFifoP_ != NULL) {
			int flags = 0;
			flags |= SV_FIFO_FLAG_VSYNCWAIT;
			flags |= SV_FIFO_FLAG_FLUSH;

			sv_fifo_buffer* sv_buffer;
			res = sv_fifo_getbuffer(cam_handleP_, inputFifoP_, &sv_buffer,
					NULL, flags);
			if (res != SV_OK) {
				cout << "sv_fifo_getbuffer : " << sv_geterrortext(res) << endl;
			}

			sv_buffer->dma.addr = (char*) image.GetImageData();
			sv_buffer->dma.size = image.GetWidth() * image.GetHeight()
					* image.GetChannelCount();

			res = sv_fifo_putbuffer(cam_handleP_, inputFifoP_, sv_buffer, NULL);
			if (res != SV_OK) {
				cout << "sv_fifo_putbuffer : " << sv_geterrortext(res) << endl;
			}

		} else {
			BIASERR("call InitFiFo(...) first");
		}
	} else {
		BIASERR("call OpenDevice(...) first " << endl);
	}

	return res;
}

int VideoSource_Centaurus::InitImage(BIAS::ImageBase &image) {

	bool alreadyOpend = false;
	if (cam_handleP_ == NULL) {
		OpenDevice();
	} else {
		alreadyOpend = true;
	}

	sv_storageinfo storageInfo;
	int res = sv_storage_status(cam_handleP_, 0, NULL, &storageInfo,
			sizeof(storageInfo), 0);
	if (res != SV_OK) {
		BIASERR("sv_storage_status : " << sv_geterrortext(res) << endl);
	}

	int colorMode = storageInfo.colormode;

	int width = storageInfo.xsize;
	int height = storageInfo.ysize;

	switch (colorMode) {
	case SV_COLORMODE_YUV422_YUYV:
		image.Init(width, height, 2, ImageBase::ST_unsignedchar);
		image.SetColorModel(ImageBase::CM_YUYV422);
		cout << "color mode :  YUV422_YUYV " << endl;
		break;
	default:
		break;
	}

	int videoMode = storageInfo.videomode;
	switch (videoMode & SV_MODE_MASK) {
	case SV_MODE_SMPTE274_25sF:
		cout << "video mode : SV_MODE_SMPTE274_25sF " << endl;
		break;
	default:
		break;
	}

	int fps = storageInfo.fps;
	cout << "FPS : " << fps << endl;

	if (!alreadyOpend) {
		CloseDevice();
	}

	return res;
}
