/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "VideoSource_DCAM_XB3.hh"

#ifdef WIN32

#include <wtypes.h>
#include <1394Camera.h>
#include <Base/Debug/TimeMeasure.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>

#else

#include <dc1394/control.h>

#include "sys/time.h"
#include "unistd.h"
#include <sys/stat.h>

#endif // WIN32

#define WIDTH 1280
#define HEIGHT 960
#define WIDTH_S 640
#define HEIGHT_S 480

using namespace BIAS;
using namespace std;

int j = 0;


VideoSource_DCAM_XB3::VideoSource_DCAM_XB3(int Card): VideoSource_DCAM(Card)
{
}

int VideoSource_DCAM_XB3::GrabSingle(Camera <unsigned char> &allImages,
                                     Camera <unsigned char> &imageLeft,
                                     Camera <unsigned char> &imageMiddle,
                                     Camera <unsigned char> &imageRight)
{
  // fetch complete image from cam
  VideoSource_DCAM::GrabSingle(allImages);

  // create 3 images out of source image
  unsigned char* grabedPixels = allImages.GetImageData();
  unsigned char* sinkPixels = NULL;
  bool skip;

  for(unsigned int j=0; j<3; j++)
  {
    switch(j)
    {
      case 0:
        if(!imageRight.IsEmpty())
        {
          sinkPixels = imageRight.GetImageData();
          skip = false;
        }
        else
        {
//          cout<<"\tEmpty image given: skip right image"<<endl;
          skip = true;
        }
        break;
      case 1:
        if(!imageMiddle.IsEmpty())
        {
          sinkPixels = imageMiddle.GetImageData();
          skip = false;
        }
        else
        {
//          cout<<"\tEmpty image given: skip middle image"<<endl;
          skip = true;
        }
        break;
      case 2:
        if(!imageLeft.IsEmpty())
        {
          sinkPixels = imageLeft.GetImageData();
          skip = false;
        }
        else
        {
//          cout<<"\tEmpty image given: skip left image"<<endl;
          skip = true;
        }
        break;
      default:
        skip = true;
        break;
    }
    if(!skip)
    {
      int
        size = WIDTH*HEIGHT;
      for(int index = 0; index < size; index++)
      {
        sinkPixels[index] = grabedPixels[index*3 + j];
      }
    }
  }
  return 0;
}


int VideoSource_DCAM_XB3::GrabSingleAsSmallRGB(Camera <unsigned char> &allImages,
                                     Camera <unsigned char> &imageLeft,
                                     Camera <unsigned char> &imageMiddle,
                                     Camera <unsigned char> &imageRight)
{
  // fetch complete image from cam
  VideoSource_DCAM::GrabSingle(allImages);

  // create 3 images out of source image
  unsigned char* grabedPixels = allImages.GetImageData();
  unsigned char* sinkPixels = NULL;
  bool skip;

  for(unsigned int j=0; j<3; j++)
  {
    switch(j)
    {
      case 0:
        if(!imageRight.IsEmpty())
        {
          sinkPixels = imageRight.GetImageData();
          skip = false;
        }
        else
        {
//          cout<<"\tEmpty image given: skip right image"<<endl;
          skip = true;
        }
        break;
      case 1:
        if(!imageMiddle.IsEmpty())
        {
          sinkPixels = imageMiddle.GetImageData();
          skip = false;
        }
        else
        {
//          cout<<"\tEmpty image given: skip middle image"<<endl;
          skip = true;
        }
        break;
      case 2:
        if(!imageLeft.IsEmpty())
        {
          sinkPixels = imageLeft.GetImageData();
          skip = false;
        }
        else
        {
//          cout<<"\tEmpty image given: skip left image"<<endl;
          skip = true;
        }
        break;
      default:
        skip = true;
        break;
    }
    if(!skip)
    {
      const int
        offsetR = WIDTH*3,
        offsetB = 3;
      int
        index = 0,
        indexG = j;
      for(int j = 0; j < HEIGHT_S; j++)
      {
        for(int i = 0; i < WIDTH_S; i++)
        {
          // R
          sinkPixels[index++] = grabedPixels[indexG+offsetR];
          // G
          sinkPixels[index++] = grabedPixels[indexG];
          // B
          sinkPixels[index++] = grabedPixels[indexG+offsetB];
          indexG += 6;
        }
        indexG += offsetR;
      }
    }
  }
  return 0;
}
