/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIASVIDEOSOURCE_DCAM_XB3_HH__
#define __BIASVIDEOSOURCE_DCAM_XB3_HH__

#include <VideoSource/VideoSource_Base.hh>
#include <VideoSource/VideoSource_DCAM.hh>

namespace BIAS {

/**
  \class VideoSource_DCAM_XB3
  \ingroup g_videosource
  \author Daniel Jung
  \brief VideoSource PtGrey Bublebee XB3 
 */
  class BIASVideoSource_EXPORT VideoSource_DCAM_XB3 : public VideoSource_DCAM{


  public:
    VideoSource_DCAM_XB3(int Card = 0);
    //~VideoSource_DCAM_XB3();

    /**
      @brief Grab single images as bayer pattern GBRG. Left, middle and right images HAVE to be 1280x960, 1 channel, color model GBRG and unsigned char or must be omitted (pass empty image)!
    **/
    int GrabSingle(BIAS::Camera <unsigned char> &allImages,
                   BIAS::Camera <unsigned char> &imageLeft,
                   BIAS::Camera <unsigned char> &imageMiddle,
                   BIAS::Camera <unsigned char> &imageRight
                   );

    /**
      @brief Grab single images as RGB. No interpolation on bayer pattern. 50% of green values are dropped. Left, middle and right images HAVE to be 640x480, 3 channels, interleaved, color model RGB and unsigned char or must be omitted (pass empty image)!
      @author Daniel Jung
      @date 08/2008
    **/
    int GrabSingleAsSmallRGB(BIAS::Camera <unsigned char> &allImages,
        BIAS::Camera <unsigned char> &imageLeft,
        BIAS::Camera <unsigned char> &imageMiddle,
        BIAS::Camera <unsigned char> &imageRight);
  };


} // namespace BIAS

#endif // __MIPVIDEOSOURCE_DCAM_XB3_HH__
