/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIASVIDEOSOURCE_DISK_HH__
#define __BIASVIDEOSOURCE_DISK_HH__



#include <VideoSource/VideoSource_Base.hh>
#ifdef BIAS_HAVE_FFMPEG
#  include <VideoSource/FFMPEGVideo.hh>
#endif

#include "Base/Debug/TimeMeasure.hh"


namespace BIAS{
/** 
    \class VideoSource_Disk 
    \ingroup g_videosource
    \author evers
    \brief This class simulates a video device by loading images from disk.
*/
  class BIASVideoSource_EXPORT VideoSource_Disk : public VideoSource{
   
  public: 
    VideoSource_Disk();
    virtual ~VideoSource_Disk();
  
    /** 
     * @brief this opens all images given by prefix, postfix, Digitcount and  PictureCount
     * e.g. myimages_00001.mip - myimages_00100.mip
     * prefix = myimages_, postfix = .mip, digitcount = 5, picturecount = 101
     * @param [in] Prefix, image prefix such as myimages_
     * @param [in] Postfix, image postfix such as .mip
     * @param [in] DigitCount, number of digits in number, e.g. 5
     * @param [in] PictureCount, number of images
     * @param [in] StartCount, number to start counting
     */
    virtual int OpenDevice(std::string Prefix, std::string Postfix, int DigitCount, 
														int PictureCount,int StartCount=0);
    /** 
     * @brief this opens all images given in the vector of strings
     */
    virtual int OpenDevice(const std::vector<std::string> &FileNames);
    
    int CloseDevice();
  
    int SetColorMode(BIAS::ImageBase::EColorModel mode);

    void SetLoopMode(bool l=true){Loop_ = l;};
    
    void SetReverseMode(bool r=true){bReverse_=r;};

    void Rewind() {ActiveFrame_ = 0;};

    virtual int GrabSingle(BIAS::Camera <unsigned char> &image);
    virtual int GrabSingle(BIAS::Camera <float> &image);
#ifdef BUILD_IMAGE_USHORT
    virtual int GrabSingle(BIAS::Camera <unsigned short int> &image);
#endif
    
    /** Disk has also a imagebase grabber for e.g.g float images
     */
    virtual int GrabSingle(BIAS::ImageBase &image);

    std::string GetNextFilename() const {
      if (ActiveFrame_ <FileNames_.size()) return FileNames_[ActiveFrame_];
      else return "";};
    std::string GetLastFilename() const {
      return ActiveFrame_ >0 ? FileNames_[ActiveFrame_-1] : "" ;};

    std::vector<std::string> &GetFilenamesRef() {return FileNames_;};
    
    unsigned int GetNumFramesRemaining() const;

    /** @brief zero based counting, returns next frame */
    unsigned int GetActiveFrameNumber() const { return ActiveFrame_;}

    void SetActiveFrameNumber(unsigned int frame) { ActiveFrame_ = frame; }

    /** @brief pushback new filenames on internal filename vector
        @author koeser 12/2005 */
    inline void AddFilenames(const std::vector<std::string> &FileNames) {
      FileNames_.reserve(FileNames_.size()+FileNames.size());
      for (unsigned int i=0; i<FileNames.size(); i++){
        FileNames_.push_back(FileNames[i]);
      }
    }

    void SetGenerateNewUIDs(bool s) {GenerateNewUIDs_=s;};
    void SetKeepFPS(bool s) {KeepFPS_ = s;};
    int PreGrab();
    int PostGrab() {return true;};
    void SetFrameCounterAsTimeStamp(const bool b) {FrameCounterAsTimeStamp_=b;}

    /** 
        If set to 'true' ParseMetaData will be called after load-image.
        !!!Overrides timestamp from SetFrameCounterAsTimeStamp!!!
    **/
    inline void SetUseMetaData(const bool b) {bUseMetaData_=b;}
  
  protected:
    // exports for dll (cmenk)
    //BIAS_EXPORT_STL_VECTOR( BIASVideoSource_EXPORT, std::string )

    BIAS::TimeMeasure timer_;
    // struct timeval lastTime_; -- TimeMeasure is used instead

    // for doublebuffering this swich is used to select the frame
    unsigned int ActiveFrame_;
    bool Loop_;
    bool bReverse_;
    bool bReversePlaying_;
    bool bUseMetaData_;
    std::vector<std::string> FileNames_;
    bool KeepFPS_;
    bool GenerateNewUIDs_;
    bool FrameCounterAsTimeStamp_;
    bool bVideo_;

#ifdef BIAS_HAVE_FFMPEG
    FFMPEGVideo Video_;
#endif
  };


} // namespace MIP

#endif
