/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIASVIDEOSOURCE_DISKPMD_HH__
#define __BIASVIDEOSOURCE_DISKPMD_HH__

#include <VideoSource/VideoSource_Disk.hh>
#include <Image/Camera.hh>
#include <bias_config.h>
#include <vector>
#include <Image/PMDImageProc.hh>
#include <Base/Image/PMDImageIO.hh>

/* no longer in use -- timemeasure instead
#include <sys/time.h>
*/

namespace BIAS {

  /** @class VideSource_DiskPMD
      @ingroup g_videosource
      @brief reads PMD-tiffs with depth images and modulation coefficients
      @author streckel 05/2006 */
  class BIASVideoSource_EXPORT VideoSource_DiskPMD : public VideoSource_Disk {

  public:
    VideoSource_DiskPMD();

    virtual ~VideoSource_DiskPMD();

#ifdef BIAS_HAVE_XML2
    /** Read the images from an XML-file containing a whole image sequence.
    */
    virtual int OpenDevice(std::string &FileNames);
#endif

    /** The filenames vector should contain the names of PMD-TIFFS
    */
    virtual int OpenDevice(const std::vector<std::string> &FileNames);

    virtual int OpenDevice(); 

    /** Returns the 2D image, and reads depth and modulation coefficients */
    virtual int GrabSingle(Camera<unsigned char> &image);

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleDepth(Camera<float> &image)
    { 
      if (!DepthImg_.IsEmpty()) {
        image = DepthImg_; 
        return 0;
      } 
      return -1;
    }

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleModCoeff(Camera<float> &image)
    {       
      if (!ModCoeffImg_.IsEmpty()) {
        image = ModCoeffImg_; 
        return 0;
      } 
      return -1;
    }

    void SetGenerateNewUIDs(bool s) {  
      BIASERR("Always generating new UUIDS for PMD images");
      BIASABORT;
    }

  protected:
    VideoSource_Disk *vsd_;

#ifdef BIAS_HAVE_XML2
    std::vector<Image<unsigned char> > vecImg2D_;
    std::vector<Image<float> > vecImgDepth_;
    std::vector<Image<float> > vecImgModCoeff_;
    std::vector<PMDImageMetaData> vecMetaData2D_;
    std::vector<PMDImageMetaData> vecMetaDataDepth_;
#endif

    //BIAS::Camera<unsigned char> 2DImg_;
    BIAS::Camera<float> DepthImg_, ModCoeffImg_;


  };

} // namespace BIAS

#endif
