/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef  WIN32
#include <unistd.h>
#endif //WIN32
#include <Base/Common/W32Compat.hh>

#include <string>
#include <sstream>
#include <iostream>

#include <VideoSource/VideoSource_Disk.hh>
#include "VideoSource_Disk_Bayer.hh"
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Image/Camera.hh>

#include <Base/Common/BIASpragma.hh>

using namespace BIAS;
using namespace std;

VideoSource_Disk_Bayer::VideoSource_Disk_Bayer(): VideoSource_Disk()
{
   ColorMode_ = BIAS::ImageBase::CM_Bayer_GBRG;
}


VideoSource_Disk_Bayer::~VideoSource_Disk_Bayer()
{
}


int VideoSource_Disk_Bayer::OpenDevice(const vector<string> &FileNames)
{
//   BIASDOUT(D_VSO_DISK,"entering VideoSource_Disk_Bayer::OpenDevice(vector<string>)");
//  Camera<unsigned char> Image;
  // load first image and initialize some internal variables
  ImageBase Image;
  BIASDOUT(D_VSO_DISK,"FileNames[0] : "<<FileNames[0]);
  // Image.SetDebugLevel(D_COPYIN | D_CONVERT);

  if (ImageIO::Load(FileNames[0],Image)!=0){
    cout<<"pwd: ";system("pwd");
    BIASERR("unable to read "<<FileNames[0]);
    BIASBREAK;
    return -1;
  }
  Width_  = Image.GetWidth();
  Height_ = Image.GetHeight();
  ColorChannels_ = Image.GetChannelCount();
  // because of bayer image, colormode has to be set manually
//  ColorMode_ = Image.GetColorModel();
  BytesPerPixel_ = Image.GetDepth();
  if (FileNames_.size() != 0)
    FileNames_.erase(FileNames_.begin(), FileNames_.end());

  for (vector<string>::const_iterator i = FileNames.begin();
    i != FileNames.end();i++)
  {
    FileNames_.push_back(*i);

#if 0
    const char *fName2 =(*i).c_str();
    //ImageIO::Read(fName2,Image);
    ImageIO::Load(fName2,Image);
    if (i != FileNames.begin()){
      if (Width_ != Image.GetWidth() || Height_ != Image.GetHeight() ||
        ColorChannels_ != Image.GetChannelCount() ){
          BIASERR("VideoSource_Disk_Bayer::OpenDevice() Image is "<<*i<<
            " different from first image in sequence");
          return -1;
        }
    }
#endif
  }

  CompleteInitialized_ = true;
  ActiveFrame_ = 0;
  /* old version -- without timemeasure.hh
  gettimeofday(&lastTime_,NULL);
  */
  return 0;
}

//int VideoSource_Disk_Bayer::PreGrab()
//{
//  VideoSource::PreGrab();
//
//  timer_.Start();
//  return 0;
//}



//int VideoSource_Disk_Bayer::CloseDevice()
//{
//  if (Grabbing_) {
//    BIASERR("VideoSource_Disk_Bayer::CloseDevice(): Grabbing is running, but closing anyway");
//  }
//  Grabbing_ = false;
//  PostGrab();
//  CompleteInitialized_ = false;
//  if (FileNames_.size() != 0)
//    FileNames_.erase(FileNames_.begin(), FileNames_.end());
//  return 0;
//}


int  VideoSource_Disk_Bayer::SetColorMode(ImageBase::EColorModel mode)
{

  if(mode == ImageBase::CM_Bayer_BGGR ||
     mode == ImageBase::CM_Bayer_GBRG ||
     mode == ImageBase::CM_Bayer_GRBG ||
     mode == ImageBase::CM_Bayer_RGGB){
    ColorMode_ = mode;
  } else {
    BIASERR("So far this class is implemented to handle Bayer patterns only!");
    return -1;
  }

  return 0;
}

int VideoSource_Disk_Bayer::GrabSingle(Camera<unsigned char> &image)
{
  BIASDOUT(D_VSO_DISK,"VideoSource_Disk_Bayer::GrabSingle()");

  if (ActiveFrame_ == FileNames_.size()){
    BIASERR("VideoSource_Disk_Bayer: End of input. SetLoop(true) or Rewind()");
    return -1;
  }
  int res;
  if (!Active_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object not active, use PreGrab()");
    return -1;
  }

  if (Grabbing_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object is already grabbing()");
    return -1;
  }

  Grabbing_ = true;
  Camera<unsigned char> bayerImage;
  res = ImageIO::Load(FileNames_[ActiveFrame_], bayerImage);
  bayerImage.SetColorModel(ColorMode_);
  ImageConvert::ToRGB(bayerImage,image);

  if (res<0) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): Unable to read image: "<<FileNames_[ActiveFrame_]);
    return res;
  }

  // set time to _now_
  if (FrameCounterAsTimeStamp_) image.SetTime(ActiveFrame_,0);
  else {
    timeval tv;
    gettimeofday(&tv, NULL);
    image.SetTime(tv.tv_sec,tv.tv_usec);
  }

  //set meta data from loaded image, if activated
  if (bUseMetaData_) image.ParseMetaData();

  if(ActiveFrame_<FileNames_.size()-1){
    BIASDOUT(D_VSO_DISK,"GrabSingle() "<<ActiveFrame_<<"  "<<
             FileNames_[ActiveFrame_]);
    if(bReversePlaying_){
      if(ActiveFrame_>0)
        ActiveFrame_--;
      else
        {
          bReversePlaying_=false;
          ActiveFrame_++;
        }
    }
    else{
      ActiveFrame_++;
    }
  }
  else if (Loop_) {
    ActiveFrame_=0;
    BIASDOUT(D_VSO_DISK,"GrabSingle() restarting with first file");
  }
  else if(bReverse_){

    bReversePlaying_=true;
    ActiveFrame_--;
  }
  else ActiveFrame_++;

  Grabbing_ = false;
  BIASDOUT(D_VSO_DISK,"leaving GrabSingle()");
  // check time and sleep while enough time elapsed
  double secPerFrame=1.0/FramesPerSecond_;

  //struct timeval now;         -- displaced by TimeMeasure
  //double secs=0.0;            --         -"-
  //while (secs<=secPerFrame) { --         -"-
  if(KeepFPS_) {
    timer_.Stop();
    if (FramesPerSecond_ != 0)
    {
      while (timer_.GetRealTime()<=(1E06*secPerFrame))
      {

        timer_.Start();
#ifdef   WIN32
        Sleep(1);        //go sleeping for 1 msec
#else  //not WIN32
        usleep(1000);     //go sleeping for 1 msec
#endif //WIN32
        timer_.Stop();

      }
    }
    timer_.Reset();
    timer_.Start();
  }
  if (GenerateNewUIDs_) {
    image.SetUID(BIAS::UUID::GenerateUUID());
  }
  //lastTime_=now;   -- displaced by TimeMeasure

  return 0;
}

int VideoSource_Disk_Bayer::GrabSingle(Camera<float> &image)
{
  BIASDOUT(D_VSO_DISK,"VideoSource_Disk_Bayer::GrabSingle()");

  if (ActiveFrame_ == FileNames_.size()){
    BIASERR("VideoSource_Disk_Bayer: End of input. SetLoop(true) or Rewind()");
    return -1;
  }
  int res;
  if (!Active_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object not active, use PreGrab()");
    return -1;
  }

  if (Grabbing_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object is already grabbing()");
    return -1;
  }

  Grabbing_ = true;

  Camera<float> bayerImage;
  res = ImageIO::Load(FileNames_[ActiveFrame_], bayerImage);
  bayerImage.SetColorModel(ColorMode_);
  ImageConvert::ToRGB(bayerImage,image);


  if (res<0) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): Unable to read image: "<<FileNames_[ActiveFrame_]);
    return res;
  }

  // set time to _now_
  if (FrameCounterAsTimeStamp_) image.SetTime(ActiveFrame_,0);
  else {
    timeval tv;
    gettimeofday(&tv, NULL);
    image.SetTime(tv.tv_sec,tv.tv_usec);
  }

  //set meta data from loaded image, if activated
  if (bUseMetaData_) image.ParseMetaData();

  if(ActiveFrame_<FileNames_.size()-1){
    BIASDOUT(D_VSO_DISK,"GrabSingle() "<<ActiveFrame_<<"  "<<
             FileNames_[ActiveFrame_]);
    if(bReversePlaying_){
      if(ActiveFrame_>0)
        ActiveFrame_--;
      else
        {
          bReversePlaying_=false;
          ActiveFrame_++;
        }
    }
    else{
      ActiveFrame_++;
    }
  }
  else if (Loop_) {
    ActiveFrame_=0;
    BIASDOUT(D_VSO_DISK,"GrabSingle() restarting with first file");
  }
  else if(bReverse_){

    bReversePlaying_=true;
    ActiveFrame_--;
  }
  else ActiveFrame_++;

  Grabbing_ = false;
  BIASDOUT(D_VSO_DISK,"leaving GrabSingle()");
  // check time and sleep while enough time elapsed
  double secPerFrame=1.0/FramesPerSecond_;

  //struct timeval now;         -- displaced by TimeMeasure
  //double secs=0.0;            --         -"-
  //while (secs<=secPerFrame) { --         -"-
  if(KeepFPS_) {
    timer_.Stop();

    if (FramesPerSecond_ != 0)
    {
      while (timer_.GetRealTime()<=(1E06*secPerFrame))
      {

        timer_.Start();
#ifdef   WIN32
        Sleep(1);        //go sleeping for 1 msec
#else  //not WIN32
        usleep(1000);     //go sleeping for 1 msec
#endif //WIN32
        timer_.Stop();

      }
      timer_.Reset();
      timer_.Start();
    }
  }
  if (GenerateNewUIDs_) {
    image.SetUID(BIAS::UUID::GenerateUUID());
  }
  //lastTime_=now;   -- displaced by TimeMeasure

  return 0;
}

#ifdef BUILD_IMAGE_USHORT
int VideoSource_Disk_Bayer::GrabSingle(Camera<unsigned short> &image)
{
  BIASDOUT(D_VSO_DISK,"VideoSource_Disk_Bayer::GrabSingle()");

  if (ActiveFrame_ == FileNames_.size()){
    BIASERR("VideoSource_Disk_Bayer: End of input. SetLoop(true) or Rewind()");
    return -1;
  }
  int res;
  if (!Active_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object not active, use PreGrab()");
    return -1;
  }

  if (Grabbing_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object is already grabbing()");
    return -1;
  }

  Grabbing_ = true;

  Camera<unsigned short> bayerImage;
  res = ImageIO::Load(FileNames_[ActiveFrame_], bayerImage);
  bayerImage.SetColorModel(ColorMode_);
  ImageConvert::ToRGB(bayerImage,image);

  if (res<0) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): Unable to read image: "<<FileNames_[ActiveFrame_]);
    return res;
  }

  // set time to _now_
  if (FrameCounterAsTimeStamp_) image.SetTime(ActiveFrame_,0);
  else {
    timeval tv;
    gettimeofday(&tv, NULL);
    image.SetTime(tv.tv_sec,tv.tv_usec);
  }

  //set meta data from loaded image, if activated
  if (bUseMetaData_) image.ParseMetaData();

  if(ActiveFrame_<FileNames_.size()-1){
    BIASDOUT(D_VSO_DISK,"GrabSingle() "<<ActiveFrame_<<"  "<<
             FileNames_[ActiveFrame_]);
    if(bReversePlaying_){
      if(ActiveFrame_>0)
        ActiveFrame_--;
      else
        {
          bReversePlaying_=false;
          ActiveFrame_++;
        }
    }
    else
      ActiveFrame_++;
  }
  else if (Loop_) {
    ActiveFrame_=0;
    BIASDOUT(D_VSO_DISK,"GrabSingle() restarting with first file");
  }
  else if(bReverse_){
    bReversePlaying_=true;
    ActiveFrame_--;
  }
  else ActiveFrame_++;

  Grabbing_ = false;
  BIASDOUT(D_VSO_DISK,"leaving GrabSingle()");
  // check time and sleep while enough time elapsed
  double secPerFrame=1.0/FramesPerSecond_;

  //struct timeval now;         -- displaced by TimeMeasure
  //double secs=0.0;            --         -"-
  //while (secs<=secPerFrame) { --         -"-
  if(KeepFPS_) {
    timer_.Stop();

    if (FramesPerSecond_ != 0)
    {
      while (timer_.GetRealTime()<=(1E06*secPerFrame))
      {
        timer_.Start();
#ifdef   WIN32
        Sleep(1);        //go sleeping for 1 msec
#else  //not WIN32
        usleep(1000);     //go sleeping for 1 msec
#endif //WIN32
        timer_.Stop();
      }
    }
    timer_.Reset();
    timer_.Start();
  }
  if (GenerateNewUIDs_) {
    image.SetUID(BIAS::UUID::GenerateUUID());
  }
  //lastTime_=now;   -- displaced by TimeMeasure

  return 0;
}
#endif

int VideoSource_Disk_Bayer::GrabSingle(ImageBase &image)
{
  BIASDOUT(D_VSO_DISK,"VideoSource_Disk_Bayer::GrabSingle()");

  if (ActiveFrame_ == FileNames_.size()){
    BIASERR("VideoSource_Disk_Bayer: End of input. SetLoop(true) or Rewind()");
    return -1;
  }
  int res;
  if (!Active_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object not active, use PreGrab()");
    return -1;
  }

  if (Grabbing_) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): VideoSource Object is already grabbing()");
    return -1;
  }

  Grabbing_ = true;

  ImageBase bayerImage;
  res = ImageIO::Load(FileNames_[ActiveFrame_], bayerImage);
  bayerImage.SetColorModel(ColorMode_);
  ImageConvert::Convert (bayerImage, image, ImageBase::CM_RGB, bayerImage.IsPlanar());



  if (res<0) {
    BIASERR("VideoSource_Disk_Bayer::GrabSingle(): Unable to read image: "<<FileNames_[ActiveFrame_]);
    return res;
  }

  if(ActiveFrame_<FileNames_.size()-1){
    BIASDOUT(D_VSO_DISK,"GrabSingle() "<<ActiveFrame_<<"  "<<
             FileNames_[ActiveFrame_]);
    if(bReversePlaying_){
      if(ActiveFrame_>0)
        ActiveFrame_--;
      else
        {
          bReversePlaying_=false;
          ActiveFrame_++;
        }
    }
    else
      ActiveFrame_++;
  }
  else if (Loop_) {
    ActiveFrame_=0;
    BIASDOUT(D_VSO_DISK,"GrabSingle() restarting with first file");
  }
  else if(bReverse_){
    bReversePlaying_=true;
    ActiveFrame_--;
  }
  else ActiveFrame_++;

  Grabbing_ = false;
  BIASDOUT(D_VSO_DISK,"leaving GrabSingle()");
  // check time and sleep while enough time elapsed
  double secPerFrame=1.0/FramesPerSecond_;

  //struct timeval now;         -- displaced by TimeMeasure
  //double secs=0.0;            --         -"-
  //while (secs<=secPerFrame) { --         -"-
  if(KeepFPS_) {
    timer_.Stop();

    if (FramesPerSecond_ != 0)
    {
      while (timer_.GetRealTime()<=(1E06*secPerFrame))
      {
        timer_.Start();
#ifdef   WIN32
        Sleep(1);        //go sleeping for 1 msec
#else  //not WIN32
        usleep(1000);     //go sleeping for 1 msec
#endif //WIN32
        timer_.Stop();
      }
    }
    timer_.Reset();
    timer_.Start();
  }
    if (GenerateNewUIDs_) {

    image.SetUID(BIAS::UUID::GenerateUUID());
  }
  //lastTime_=now;   -- displaced by TimeMeasure

  return 0;
}





