/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __BIASVIDEOSOURCE_SHMPMD_HH__
#define __BIASVIDEOSOURCE_SHMPMD_HH__



#include <iostream>
#include <sstream>
#include <VideoSource/VideoSource_Shm.hh>

#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#  include <windows.h>
#endif


namespace BIAS{



  /**
     \class VideoSource_ShmPMD
     \ingroup g_videosource
     \author Birger Streckel
     \brief This class implements a video streaming client using shared memory to 
      receive images from another application. It also transfers Depth and 
      Amplitude data coming from a PMD camera.
  */
  class BIASVideoSource_EXPORT VideoSource_ShmPMD : public VideoSource_Shm {

  public: 
 
    VideoSource_ShmPMD();
    ~VideoSource_ShmPMD();
    int GrabSingle(BIAS::Camera <unsigned char> &image);

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleDepth(Camera<float> &image)
    { 
      if (!DepthImg_.IsEmpty()) {
        image = DepthImg_; 
        return 0;
      } 
      return -1;
    }

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleModCoeff(Camera<float> &image)
    {       
      if (!ModCoeffImg_.IsEmpty()) {
        image = ModCoeffImg_; 
        return 0;
      } 
      return -1;
    }

 
 protected:
    /*!\brief prohibit default function (move to 'public' if needed) */
  VideoSource_ShmPMD(const VideoSource_ShmPMD &source);
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoSource_ShmPMD& operator =(const VideoSource_ShmPMD &source);

 BIAS::Camera<float> DepthImg_, ModCoeffImg_;

  };


} // namespace MIP

#endif
