#include "VideoSource_uEye.hh"
#include <Base/Image/ImageIO.hh>
#include <stdio.h>
#include <Base/Image/ImageBase.hh>

using namespace BIAS;
using namespace std;




VideoSource_uEye::VideoSource_uEye(): VideoSource()
{
  //set to the defaults of our ueye
  VideoSource::SetSize(752,480,1);
  VideoSource::SetColorModel(BIAS::ImageBase::CM_Grey);
  IsControllable_ = true;

  m_bIsInited    = false;
  m_fGain =0.0;
  m_fShutter = 0.00;

  //set for cutout
  m_dWidth      = 752;
  m_dHeight     = 480;
  m_dDepth      = 1;

  m_dPixelClock  =0;

  m_dLeftPos    = 0;
  m_dTopPos     = 0;
 
  m_dBitsPerPixel = 8;
  m_dMemID        = 0;
  m_dColorMode    = 0;
  m_CameraHandle  = 0;
 
  m_dSrcWidth     = 752;
  m_dSrcHeight    = 480;
  m_dSrcDepth     = 1;
 
  m_pImageBuffer  = NULL;
  m_pSwapBuffer   = NULL;
  
  minContrast_ = 0; //?
  maxContrast_ = 40;
  minGain_=0; //db
  maxGain_ = 56;
  minShutter_ = 0; //us
  maxShutter_ = 1000;
}



VideoSource_uEye::~VideoSource_uEye()
{

}

int VideoSource_uEye::InitImage(BIAS::ImageBase &Image)
{
    Image.Init(GetWidth(),GetHeight(),GetColorChannels());
    return 0;
}

void VideoSource_uEye::SetSize(int w, int h, int bytesperpixel)
{
  if( (w!=0) && (h!=0) && (bytesperpixel!=0))
  {  
    VideoSource::SetSize(w,h,bytesperpixel);
  }
}

int VideoSource_uEye::OpenDevice()
{
  int pNumCameras = 0;
  int ret = -1;
  ret = m_ueye.GetNumberOfCameras(&pNumCameras);
  if(pNumCameras <= 0 || ret != IS_SUCCESS)
    return -1;
  else
  { 
   ret = OpenCamera_();
   if(ret != IS_SUCCESS)
     BIASERR("Open Camera failed!");
  }

  return ret;
}


int VideoSource_uEye::CloseDevice()
{
  int ret =0;
  m_ueye.EnableMessage(IS_FRAME , NULL);
	m_ueye.StopLiveVideo(IS_FORCE_VIDEO_STOP );
	m_ueye.ExitCamera();
  m_ueye.FreeImageMem( m_pImageBuffer,m_dMemID);
  ret = m_ueye.ExitCamera();
 // if(m_pSwapBuffer!=NULL)
  //  delete m_pSwapBuffer;
  return ret;
}

//Shutter is given in seconds.
int VideoSource_uEye::SetShutter(float exptime)
{
  // we need milliseconds here
  exptime *= 100.0;
  double newexptime = 0.0;
  int ret = -1;
  m_fShutter = exptime;

  if(m_bIsInited)
    ret = m_ueye.SetExposureTime((double)exptime,&newexptime);

  return ret;
}

float VideoSource_uEye::GetShutter()
{
  float shutter=0.0;
  if(m_bIsInited)
    shutter = (float)m_ueye.GetExposureTime();
  else
    shutter = 0.0;
  return shutter/100.0f;
}

int VideoSource_uEye::SetGain(float g)
{
  int ret =-1;
  m_fGain = g;
  if(m_bIsInited)
    ret = m_ueye.SetGain((double)g);
  return ret;
}

float VideoSource_uEye::GetGain()
{
  float gain=0.0;
  if(m_bIsInited)
    gain = (float)m_ueye.GetGain();
  else
    gain = 0.0;
  return gain;
}

 void VideoSource_uEye::SetFPS(float fps)
{
  double newfps;
  m_ueye.SetFrameRate((double)fps,&newfps);
}

int VideoSource_uEye::GrabSingle(BIAS::Camera <unsigned char> &image)
{ 
  int ret = -1;
  ret = m_ueye.FreezeVideo(IS_WAIT);
  //ret = m_ueye.GetActiveImageMem((char **)&m_pSwapBuffer,&m_dMemID);
  ret = m_ueye.GetActSeqBuffer((char **)&m_pSwapBuffer,
                               &m_dMemID);

  if(ret == IS_SUCCESS){
    image.CopyIn_NoInit(m_pSwapBuffer);
    image.SetUID(BIAS::UUID::GenerateUUID());
  }
  return ret;
}


/**
   @brief Disable:mode = 0 
   Weak: mode =1 
   Strong mode =2
   @author ischiller
*/
int VideoSource_uEye::SetEdgeEnhancement(int mode){

  if(mode == 0){
    mode = IS_EDGE_EN_DISABLE;
  } 
  else if(mode == 1){
    mode = IS_EDGE_EN_WEAK;
  } 
  else if(mode == 1){
    mode = IS_EDGE_EN_STRONG;
  } 
  else
    mode = IS_EDGE_EN_DISABLE;

 
  return m_ueye.SetEdgeEnhancement(mode);
}


int VideoSource_uEye::PreGrab()
{
    return 0;
}


int VideoSource_uEye::PostGrab()
{
	return 0;
}

int VideoSource_uEye::SetLeftTop(unsigned int left, unsigned int top)
{
  int ret = -1;
  m_dLeftPos = left;
  m_dTopPos = top;
  if(m_dLeftPos != 0 || m_dTopPos != 0)
  {
    if(m_dLeftPos <0) m_dLeftPos = 0;
    if(m_dTopPos < 0) m_dTopPos = 0;
    ret = m_ueye.SetImagePos(m_dLeftPos,m_dTopPos);
  }
  return 0;
}

int VideoSource_uEye::SetPixelClock(int dClock)
{
  m_dPixelClock = dClock;
  m_ueye.SetPixelClock(m_dPixelClock);
  return 0;
}

int VideoSource_uEye::OpenCamera_()
{
	int dRet = IS_SUCCESS;
  m_ueye.EnableMessage(IS_FRAME , NULL);
  m_ueye.ExitCamera();
		
	// open next camera
  dRet = m_ueye.InitCamera(0);		// init cam
  m_bIsInited = true;

	// continue when camera is sucessfully initialized
	if( dRet == IS_SUCCESS )
	{
    SENSORINFO sensorInfo;
	  dRet = m_ueye.GetSensorInfo(&sensorInfo );

		// enable the dialog based error report
		if( dRet != IS_SUCCESS )
		{
			BIASERR( "ERROR: Can not enable the automatic uEye error report!" );
			return false;
		}

  	// uEye setup
		// setup the color depth to the current windows setting
    m_dBitsPerPixel =8;
    m_ueye.SetColorMode(IS_SET_CM_Y8);  
    
    m_ueye.SetDisplayMode(IS_SET_DM_DIB);

    m_dSrcWidth = sensorInfo.nMaxWidth;
    m_dSrcHeight = sensorInfo.nMaxHeight;
    m_dSrcWidth &= 0xFFFC;
    m_dSrcHeight &= 0xFFFC;

    m_dWidth = (unsigned int)GetWidth();
    m_dHeight = (unsigned int)GetHeight();
    m_dDepth = (unsigned int)GetColorChannels();

    if(m_dSrcWidth != m_dWidth || m_dSrcHeight !=  m_dHeight)
    {
      dRet = m_ueye.SetImageSize(m_dWidth,m_dHeight);    
    }
    if(m_dLeftPos != 0 || m_dTopPos != 0)
    {
      if(m_dLeftPos <0) m_dLeftPos = 0;
      if(m_dTopPos < 0) m_dTopPos = 0;
      dRet = m_ueye.SetImagePos(m_dLeftPos,m_dTopPos);
    }
    //Set pixelclock
    if(m_dPixelClock != 0)
      m_ueye.SetPixelClock(m_dPixelClock);
    //set gain
    if( m_fGain != 0.0)
      m_ueye.SetGain(m_fGain);
    //set shutter
    double newexptime = 0.0;
    if( m_fShutter != 0.0)
      m_ueye.SetExposureTime(m_fShutter,&newexptime);
   
    //allocate the image buffer
    dRet = m_ueye.AllocImageMem(m_dWidth, m_dHeight, m_dBitsPerPixel, 
                                &m_pImageBuffer, &m_dMemID);
    //set the image mem
    dRet = m_ueye.SetImageMem(m_pImageBuffer, m_dMemID);
  	
    //set white balance -> Failes for our camera
    dRet = m_ueye.SetWhiteBalance(IS_SET_WB_AUTO_ENABLE);
    // start live video
    dRet = m_ueye.CaptureVideo(IS_FORCE_VIDEO_START);
  
		return dRet;
	}
	else
	{
		BIASERR("No uEye camera could be opened !");
  }

	return IS_CANT_OPEN_DEVICE;
}

